/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef GSCULPTAPPCPP
#define GSCULPTAPPCPP

#include "GSculptApp.h"


#include <gtk/gtkmarshal.h>

#include <gtk/gtksignal.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtktable.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkfilesel.h>



#include "fileio.h"

#include "messageBox.h"


#include "gSculptConfig.h"





GSculptApp::GSculptApp(GtkSignalFunc closeCallbackFunc,
							  gpointer closeCallbackData)
{
	modellerLockFlag = false;
	unsavedDataFlag = false;
	unsavedWarningOpen = false;

	closeFunc = closeCallbackFunc;
	closeData = closeCallbackData;

	fileName = "";





	objectCollection = new ObjectCollection();


	GtkAttachOptions xOpt = (GtkAttachOptions)(GTK_FILL | GTK_EXPAND);
	GtkAttachOptions yOpt = (GtkAttachOptions)(GTK_FILL | GTK_EXPAND);



	titleLabel = gtk_label_new(
		( string("gSculpt v") + getVersionAsString() ).c_str() );
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);



	modellerButton = gtk_button_new_with_label("Modeller");
	gtk_widget_ref(modellerButton);
	gtk_widget_show(modellerButton);
	gtk_signal_connect( GTK_OBJECT(modellerButton), "clicked",
							  GTK_SIGNAL_FUNC(modellerButtonClicked), this );

	
	openButton = gtk_button_new_with_label("Open");
	gtk_widget_ref(openButton);
	gtk_widget_show(openButton);
	gtk_signal_connect( GTK_OBJECT(openButton), "clicked",
							  GTK_SIGNAL_FUNC(openButtonClicked), this );

	saveButton = gtk_button_new_with_label("Save");
	gtk_widget_ref(saveButton);
	gtk_widget_show(saveButton);
	gtk_signal_connect( GTK_OBJECT(saveButton), "clicked",
							  GTK_SIGNAL_FUNC(saveButtonClicked), this );

	saveAsButton = gtk_button_new_with_label("Save As");
	gtk_widget_ref(saveAsButton);
	gtk_widget_show(saveAsButton);
	gtk_signal_connect( GTK_OBJECT(saveAsButton), "clicked",
							  GTK_SIGNAL_FUNC(saveAsButtonClicked), this );

	exportExplicitButton = gtk_button_new_with_label("Export Explicit");
	gtk_widget_ref(exportExplicitButton);
	gtk_widget_show(exportExplicitButton);
	gtk_signal_connect( GTK_OBJECT(exportExplicitButton), "clicked",
							  GTK_SIGNAL_FUNC(exportExplicitButtonClicked), this );

	quitButton = gtk_button_new_with_label("Quit");
	gtk_widget_ref(quitButton);
	gtk_widget_show(quitButton);
	gtk_signal_connect( GTK_OBJECT(quitButton), "clicked",
							  GTK_SIGNAL_FUNC(quitButtonClicked), this );




	copyrightLabel = gtk_label_new("(C)Copyright Geoffrey French 1999 - 2001");
	gtk_widget_ref(copyrightLabel);
	gtk_widget_show(copyrightLabel);

	




	table = gtk_table_new(17, 7, true);
	gtk_widget_ref(table);

	gtk_table_attach( GTK_TABLE(table), titleLabel, 1, 6, 1, 2,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), modellerButton, 3, 4, 4, 5,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), openButton, 2, 3, 6, 7,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), saveButton, 3, 4, 6, 7,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), saveAsButton, 4, 5, 6, 7,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), exportExplicitButton, 3, 4, 8, 9,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), quitButton, 3, 4, 11, 12,
							xOpt, yOpt, 0, 0 );
	gtk_table_attach( GTK_TABLE(table), copyrightLabel, 1, 6, 14, 15,
							xOpt, yOpt, 0, 0 );

	gtk_widget_show(table);



	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_ref(window);
	gtk_container_add( GTK_CONTAINER(window), table );
	gtk_window_set_title( GTK_WINDOW(window),
			( string("gSculpt v") + getVersionAsString() ).c_str() );

	gtk_signal_connect( GTK_OBJECT(window), "delete_event",
							  GTK_SIGNAL_FUNC(windowDelete), this );
	gtk_signal_connect( GTK_OBJECT(window), "destroy",
							  GTK_SIGNAL_FUNC(windowDestroy), this );

	gtk_widget_show(window);		
}


GSculptApp::~GSculptApp()
{
	gtk_widget_unref(titleLabel);
	gtk_widget_unref(modellerButton);
	gtk_widget_unref(openButton);
	gtk_widget_unref(saveButton);
	gtk_widget_unref(saveAsButton);
	gtk_widget_unref(exportExplicitButton);
	gtk_widget_unref(quitButton);
	gtk_widget_unref(copyrightLabel);

	gtk_widget_unref(window);
	gtk_widget_unref(table);


	delete objectCollection;
}



void GSculptApp::modellerButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	if ( !klass->modellerLockFlag )
	{
		klass->modellerLockFlag = true;
		klass->unsavedDataFlag = true;
		
		klass->modellerWindow =
			new CObjectEditor( "gSculpt object modeller", klass->objectCollection,
									 GTK_SIGNAL_FUNC(modellerClosed), klass );

		klass->modellerWindow->run();
	}
	else
	{
		messageBox( "Could not start modeller", "Modeller window already open" );
	}		
}

void GSculptApp::modellerClosed(CObjectEditor *ed, GSculptApp *klass)
{
	delete klass->modellerWindow;

	klass->modellerLockFlag = false;
}


void GSculptApp::openButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	if ( klass->unsavedDataFlag )
	{
		klass->unsavedDataWarning( GTK_SIGNAL_FUNC(open), "Open" );
	}
	else
	{
		open( widget, klass );
	}
}

void GSculptApp::saveButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	klass->save();
}

void GSculptApp::saveAsButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	klass->saveAs();
}

void GSculptApp::exportExplicitButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	klass->exportExplicit();
}

void GSculptApp::quitButtonClicked(GtkWidget *widget, GSculptApp *klass)
{
	if ( klass->unsavedDataFlag )
	{
		klass->unsavedDataWarning( GTK_SIGNAL_FUNC(quit), "Quit" );
	}
	else
	{
		gtk_widget_destroy( klass->window );
	}
}


gint GSculptApp::windowDelete(GtkWidget *widget, GdkEvent *event,
										GSculptApp *klass)
{
	if ( klass->unsavedDataFlag )
	{
		klass->unsavedDataWarning( GTK_SIGNAL_FUNC(quit), "Quit" );
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


void GSculptApp::windowDestroy(GtkWidget *widget, GSculptApp *klass)
{
	GtkSignalFunc f = klass->closeFunc;

	gtk_marshal_NONE__NONE( (GtkObject*)klass, f, klass->closeData,
									(GtkArg*)NULL );
}


void GSculptApp::unsavedDataWarning(GtkSignalFunc responseFunc,
												const string& function)
{
	if ( unsavedWarningOpen )
	{
		return;
	}



	GtkWidget *unsDialog, *unsMessage,
				 *unsQuitButton, *unsCancelButton;
	

	unsDialog = gtk_dialog_new();
	gtk_window_set_title( GTK_WINDOW(unsDialog), "Warning" );


	unsMessage = gtk_label_new("You have not saved your work.");
	gtk_widget_show(unsMessage);

	unsQuitButton = gtk_button_new_with_label( function.c_str() );
	gtk_widget_show(unsQuitButton);
	gtk_signal_connect( GTK_OBJECT(unsQuitButton), "clicked",
							  GTK_SIGNAL_FUNC(unsavedClosed), this );
	gtk_signal_connect( GTK_OBJECT(unsQuitButton), "clicked",
							  responseFunc, this );
	gtk_signal_connect_object( GTK_OBJECT(unsQuitButton), "clicked",
										GTK_SIGNAL_FUNC(gtk_widget_destroy),
										GTK_OBJECT(unsDialog) );

	unsCancelButton = gtk_button_new_with_label("Cancel");
	gtk_widget_show(unsCancelButton);
	gtk_signal_connect( GTK_OBJECT(unsCancelButton), "clicked",
							  GTK_SIGNAL_FUNC(unsavedClosed), this );
	gtk_signal_connect_object( GTK_OBJECT(unsCancelButton), "clicked",
										GTK_SIGNAL_FUNC(gtk_widget_destroy),
										GTK_OBJECT(unsDialog) );




	gtk_box_pack_start( GTK_BOX( GTK_DIALOG(unsDialog)->vbox ),
							  unsMessage, true, true, 0 );


	gtk_box_pack_start( GTK_BOX( GTK_DIALOG(unsDialog)->action_area ),
							  unsQuitButton, true, true, 20 );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG(unsDialog)->action_area ),
							  unsCancelButton, true, true, 20 );


	gtk_widget_show(unsDialog);

	unsavedWarningOpen = true;
}


void GSculptApp::unsavedClosed(GtkWidget *widget, GSculptApp *klass)
{
	klass->unsavedWarningOpen = false;
}

void GSculptApp::quit(GtkWidget *widget, GSculptApp *klass)
{
	gtk_widget_destroy( klass->window );
}



void GSculptApp::fileSelection(const string& fileSelectionTitle,
										 GtkSignalFunc okResponse,
										 GtkSignalFunc cancelResponse)
{
	GtkWidget *fileSelection;


	fileSelection = gtk_file_selection_new( fileSelectionTitle.c_str() );
	gtk_widget_show(fileSelection);



	FileSelectionStruct *fs = new FileSelectionStruct;
	fs->widget = fileSelection;
	fs->klass = this;




	if ( okResponse != NULL )
	{
		gtk_signal_connect(
			GTK_OBJECT( GTK_FILE_SELECTION(fileSelection)->ok_button ),
			"clicked", okResponse, fs );
	}
	gtk_signal_connect_object(
				GTK_OBJECT( GTK_FILE_SELECTION(fileSelection)->ok_button ),
				"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(fileSelection) );
	if ( cancelResponse != NULL )
	{
		gtk_signal_connect(
			GTK_OBJECT( GTK_FILE_SELECTION(fileSelection)->cancel_button ),
			"clicked", okResponse, fs );
	}
	gtk_signal_connect_object(
				GTK_OBJECT( GTK_FILE_SELECTION(fileSelection)->cancel_button ),
				"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(fileSelection) );
	gtk_signal_connect( GTK_OBJECT(fileSelection), "destroy",
							  GTK_SIGNAL_FUNC(fileSelectionDestroy), fs );
}



void GSculptApp::fileSelectionDestroy(GtkWidget *widget,
												  FileSelectionStruct *fs)
{
	delete fs;
}


void GSculptApp::open(GtkWidget *widget, GSculptApp *klass)
{
	klass->fileSelection( "Open", GTK_SIGNAL_FUNC(openFileOk), NULL );
}


void GSculptApp::openFileOk(GtkWidget *widget, FileSelectionStruct *fs)
{
	GSculptApp *klass = fs->klass;
	
	klass->fileName = gtk_file_selection_get_filename(
											GTK_FILE_SELECTION(fs->widget) );
	gtk_widget_destroy(fs->widget);

	FILE *f = fopen( klass->fileName.c_str(), "rb" );
	if ( !f )
	{
		messageBox("Error", "Could not open file for reading");
		return;
	}

	klass->readFile(f);

	fclose(f);
}



void GSculptApp::save()
{
	if ( fileName == "" )
	{
		saveAs();
	}
	else
	{
		writeToFileWithName( fileName, SAVETYPE_SAVE );
	}
}

void GSculptApp::saveAs()
{
	fileSelection( "Save As", GTK_SIGNAL_FUNC(saveFileOk), NULL );
}

void GSculptApp::exportExplicit()
{
	fileSelection( "Export Explicit", GTK_SIGNAL_FUNC(exportExplicitOk), NULL );
}





void GSculptApp::saveFileOk(GtkWidget *widget, FileSelectionStruct *fs)
{
	GSculptApp *klass = fs->klass;

	klass->fileName = gtk_file_selection_get_filename(
											GTK_FILE_SELECTION(fs->widget) );
	gtk_widget_destroy(fs->widget);

	klass->writeToFileWithName( klass->fileName, SAVETYPE_SAVE );
}


void GSculptApp::exportExplicitOk(GtkWidget *widget, FileSelectionStruct *fs)
{
	GSculptApp *klass = fs->klass;

	klass->fileName = gtk_file_selection_get_filename(
											GTK_FILE_SELECTION(fs->widget) );
	gtk_widget_destroy(fs->widget);

	klass->writeToFileWithName( klass->fileName, SAVETYPE_EXPORTDIRECT );
}






void GSculptApp::writeToFileWithName(const string& name, SaveType saveType)
{
	FILE *f = fopen(name.c_str(), "wb");
	if ( !f )
	{
		messageBox("Error", "Could not open file for writing");
		return;
	}

	switch(saveType)
	{
	case SAVETYPE_SAVE:
		writeToFile(f);
		break;
	case SAVETYPE_EXPORTDIRECT:
		exportExplicitToFile(f);
		break;
	default:
		break;
	}

	fclose(f);
}


void GSculptApp::readFile(FILE *f)
{
	char fileHeader[4];
	int fileVersion[4];

	fread( fileHeader, sizeof(char), 4, f );

	if ( !checkFileHeader( fileHeader ) )
	{
		messageBox("Could not read", "Not a gSculpt file");
		return;
	}

	fread(fileVersion, sizeof(int), 4, f);


	if ( !checkVersionCompatibility( fileVersion, getVersionAsInts() ) )
	{
		messageBox("Could not read", "Incompatible gSculpt version");
		return;
	}


	string gSculptFileType = readString(f);


	if ( gSculptFileType == "Generative" )
	{
		objectCollection->read(f);
	}
	else
	{
		messageBox("Could not read", "Can only read generative model files");
		return;
	}
	
	
	unsavedDataFlag = false;
}

void GSculptApp::writeToFile(FILE *f)
{
	char *fileHeader = "gSc\0";

	fwrite( fileHeader, sizeof(char), 4, f );
	fwrite( getVersionAsInts(), sizeof(int), 4, f );


	writeString( f, "Generative" );


	objectCollection->write(f);

	
	
	unsavedDataFlag = false;
}

void GSculptApp::exportExplicitToFile(FILE *f)
{
	char *fileHeader = "gSc\0";

	fwrite( fileHeader, sizeof(char), 4, f );
	fwrite( getVersionAsInts(), sizeof(int), 4, f );


	writeString( f, "ExplicitModel" );

	objectCollection->exportExplicit(f);

	
	
	unsavedDataFlag = false;
}







#endif
