/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by:					Geoffrey French
	application:		gSculpt
	last revision:		16/Apr/2000
*/
#ifndef __GSCULPTAPP_H
#define __GSCULPTAPP_H

#include <gtk/gtkwidget.h>

#include "CObjectEditor.h"
#include "ObjectCollection.h"




class GSculptApp
{
private:
	struct FileSelectionStruct {
		GtkWidget *widget;
		GSculptApp *klass;
	};

	typedef enum {
		SAVETYPE_SAVE,
		SAVETYPE_EXPORTDIRECT
	} SaveType;
	
	
	GtkWidget *window, *table;

	GtkWidget *titleLabel;

	GtkWidget *modellerButton;
	GtkWidget *openButton, *saveButton, *saveAsButton;
	GtkWidget *exportExplicitButton;
	GtkWidget *quitButton;

	GtkWidget *copyrightLabel;

	bool modellerLockFlag;
	bool unsavedDataFlag;
	bool unsavedWarningOpen;



	ObjectCollection *objectCollection;
	CObjectEditor *modellerWindow;



	GtkSignalFunc closeFunc;
	gpointer closeData;


	string fileName;


public:
	GSculptApp(GtkSignalFunc closeCallbackFunc,
				  gpointer closeCallbackData);
	~GSculptApp();

	void lockModeller();
	void unlockModeller();




private:
	static void modellerButtonClicked(GtkWidget *widget, GSculptApp *klass);
	static void modellerClosed(CObjectEditor *ed, GSculptApp *klass);

	static void openButtonClicked(GtkWidget *widget, GSculptApp *klass);
	static void saveButtonClicked(GtkWidget *widget, GSculptApp *klass);
	static void saveAsButtonClicked(GtkWidget *widget, GSculptApp *klass);
	static void exportExplicitButtonClicked(GtkWidget *widget, GSculptApp *klass);
	static void quitButtonClicked(GtkWidget *widget, GSculptApp *klass);

	static gint windowDelete(GtkWidget *widget, GdkEvent *event,
									 GSculptApp *klass);
	static void windowDestroy(GtkWidget *widget, GSculptApp *klass);


	void unsavedDataWarning(GtkSignalFunc responseFunc,
									const string& function);
	static void unsavedClosed(GtkWidget *widget, GSculptApp *klass);


	static void quit(GtkWidget *widget, GSculptApp *klass);


	void fileSelection(const string& fileSelectionTitle,
						 	 GtkSignalFunc okResponse,
						 	 GtkSignalFunc cancelResponse);

	static void fileSelectionDestroy(GtkWidget *widget,
												FileSelectionStruct *fs);

	static void open(GtkWidget *widget, GSculptApp *klass);
	static void openFileOk(GtkWidget *widget, FileSelectionStruct *fs);


	void save();
	void saveAs();
	void exportExplicit();


	static void saveFileOk(GtkWidget *widget, FileSelectionStruct *fs);
	static void exportExplicitOk(GtkWidget *widget, FileSelectionStruct *fs);

	void writeToFileWithName(const string& name, SaveType saveType);
	void readFile(FILE *f);
	void writeToFile(FILE *f);
	void exportExplicitToFile(FILE *f);
};




#endif

