/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef TRIANGULATECPP
#define TRIANGULATECPP

#include "Triangulate.h"

#include "CompGeometry.h"


bool isEar(const array<Point3>& vertices, int vertexA, int vertexB, int vertexC)
{
	Point3 pa = vertices[vertexA],
			 pb = vertices[vertexB],
			 pc = vertices[vertexC];
	
	for (int i = 0; i < vertices.size(); i++)
	{
		if (i == vertexA || i == vertexB || i == vertexC)
		{
			continue;
		}

		if ( triangleContainsPoint(pa, pb, pc, vertices[i]) )
		{
			return false;
		}
	}

	return true;
}

struct __triangulate__Vertex {
	Point3 p;
	int n;
};


bool triangulate(const array<Point3>& vertices, array<CGTriangle>& tris)
{
	if (vertices.size() < 3)
	{
		return false;
	}

	array<__triangulate__Vertex> poly;
	poly.reserve( vertices.size() );

	if ( !isPolyClockwise(vertices) )
	{
		for (int i = 0; i < vertices.size(); i++)
		{
			__triangulate__Vertex v;
			int x = (vertices.size() - 1) - i;
			v.p = vertices[x];
			v.n = x;
			poly.push_back(v);
		}
	}
	else
	{
		for (int i = 0; i < vertices.size(); i++)
		{
			__triangulate__Vertex v;
			v.p = vertices[i];
			v.n = i;
			poly.push_back(v);
		}
	}

	int a, b, c;
	bool earFound;


	while (poly.size() > 2)
	{
		double area;

		earFound = false;
		a = poly.size() - 2;
		b = poly.size() - 1;
		for (c = 0; c < poly.size(); c++)
		{
			area = areaOfTriangle( poly[a].p, poly[b].p, poly[c].p );

			if (area == 0.0)
			{
				return false;
			}

			if ( isTriangleClockwise( poly[a].p, poly[b].p, poly[c].p ) )
			{
				if (isEar( vertices, poly[a].n, poly[b].n, poly[c].n ))
				{
					CGTriangle t;

					t.a = poly[a].n;
					t.b = poly[b].n;
					t.c = poly[c].n;

					tris.push_back(t);
			
					poly.erase(b);

					earFound = true;
					
					break;
				}
			}
			a = b;
			b = c;
		}

		if (!earFound)
		{
			return false;
		}
	}

	return true;
}



#endif

