/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCOPY_CPP
#define __TOOLCOPY_CPP

#include <strstream>
#include <math.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkclist.h>


#include "ToolCopy.h"

#include "JournalMesh.h"
#include "CMesh.h"

#include "messageBox.h"

#include "Point3.h"





void* __create_ProcCopy()
{
	return (void*)( new ProcCopy() );
}

ObjectType type_ProcCopy( "ProcCopy", "JournalProcess",
								  __create_ProcCopy );




ProcCopy::ProcCopy(UniqueId source) : JournalProcess()
{
	sourceObjectId = source;
	setSourceObject();
}

ProcCopy::~ProcCopy()
{
}



void ProcCopy::read(FILE *f)
{
	JournalProcess::read(f);

	unsetSourceObject();

	fread( &sourceObjectId, sizeof(UniqueId), 1, f );

	setSourceObject();
}

void ProcCopy::write(FILE *f)
{
	JournalProcess::write(f);

	fwrite( &sourceObjectId, sizeof(UniqueId), 1, f );
}





string ProcCopy::getDescription() const
{
	return "Copy";
}

const ObjectType* ProcCopy::getType() const
{
	return &type_ProcCopy;
}

const ObjectType* ProcCopy::getTypeStatic()
{
	return &type_ProcCopy;
}



const ObjectType* ProcCopy::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* ProcCopy::clone() const
{
	return new ProcCopy( sourceObjectId );
}



JournalCollection* ProcCopy::getJournalCollection()
{
	Journal *journ = (Journal*)getOwner();

	if ( journ != NULL )
	{
		return journ->getOwner();
	}
	else
	{
		return NULL;
	}
}

const JournalCollection* ProcCopy::getJournalCollection() const
{
	const Journal *journ = (Journal*)getOwner();

	if ( journ != NULL )
	{
		return journ->getOwner();
	}
	else
	{
		return NULL;
	}
}


UniqueId ProcCopy::getSourceObjectId() const
{
	return sourceObjectId;
}

	
void ProcCopy::setSourceObjectId(UniqueId i)
{
	unsetSourceObject();
	sourceObjectId = i;
	setSourceObject();
	modified();
}


UniqueId ProcCopy::getMyObjectId() const
{
	const Journal *journ = (Journal*)getOwner();
	const JournalCollection *coll = getJournalCollection();

	if ( coll != NULL )
	{
		return coll->getUniqueId(journ);
	}
	else
	{
		return -1;
	}
}


const ObjectType* ProcCopy::_getOutputType(const ObjectType* inputType) const
{
	const Journal *journ = getSourceJournal();

	if ( journ != NULL )
	{
		return journ->getResultType();
	}
	else
	{
		return JournalProcess::getTypeStatic();
	}
}

JournalObject* ProcCopy::_process(JournalObject *o,
											 JournalTaskProgress *progress)
{
	delete o;

	Journal *journ = getSourceJournal();

	if ( journ != NULL )
	{
		JournalTaskProgress nestedProgress;
		JournalObject *result;

		if ( progress != NULL )
		{
			progress->lock();
			progress->setNestedTask( &nestedProgress );
			progress->unlock();
			result = journ->getResultUnthreaded( &nestedProgress );
		}
		else
		{
			result = journ->getResultUnthreaded(NULL);
		}

		if ( progress != NULL )
		{
			progress->lock();
			progress->setNestedTask(NULL);
			progress->unlock();
		}

		return result;
	}
	else
	{
		return new JournalObject();
	}
}


Journal* ProcCopy::getSourceJournal()
{
	JournalCollection *coll = getJournalCollection();
	if ( coll != NULL )
	{
		return coll->getObject(sourceObjectId);
	}
	else
	{
		return NULL;
	}
}

const Journal* ProcCopy::getSourceJournal() const
{
	const JournalCollection *coll = getJournalCollection();
	if ( coll != NULL )
	{
		return coll->getObject(sourceObjectId);
	}
	else
	{
		return NULL;
	}
}

void ProcCopy::setSourceObject()
{
	Journal *journ = getSourceJournal();

	if ( journ != NULL )
	{
		journ->addListener(this);
	}

	UniqueId myId = getMyObjectId();
	if ( myId != -1 )
	{
		JournalCollection *coll = getJournalCollection();
		if ( coll != NULL )
		{
			coll->setDependancy( myId, sourceObjectId );
		}
	}
}
	
void ProcCopy::unsetSourceObject()
{
	Journal *journ = getSourceJournal();

	if ( journ != NULL )
	{
		journ->removeListener(this);
	}

	UniqueId myId = getMyObjectId();
	if ( myId != -1 )
	{
		JournalCollection *coll = getJournalCollection();
		if ( coll != NULL )
		{
			coll->removeDependancy(myId);
		}
	}
}	

void ProcCopy::journalModified(Journal *journ)
{
	modified();
}







//**************************************************


ToolCopy::ToolCopy(CadView *v)
						 : ProcessTool( v, "Copy", "Copy Object", 1 )
{
	objectListLabel = gtk_label_new("Objects:");
	gtk_widget_ref(objectListLabel);
	gtk_widget_show(objectListLabel);


	selectButton = gtk_button_new_with_label("Select");
	gtk_widget_ref(selectButton);
	gtk_widget_show(selectButton);
	gtk_signal_connect( GTK_OBJECT(selectButton), "clicked",
							  GTK_SIGNAL_FUNC(selectClicked), this );


	sourceNameLabel = gtk_label_new("");
	gtk_widget_ref(sourceNameLabel);
	gtk_widget_show(sourceNameLabel);


	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);
	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.add(objectListLabel);
	listPosition = layout.getPosition();
	layout.addAsBlock( NULL, ISize2( 1, 8 ) );
	layout.lineSpace();
	layout.add(selectButton);
	layout.lineSpace();
	layout.add(sourceNameLabel);
	layout.lineSpace();
	layout.addAsLine(doneButton);
}

ToolCopy::~ToolCopy()
{
	gtk_widget_unref(objectListLabel);
	gtk_widget_unref(selectButton);
	gtk_widget_unref(sourceNameLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolCopy::getProcessType() const
{
	return ProcCopy::getTypeStatic();
}




void ToolCopy::initialise()
{
	ProcessTool::initialise();

	if ( hasProcess() )
	{
		ProcCopy *proc = (ProcCopy*)journalProcess;
		ObjectCollection *collection =
				(ObjectCollection*)proc->getJournalCollection();

		objectList = collection->getListManager();

		gtk_widget_show( objectList->getWidget() );


		layout.setPosition(listPosition);
		layout.addAsBlock( objectList->getWidget(), ISize2( 1, 8 ) );	


		string name = collection->getName( proc->getSourceObjectId() );
		gtk_label_set_text( GTK_LABEL(sourceNameLabel), name.c_str() );
	}
}

void ToolCopy::shutdown()
{
	ProcessTool::shutdown();

	ProcCopy *proc = (ProcCopy*)journalProcess;
	ObjectCollection *collection =
			(ObjectCollection*)proc->getJournalCollection();

	layout.remove( objectList->getWidget() );
	collection->putbackListManager(objectList);	
}



void ToolCopy::select()
{
	ProcCopy *proc = (ProcCopy*)journalProcess;
	ObjectCollection *collection =
			(ObjectCollection*)proc->getJournalCollection();

	string choice = objectList->getSelection();
	UniqueId choiceId = collection->getUniqueId(choice);


	if ( collection->hasDependancyPath( proc->getMyObjectId(), choiceId ) )
	{
		messageBox( "Circular dependancy",
			"Performing this operation would result in circular dependancy" );
	}
	else
	{
		gtk_label_set_text( GTK_LABEL(sourceNameLabel), choice.c_str() );
		proc->setSourceObjectId(choiceId);
		modified();
	}
}



void ToolCopy::selectClicked(GtkWidget *widget, ToolCopy *klass)
{
	klass->select();
}

void ToolCopy::doneClicked(GtkWidget *widget, ToolCopy *klass)
{
	klass->done();
}



#endif

