/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCOPY_H
#define __TOOLCOPY_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "Journal.h"
#include "MTool.h"
#include "ProcessTool.h"
#include "NumericEntry.h"
#include "ObjectCollection.h"
#include "ListManager.h"




class ProcCopy : public JournalProcess, public JournalListener
{
private:
	UniqueId sourceObjectId;
	
public:
	ProcCopy(UniqueId source = -1);
	virtual ~ProcCopy();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;


	JournalCollection* getJournalCollection();
	const JournalCollection* getJournalCollection() const;
	UniqueId getSourceObjectId() const;

	void setSourceObjectId(UniqueId i);

	UniqueId getMyObjectId() const;

protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);

	Journal* getSourceJournal();
	const Journal* getSourceJournal() const;
	void setSourceObject();
	void unsetSourceObject();

	virtual void journalModified(Journal *journ);
};




class ToolCopy : public ProcessTool
{
private:
	GtkWidget *objectListLabel;
	ListManager *objectList;	
	GtkWidget *selectButton;
	GtkWidget *sourceNameLabel;
	GtkWidget *doneButton;

	IPoint2 listPosition;

public:
	ToolCopy(CadView *v);
	~ToolCopy();


	virtual const ObjectType* getProcessType() const;



protected:
	virtual void initialise();
	virtual void shutdown();

	void select();


private:
	static void selectClicked(GtkWidget *widget, ToolCopy *klass);
	static void doneClicked(GtkWidget *widget, ToolCopy *klass);
};


#endif
