/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCREATECUBE_CPP
#define __TOOLCREATECUBE_CPP

#include <strstream>
#include <math.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>


#include "ToolCreateCube.h"

#include "JournalMesh.h"
#include "CMesh.h"

#include "Point3.h"





void* __create_ProcCreateCube()
{
	return (void*)( new ProcCreateCube() );
}

ObjectType type_ProcCreateCube( "ProcCreateCube", "ProcAxis",
											__create_ProcCreateCube );




ProcCreateCube::ProcCreateCube() : JournalProcess()
{
	xSegments = ySegments = zSegments = 1;
	width = height = depth = 100.0;
}

ProcCreateCube::ProcCreateCube(int xSegs, int ySegs, int zSegs,
										 double cubeWidth, double cubeHeight,
										 double cubeDepth)
										 : JournalProcess()
{
	xSegments = xSegs;
	ySegments = ySegs;
	zSegments = zSegs;
	width = cubeWidth;
	height = cubeHeight;
	depth = cubeDepth;
}

ProcCreateCube::~ProcCreateCube()
{
}



void ProcCreateCube::read(FILE *f)
{
	JournalProcess::read(f);

	fread(&xSegments, sizeof(int), 1, f);
	fread(&ySegments, sizeof(int), 1, f);
	fread(&zSegments, sizeof(int), 1, f);
	fread(&width, sizeof(double), 1, f);
	fread(&height, sizeof(double), 1, f);
	fread(&depth, sizeof(double), 1, f);
}

void ProcCreateCube::write(FILE *f)
{
	JournalProcess::write(f);

	fwrite(&xSegments, sizeof(int), 1, f);
	fwrite(&ySegments, sizeof(int), 1, f);
	fwrite(&zSegments, sizeof(int), 1, f);
	fwrite(&width, sizeof(double), 1, f);
	fwrite(&height, sizeof(double), 1, f);
	fwrite(&depth, sizeof(double), 1, f);
}





string ProcCreateCube::getDescription() const
{
	return "Cube";
}

const ObjectType* ProcCreateCube::getType() const
{
	return &type_ProcCreateCube;
}

const ObjectType* ProcCreateCube::getTypeStatic()
{
	return &type_ProcCreateCube;
}



const ObjectType* ProcCreateCube::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* ProcCreateCube::clone() const
{
	return new ProcCreateCube(xSegments, ySegments, zSegments,
									  width, height, depth);
}


int ProcCreateCube::getXSegments() const
{
	return xSegments;
}

int ProcCreateCube::getYSegments() const
{
	return ySegments;
}

int ProcCreateCube::getZSegments() const
{
	return zSegments;
}

double ProcCreateCube::getWidth() const
{
	return width;
}

double ProcCreateCube::getHeight() const
{
	return height;
}

double ProcCreateCube::getDepth() const
{
	return depth;
}

	
void ProcCreateCube::setXSegments(int n)
{
	xSegments = n;
	modified();
}

void ProcCreateCube::setYSegments(int n)
{
	ySegments = n;
	modified();
}

void ProcCreateCube::setZSegments(int n)
{
	zSegments = n;
	modified();
}

void ProcCreateCube::setWidth(double w)
{
	width = w;
	modified();
}

void ProcCreateCube::setHeight(double h)
{
	height = h;
	modified();
}

void ProcCreateCube::setDepth(double d)
{
	depth = d;
	modified();
}


const ObjectType* ProcCreateCube::_getOutputType(const ObjectType* inputType) const
{
	return JournalMesh::getTypeStatic();
}

JournalObject* ProcCreateCube::_process(JournalObject *o,
													 JournalTaskProgress *progress)
{
	delete o;

	JournalMesh *jMesh = new JournalMesh();
	CMesh *mesh = jMesh->getMesh();

	vector<CVertex*> vertexPtrs;


	double halfWidth = width * 0.5;
	double halfHeight = height * 0.5;
	double halfDepth = depth * 0.5;

	mesh->addVertex( Point3( -halfWidth, -halfHeight, halfDepth ) );
	mesh->addVertex( Point3( halfWidth, -halfHeight, halfDepth ) );
	mesh->addVertex( Point3( -halfWidth, halfHeight, halfDepth ) );
	mesh->addVertex( Point3( halfWidth, halfHeight, halfDepth ) );
	mesh->addVertex( Point3( -halfWidth, -halfHeight, -halfDepth ) );
	mesh->addVertex( Point3( halfWidth, -halfHeight, -halfDepth ) );
	mesh->addVertex( Point3( -halfWidth, halfHeight, -halfDepth ) );
	mesh->addVertex( Point3( halfWidth, halfHeight, -halfDepth ) );


	vector<int> frontLeftEdge, frontRightEdge, frontTopEdge, frontBottomEdge,
					backLeftEdge, backRightEdge, backTopEdge, backBottomEdge,
					leftTopEdge, leftBottomEdge, rightTopEdge, rightBottomEdge;

	//mesh->vertices();
	mesh->reserveFaces( ( xSegments * ySegments +
								 xSegments * zSegments +
								 ySegments * zSegments ) << 1 );


	createEdge( mesh, 2, 3, xSegments, frontTopEdge );
	createEdge( mesh, 0, 1, xSegments, frontBottomEdge );
	createEdge( mesh, 2, 0, ySegments, frontLeftEdge );
	createEdge( mesh, 3, 1, ySegments, frontRightEdge );
	
	createEdge( mesh, 7, 6, xSegments, backTopEdge );
	createEdge( mesh, 5, 4, xSegments, backBottomEdge );
	createEdge( mesh, 6, 4, ySegments, backLeftEdge );
	createEdge( mesh, 7, 5, ySegments, backRightEdge );

	createEdge( mesh, 6, 2, zSegments, leftTopEdge );
	createEdge( mesh, 4, 0, zSegments, leftBottomEdge );
	createEdge( mesh, 3, 7, zSegments, rightTopEdge );
	createEdge( mesh, 1, 5, zSegments, rightBottomEdge );
	

	mesh->addPatch(6);


	//front
	createSide( mesh, frontTopEdge, frontBottomEdge,
					frontLeftEdge, frontRightEdge, 0 );

	//right
	createSide( mesh, rightTopEdge, rightBottomEdge,
					frontRightEdge, backRightEdge, 1 );

	//back
	createSide( mesh, backTopEdge, backBottomEdge,
					backRightEdge, backLeftEdge, 2 );

	//left
	createSide( mesh, leftTopEdge, leftBottomEdge,
					backLeftEdge, frontLeftEdge, 3 );

	//top
	createSide( mesh, reverseVertexList( backTopEdge ), frontTopEdge,
					leftTopEdge, reverseVertexList( rightTopEdge ), 4 );

	//bottom
	createSide( mesh, frontBottomEdge, reverseVertexList( backBottomEdge ),
					reverseVertexList( leftBottomEdge ), rightBottomEdge, 5 );
	


	return (JournalObject*)jMesh;
}

void ProcCreateCube::createEdge(CMesh *mesh, int startVertex, int endVertex,
										  int sections, vector<int>& vertices)
{
	Point3 start = mesh->vertices().get( startVertex ).getWorld();
	Point3 end = mesh->vertices().get( endVertex ).getWorld();

	Vector3 edgeVector = end - start;
	Vector3 delta = edgeVector * ( 1.0 / (double)sections );


	Point3 position = start + delta;


	vertices.reserve( sections + 1 );

	vertices.push_back( startVertex );

	for (int i = 1; i < sections; i++)
	{
		vertices.push_back( mesh->vertices().size() );

		mesh->addVertex( position );

		position += delta;
	}

	vertices.push_back( endVertex );
}

void ProcCreateCube::createSide(CMesh *mesh,
										  const vector<int>& topEdge,
										  const vector<int>& bottomEdge,
										  const vector<int>& leftEdge,
										  const vector<int>& rightEdge,
										  int patch)
{
	vector<int> vertices;

	vertices.reserve( topEdge.size() * leftEdge.size() );
	
	for (int i = 0; i < (int)topEdge.size(); i++)
	{
		vertices.push_back( topEdge[i] );
	}


	for (int i = 1; i < (int)( leftEdge.size() - 1 ); i++)
	{
		vector<int> edgeVertices;
		
		createEdge( mesh, leftEdge[i], rightEdge[i],
						topEdge.size() - 1, edgeVertices );

		for (int j = 0; j < (int)edgeVertices.size(); j++)
		{
			vertices.push_back( edgeVertices[j] );
		}
	}
	

	for (int i = 0; i < (int)bottomEdge.size(); i++)
	{
		vertices.push_back( bottomEdge[i] );
	}



	int horizontalVertices = topEdge.size();
	int horizontalFaces = topEdge.size() - 1;
	int verticalFaces = leftEdge.size() - 1;

	double u = 0.0, uInc = 1.0 / (double)horizontalFaces;
	double v = 0.0, vInc = 1.0 / (double)verticalFaces;

	for (int y = 0; y < verticalFaces; y++)
	{
		u = 0.0;
		
		for (int x = 0; x < horizontalFaces; x++)
		{
			int a = y * horizontalVertices + x;
			int b = ( y + 1 ) * horizontalVertices + x;
			int c = ( y + 1 ) * horizontalVertices + x + 1;
			int d = y * horizontalVertices + x + 1;

			mesh->addFace( vertices[a], Point3( u, v, 0.0 ),
								vertices[b], Point3( u, v + vInc, 0.0 ),
								vertices[c], Point3( u + uInc, v + vInc, 0.0 ),
								vertices[d], Point3( u + uInc, v, 0.0 ), patch );

			u += uInc;
		}
		
		v += vInc;
	}
}

vector<int> ProcCreateCube::reverseVertexList(const vector<int>& vertices) const
{
	vector<int> result;

	result.reserve( vertices.size() );

	for (int i = vertices.size() - 1; i >= 0; i--)
	{
		result.push_back( vertices[i] );
	}

	return result;
}





//**************************************************


ToolCreateCube::ToolCreateCube(CadView *v)
										 : ProcessTool(v, "Cube", "Create Cube", 2)
{
	xSegsLabel = gtk_label_new("U Segs:");
	gtk_widget_ref(xSegsLabel);
	gtk_widget_show(xSegsLabel);
	
	xSegsEntry = new NumericEntry( 1.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
											 GTK_SIGNAL_FUNC(xSegsChanged), this );
	gtk_widget_show( xSegsEntry->getWidget() );



	ySegsLabel = gtk_label_new("V Segs:");
	gtk_widget_ref(ySegsLabel);
	gtk_widget_show(ySegsLabel);
	ySegsEntry = new NumericEntry( 1.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
											 GTK_SIGNAL_FUNC(ySegsChanged), this );
	gtk_widget_show( ySegsEntry->getWidget() );




	zSegsLabel = gtk_label_new("V Segs:");
	gtk_widget_ref(zSegsLabel);
	gtk_widget_show(zSegsLabel);
	zSegsEntry = new NumericEntry( 1.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
											 GTK_SIGNAL_FUNC(zSegsChanged), this );
	gtk_widget_show( zSegsEntry->getWidget() );




	widthLabel = gtk_label_new("Width:");
	gtk_widget_ref(widthLabel);
	gtk_widget_show(widthLabel);
	widthEntry = new NumericEntry( 100.0, 0.0, 1000000000.0, 0.5, 50.0, 0.5, 4,
											 GTK_SIGNAL_FUNC(widthChanged), this );
	gtk_widget_show( widthEntry->getWidget() );



	heightLabel = gtk_label_new("Height:");
	gtk_widget_ref(heightLabel);
	gtk_widget_show(heightLabel);
	heightEntry = new NumericEntry( 40.0, 0.0, 1000000000.0, 0.5, 50.0, 0.5, 4,
											  GTK_SIGNAL_FUNC(heightChanged), this );
	gtk_widget_show( heightEntry->getWidget() );



	depthLabel = gtk_label_new("Height:");
	gtk_widget_ref(depthLabel);
	gtk_widget_show(depthLabel);
	depthEntry = new NumericEntry( 40.0, 0.0, 1000000000.0, 0.5, 50.0, 0.5, 4,
											 GTK_SIGNAL_FUNC(depthChanged), this );
	gtk_widget_show( depthEntry->getWidget() );



	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.add(xSegsLabel);
	layout.add( xSegsEntry->getWidget() );
	layout.add(ySegsLabel);
	layout.add( ySegsEntry->getWidget() );
	layout.add(zSegsLabel);
	layout.add( zSegsEntry->getWidget() );
	layout.add(widthLabel);
	layout.add( widthEntry->getWidget() );
	layout.add(heightLabel);
	layout.add( heightEntry->getWidget() );
	layout.add(depthLabel);
	layout.add( depthEntry->getWidget() );
	layout.lineSpace();
	layout.addAsLine(doneButton);
}

ToolCreateCube::~ToolCreateCube()
{
	delete xSegsEntry;
	delete ySegsEntry;
	delete zSegsEntry;
	delete widthEntry;
	delete heightEntry;
	delete depthEntry;
	
	gtk_widget_unref(xSegsLabel);
	gtk_widget_unref(ySegsLabel);
	gtk_widget_unref(zSegsLabel);
	gtk_widget_unref(widthLabel);
	gtk_widget_unref(heightLabel);
	gtk_widget_unref(depthLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolCreateCube::getProcessType() const
{
	return ProcCreateCube::getTypeStatic();
}



void ToolCreateCube::initialise()
{
	ProcessTool::initialise();

	if ( hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)journalProcess;

		xSegsEntry->setValue( proc->getXSegments() );
		ySegsEntry->setValue( proc->getYSegments() );
		zSegsEntry->setValue( proc->getZSegments() );
		widthEntry->setValue( proc->getWidth() );
		heightEntry->setValue( proc->getHeight() );
		depthEntry->setValue( proc->getDepth() );
	}
}



void ToolCreateCube::xSegsChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setXSegments( klass->xSegsEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolCreateCube::ySegsChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setYSegments( klass->ySegsEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolCreateCube::zSegsChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setZSegments( klass->zSegsEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolCreateCube::widthChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setWidth( klass->widthEntry->getValue() );
		klass->modified();
	}
}

void ToolCreateCube::heightChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setHeight( klass->heightEntry->getValue() );
		klass->modified();
	}
}

void ToolCreateCube::depthChanged(NumericEntry *widget, ToolCreateCube *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreateCube *proc = (ProcCreateCube*)klass->journalProcess;
		proc->setDepth( klass->depthEntry->getValue() );
		klass->modified();
	}
}

void ToolCreateCube::doneClicked(GtkWidget *widget, ToolCreateCube *klass)
{
	klass->done();
}



#endif

