/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCREATECUBE_H
#define __TOOLCREATECUBE_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "NumericEntry.h"
#include "CMesh.h"



class ProcCreateCube : public JournalProcess
{
private:
	int xSegments, ySegments, zSegments;
	double width, height, depth;
	
public:
	ProcCreateCube();
	ProcCreateCube(int xSegs, int ySegs, int zSegs,
						double cubeWidth, double cubeHeight, double cubeDepth);
	virtual ~ProcCreateCube();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;

	int getXSegments() const;
	int getYSegments() const;
	int getZSegments() const;
	double getWidth() const;
	double getHeight() const;
	double getDepth() const;
	void setXSegments(int n);
	void setYSegments(int n);
	void setZSegments(int n);
	void setWidth(double w);
	void setHeight(double h);
	void setDepth(double d);
	
protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);

	void createEdge(CMesh *mesh,  int startVertex, int endVertex,
						 int sections, vector<int>& vertices);

	void createSide(CMesh *mesh,
						 const vector<int>& topEdge,
						 const vector<int>& bottomEdge,
						 const vector<int>& leftEdge,
						 const vector<int>& rightEdge,
						 int patch );

	vector<int> reverseVertexList(const vector<int>& vertices) const;
};




class ToolCreateCube : public ProcessTool
{
private:
	GtkWidget *xSegsLabel, *ySegsLabel, *zSegsLabel,
				 *widthLabel, *heightLabel, *depthLabel;
	NumericEntry *xSegsEntry, *ySegsEntry, *zSegsEntry,
					 *widthEntry, *heightEntry, *depthEntry;
	GtkWidget *doneButton;

public:
	ToolCreateCube(CadView *v);
	~ToolCreateCube();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();


private:
	static void xSegsChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void ySegsChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void zSegsChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void widthChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void heightChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void depthChanged(NumericEntry *widget, ToolCreateCube *klass);
	static void doneClicked(GtkWidget *widget, ToolCreateCube *klass);
};


#endif
