/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCREATEPATCH_CPP
#define __TOOLCREATEPATCH_CPP

#include <strstream>
#include <math.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>


#include "Patch.h"

#include "ToolCreatePatch.h"

#include "JournalMesh.h"
#include "CMesh.h"

#include "Point3.h"





void* __create_ProcCreatePatch()
{
	return (void*)( new ProcCreatePatch() );
}

ObjectType type_ProcCreatePatch( "ProcCreatePatch", "ProcAxis",
											__create_ProcCreatePatch );




ProcCreatePatch::ProcCreatePatch() : ProcAxis()
{
	uSegments = vSegments = 16;
	width = height = 100.0;
}

ProcCreatePatch::ProcCreatePatch(Axis a, int uSegs, int vSegs,
											double patchWidth, double patchHeight)
											: ProcAxis(a)
{
	uSegments = uSegs;
	vSegments = vSegs;
	width = patchWidth;
	height = patchHeight;
}

ProcCreatePatch::~ProcCreatePatch()
{
}



void ProcCreatePatch::read(FILE *f)
{
	ProcAxis::read(f);

	fread(&uSegments, sizeof(int), 1, f);
	fread(&vSegments, sizeof(int), 1, f);
	fread(&width, sizeof(double), 1, f);
	fread(&height, sizeof(double), 1, f);
}

void ProcCreatePatch::write(FILE *f)
{
	ProcAxis::write(f);

	fwrite(&uSegments, sizeof(int), 1, f);
	fwrite(&vSegments, sizeof(int), 1, f);
	fwrite(&width, sizeof(double), 1, f);
	fwrite(&height, sizeof(double), 1, f);
}





string ProcCreatePatch::getDescription() const
{
	return "Patch";
}

const ObjectType* ProcCreatePatch::getType() const
{
	return &type_ProcCreatePatch;
}

const ObjectType* ProcCreatePatch::getTypeStatic()
{
	return &type_ProcCreatePatch;
}



const ObjectType* ProcCreatePatch::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* ProcCreatePatch::clone() const
{
	return new ProcCreatePatch(axis, uSegments, vSegments, width, height);
}


int ProcCreatePatch::getUSegments() const
{
	return uSegments;
}

int ProcCreatePatch::getVSegments() const
{
	return vSegments;
}

double ProcCreatePatch::getWidth() const
{
	return width;
}

double ProcCreatePatch::getHeight() const
{
	return height;
}

	
void ProcCreatePatch::setUSegments(int n)
{
	uSegments = n;
	modified();
}

void ProcCreatePatch::setVSegments(int n)
{
	vSegments = n;
	modified();
}

void ProcCreatePatch::setWidth(double w)
{
	width = w;
	modified();
}

void ProcCreatePatch::setHeight(double h)
{
	height = h;
	modified();
}


const ObjectType* ProcCreatePatch::_getOutputType(
									const ObjectType* inputType) const
{
	return JournalMesh::getTypeStatic();
}

JournalObject* ProcCreatePatch::_process(JournalObject *o,
													  JournalTaskProgress *progress)
{
	delete o;

	JournalMesh *hMesh = new JournalMesh();
	CMesh *mesh = hMesh->getMesh();

	vector<CVertex*> vertexPtrs;


	Axes3 axes = getAxes();


	mesh->reserveVertices( (uSegments + 1) * (vSegments + 1) );
	mesh->reserveFaces( uSegments * vSegments );



	double x, xInc = width / (double)uSegments;
	double y = -height * 0.5, yInc = height / (double)vSegments;
	double u = 0.0, uInc = 1.0 / (double)uSegments;
	double v = 0.0, vInc = 1.0 / (double)vSegments;
	
	vector<Point3> tex;
	tex.reserve( (uSegments + 1) * (vSegments + 1) );


	for (int j = 0; j <= vSegments; j++)
	{
		x = -width * 0.5;
		u = 0.0;
		for (int i = 0; i <= uSegments; i++)
		{
			mesh->addVertex( Point3() + axes.i * x + axes.j * y, true );
			tex.push_back( Point3( u, v, 0.0 ) );
			x += xInc;
			u += uInc;
		}
		y += yInc;
		v += vInc;
	}

	mesh->addPatch();

	vector<int> faceQuads;
	patchGenerateQuadList( faceQuads, uSegments, vSegments, false, false );

	for (int i = 0; i < (int)faceQuads.size(); i += 4)
	{
		mesh->addFace( faceQuads[i], tex[ faceQuads[i] ],
							faceQuads[i+1], tex[ faceQuads[i+1] ],
							faceQuads[i+2], tex[ faceQuads[i+2] ],
							faceQuads[i+3], tex[ faceQuads[i+3] ], 0 );
	}


	return (JournalObject*)hMesh;
}







//**************************************************


ToolCreatePatch::ToolCreatePatch(CadView *v)
										   : ToolAxis(v, "Patch", "Create Patch",
										   			  2, CadToolAxisChooser::PLANE)
{
	uSegsLabel = gtk_label_new("U Segs:");
	gtk_widget_ref(uSegsLabel);
	gtk_widget_show(uSegsLabel);
	
	uSegsEntry = new NumericEntry( 16.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
											 GTK_SIGNAL_FUNC(uSegsChanged), this );
	gtk_widget_show( uSegsEntry->getWidget() );




	vSegsLabel = gtk_label_new("V Segs:");
	gtk_widget_ref(vSegsLabel);
	gtk_widget_show(vSegsLabel);
	vSegsEntry = new NumericEntry( 16.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
											 GTK_SIGNAL_FUNC(vSegsChanged), this );
	gtk_widget_show( vSegsEntry->getWidget() );




	widthLabel = gtk_label_new("Width:");
	gtk_widget_ref(widthLabel);
	gtk_widget_show(widthLabel);
	widthEntry = new NumericEntry( 100.0, 0.0, 1000000000.0, 0.5, 50.0, 0.5, 4,
											 GTK_SIGNAL_FUNC(widthChanged), this );
	gtk_widget_show( widthEntry->getWidget() );




	heightLabel = gtk_label_new("Height:");
	gtk_widget_ref(heightLabel);
	gtk_widget_show(heightLabel);
	heightEntry = new NumericEntry( 100.0, 0.0, 1000000000.0, 0.5, 50.0, 0.5, 4,
											  GTK_SIGNAL_FUNC(heightChanged), this );
	gtk_widget_show( heightEntry->getWidget() );



	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.add(uSegsLabel);
	layout.add( uSegsEntry->getWidget() );
	layout.add(vSegsLabel);
	layout.add( vSegsEntry->getWidget() );
	layout.add(widthLabel);
	layout.add( widthEntry->getWidget() );
	layout.add(heightLabel);
	layout.add( heightEntry->getWidget() );
	layout.lineSpace();
	layout.addAsLine(doneButton);
}

ToolCreatePatch::~ToolCreatePatch()
{
	delete uSegsEntry;
	delete vSegsEntry;
	delete widthEntry;
	delete heightEntry;
	
	gtk_widget_unref(uSegsLabel);
	gtk_widget_unref(vSegsLabel);
	gtk_widget_unref(widthLabel);
	gtk_widget_unref(heightLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolCreatePatch::getProcessType() const
{
	return ProcCreatePatch::getTypeStatic();
}




void ToolCreatePatch::initialise()
{
	ToolAxis::initialise();

	if ( hasProcess() )
	{
		ProcCreatePatch *proc = (ProcCreatePatch*)journalProcess;

		uSegsEntry->setValue( proc->getUSegments() );
		vSegsEntry->setValue( proc->getVSegments() );
		widthEntry->setValue( proc->getWidth() );
		heightEntry->setValue( proc->getHeight() );
	}
}



void ToolCreatePatch::uSegsChanged(NumericEntry *widget, ToolCreatePatch *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreatePatch *proc = (ProcCreatePatch*)klass->journalProcess;
		proc->setUSegments( klass->uSegsEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolCreatePatch::vSegsChanged(NumericEntry *widget, ToolCreatePatch *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreatePatch *proc = (ProcCreatePatch*)klass->journalProcess;
		proc->setVSegments( klass->vSegsEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolCreatePatch::widthChanged(NumericEntry *widget, ToolCreatePatch *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreatePatch *proc = (ProcCreatePatch*)klass->journalProcess;
		proc->setWidth( klass->widthEntry->getValue() );
		klass->modified();
	}
}

void ToolCreatePatch::heightChanged(NumericEntry *widget, ToolCreatePatch *klass)
{
	if ( klass->hasProcess() )
	{
		ProcCreatePatch *proc = (ProcCreatePatch*)klass->journalProcess;
		proc->setHeight( klass->heightEntry->getValue() );
		klass->modified();
	}
}

void ToolCreatePatch::doneClicked(GtkWidget *widget, ToolCreatePatch *klass)
{
	klass->done();
}



#endif

