/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCREATEPATCH_H
#define __TOOLCREATEPATCH_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "MTool.h"
#include "NumericEntry.h"
#include "ToolAxis.h"



class ProcCreatePatch : public ProcAxis
{
private:
	
	int uSegments, vSegments;
	double width, height;
	
public:
	ProcCreatePatch();
	ProcCreatePatch(Axis a, int uSegs, int vSegs,
						 double patchWidth, double patchHeight);
	virtual ~ProcCreatePatch();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;

	int getUSegments() const;
	int getVSegments() const;
	double getWidth() const;
	double getHeight() const;
	
	void setUSegments(int n);
	void setVSegments(int n);
	void setWidth(double w);
	void setHeight(double h);
	
protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
};




class ToolCreatePatch : public ToolAxis
{
private:
	GtkWidget *uSegsLabel, *vSegsLabel, *widthLabel, *heightLabel;
	NumericEntry *uSegsEntry, *vSegsEntry, *widthEntry, *heightEntry;
	GtkWidget *doneButton;

public:
	ToolCreatePatch(CadView *v);
	~ToolCreatePatch();


	virtual const ObjectType* getProcessType() const;



protected:
	virtual void initialise();


private:
	static void uSegsChanged(NumericEntry *widget, ToolCreatePatch *klass);
	static void vSegsChanged(NumericEntry *widget, ToolCreatePatch *klass);
	static void widthChanged(NumericEntry *widget, ToolCreatePatch *klass);
	static void heightChanged(NumericEntry *widget, ToolCreatePatch *klass);
	static void doneClicked(GtkWidget *widget, ToolCreatePatch *klass);
};


#endif
