/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLCREATETORUS_H
#define __TOOLCREATETORUS_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "MTool.h"
#include "NumericEntry.h"
#include "ToolAxis.h"



class ProcCreateTorus : public ProcAxis
{
private:
	int majSegs, minSegs;
	double majRad, minRad;
	
public:
	ProcCreateTorus();
	ProcCreateTorus(Axis a, int majS, int minS, double majR, double minR);
	virtual ~ProcCreateTorus();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;

	int getMajorSegments() const;
	int getMinorSegments() const;
	double getMajorRadius() const;
	double getMinorRadius() const;
	
	void setMajorSegments(int n);
	void setMinorSegments(int n);
	void setMajorRadius(double r);
	void setMinorRadius(double r);


protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
};




class ToolCreateTorus : public ToolAxis
{
private:
	GtkWidget *majSegsLabel, *minSegsLabel, *majRadLabel, *minRadLabel;
	NumericEntry *majSegsEntry, *minSegsEntry, *majRadEntry, *minRadEntry;
	GtkWidget *doneButton;

public:
	ToolCreateTorus(CadView *v);
	~ToolCreateTorus();


	virtual const ObjectType* getProcessType() const;

protected:
	virtual void initialise();

private:
	static void majSegsChanged(NumericEntry *widget, ToolCreateTorus *klass);
	static void minSegsChanged(NumericEntry *widget, ToolCreateTorus *klass);
	static void majRadChanged(NumericEntry *widget, ToolCreateTorus *klass);
	static void minRadChanged(NumericEntry *widget, ToolCreateTorus *klass);
	static void doneClicked(GtkWidget *widget, ToolCreateTorus *klass);
};


#endif
