/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLGROUPCREATEMESH_CPP
#define __TOOLGROUPCREATEMESH_CPP

#include "ToolGroupCreateMesh.h"

#include <gtk/gtklabel.h>



ToolGroupCreateMesh::ToolGroupCreateMesh(
				ProcessToolManager *toolManager,
				JournalEditor *journEditor, CadView *view )
		: MToolGroup(toolManager)
{
	titleLabel = gtk_label_new("Create Mesh");
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);

	journalEditor = journEditor;

	cubeButton = addToolButton( "Cube", GTK_SIGNAL_FUNC(cubeClicked) );
	patchButton = addToolButton( "Patch", GTK_SIGNAL_FUNC(patchClicked) );
	torusButton = addToolButton( "Torus", GTK_SIGNAL_FUNC(torusClicked) );

	toolCube = new ToolCreateCube(view);
	toolPatch = new ToolCreatePatch(view);
	toolTorus = new ToolCreateTorus(view);

	toolMan->addTool(toolCube->getProcessType(), toolCube);
	toolMan->addTool(toolPatch->getProcessType(), toolPatch);
	toolMan->addTool(toolTorus->getProcessType(), toolTorus);
}

ToolGroupCreateMesh::~ToolGroupCreateMesh()
{
	gtk_widget_unref(titleLabel);
	
	toolMan->eraseTool(toolCube->getProcessType());
	toolMan->eraseTool(toolPatch->getProcessType());
	toolMan->eraseTool(toolTorus->getProcessType());
	delete toolCube;
	delete toolPatch;
	delete toolTorus;
}


void ToolGroupCreateMesh::addToLayout(GridLayout *layout)
{
	MToolGroup::addToLayout(layout);

	layout->addAsLine(titleLabel);
	
	layout->add(cubeButton);
	layout->add(patchButton);
	layout->add(torusButton);
}	




void ToolGroupCreateMesh::cubeClicked( GtkWidget *widget,
													ToolGroupCreateMesh *klass )
{
	ProcCreateCube *p = new ProcCreateCube();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupCreateMesh::patchClicked( GtkWidget *widget,
													 ToolGroupCreateMesh *klass )
{
	ProcCreatePatch *p = new ProcCreatePatch();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupCreateMesh::torusClicked( GtkWidget *widget,
													 ToolGroupCreateMesh *klass )
{
	ProcCreateTorus *p = new ProcCreateTorus();
	klass->journalEditor->addProcess(p);
	delete p;
}



#endif
