/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADEDITORWINDOW_CPP
#define __CADEDITORWINDOW_CPP

#include <gtk/gtkmarshal.h>

#include <gtk/gtknotebook.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtksignal.h>
#include <gtk/gtklabel.h>


#include "CadEditorWindow.h"



CadEditorWindow::CadEditorWindow(CadView *vw, const string& wintitle)
{
	wtitle = wintitle;

	gtk_widget_show( vw->getWidget() );


	//***
	//the toolbox
	toolbox = gtk_notebook_new();
	gtk_widget_ref(toolbox);
	gtk_widget_show(toolbox);


	//***
	//work area containing tool palette and the cad view
	workhbox = gtk_hbox_new( false, 0 );
	gtk_widget_ref(workhbox);

	//add toolbox to workhbox
	gtk_box_pack_start( GTK_BOX(workhbox), toolbox, false, false, 0 );
	//add the cad view to the workhbox
	gtk_box_pack_start( GTK_BOX(workhbox), vw->getWidget(),
							  true, true, 4 );
	gtk_widget_show(workhbox);


	//***
	//the status bar
	statusBar = new StatusBar();
	gtk_widget_show( statusBar->getWidget() );


	//***
	//box containing work area and status bar
	winvbox = gtk_vbox_new(false, 0);
	gtk_widget_ref(winvbox);

	//add workhbox to winvbox
	gtk_box_pack_start( GTK_BOX(winvbox), workhbox, true, true, 2 );

	//add statusbar to winvbox
	gtk_box_pack_start( GTK_BOX(winvbox), statusBar->getWidget(),
							  false, false, 2 );

	gtk_widget_show(winvbox);
}

CadEditorWindow::~CadEditorWindow()
{
	gtk_widget_unref(toolbox);
	gtk_widget_unref(workhbox);
	gtk_widget_unref(winvbox);

	delete statusBar;
}



void CadEditorWindow::addToolset(GtkWidget *ts, const string& name,
											bool makeCurrent)
{
	gtk_notebook_append_page( GTK_NOTEBOOK(toolbox), ts, NULL);
	gtk_notebook_set_tab_label_text( GTK_NOTEBOOK(toolbox), ts, name.c_str() );
	
	if (makeCurrent)
	{
		setCurrentToolset(ts);
	}
}

void CadEditorWindow::removeToolset(GtkWidget *ts)
{
	int page = gtk_notebook_page_num( GTK_NOTEBOOK(toolbox), ts );
	if (page != -1)
	{
		gtk_notebook_remove_page( GTK_NOTEBOOK(toolbox), page );
	}
}

void CadEditorWindow::setCurrentToolset(GtkWidget *ts)
{
	int page = gtk_notebook_page_num( GTK_NOTEBOOK(toolbox), ts );
	gtk_notebook_set_page( GTK_NOTEBOOK(toolbox), page );
}

void CadEditorWindow::addToolset(Toolset *toolset, const string& name,
											bool makeCurrent)
{
	addToolset( toolset->getWidget(), name, makeCurrent );
}


StatusBar* CadEditorWindow::getStatusBar()
{
	return statusBar;
}



void CadEditorWindow::go(GtkSignalFunc closeCallbackFunc,
								 gpointer closeCallbackData)
{
	//***
	//the window
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title( GTK_WINDOW(win), wtitle.c_str() );
	gtk_widget_set_usize( win, 640, 480 );

	gtk_container_add(GTK_CONTAINER(win), winvbox);

	gtk_widget_show(win);

	closeFunc = closeCallbackFunc;
	closeData = closeCallbackData;

	gtk_signal_connect(GTK_OBJECT(win), "destroy",
							 GTK_SIGNAL_FUNC(win_destroy), this);
}

void CadEditorWindow::stop()
{
}



void CadEditorWindow::win_destroy(GtkWidget *widget, CadEditorWindow *klass)
{
	klass->stop();
	
	gtk_marshal_NONE__NONE( (GtkObject*)klass, klass->closeFunc, klass->closeData,
									(GtkArg*)NULL );
}





#endif
