/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADEVENTHANDLERS_CPP
#define __CADEVENTHANDLERS_CPP

#include "PointerStatus.h"
#include "DrawingArea.h"
#include "CadArea.h"

#include "CadEventHandlers.h"



/*
*******************************************************************************
											CadCursorPosition
*******************************************************************************
*/

CadCursorPosition::CadCursorPosition()
{
}

CadCursorPosition::CadCursorPosition(CadArea *cadArea)
{
	screenInt = cadArea->getPointerStatus().getPosition();
	screenFloat = Point3( (double)screenInt.x, (double)screenInt.y, 0.0 );
	world = cadArea->transformScreenWorld( screenFloat );
}


IPoint2 CadCursorPosition::getScreenInt()
{
	return screenInt;
}

Point3 CadCursorPosition::getScreenFloat()
{
	return screenFloat;
}

Point3 CadCursorPosition::getWorld()
{
	return world;
}





/*
*******************************************************************************
										CadEventHandler
*******************************************************************************
*/


CadEventHandler::CadEventHandler(CadArea *c) : CAEventHandler(c)
{
}


void CadEventHandler::evPrimaryButtonDown()
{
}
 
void CadEventHandler::evSecondaryButtonDown()
{
}

void CadEventHandler::evTertiaryButtonDown()
{
}

void CadEventHandler::evPrimaryButtonUp()
{
}

void CadEventHandler::evSecondaryButtonUp()
{
}

void CadEventHandler::evTertiaryButtonUp()
{
}

void CadEventHandler::evButtonDown(int button)
{
	const PointerStatus &ps = cadArea->getPointerStatus();

	if ( ps.isShift() || ps.isControl() )
	{
		return;
	}

	switch(button)
	{
	case 1:
		evPrimaryButtonDown();
		break;
	case 2:
		evSecondaryButtonDown();
		break;
	case 3:
		evTertiaryButtonDown();
		break;
	default:
		break;
	}
}

void CadEventHandler::evButtonUp(int button)
{
	const PointerStatus &ps = cadArea->getPointerStatus();

	if ( ps.isShift() || ps.isControl() )
	{
		return;
	}

	switch(button)
	{
	case 1:
		evPrimaryButtonUp();
		break;
	case 2:
		evSecondaryButtonUp();
		break;
	case 3:
		evTertiaryButtonUp();
		break;
	default:
		break;
	}
}

void CadEventHandler::evMotion(const IPoint2& p)
{
	currentCursorPosition = CadCursorPosition(cadArea);
}





/*
*******************************************************************************
										CadClickAndDragEventHandler
*******************************************************************************
*/

CadClickAndDragEventHandler::CadClickAndDragEventHandler(CadArea *c)
																			: CadEventHandler(c)
{
	runningMode = RUNMODE_NOTRUNNING;
}

CadClickAndDragEventHandler::~CadClickAndDragEventHandler()
{
}


void CadClickAndDragEventHandler::start()
{
	startCursorPosition = CadCursorPosition(cadArea);
}

void CadClickAndDragEventHandler::finish(bool terminated)
{
}


void CadClickAndDragEventHandler::function()
{
}


void CadClickAndDragEventHandler::evMotion(const IPoint2& p)
{
	CadEventHandler::evMotion(p);
	
	if (runningMode != RUNMODE_NOTRUNNING)
	{
		function();
	}
}

void CadClickAndDragEventHandler::evLeave(const IPoint2& p)
{
	if (runningMode == RUNMODE_CLICK)
	{
		runningMode = RUNMODE_NOTRUNNING;
		finish(true);
	}
}


void CadClickAndDragEventHandler::evPrimaryButtonDown()
{
	if (runningMode == RUNMODE_NOTRUNNING)
	{
		runningMode = RUNMODE_CLICKANDHOLD;
		start();
	}
}

void CadClickAndDragEventHandler::evSecondaryButtonDown()
{
	switch(runningMode)
	{
	case RUNMODE_NOTRUNNING:
		runningMode = RUNMODE_CLICK;
		start();
		break;
	case RUNMODE_CLICK:
		runningMode = RUNMODE_NOTRUNNING;
		function();
		finish(false);
		break;
	default:
		break;
	}
}

void CadClickAndDragEventHandler::evTertiaryButtonDown()
{
	if (runningMode != RUNMODE_NOTRUNNING)
	{
		runningMode = RUNMODE_NOTRUNNING;
		finish(true);
	}
}

void CadClickAndDragEventHandler::evPrimaryButtonUp()
{
	if (runningMode == RUNMODE_CLICKANDHOLD)
	{
		runningMode = RUNMODE_NOTRUNNING;
		function();
		finish(false);
	}
}

void CadClickAndDragEventHandler::evSecondaryButtonUp()
{
}

void CadClickAndDragEventHandler::evTertiaryButtonUp()
{
}






#endif
