/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by :					Geoffrey French
	application :		gSculpt
	last revision :	30/Jun/2000
	title :				CAD Function Base Classes
*/
#ifndef __CADEVENTHANDLERS_H
#define __CADEVENTHANDLERS_H

#include "CadArea.h"



/*
*******************************************************************************
											CadCursorPosition
*******************************************************************************
*/


class CadCursorPosition
{
private:
	IPoint2 screenInt;
	Point3 screenFloat, world;

public:
	CadCursorPosition();
	CadCursorPosition(CadArea *cadArea);

	IPoint2 getScreenInt();
	Point3 getScreenFloat();
	Point3 getWorld();
};




/*
*******************************************************************************
											CadEventHandler
*******************************************************************************
*/


class CadEventHandler : public CAEventHandler
{
protected:
	CadCursorPosition currentCursorPosition;
	
public:
	CadEventHandler(CadArea *c);

protected:
	virtual void evPrimaryButtonDown();
	virtual void evSecondaryButtonDown();
	virtual void evTertiaryButtonDown();
	virtual void evPrimaryButtonUp();
	virtual void evSecondaryButtonUp();
	virtual void evTertiaryButtonUp();

public:
	virtual void evButtonDown(int button);
	virtual void evButtonUp(int button);
	virtual void evMotion(const IPoint2& p);
};





/*
*******************************************************************************
										CadClickAndDragEventHandler
*******************************************************************************
*/

class CadClickAndDragEventHandler : public CadEventHandler
{
protected:
	typedef enum {
		RUNMODE_NOTRUNNING,
		RUNMODE_CLICKANDHOLD,
		RUNMODE_CLICK
	} RMode;

	RMode runningMode;
	CadCursorPosition startCursorPosition;


	virtual void start();
	virtual void finish(bool terminated);
	
	virtual void function();

	virtual void evMotion(const IPoint2& p);
	virtual void evLeave(const IPoint2& p);

	virtual void evPrimaryButtonDown();
	virtual void evSecondaryButtonDown();
	virtual void evTertiaryButtonDown();
	virtual void evPrimaryButtonUp();
	virtual void evSecondaryButtonUp();
	virtual void evTertiaryButtonUp();

public:
	CadClickAndDragEventHandler(CadArea *c);
	virtual ~CadClickAndDragEventHandler();
};




#endif
