/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CFACE_CPP
#define __CFACE_CPP

#include "CFace.h"


/*
*******************************************************************************
												CFace
*******************************************************************************
*/


CFace::Vertex::Vertex()
{
	vertexNum = 0;
	texture = Point3();
}

CFace::Vertex::Vertex(int n, const Point3& t)
{
	vertexNum = n;
	texture = t;
}



CFace::CFace()
{
}

CFace::CFace(Vertex a, Vertex b, Vertex c, int patch)
{
	vts.reserve(3);
	vts.push_back(a);
	vts.push_back(b);
	vts.push_back(c);

	patchIndex = patch;
}

CFace::CFace(int va, const Point3& ta,
				 int vb, const Point3& tb,
				 int vc, const Point3& tc, int patch)
{
	vts.reserve(3);
	vts.push_back( Vertex(va, ta ) );
	vts.push_back( Vertex(vb, tb ) );
	vts.push_back( Vertex(vc, tc ) );

	patchIndex = patch;
}

CFace::CFace(Vertex a, Vertex b, Vertex c, Vertex d, int patch)
{
	vts.reserve(4);
	vts.push_back(a);
	vts.push_back(b);
	vts.push_back(c);
	vts.push_back(d);

	patchIndex = patch;
}

CFace::CFace(int va, const Point3& ta,
				 int vb, const Point3& tb,
				 int vc, const Point3& tc,
				 int vd, const Point3& td, int patch)
{
	vts.reserve(4);
	vts.push_back( Vertex(va, ta ) );
	vts.push_back( Vertex(vb, tb ) );
	vts.push_back( Vertex(vc, tc ) );
	vts.push_back( Vertex(vd, td ) );

	patchIndex = patch;
}



void CFace::read(FILE *f)
{
	vts.clear();

	int numVertices;
	fread( &numVertices, sizeof(int), 1, f );

	for (int i = 0; i < numVertices; i++)
	{
		int vertexNumber;
		Vector3 tex;
		
		fread( &vertexNumber, sizeof(int), 1, f );
		tex.read(f);

		Vertex v;
		v.vertexNum = vertexNumber;
		v.texture = tex;
		
		vts.push_back(v);
	}

	fread( &patchIndex, sizeof(int), 1, f );
}

void CFace::write(FILE *f) const
{
	int numVertices = vts.size();
	fwrite( &numVertices, sizeof(int), 1, f );

	for (int i = 0; i < numVertices; i++)
	{
		const Vertex &v = vts[i];
		
		int vertexNumber = v.vertexNum;
		fwrite( &vertexNumber, sizeof(int), 1, f );
		v.texture.write(f);
	}

	fwrite( &patchIndex, sizeof(int), 1, f );
}



void CFace::addVertex(Vertex vtx)
{
	vts.push_back(vtx);
}


void CFace::copyFrom(const CFace *f)
{
	vts.resize( f->vts.size() );

	for (int i = 0; i < (int)f->vts.size(); i++)
	{
		vts[i] = f->vts[i];
	}
}


const CFace::Vertex& CFace::operator[](int i) const
{
	return vts[i];
}

int CFace::size() const
{
	return vts.size();
}


int CFace::getPatchIndex() const
{
	return patchIndex;
}


#endif
