/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/************************
 Licensed under the terms
 and conditions described
 in the file 'COPYING'
 ************************
	by:					Geoffrey French
	application:		gSculpt
	last revision:		16/Apr/2000
*/
#ifndef __HERMITESURFACECONTROLPOINT_H
#define __HERMITESURFACECONTROLPOINT_H

#include <stdio.h>

#include "Point3.h"
#include "Transformation.h"
#include "Frame.h"
#include "Matrix4.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"
#include "ViewFrustrum.h"




class HermiteSurfaceControlPoint
{
public:
	typedef enum {
		POSITION_BIT = 1,
		STANGENT_BIT = 2,
		TTANGENT_BIT = 4,
		TWISTVEC_BIT = 8
	} TransformMask;
	
private:
	Point3 position;
	Vector3 sTangent, tTangent, twistVector;
	Point3 screenPosition, screenSTan, screenTTan, screenTwist;
	bool markFlag;
	long visData;
	Frame fr;

public:
	HermiteSurfaceControlPoint();
	HermiteSurfaceControlPoint(const Point3& pos,
										const Vector3& sTan, const Vector3& tTan,
										const Vector3& twistVec);


	void read(FILE *f);
	void write(FILE *f) const;



	//mark, unmark, check marked status
	void mark();
	void unmark();
	bool isMarked() const;

	//set/clear/check visibility bits
	bool checkVisibilityBit(int b) const;
	long getVisibilityData() const;
	bool isVisible() const;

	//set and get geometry
	void setGeometry(const Point3& pos, const Vector3& sTan,
						  const Vector3& tTan, const Vector3& twistVec);
	void setPosition(const Point3& pos);
	void setSTangent(const Vector3& sTan);
	void setTTangent(const Vector3& tTan);
	void setTwistVector(const Vector3& twistVec);
	const Point3& getPosition() const;
	const Vector3& getSTangent() const;
	const Vector3& getTTangent() const;
	const Vector3& getTwistVector() const;
	const Point3& getScreenPosition() const;
	const Point3& getScreenSTangent() const;
	const Point3& getScreenTTangent() const;
	const Point3& getScreenTwist() const;
	

	void copyFrom(const HermiteSurfaceControlPoint *v);

	

	//transform vertex (transform world coord)
	void transform(const Matrix4& m, TransformMask mask);
	void transform(const Transformation *x, TransformMask mask);

	//select vertex
	void select(SelectPredicate predicate, const SelectRegion *region);


	//update the screen coordinate, and visibility data:
	//transforms world coord to get screen coord using w2s if necessary
	void update(const Transformation *w2s, const ViewFrustrum& frus,
					const Frame& currentFrame);


	HermiteSurfaceControlPoint backup() const;
	void restore(const HermiteSurfaceControlPoint& bak);
};



#endif
