/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/************************
 Licensed under the terms
 and conditions described
 in the file 'COPYING'
 ************************/
#ifndef __HERMITESURFACEVERTEX_CPP
#define __HERMITESURFACEVERTEX_CPP

#include "HermiteSurfaceVertex.h"





HermiteSurfaceVertex::HermiteSurfaceVertex()
{
	fr = 0;
	visData = 0;
}

HermiteSurfaceVertex::HermiteSurfaceVertex(const Point3& v)
{
	wld = scr = v;
	fr = 0;
	visData = 0;
}





bool HermiteSurfaceVertex::checkVisibilityBit(int b) const
{
	return visData & (1 << b);
}

long HermiteSurfaceVertex::getVisibilityData() const
{
	return visData;
}

bool HermiteSurfaceVertex::isVisible() const
{
	return visData == 0;
}


void HermiteSurfaceVertex::setWorld(const Point3& v)
{
	wld = scr = v;
	fr = 0;
}

Point3 HermiteSurfaceVertex::getWorld() const
{
	return wld;
}

Point3 HermiteSurfaceVertex::getScreen() const
{
	return scr;
}



void HermiteSurfaceVertex::copyFrom(const HermiteSurfaceVertex *v)
{
	wld = v->wld;
}


void HermiteSurfaceVertex::transform(const Matrix4& m)
{
	scr = wld = wld * m;
	fr = 0;
}

void HermiteSurfaceVertex::transform(const Transformation *x)
{
	scr = wld = x->transformPoint(wld);
	fr = 0;
}


void HermiteSurfaceVertex::update(const Transformation *w2s,
											 const ViewFrustrum& frus,
											 const Frame& currentFrame)
{
	if (fr != currentFrame)
	{
		scr = w2s->transformPoint(wld);
		visData = frus.pointBitList(wld);
		fr = currentFrame;
	}
}



HermiteSurfaceVertex HermiteSurfaceVertex::backup() const
{
	HermiteSurfaceVertex v;
	v.wld = wld;
	
	return v;
}

void HermiteSurfaceVertex::restore(const HermiteSurfaceVertex& bak)
{
	setWorld( bak.wld );
}

	


#endif
