/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/************************
 Licensed under the terms
 and conditions described
 in the file 'COPYING'
 ************************
	by:					Geoffrey French
	application:		gSculpt
	last revision:		16/Apr/2000
*/
#ifndef __HERMITESURFACEVERTEX_H
#define __HERMITESURFACEVERTEX_H

#include <stdio.h>

#include "Point3.h"
#include "Transformation.h"
#include "Frame.h"
#include "ViewFrustrum.h"
#include "Matrix4.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"




class HermiteSurfaceVertex
{
private:
	Point3 wld, scr;
	long visData;
	Frame fr;

public:
	HermiteSurfaceVertex();
	HermiteSurfaceVertex(const Point3& v);




	//set/clear/check visibility bits
	bool checkVisibilityBit(int b) const;
	long getVisibilityData() const;
	bool isVisible() const;

	//set world, get world
	void setWorld(const Point3& v);
	Point3 getWorld() const;
	Point3 getScreen() const;


	void copyFrom(const HermiteSurfaceVertex *v);

	

	//transform vertex (transform world coord)
	void transform(const Matrix4& m);
	void transform(const Transformation *x);

	//update the screen coordinate, and visibility data:
	//transforms world coord to get screen coord using w2s if necessary
	void update(const Transformation *w2s, const ViewFrustrum& frus,
					const Frame& currentFrame);


	HermiteSurfaceVertex backup() const;
	void restore(const HermiteSurfaceVertex& bak);
};



#endif
