/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __JOURNALPROCESS_CPP
#define __JOURNALPROCESS_CPP


#include "JournalProcess.h"






JournalProcessIncompatibleObjectTypeException::
	JournalProcessIncompatibleObjectTypeException(const ObjectType *i,
				const ObjectType *r, const ObjectType *pType)
{
	input = i;
	requested = r;
	processType = pType;
}




JournalProcessIncompatibleDestinationObjectTypeException::
	JournalProcessIncompatibleDestinationObjectTypeException(const ObjectType *i,
				const ObjectType *o, const ObjectType *d, const ObjectType *pType)
{
	input = i;
	output = o;
	dest = d;
	processType = pType;
}







void JournalProcessOwner::processModified(JournalProcess *p)
{
}






void* __create_JournalProcess()
{
	return (void*)( new JournalProcess() );
}


ObjectType type_JournalProcess("JournalProcess", "", __create_JournalProcess);









JournalProcess::JournalProcess()
{
	owner = NULL;
}

JournalProcess::~JournalProcess()
{
}



void JournalProcess::read(FILE *f)
{
}

void JournalProcess::write(FILE *f)
{
}




string JournalProcess::getDescription() const
{
	return "None";
}

const ObjectType* JournalProcess::getType() const
{
	return &type_JournalProcess;
}

const ObjectType* JournalProcess::getTypeStatic()
{
	return &type_JournalProcess;
}

string JournalProcess::getTypeName() const
{
	return getType()->getName();
}




void JournalProcess::setOwner(JournalProcessOwner *processOwner)
{
	owner = processOwner;
}

const JournalProcessOwner* JournalProcess::getOwner() const
{
	return owner;
}



bool JournalProcess::checkInputType(const ObjectType* inputType) const
{
	return inputType->isSubclassOf( getInputType() );
}

const ObjectType* JournalProcess::getOutputType(
								const ObjectType* inputType) const
{
	if ( checkInputType(inputType) )
	{
		return _getOutputType(inputType);
	}
	else
	{
		return NULL;
	}
}


JournalObject* JournalProcess::process(JournalObject *o,
													JournalTaskProgress *progress)
{
	if ( !checkInputType( o->getType() ) )
	{
		throw JournalProcessIncompatibleObjectTypeException( o->getType(),
				getInputType(), getType() );
		return NULL;
	}
	else
	{
		return _process(o, progress);
	}
}

void JournalProcess::processInto(JournalObject *o, JournalObject *dst,
											JournalTaskProgress *progress)
{
	if ( checkInputType( o->getType() ) )
	{
		const ObjectType *outType = _getOutputType( o->getType() );

		if ( outType == NULL )
		{
			return;
		}
		else
		{
			if ( dst->getType()->isSubclassOf( outType ) )
			{
				_processInto(o, dst, progress);
			}
			else
			{
				throw JournalProcessIncompatibleDestinationObjectTypeException(
					o->getType(), outType, dst->getType(), getType() );
			}
		}
	}
	else
	{
		throw JournalProcessIncompatibleObjectTypeException( o->getType(),
				getInputType(), getType() );
	}
}





const ObjectType* JournalProcess::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* JournalProcess::clone() const
{
	return new JournalProcess();
}






const ObjectType* JournalProcess::_getOutputType(
													const ObjectType* inputType) const
{
	return inputType;
}

JournalObject* JournalProcess::_process(JournalObject *o,
													 JournalTaskProgress *progress)
{
	return o;
}

void JournalProcess::_processInto(JournalObject *o, JournalObject *dst,
											 JournalTaskProgress *progress)
{
}


void JournalProcess::modified()
{
	if (owner != NULL)
	{
		owner->processModified(this);
	}
}


#endif
