/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __JOURNALPROCESS_H
#define __JOURNALPROCESS_H

#include <stdio.h>



#include "JournalTaskProgress.h"
#include "JournalObject.h"




class JournalProcessIncompatibleObjectTypeException
{
public:
	const ObjectType *input, *requested, *processType;

	JournalProcessIncompatibleObjectTypeException(const ObjectType *i,
				const ObjectType *r, const ObjectType *pType);
};





class JournalProcessIncompatibleDestinationObjectTypeException
{
public:
	const ObjectType *input, *output, *dest, *processType;

	JournalProcessIncompatibleDestinationObjectTypeException(const ObjectType *i,
				const ObjectType *o, const ObjectType *d, const ObjectType *pType);
};







class JournalProcess;


class JournalProcessOwner
{
public:
	virtual void processModified(JournalProcess *p);
};



class JournalProcess
{
private:
	JournalProcessOwner *owner;
	
public:
	JournalProcess();
	virtual ~JournalProcess();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();
	string getTypeName() const;


	void setOwner(JournalProcessOwner *processOwner);
	const JournalProcessOwner* getOwner() const;

	bool checkInputType(const ObjectType* inputType) const;
	const ObjectType* getOutputType(const ObjectType* inputType) const;

	JournalObject* process(JournalObject *o, JournalTaskProgress *progress);
	void processInto(JournalObject *o, JournalObject *dst,
						  JournalTaskProgress *progress);



	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;


protected:
	virtual const ObjectType* _getOutputType(
		const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
	virtual void _processInto(JournalObject *o, JournalObject *dst,
									  JournalTaskProgress *progress);

	void modified();
};



#endif
