/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __JOURNALTASKMONITOR_CPP
#define __JOURNALTASKMONITOR_CPP

#include <gtk/gtkmarshal.h>

#include <gtk/gtkhbox.h>
#include <gtk/gtkprogressbar.h>
#include <gtk/gtkmain.h>

#include <stdio.h>


#include "JournalTaskMonitor.h"




JournalTaskMonitor::JournalTaskMonitor(StatusBar *statBar)
{
	statusBar = statBar;
}


void JournalTaskMonitor::start(JournalTaskProgress *p,
										 GtkSignalFunc finishedFunction,
										 gpointer finishedData)
{
	progress = p;
	

	countStart = 0;
	showingProgressFlag = false;

	finFunc = finishedFunction;
	finData = finishedData;


	progressBar = gtk_progress_bar_new();
	gtk_grab_add(progressBar);


	startId = gtk_timeout_add( 40, (GtkFunction)waitToStartFunction, this );
	idleId = gtk_idle_add( (GtkFunction)idleFunction, this );
}




void JournalTaskMonitor::startProgressBars()
{
	statusBar->setLabelText("Computing:");
	gtk_widget_show(progressBar);
	statusBar->add(progressBar);

	updateId = gtk_timeout_add( 18, (GtkFunction)updateProgressFunction, this );
	showingProgressFlag = true;
}

void JournalTaskMonitor::finished()
{
	gtk_grab_remove(progressBar);


	if (showingProgressFlag)
	{
		gtk_timeout_remove( updateId );
		statusBar->remove(progressBar);
	}
	else
	{
		gtk_timeout_remove( startId );
	}

	gtk_idle_remove(idleId);


	statusBar->setLabelText("Ready");

	gtk_marshal_NONE__NONE( (GtkObject*)this, finFunc, finData,
									(GtkArg*)NULL );
}

void JournalTaskMonitor::updateProgress()
{
	progress->lock();
	float frac = progress->getFraction();
	progress->unlock();

	gtk_progress_bar_update( GTK_PROGRESS_BAR(progressBar), frac );
}




gint JournalTaskMonitor::waitToStartFunction(JournalTaskMonitor *klass)
{
	klass->countStart++;

	if ( klass->countStart >= 2 )
	{
		gtk_timeout_remove( klass->startId );
		klass->startProgressBars();
	}

	return klass->countStart < 2;
}

gint JournalTaskMonitor::idleFunction(JournalTaskMonitor *klass)
{
	if ( klass->progress->isComplete() )
	{
		klass->finished();
		return false;
	}
	else
	{
		return true;
	}
}

gint JournalTaskMonitor::updateProgressFunction(JournalTaskMonitor *klass)
{
	klass->updateProgress();

	return true;
}




#endif
