/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __JOURNALTASKPROGRESS_CPP
#define __JOURNALTASKPROGRESS_CPP

#include "JournalTaskProgress.h"



JournalTaskProgress::JournalTaskProgress()
{
	taskMutex = new GSCMutex();
	subtaskProgress = subtaskLength = subtasksComplete = numSubtasks = 0;
	completeFlag = false;
	nestedTask = NULL;
}

JournalTaskProgress::~JournalTaskProgress()
{
	delete taskMutex;
}



void JournalTaskProgress::lock()
{
	taskMutex->lock();
}

void JournalTaskProgress::unlock()
{
	taskMutex->unlock();
}



void JournalTaskProgress::reset()
{
	subtaskProgress = subtaskLength = subtasksComplete = numSubtasks = 0;
	completeFlag = false;
}


void JournalTaskProgress::setNestedTask(JournalTaskProgress *task)
{
	nestedTask = task;
}

JournalTaskProgress* JournalTaskProgress::getNestedTask()
{
	return nestedTask;
}



void JournalTaskProgress::setSubtaskDescription(const string& s)
{
	subtaskDescription = s;
}

string JournalTaskProgress::getSubtaskDescription()
{
	return subtaskDescription;
}


void JournalTaskProgress::setSubtaskLength(int l)
{
	subtaskLength = l;
}

void JournalTaskProgress::setSubtaskProgress(int p)
{
	subtaskProgress = p;
}

void JournalTaskProgress::setSubtasksComplete(int t)
{
	subtasksComplete = t;
}

void JournalTaskProgress::setNumSubtasks(int n)
{
	numSubtasks = n;
}

void JournalTaskProgress::setComplete(bool c)
{
	completeFlag = c;
}


int JournalTaskProgress::getSubtaskLength() const
{
	return subtaskLength;
}

int JournalTaskProgress::getSubtaskProgress() const
{
	return subtaskProgress;
}

int JournalTaskProgress::getSubtasksComplete() const
{
	return subtasksComplete;
}

int JournalTaskProgress::getNumSubtasks() const
{
	return numSubtasks;
}

bool JournalTaskProgress::isComplete() const
{
	return completeFlag;
}


float JournalTaskProgress::getFraction() const
{
	float perSubtask = 0.0;
	float perSubtaskUnit = 0.0;
	float nestedTaskFrac = 0.0;

	if ( numSubtasks > 0 )
	{
		perSubtask = 1.0 / (float)numSubtasks;
	}

	if ( subtaskLength > 0 )
	{
		perSubtaskUnit = 1.0 / (float)subtaskLength;
	}

	float subtasks = (float)subtasksComplete * perSubtask;
	float currentSubtask = (float)subtaskProgress * perSubtaskUnit;

	if ( nestedTask != NULL && subtaskLength > 0 )
	{
		nestedTask->lock();
		nestedTaskFrac = nestedTask->getFraction();
		nestedTask->unlock();
	}

	
	return subtasks +
			 currentSubtask * perSubtask +
			 nestedTaskFrac * perSubtaskUnit;
}




#endif
