/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __JOURNALTHREADS_CPP
#define __JOURNALTHREADS_CPP

#include <list>
#include <glib.h>

#include "gSculptConfig.h"

#include "MultiThreading.h"

#include "JournalThreads.h"


#include "Journal.h"



list<JournalTask*> __jthreads_taskList;

GSCMutex *__jthreads_taskListMutex;
GSCCond *__jthreads_taskListCond;

GSCMutex *__jthreads_runningMutex;




JournalTask* __jthreads_threadGetTask()
{
	__jthreads_taskListMutex->lock();

	__jthreads_taskListCond->wait( __jthreads_taskListMutex );
	JournalTask *task = __jthreads_taskList.front();
	__jthreads_taskList.pop_front();
	__jthreads_taskListMutex->unlock();

	return task;
}



static void __jthreads_threadFunction(gpointer data)
{
	while (1)
	{
		JournalTask *task = __jthreads_threadGetTask();

		if (task == NULL)
		{
			__jthreads_runningMutex->unlock();
			return;
		}
		else
		{
			task->run();
		}
	}
}



void initJournalThreads()
{
	if ( ! gSculptConfigIsMultiThreaded() )
	{
		return;
	}
	
	__jthreads_taskListMutex = new GSCMutex();
	__jthreads_taskListCond = new GSCCond();
	
	__jthreads_runningMutex = new GSCMutex();

	__jthreads_runningMutex->lock();
	newThread( (GHookFunc)__jthreads_threadFunction, NULL );
}

void shutdownJournalThreads()
{
	if ( ! gSculptConfigIsMultiThreaded() )
	{
		return;
	}
	
	addJournalTask(NULL);
	__jthreads_runningMutex->lock();
	__jthreads_runningMutex->unlock();

	delete __jthreads_taskListMutex;
	delete __jthreads_taskListCond;
	delete __jthreads_runningMutex;
}



void addJournalTask(JournalTask *task)
{
	if ( ! gSculptConfigIsMultiThreaded() )
	{
		return;
	}

	__jthreads_taskListMutex->lock();
	__jthreads_taskList.push_back(task);
	__jthreads_taskListCond->signal();
	__jthreads_taskListMutex->unlock();
}




#endif
