/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLGROUPLOFTPOLY_CPP
#define __TOOLGROUPLOFTPOLY_CPP

#include "ToolGroupLoftPoly.h"

#include <gtk/gtklabel.h>



ToolGroupLoftPoly::ToolGroupLoftPoly(ProcessToolManager *toolManager,
												 JournalEditor *journEditor,
												 CadView *view,
												 ObjPainterManager *painterManager)
												 : MToolGroup(toolManager)
{
	titleLabel = gtk_label_new("Loft");
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);

	journalEditor = journEditor;

	extrudeButton = addToolButton( "Extrude", GTK_SIGNAL_FUNC(extrudeClicked) );
	spinButton = addToolButton( "Spin", GTK_SIGNAL_FUNC(spinClicked) );

	toolExtrude = new ToolLoftPolyExtrude(view);
	toolSpin = new ToolLoftPolySpin(view);

	toolMan->addTool(toolExtrude->getProcessType(), toolExtrude);
	toolMan->addTool(toolSpin->getProcessType(), toolSpin);
}

ToolGroupLoftPoly::~ToolGroupLoftPoly()
{
	gtk_widget_unref(titleLabel);
	
	toolMan->eraseTool(toolExtrude->getProcessType());
	toolMan->eraseTool(toolSpin->getProcessType());
	delete toolExtrude;
	delete toolSpin;
}



void ToolGroupLoftPoly::addToLayout(GridLayout *layout)
{
	MToolGroup::addToLayout(layout);

	layout->addAsLine(titleLabel);

	layout->add(extrudeButton);
	layout->add(spinButton);
}




void ToolGroupLoftPoly::extrudeClicked(GtkWidget *widget,
													ToolGroupLoftPoly *klass)
{
	ProcLoftPolyExtrude *p = new ProcLoftPolyExtrude();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupLoftPoly::spinClicked(GtkWidget *widget,
												ToolGroupLoftPoly *klass)
{
	ProcLoftPolySpin *p = new ProcLoftPolySpin();
	klass->journalEditor->addProcess(p);
	delete p;
}



#endif
