/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLLOFTEXTRUDE_H
#define __TOOLLOFTEXTRUDE_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "NumericEntry.h"



class ProcLoftPolyExtrude : public JournalProcess
{
private:
	int segments;
	double extrudeLength;
	bool flattenFlag, capStartFlag, capEndFlag;
	
public:
	ProcLoftPolyExtrude();
	ProcLoftPolyExtrude(int extrudeSegs, double length, bool flatten,
							  bool capStart, bool capEnd);
	virtual ~ProcLoftPolyExtrude();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;

	int getExtrudeSegments() const;
	double getExtrudeLength() const;
	bool getFlatten() const;
	bool getCapStart() const;
	bool getCapEnd() const;
	void setExtrudeSegments(int n);
	void setExtrudeLength(double l);
	void setFlatten(bool flatten);
	void setCapStart(bool capStart);
	void setCapEnd(bool capEnd);
	
protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
};




class ToolLoftPolyExtrude : public ProcessTool
{
private:
	GtkWidget *flattenCheck;
	
	GtkWidget *segmentsLabel, *lengthLabel;
	NumericEntry *segmentsEntry, *lengthEntry;

	GtkWidget *capStartCheck, *capEndCheck;
	
	GtkWidget *doneButton;

public:
	ToolLoftPolyExtrude(CadView *v);
	~ToolLoftPolyExtrude();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();


private:
	static void segmentsChanged(NumericEntry *widget, ToolLoftPolyExtrude *klass);
	static void lengthChanged(NumericEntry *widget, ToolLoftPolyExtrude *klass);
	static void flattenClicked(GtkWidget *widget, ToolLoftPolyExtrude *klass);
	static void capStartClicked(GtkWidget *widget, ToolLoftPolyExtrude *klass);
	static void capEndClicked(GtkWidget *widget, ToolLoftPolyExtrude *klass);
	static void doneClicked(GtkWidget *widget, ToolLoftPolyExtrude *klass);
};


#endif
