/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef CGRIDCPP
#define CGRIDCPP

#include "CGrid.h"

#include <iostream>

/*
*******************************************************************************
													CGrid
*******************************************************************************
*/

CGrid::CGrid() : Grid()
{
	unsigned short r = 8192, g = 16384, b = 32768;
	colours.resize( nLevels() );
	for (int i = nLevels() - 1; i >= 0; i--)
	{
		colours[i] = DAColour(r, g, b);
		r >>= 1;
		g >>= 1;
		b >>= 1;
	}
}

CGrid::CGrid(const Ruler& rul, bool useSmallest, double multiplier)
				 : Grid(rul, useSmallest, multiplier)
{
	unsigned short r = 8192, g = 16384, b = 32768;
	colours.resize( nLevels() );
	for (int i = nLevels() - 1; i >= 0; i--)
	{
		colours[i] = DAColour(r, g, b);
		r >>= 1;
		g >>= 1;
		b >>= 1;
	}
}

CGrid::CGrid(const Ruler& rul, bool useSmallest, double multiplier,
				 const array<DAColour>& cols)
				 : Grid(rul, useSmallest, multiplier)
{
	int j = 0;

	if (cols.size() == 0)
	{
		unsigned short r = 8192, g = 16384, b = 32768;
		colours.resize( nLevels() );
		for (int i = nLevels() - 1; i >= 0; i--)
		{
			colours[i] = DAColour(r, g, b);
			r >>= 1;
			g >>= 1;
			b >>= 1;
		}
	}
	else
	{
		for (int i = 0; i < nLevels(); i++)
		{
			colours.push_back( cols[j] );
			j++;
			if (j >= cols.size())
			{
				j--;
			}
		}
	}
}


DAColour& CGrid::getGridColour(int gridNo)
{
	return colours[gridNo];
}



#endif
