/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef GRIDCPP
#define GRIDCPP

#include "Grid.h"




Grid::Grid()
{
	usm = true;
	mul = 0.3;
	setCorners( Point3(0.0, 0.0, 0.0), Point3(1.0, 1.0, 0.0) );
}

Grid::Grid(const Ruler& rl, bool useSmallest, double multiplier)
{
	r = rl;
	usm = useSmallest;
	mul = multiplier;
	setCorners( Point3(0.0, 0.0, 0.0), Point3(1.0, 1.0, 0.0) );
}



void Grid::setCorners(const Point3& p, const Point3& q)
{
	double w, h, size;
	
	p1.x = p.x < q.x ? p.x : q.x;
	p1.y = p.y < q.y ? p.y : q.y;
	p2.x = p.x > q.x ? p.x : q.x;
	p2.y = p.y > q.y ? p.y : q.y;


	w = p2.x - p1.x;
	h = p2.y - p1.y;

	if (usm)
	{
		size = w < h ? w : h;
	}
	else
	{
		size = w > h ? w : h;
	}

	size *= mul;

	r.setLength(size);
}


array<Line3> Grid::generateHLines(int level) const
{
	array<double> ticks;
	array<Line3> result;

	ticks = r.generateTicks(p1.x, p2.x, level);
	for (int i = 0; i < ticks.size(); i++)
	{
		result.push_back( Line3( Point3(ticks[i], p1.y, 0.0),
										 Point3(ticks[i], p2.y, 0.0) ) );
	}

	return result;
}

array<Line3> Grid::generateVLines(int level) const
{
	array<double> ticks;
	array<Line3> result;

	ticks = r.generateTicks(p1.y, p2.y, level);
	for (int i = 0; i < ticks.size(); i++)
	{
		result.push_back( Line3( Point3(p1.x, ticks[i], 0.0),
										 Point3(p2.x, ticks[i], 0.0) ) );
	}

	return result;
}


int Grid::nLevels() const
{
	return r.nLevels();
}

const Ruler& Grid::getRuler() const
{
	return r;
}




	


#endif
