/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef LOGSCALECPP
#define LOGSCALECPP

#include "LogScale.h"



LogScale::Level::Level()
{
	ls = NULL;
}

LogScale::Level::Level(const LogScale *l, int maj, int min)
{
	ls = l;
	major = maj;
	minor = min;
}


LogScale::Level& LogScale::Level::operator++()
{
	minor++;
	if (minor >= ls->powers.size())
	{
		minor = 0;
		major++;
	}

	return *this;
}

LogScale::Level LogScale::Level::operator++(int)
{
	Level temp;
	(*this)++;
	return temp;
}

LogScale::Level& LogScale::Level::operator+=(int i)
{
	minor += i;
	while (minor >= ls->powers.size())
	{
		minor -= ls->powers.size();
		major++;
	}

	return *this;
}

LogScale::Level LogScale::Level::operator+(int i) const
{
	Level l = *this;
	l += i;
	return l;
}



LogScale::Level& LogScale::Level::operator--()
{
	minor--;
	if (minor < 0)
	{
		minor = ls->powers.size() - 1;
		major--;
	}

	return *this;
}

LogScale::Level LogScale::Level::operator--(int)
{
	Level temp;
	(*this)--;
	return temp;
}

LogScale::Level& LogScale::Level::operator-=(int i)
{
	minor -= i;
	while (minor < 0)
	{
		minor += ls->powers.size();
		major--;
	}

	return *this;
}

LogScale::Level LogScale::Level::operator-(int i) const
{
	Level l = *this;
	l -= i;
	return l;
}


double LogScale::Level::getValue() const
{
	return pow( ls->base, (double)major + ls->powers[minor] );
}	

double LogScale::Level::getPower() const
{
	return (double)major + ls->powers[minor];
}	




LogScale::LogScale()
{
	base = 10.0;
	powers.push_back( log(2.0) / log(10.0) );
	powers.push_back( log(5.0) / log(10.0) );
	powers.push_back( 1.0 );
}

LogScale::LogScale(double bs, const array<double> pwr)
{
	base = bs;
	powers = pwr;
}



LogScale::Level LogScale::getSmaller(double v) const
{
	return find( log(v) / log(base) );
}

LogScale::Level LogScale::getNearest(double v) const
{
	double p = log(v) / log(base);
	Level l = find(p);
	Level n = l;
	++n;

	if (( n.getPower() - p ) < ( p - l.getPower() ))
	{
		return n;
	}
	else
	{
		return l;
	}
}

LogScale::Level LogScale::getLarger(double v) const
{
	Level l = find( log(v) / log(base) );
	++l;
	return l;
}





LogScale::Level LogScale::find(double pwr) const
{
	int intPower = (int)pwr;
	double fracPower;

	int minor;
	
	//round down negatives (ie -1.5 would normally round to -1.0, however it
	//should round to -2.0, for this purpose anyway)
	if (pwr < 0.0)
	{
		intPower -= 1;
	}

	//calculate fractional part
	fracPower = pwr - (double)intPower;

	//find which is the largest scale
	minor = -1;
	do
	{
		minor++;
	} while (powers[minor] < fracPower);

	//minor will be 1 larger than wanted, so reduce it
	minor--;
	//if minor was 0, then it will be < 0, so:
	if (minor < 0)
	{
		//make it the last level
		minor = powers.size() - 1;
		//decrease the int power
		intPower--;
	}

	return Level(this, intPower, minor);	
}





#endif
