/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef AXES3CPP
#define AXES3CPP

#include "Axes3.h"



Axes3::Axes3()
{
	i = Vector3( 1.0, 0.0, 0.0 );
	j = Vector3( 0.0, 1.0, 0.0 );
	k = Vector3( 0.0, 0.0, 1.0 );
}

Axes3::Axes3(const Vector3& ai, const Vector3& aj, const Vector3& ak)
{
	i = ai;
	j = aj;
	k = ak;
}

Axes3::Axes3(Axis axesIdentifier)
{
	switch(axesIdentifier)
	{
	case AXIS_X:
		i = Vector3(0.0, 0.0, -1.0);
		j = Vector3(0.0, 1.0, 0.0);
		k = Vector3(1.0, 0.0, 0.0);
		break;
	case AXIS_Y:
		i = Vector3(1.0, 0.0, 0.0);
		j = Vector3(0.0, 0.0, -1.0);
		k = Vector3(0.0, 1.0, 0.0);
		break;
	case AXIS_Z:
		i = Vector3(1.0, 0.0, 0.0);
		j = Vector3(0.0, 1.0, 0.0);
		k = Vector3(0.0, 0.0, 1.0);
		break;
	default:
		i = Vector3(1.0, 0.0, 0.0);
		j = Vector3(0.0, 1.0, 0.0);
		k = Vector3(0.0, 0.0, 1.0);
		break;
	}		
}



Axes3 Axes3::operator+(const Axes3& a) const
{
	return Axes3( i + a.i, j + a.j, k + a.k );
}

Axes3& Axes3::operator+=(const Axes3& a)
{
	i += a.i;
	j += a.j;
	k += a.k;
	return *this;
}

Axes3 Axes3::operator*(double t) const
{
	return Axes3( i * t, j * t, k * t );
}

Axes3& Axes3::operator*=(double t)
{
	i *= t;
	j *= t;
	k *= t;
	return *this;
}


Axes3 Axes3::operator~() const
{
	Matrix4 m = ~( matrixTransformation() );
	return Axes3( Vector3(1.0, 0.0, 0.0) * m,
					  Vector3(0.0, 1.0, 0.0) * m,
					  Vector3(0.0, 0.0, 1.0) * m );
}



Matrix4 Axes3::matrixTransformation() const
{
	return Matrix4( i, j, k );
}

Matrix4 Axes3::toAxesTransformation(const Axes3& a) const
{
	return Matrix4_spaceToSpace( i, j, k, a.i, a.j, a.k );
}



Axes3 operator*(const Axes3& a, const Matrix4& m)
{
	return Axes3( (a.i * m).getNormalised(),
					  (a.j * m).getNormalised(),
					  (a.k * m).getNormalised() );
}




#endif
