/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __BBOX_H
#define __BBOX_H

#include <vector>

#include "array.h"

#include "Point3.h"
#include "Plane.h"
#include "Line3.h"




class BBox
{
private:
	Point3 l, u;

public:
	BBox();
	BBox(const Point3& pt);
	BBox(const Point3& lower, const Point3& upper);
	BBox(const array<Point3>& pts);
	BBox(const vector<Point3>& pts);
	
	//if necessary, alter the bounding box so that p is within it's bounds
	void addPoint(const Point3& p);

	//check if p is within the bounding box
	bool check(const Point3& p) const;

	//check against plane p;
	int check(const Plane& p) const;

	//check to see if the line from st to en intersects *this
	bool intersect(const Line3& ln) const;

	//get lower and upper corners
	Point3 getLower() const;
	Point3 getUpper() const;

	//get the centre
	Point3 getCentre() const;

	//get one of the (eight) corners
	Point3 getPoint(int i) const;


	//get dimensions
	double getWidth() const;
	double getHeight() const;
	double getDepth() const;	
};




#endif
