/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef GEOMETRYCPP
#define GEOMETRYCPP

#include "Geometry.h"




IVector2::IVector2()
{
	x = y = 0;
}

IVector2::IVector2(int ix, int iy)
{
	x = ix;
	y = iy;
}

IVector2::IVector2(const Vector3& v)
{
	x = (int)(v.x + 0.5);
	y = (int)(v.x + 0.5);
}


IVector2 IVector2::operator+(const IVector2& v) const
{
	return IVector2(x + v.x, y + v.y);
}

IVector2& IVector2::operator+=(const IVector2& v)
{
	x += v.x;
	y += v.y;
	return *this;
}

IVector2 IVector2::operator-(const IVector2& v) const
{
	return IVector2(x - v.x, y - v.y);
}

IVector2& IVector2::operator-=(const IVector2& v)
{
	x -= v.x;
	y -= v.y;
	return *this;
}




IPoint2::IPoint2()
{
	x = y = 0;
}

IPoint2::IPoint2(int ix, int iy)
{
	x = ix;
	y = iy;
}

IPoint2::IPoint2(const IVector2& v)
{
	x = v.x;
	y = v.y;
}

IPoint2::IPoint2(const Point3& p)
{
	x = (int)(p.x + 0.5);
	y = (int)(p.y + 0.5);
}


IPoint2 IPoint2::operator+(const IVector2& v) const
{
	return IPoint2(x + v.x, y + v.y);
}

IPoint2& IPoint2::operator+=(const IVector2& v)
{
	x += v.x;
	y += v.y;
	return *this;
}

IVector2 IPoint2::operator-(const IPoint2& p) const
{
	return IVector2(x - p.x, y - p.y);
}

IVector2 IPoint2::toVector2() const
{
	return IVector2(x, y);
}




ISize2::ISize2()
{
	w = h = 0;
}

ISize2::ISize2(int iw, int ih)
{
	w = iw;
	h = ih;
}


bool ISize2::check(const IPoint2& p)
{
	return (p.x >= 0) && (p.y >= 0) && (p.x < w) && (p.y < h);
}




ILine2::ILine2()
{
}

ILine2::ILine2(const IPoint2& ip1, const IPoint2& ip2)
{
	p1 = ip1;
	p2 = ip2;
}

ILine2::ILine2(int x1, int y1, int x2, int y2)
{
	p1 = IPoint2(x1, y1);
	p2 = IPoint2(x2, y2);
}




IRect2::IRect2()
{
	x = y = w = h = 0;
}

IRect2::IRect2(const IPoint2& p, const ISize2& s)
{
	x = p.x;
	y = p.y;
	w = s.w;
	h = s.h;
}

IRect2::IRect2(int ix, int iy, int iw, int ih)
{
	x = ix;
	y = iy;
	w = iw;
	h = ih;
}

IRect2::IRect2(const IPoint2& p1, const IPoint2& p2)
{
	x = p1.x;
	y = p1.y;
	w = (p2.x - x) + 1;
	h = (p2.y - y) + 1;
}


bool IRect2::check(const IPoint2& p) const
{
	return ISize2(w, h).check(p - IPoint2(x, y) );
}


















#endif
