/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __GEOMETRY_H
#define __GEOMETRY_H

#include "Vector3.h"
#include "Point3.h"



class IVector2
{
public:
	int x, y;


	IVector2();
	IVector2(int ix, int iy);
	IVector2(const Vector3& v);

	IVector2 operator+(const IVector2& v) const;
	IVector2& operator+=(const IVector2& v);
	IVector2 operator-(const IVector2& v) const;
	IVector2& operator-=(const IVector2& v);	
};



class IPoint2
{
public:
	int x, y;


	IPoint2();
	IPoint2(int ix, int iy);
	IPoint2(const IVector2& v);
	IPoint2(const Point3& p);

	IPoint2 operator+(const IVector2& v) const;
	IPoint2& operator+=(const IVector2& v);
	IVector2 operator-(const IPoint2& p) const;

	IVector2 toVector2() const;
};



class ISize2
{
public:
	int w, h;


	ISize2();
	ISize2(int iw, int ih);

	bool check(const IPoint2& p);
};




class ILine2
{
public:
	IPoint2 p1, p2;


	ILine2();
	ILine2(const IPoint2& ip1, const IPoint2& ip2);
	ILine2(int x1, int y1, int x2, int y2);
};




class IRect2
{
public:
	int x, y, w, h;


	IRect2();
	IRect2(const IPoint2& p, const ISize2& s);
	IRect2(int ix, int iy, int iw, int ih);
	IRect2(const IPoint2& p1, const IPoint2& p2);


	bool check(const IPoint2& p) const;
};






#endif
