/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef LINE3CPP
#define LINE3CPP

#include "Line3.h"



Line3::Line3()
{
}

Line3::Line3(const Point3& start, const Point3& end)
{
	s = start;
	e = end;
}

Line3::Line3(const Point3& start, const Vector3& direction)
{
	s = start;
	e = s + direction;
}


Vector3 Line3::direction() const
{
	return e - s;
}

bool Line3::intersect(const Plane& p, Point3& intersection) const
{
	double n_dot_dir = p.n.dotProduct(direction());
	double t;

	if (n_dot_dir == 0.0)
	{
		return false;
	}

	t = (p.d - s.dotProduct(p.n)) / n_dot_dir;
	intersection = s + (direction() * t);

	return true;
}


bool Line3::clip(const Plane& p)
{
	double n_dot_dir = p.n.dotProduct( direction() );
	double t;
	Point3 intersection;
	

	if (n_dot_dir == 0.0)
	{
		return true;
	}

	t = (p.d - s.dotProduct(p.n)) / n_dot_dir;

	if (t < 0.0)
	{
		return s > p;
	}
	else if (t > 1.0)
	{
		return e > p;
	}
	
	intersection = s + (direction() * t);

	if ( p.d > s.dotProduct(p.n) )
	{
		s = intersection;
	}
	else
	{
		e = intersection;
	}
	return true;
}


Line3 operator*(const Line3& l, const Matrix4& m)
{
	return Line3(l.s * m, l.e * m);
}



#endif
