/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#include "Plane.h"



Plane::Plane()
{
	n = Vector3(0.0, 0.0, 1.0);
	d = 0.0;
}

Plane::Plane(const Vector3& normal, double distance)
{
	n = normal;
	d = distance;
}

Plane::Plane(const Vector3& normal, const Point3& a)
{
	n = normal;
	d = a.dotProduct(n);
}

Plane::Plane(const Point3& a, const Vector3& u, const Vector3& v)
{
	n = u.crossProduct(v).getNormalised();
	d = a.dotProduct(n);
}

Plane::Plane(const Point3& a, const Point3& b, const Point3& c)
{
	Vector3 u = b - a;
	Vector3 v = c - a;
	n = u.crossProduct(v).getNormalised();
	d = a.dotProduct(n);
}

bool Plane::operator!=(const Plane& cmp) const
{
	return n != cmp.n || d != cmp.d;
}

double Plane::dist(const Point3& p) const
{
	return p.dotProduct(n) - d;
}

Point3 Plane::point() const
{
	return Point3(n * d);
}

Plane Plane::operator-() const
{
	return Plane(-n, -d);
}

bool Plane::intersect(const Point3& st, const Vector3& dir, double& t) const
{
	double dir_dot_n = dir.dotProduct(n);

	if (dir_dot_n != 0.0)
	{
		t = (d - st.dotProduct(n)) / dir_dot_n;
		return true;
	}
	else
	{
		return false;
	}
}

bool Plane::intersect(const Point3& st, const Point3& en, double& t) const
{
	if (intersect(st, en - st, t))
	{
		if (t >= 0.0 && t <= 1.0)
		{
			return true;
		}
	}
	return false;
}


bool operator==(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) == pl.d;
}

bool operator==(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) == pl.d;
}

bool operator!=(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) != pl.d;
}

bool operator!=(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) != pl.d;
}

bool operator<(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) > pl.d;
}

bool operator<(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) < pl.d;
}

bool operator<=(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) >= pl.d;
}


bool operator<=(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) <= pl.d;
}

bool operator>(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) < pl.d;
}

bool operator>(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) > pl.d;
}

bool operator>=(const Plane& pl, const Point3& pt)
{
	return pt.dotProduct(pl.n) <= pl.d;
}

bool operator>=(const Point3& pt, const Plane& pl)
{
	return pt.dotProduct(pl.n) >= pl.d;
}



