/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef POINT3CPP
#define POINT3CPP

#include <math.h>

#include "Point3.h"



Point3::Point3()
{
	x = y = z = 0.0;
}

Point3::Point3(double ix, double iy, double iz)
{
	x = ix;
	y = iy;
	z = iz;
}

Point3::Point3(const Vector3& v)
{
	x = v.x;
	y = v.y;
	z = v.z;
}



void Point3::read(FILE *f)
{
	fread( &x, sizeof(double), 1, f );
	fread( &y, sizeof(double), 1, f );
	fread( &z, sizeof(double), 1, f );
}

void Point3::write(FILE *f) const
{
	fwrite( &x, sizeof(double), 1, f );
	fwrite( &y, sizeof(double), 1, f );
	fwrite( &z, sizeof(double), 1, f );
}


bool Point3::operator==(const Point3& p) const
{
	return (x == p.x) && (y == p.y) && (z == p.z);
}

bool Point3::operator!=(const Point3& p) const
{
	return (x != p.x) || (y != p.y) || (z != p.z);
}


Point3 Point3::operator+(const Vector3& v) const
{
	return Point3(x + v.x, y + v.y, z + v.z);
}

Point3& Point3::operator+=(const Vector3& v)
{
	x += v.x;
	y += v.y;
	z += v.z;
	return *this;
}

Point3 Point3::operator-(const Vector3& v) const
{
	return Point3(x - v.x, y - v.y, z - v.z);
}

Point3& Point3::operator-=(const Vector3& v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	return *this;
}

Vector3 Point3::operator-(const Point3& v) const
{
	return Vector3(x - v.x, y - v.y, z - v.z);
}


Vector3 Point3::toVector3() const
{
	return Vector3(x, y, z);
}


double Point3::dotProduct(const Vector3& v) const
{
	return (x * v.x) + (y * v.y) + (z * v.z);
}

double Point3::distanceSquared(const Point3& p) const
{
	return (*this - p).lengthSquared();
}

double Point3::distanceTo(const Point3& p) const
{
	return (*this - p).length();
}



#endif
