/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef VECTOR3H
#define VECTOR3H

#include <stdio.h>


typedef enum {
	AXIS_X,
	AXIS_Y,
	AXIS_Z
} Axis;



class Vector3
{
public:
	double x, y, z;


	Vector3();
	Vector3(double ix, double iy, double iz);
	Vector3(Axis a);


	void read(FILE *f);
	void write(FILE *f) const;


	bool operator==(const Vector3& v) const;
	bool operator!=(const Vector3& v) const;

	Vector3 operator+(const Vector3& v) const;
	Vector3& operator+=(const Vector3& v);
	Vector3 operator-(const Vector3& v) const;
	Vector3& operator-=(const Vector3& v);
	Vector3 operator*(double f) const;
	Vector3& operator*=(double f);
	Vector3 operator-() const;

	double dotProduct(const Vector3& v) const;
	Vector3 crossProduct(const Vector3& v) const;
	double lengthSquared() const;
	double length() const;
	double manhattanLength() const;
	void normalise();
	void normalise(double l);
	Vector3 getNormalised() const;
	Vector3 getNormalised(double l) const;
};



#endif
