/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef VIEWFRUSTRUMCPP
#define VIEWFRUSTRUMCPP

#include "ViewFrustrum.h"



ViewFrustrum::ViewFrustrum()
{
}

ViewFrustrum::ViewFrustrum(const Vector3& viewDirection, const array<Point3>& corners)
{
	array<Point3>::const_dual_iterator iter = corners.circularDual();

	pl.reserve(corners.size());

	for (int i = 0; i < corners.size(); i++)
	{
		pl.push_back( Plane( *(iter.first), viewDirection, *(iter.second) - *(iter.first) ) );
		++iter;
	}
}

ViewFrustrum::ViewFrustrum(const array<Plane>& planes)
{
	pl = planes;
}


bool ViewFrustrum::checkPoint(const Point3& p) const
{
	for (int i = 0; i < pl.size(); i++)
	{
		if (p < pl[i])
		{
			return false;
		}
	}
	return true;
}

long ViewFrustrum::pointBitList(const Point3& p) const
{
	long res = 0;
	
	for (int i = 0; i < pl.size(); i++)
	{
		if (p < pl[i])
		{
			res |= 1 << i;
		}
	}
	
	return res;
}


bool ViewFrustrum::clip(Line3& l) const
{
	for (int i = 0; i < pl.size(); i++)
	{
		if ( !l.clip( pl[i] ) )
		{
			return false;
		}
	}

	return true;
}



ViewFrustrum operator*(ViewFrustrum f, const Matrix4& m)
{
	for (int i = 0; i < f.pl.size(); i++)
	{
		f.pl[i] = f.pl[i] * m;
	}
	return f;
}


#endif
