/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CMESH_CPP
#define __CMESH_CPP

#include "CMesh.h"


/*
*******************************************************************************
											CMesh
*******************************************************************************
*/


CMesh::CMesh() : CModel()
{
	numPatches = 0;
}

/*CMesh::CMesh(const CMesh& m) : CModel()
{
	map<const CVertex*, CVertex*> vertexMapping;

	v.duplicate( m.v, vertexMapping );
	f.duplicate( m.f, vertexMapping );

	computeEdgeList();
}*/

CMesh::~CMesh()
{
}



/*CMesh& CMesh::operator=(const CMesh& m)
{
	map<const CVertex*, CVertex*> vertexMapping;

	v.duplicate( m.v, vertexMapping );
	f.duplicate( m.f, vertexMapping );

	computeEdgeList();

	return *this;
}*/



void CMesh::addFaceEdges(CFace &fc)
{
	int a, b;

	int i = fc.size() - 1;
	a = fc[i].vertexNum;
		
	for (int j = 0; j < fc.size(); j++)
	{
		b = fc[j].vertexNum;

		CEdge edge = CEdge(a, b);

		if ( e.count(edge) == 0 )
		{
			e.insert(edge);
		}

		a = b;
		i = j;
	}
}

void CMesh::computeEdgeList()
{
	e.clear();
	
	for (CFaceContainer::iterator iter = f.begin();
		  iter != f.end();
		  ++iter)
	{
		addFaceEdges( *iter );
	}
}


void CMesh::read(FILE *fh)
{
	v.read(fh);
	f.read(fh);

	fread( &numPatches, sizeof(int), 1, fh );

	computeEdgeList();
}

void CMesh::write(FILE *fh) const
{
	v.write(fh);
	f.write(fh);

	fwrite( &numPatches, sizeof(int), 1, fh );
}


CVertex* CMesh::addVertex(const Point3& location, bool marked)
{
	v.push_back( CVertex(location) );

	if (marked)
	{
		v.back().mark();
	}
	
	return &v.back();
}

CFace* CMesh::addFace(int va, const Point3& ta,
							 int vb, const Point3& tb,
							 int vc, const Point3& tc, int patch)
{
	f.push_back( CFace( va, ta, vb, tb, vc, tc, patch ) );
	addFaceEdges( f.back() );
	return &f.back();
}

CFace* CMesh::addFace(int va, const Point3& ta,
							 int vb, const Point3& tb,
							 int vc, const Point3& tc,
							 int vd, const Point3& td, int patch)
{
	f.push_back( CFace( va, ta, vb, tb, vc, tc, vd, td, patch ) );
	addFaceEdges( f.back() );
	return &f.back();
}

CFace* CMesh::addFace(const CFace& fc)
{
	f.push_back(fc);
	addFaceEdges( f.back() );
	return &f.back();
}

void CMesh::removeFace(CFace* fc)
{
	for (CFaceContainer::iterator iter = f.begin();
		  iter != f.end();
		  ++iter)
	{
		if (&(*iter) == fc)
		{
			f.erase(iter);
			computeEdgeList();
			return;
		}
	}
}

void CMesh::addPatch(int patches)
{
	numPatches += patches;
}



void CMesh::copyFrom(const CMesh *m)
{
	v.copyFrom( &(m->v) );
	f.copyFrom( &(m->f) );
	e.copyFrom( &(m->e) );
}



void CMesh::reserveVertices(int s)
{
	v.reserve(s);
}

void CMesh::reserveFaces(int s)
{
	f.reserve(s);
}



CVertexContainer& CMesh::vertices()
{
	return v;
}

const CVertexContainer& CMesh::vertices() const
{
	return v;
}


CFaceContainer& CMesh::faces()
{
	return f;
}

const CFaceContainer& CMesh::faces() const
{
	return f;
}


set<CEdge>& CMesh::edges()
{
	return e;
}

const set<CEdge>& CMesh::edges() const
{
	return e;
}



void CMesh::transform(const Matrix4& m, bool selectedOnly)
{
	v.transform(m, selectedOnly);
}

void CMesh::transform(const Transformation *x, bool selectedOnly)
{
	v.transform(x, selectedOnly);
}

void CMesh::transformBackup(bool selectedOnly)
{
	xformBak = v.backup(selectedOnly);
}

void CMesh::transformRestore()
{
	v.restore(xformBak);
}


void CMesh::select(SelectPredicate predicate, const SelectRegion *region)
{
	v.select( predicate, region );
}

void CMesh::selectBackup()
{
	selectBak = v.backup(false);
}

void CMesh::selectRestore()
{
	v.restore(selectBak);
}


BBox CMesh::getBBox(bool selectedOnly) const
{
	return v.getBBox(selectedOnly);
}

BBox CMesh::getBBox(const Matrix4& m, bool selectedOnly) const
{
	return v.getBBox(m, selectedOnly);
}

BBox CMesh::getBBox(const Transformation *x, bool selectedOnly) const
{
	return v.getBBox(x, selectedOnly);
}


#endif
