/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by :					Geoffrey French
	application :		gSculpt
	last revision :	02/Jul/2000
	title :				Model
*/
#ifndef __CMESH_H
#define __CMESH_H

#include <set>


#include "CModel.h"

#include "CVertex.h"
#include "CVertexContainer.h"
#include "CFace.h"
#include "CFaceContainer.h"
#include "CEdge.h"
#include "CEdgeContainer.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"

#include "CadArea.h"

#include "Point3.h"


/*
*******************************************************************************
												CMesh
*******************************************************************************
*/

class CMesh : public CModel
{
private:
	CVertexContainer v;
	CFaceContainer f;
	CEdgeContainer e;
	CVertexContainer::VertexBackup xformBak, selectBak;
	int numPatches;

public:
	CMesh();
	//CMesh(const CMesh& m);
	virtual ~CMesh();


	//CMesh& operator=(const CMesh& m);


private:
	void addFaceEdges(CFace &fc);
	void computeEdgeList();


public:
	virtual void read(FILE *fh);
	virtual void write(FILE *fh) const;


	CVertex* addVertex(const Point3& location, bool marked = true);
	CFace* addFace(int va, const Point3& ta,
						int vb, const Point3& tb,
						int vc, const Point3& tc, int patch);
	CFace* addFace(int va, const Point3& ta,
						int vb, const Point3& tb,
						int vc, const Point3& tc,
						int vd, const Point3& td, int patch);
	CFace* addFace(const CFace& fc);
	void removeFace(CFace* fc);
	void addPatch(int patches = 1);


	void reserveVertices(int s);
	void reserveFaces(int s);


	void copyFrom(const CMesh *m);


	CVertexContainer& vertices();
	const CVertexContainer& vertices() const;

	CFaceContainer& faces();
	const CFaceContainer& faces() const;

	set<CEdge>& edges();
	const set<CEdge>& edges() const;



	virtual void transform(const Matrix4& m, bool selectedOnly = true);
	virtual void transform(const Transformation *x, bool selectedOnly = true);
	virtual void transformBackup(bool selectedOnly = true);
	virtual void transformRestore();

	virtual void select(SelectPredicate predicate, const SelectRegion *region);
	virtual void selectBackup();
	virtual void selectRestore();

	virtual BBox getBBox(bool selectedOnly = true) const;
	virtual BBox getBBox(const Matrix4& m, bool selectedOnly = true) const;
	virtual BBox getBBox(const Transformation *x, bool selectedOnly = true) const;
};


#endif
