/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __MESHPAINTER_CPP
#define __MESHPAINTER_CPP

#include "MeshPainter.h"

#include "ModelDraw.h"




MeshPainter::MeshPainter(CadArea *c) : ObjectPainter(c)
{
	mesh = NULL;
}


void MeshPainter::setMesh(CMesh *m)
{
	mesh = m;
}

void MeshPainter::setObject(JournalObject *o)
{
	ObjectPainter::setObject(o);
	JournalMesh *m = (JournalMesh*)o;
	mesh = m->getMesh();
}


ObjectPainter* MeshPainter::clone()
{
	return new MeshPainter(cadArea);
}


void MeshPainter::paint()
{
	if (mesh != NULL)
	{
		drawMesh();
	}
}

void MeshPainter::drawMesh()
{
	CVertexContainer &v = mesh->vertices();
	set<CEdge> &e = mesh->edges();
	
	IPoint2 p;

	if (v.size() == 0)
	{
		return;
	}

	for (CVertexContainer::iterator iter = v.begin();
		  iter != v.end();
		  iter++)
	{
		(*iter).update( cadArea->getWorldScreenTransformation(),
							 cadArea->getViewFrustrum(), cadArea->getFrame() );
	}

	//draw edges

	for (set<CEdge>::const_iterator iter = e.begin();
		  iter != e.end();
		  iter++)
	{
		drawEdge( &(*iter), cadArea, v );
	}


	if ( !backgroundFlag )
	{
		for (CVertexContainer::const_iterator iter = v.begin();
			  iter != v.end();
			  iter++)
		{
			if ( !(*iter).isVisible() )
			{
				continue;
			}
		
			p = IPoint2( (*iter).getScreen() );
			if ( (*iter).isMarked() )
			{
				modelDrawMarkedVertex( cadArea, p );
			}
			else
			{
				modelDrawUnmarkedVertex( cadArea, p );
			}
		}
	}
}

void MeshPainter::drawEdge(const CEdge *edge, CadArea *cadArea,
									const CVertexContainer& v)
{
	ILine2 sl;
	Line3 wl;
	IPoint2 p1, p2;
	
	const CEdge &ed = *edge;
	
	const CVertex *s = &( v[ ed[0] ] );
	const CVertex *e = &( v[ ed[1] ] );


	if ((s->getVisibilityData() != 0) || (e->getVisibilityData() != 0))
	{
		wl = Line3( s->getWorld(), e->getWorld() );
		if ( !cadArea->getViewFrustrum().clip(wl) )
		{
			return;
		}
		else
		{
			p1 = cadArea->transformWorldScreenInt2( wl.s );
			p2 = cadArea->transformWorldScreenInt2( wl.e );
			sl = ILine2( p1, p2 );
		}
	}
	else
	{
		sl = ILine2( IPoint2( s->getScreen() ),
						 IPoint2( e->getScreen() ) );
	}

	if (backgroundFlag)
	{
		modelDrawEdgeBackground(cadArea, sl);
	}
	else
	{
		modelDrawEdge(cadArea, sl);
	}
}



#endif
