/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __MULTITHREADING_CPP
#define __MULTITHREADING_CPP


#include "MultiThreading.h"

#include "gSculptConfig.h"




#ifdef GSCULPT_CONFIG_MULTITHREADED
#include <pthread.h>
#endif





void initThreads()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	if ( !g_thread_supported() )
	{
		g_thread_init(NULL);
	}
#endif
}


void newThread(GHookFunc func, gpointer data)
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	pthread_t thread;
	pthread_attr_t threadAttributes;

	pthread_attr_init(&threadAttributes);

	pthread_create( &thread, &threadAttributes,
						 (void *(*)(void*))func, data );
#endif
}



GSCMutex::GSCMutex()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	mutex = g_mutex_new();
#endif
}

GSCMutex::~GSCMutex()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_mutex_free( mutex );
#endif
}


void GSCMutex::lock()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_mutex_lock( mutex );
#endif
}

void GSCMutex::unlock()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_mutex_unlock( mutex );
#endif
}




GSCCond::GSCCond()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	cond = g_cond_new();
#endif
}

GSCCond::~GSCCond()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_cond_free( cond );
#endif
}


void GSCCond::signal()
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_cond_signal( cond );
#endif
}

void GSCCond::wait(GSCMutex *mutex)
{
#ifdef GSCULPT_CONFIG_MULTITHREADED
	g_cond_wait( cond, mutex->mutex );
#endif
}




#endif
