/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __COBJECTEDITOR_CPP
#define __COBJECTEDITOR_CPP

#include "CObjectEditor.h"

#include <math.h>



CObjectEditor::CObjectEditor(const string& wintitle, ObjectCollection *objColl,
									  GtkSignalFunc closeCallbackFunc,
									  gpointer closeCallbackData)
	: CadEditor( new CadView(), wintitle, closeCallbackFunc, closeCallbackData )
{
	objects = objColl;
	
	view = vw;


	painterManager = new ObjPainterManager();

	polyPainter = new PolylinePainter( view->getCadArea() );
	painterManager->addPainter( JournalPolyline::getTypeStatic(), polyPainter );

	meshPainter = new MeshPainter( view->getCadArea() );
	painterManager->addPainter( JournalMesh::getTypeStatic(), meshPainter );



	toolMan = new ProcessToolManager();


	objectEditorToolset = new ToolsetObjectEdit( win, objects, painterManager,
																view, toolMan,
																win->getStatusBar() );
	win->addToolset(objectEditorToolset, "Obj");

	createObjectToolkit = new ToolkitCreateObject( win, view, objectEditorToolset,
																  toolMan, objColl );
	polylineToolkit = new ToolkitPolyline( win, view, objectEditorToolset,
														toolMan, painterManager );
	meshToolkit = new ToolkitMesh( win, view, objectEditorToolset,
											 toolMan, painterManager );


	viewToolset = new ToolsetView(win, view);
	win->addToolset(viewToolset, "View");


	objectEditorToolset->addToolkit( JournalObject::getTypeStatic(),
												createObjectToolkit );
	objectEditorToolset->addToolkit( JournalPolyline::getTypeStatic(),
												polylineToolkit );
	objectEditorToolset->addToolkit( JournalMesh::getTypeStatic(),
												meshToolkit );
}

CObjectEditor::~CObjectEditor()
{
}


void CObjectEditor::shutdown()
{
	CadEditor::shutdown();
	delete painterManager;
	delete polyPainter;
	delete meshPainter;
	
	delete objectEditorToolset;
	delete createObjectToolkit;
	delete polylineToolkit;
	delete meshToolkit;

	delete viewToolset;

	delete toolMan;

	delete view;
}


#endif
