/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by:					Geoffrey French
	application:		gSculpt
	last revision:		22/Sep/2000
*/
#ifndef __OBJECTCOLLECTION_H
#define __OBJECTCOLLECTION_H

#include <stdio.h>

#include <string>
#include <map>
#include <set>
#include <list>

#include "UniqueId.h"

#include "fileio.h"

#include "Journal.h"
#include "ListManager.h"




class ObjectCollection : public JournalCollection
{
private:
	struct Entry {
		Journal *journal;
		string name;
		set<UniqueId> dependants;
		UniqueId dependancy;
	};

	typedef map<UniqueId,Entry> EntryTable;
	typedef map<string,UniqueId> IdTable;
	typedef list<ListManager*> LsManList;

	UniqueId uniqueIdCounter;

	EntryTable objects;
	IdTable nameToId;
	
	list<ListManager*> listManagers;

public:
	ObjectCollection();
	virtual ~ObjectCollection();


	void read(FILE *f);
	void write(FILE *f);
	void exportExplicit(FILE *f);



	bool exists(const string& name) const;
	bool exists(UniqueId i) const;
	void add(const string& name);
	void remove(const string& name);
	string getSelectionName(ListManager *lsMan) const;
	Journal* getObject(const string& name);
	const Journal* getObject(const string& name) const;
	virtual Journal* getObject(UniqueId i);
	virtual const Journal* getObject(UniqueId i) const;
	Journal* getSelection(ListManager *lsMan);

	UniqueId getUniqueId(const string& name) const;
	virtual UniqueId getUniqueId(const Journal *journal) const;
	string getName(UniqueId i) const;

	ListManager* getListManager();
	void putbackListManager(ListManager *lsMan);

	bool hasDependants(UniqueId objectId) const;
	bool hasDependant(UniqueId objectId, UniqueId dependantId) const;
	bool hasDependancyPath(UniqueId start, UniqueId end) const;
	virtual void setDependancy(UniqueId objectId, UniqueId dependancyId);
	virtual void removeDependancy(UniqueId objectId);
	bool getDependantNames(UniqueId objectId, set<string>& dependantNames) const;

private:
	UniqueId createUniqueId();
};




#endif
