/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSETOBJECTEDIT_CPP
#define __TOOLSETOBJECTEDIT_CPP

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkclist.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkarrow.h>

#include <vector>
#include <set>


#include "messageBox.h"


#include "ToolsetObjectEdit.h"

#include "gSculptConfig.h"



ToolsetObjectEdit::ToolsetObjectEdit(Toolbox *tbx, ObjectCollection *objColl,
												 ObjPainterManager *pManager,
												 CadView *vw,
												 ProcessToolManager *toolManager,
												 StatusBar *statBar)
												 : Toolset("---Object---", tbx)
{
	toolMan = toolManager;
	statusBar = statBar;
	threadMonitor = new JournalTaskMonitor(statusBar);


	
	currentToolkit = NULL;
	
	currentObject.journ = NULL;
	currentObject.painter = NULL;
	
	currentTool.tool = NULL;
	currentTool.journalObject = NULL;

	painterManager = pManager;
	view = vw;


	//the layout will contain the title label and the vertical
	//packing box which contains everything else

	//the main table must NOT be homogeneous
	gtk_table_set_homogeneous( GTK_TABLE(toolTable), false );


	//create the vertical packing box; not homogeneous
	verticalBox = gtk_vbox_new( false, 0 );
	gtk_widget_ref(verticalBox);

	

	

	backgroundObjects = new BackgroundObjManager(view, painterManager);

	
	ignoreJournalListSelectEvents = false;


	objects = objColl;

	myListManager = objects->getListManager();







	objectsLabel = gtk_label_new("Objects:");
	objectsTitleTable = gtk_table_new( 2, 1, true );
	gtk_widget_ref(objectsLabel);
	gtk_widget_ref(objectsTitleTable);
	gtk_widget_show(objectsLabel);
	gtk_table_attach( GTK_TABLE(objectsTitleTable), objectsLabel,
							0, 1, 1, 2,
							(GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
							(GtkAttachOptions)GTK_FILL, 0, 0 );
	gtk_widget_show( objectsTitleTable );


	gtk_widget_show( myListManager->getWidget() );


								
	
	objectCreateButton = gtk_button_new_with_label("Create");
	objectEditButton = gtk_button_new_with_label("Edit");
	objectDeleteButton = gtk_button_new_with_label("Delete");
	objectShowButton = gtk_button_new_with_label("Show");
	objectHideButton = gtk_button_new_with_label("Hide");

	gtk_widget_ref(objectCreateButton);
	gtk_widget_ref(objectEditButton);
	gtk_widget_ref(objectDeleteButton);
	gtk_widget_ref(objectShowButton);
	gtk_widget_ref(objectHideButton);

	gtk_signal_connect( GTK_OBJECT(objectCreateButton), "clicked",
							  GTK_SIGNAL_FUNC(objectCreateClicked), this );
	gtk_signal_connect( GTK_OBJECT(objectEditButton), "clicked",
							  GTK_SIGNAL_FUNC(objectEditClicked), this );
	gtk_signal_connect( GTK_OBJECT(objectDeleteButton), "clicked",
							  GTK_SIGNAL_FUNC(objectDeleteClicked), this );
	gtk_signal_connect( GTK_OBJECT(objectShowButton), "clicked",
							  GTK_SIGNAL_FUNC(objectShowClicked), this );
	gtk_signal_connect( GTK_OBJECT(objectHideButton), "clicked",
							  GTK_SIGNAL_FUNC(objectHideClicked), this );

	gtk_widget_show( objectCreateButton );
	gtk_widget_show( objectEditButton );
	gtk_widget_show( objectDeleteButton );
	gtk_widget_show( objectShowButton );
	gtk_widget_show( objectHideButton );


	objectToolsTable = gtk_table_new(2, 3, true);
	gtk_widget_ref(objectToolsTable);
	
	objectsLayout = new GridLayout( GTK_TABLE( objectToolsTable ) );
	objectsLayout->add( objectCreateButton );
	objectsLayout->add( objectEditButton );
	objectsLayout->add( objectDeleteButton );
	objectsLayout->add( objectShowButton );
	objectsLayout->add( objectHideButton );

	gtk_widget_show( objectToolsTable );

	gtk_box_pack_start( GTK_BOX(verticalBox), objectsTitleTable,
							  false, false, 0 );
	gtk_box_pack_start( GTK_BOX(verticalBox), myListManager->getWidget(),
							  true, true, 0 );
	gtk_box_pack_start( GTK_BOX(verticalBox), objectToolsTable,
							  false, false, 0 );




	


	journalSeparator = gtk_hseparator_new();
	journalLabel = gtk_label_new("Journal:");
	journalTitleTable = gtk_table_new( 3, 1, true );
	gtk_widget_ref(journalSeparator);
	gtk_widget_ref(journalLabel);
	gtk_widget_ref(journalTitleTable);
	gtk_widget_show(journalSeparator);
	gtk_widget_show(journalLabel);
	gtk_table_attach( GTK_TABLE(journalTitleTable), journalSeparator,
							0, 1, 0, 1,
							(GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
							(GtkAttachOptions)GTK_FILL, 0, 0 );
	gtk_table_attach( GTK_TABLE(journalTitleTable), journalLabel,
							0, 1, 2, 3,
							(GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
							(GtkAttachOptions)GTK_FILL, 0, 0 );
	gtk_widget_show( journalTitleTable );
								
	
	char *titles[] = {"Action"};
	journalCListWidget = gtk_clist_new_with_titles(1, titles);
	gtk_widget_ref(journalCListWidget);
	gtk_signal_connect( GTK_OBJECT(journalCListWidget), "select_row",
							  GTK_SIGNAL_FUNC(journalCListRowSelected), this );
	gtk_widget_show(journalCListWidget);


	journalScrolledWin = gtk_scrolled_window_new( NULL, NULL );
	gtk_widget_ref(journalScrolledWin);
	gtk_scrolled_window_set_policy(
		GTK_SCROLLED_WINDOW(journalScrolledWin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(journalScrolledWin),
		journalCListWidget );
	gtk_widget_show(journalScrolledWin);



	journalModifyButton = gtk_button_new_with_label("Modify");
	journalRemoveButton = gtk_button_new_with_label("Remove");
	journalMoveUpButton = gtk_button_new();
	journalMoveDownButton = gtk_button_new();
	journalUpBox = gtk_vbox_new( false, 0 );
	journalDownBox = gtk_vbox_new( false, 0 );
	journalUpArrow = gtk_arrow_new( GTK_ARROW_UP, GTK_SHADOW_OUT );
	journalDownArrow = gtk_arrow_new( GTK_ARROW_DOWN, GTK_SHADOW_OUT );

	gtk_widget_ref(journalModifyButton);
	gtk_widget_ref(journalRemoveButton);
	gtk_widget_ref(journalMoveUpButton);
	gtk_widget_ref(journalMoveDownButton);
	gtk_widget_ref(journalUpBox);
	gtk_widget_ref(journalDownBox);
	gtk_widget_ref(journalUpArrow);
	gtk_widget_ref(journalDownArrow);

	gtk_signal_connect( GTK_OBJECT(journalModifyButton), "clicked",
							  GTK_SIGNAL_FUNC(journalModifyClicked), this );
	gtk_signal_connect( GTK_OBJECT(journalRemoveButton), "clicked",
							  GTK_SIGNAL_FUNC(journalRemoveClicked), this );
	gtk_signal_connect( GTK_OBJECT(journalMoveUpButton), "clicked",
							  GTK_SIGNAL_FUNC(journalMoveUpClicked), this );
	gtk_signal_connect( GTK_OBJECT(journalMoveDownButton), "clicked",
							  GTK_SIGNAL_FUNC(journalMoveDownClicked), this );
		
	gtk_widget_show(journalModifyButton);
	gtk_widget_show(journalRemoveButton);
	gtk_widget_show(journalUpArrow);
	gtk_widget_show(journalDownArrow);

	gtk_container_set_border_width( GTK_CONTAINER(journalUpBox), 3 );
	gtk_container_set_border_width( GTK_CONTAINER(journalDownBox), 3 );
	gtk_container_add( GTK_CONTAINER(journalUpBox), journalUpArrow );
	gtk_container_add( GTK_CONTAINER(journalDownBox), journalDownArrow );
	gtk_widget_show(journalUpBox);
	gtk_widget_show(journalDownBox);

	gtk_container_add( GTK_CONTAINER(journalMoveUpButton), journalUpBox );
	gtk_container_add( GTK_CONTAINER(journalMoveDownButton), journalDownBox );
	gtk_widget_show(journalMoveUpButton);
	gtk_widget_show(journalMoveDownButton);


	journalToolsTable = gtk_table_new( 2, 3, true );
	gtk_widget_ref( journalToolsTable );

	journalLayout = new GridLayout( GTK_TABLE(journalToolsTable) );
	journalLayout->add( journalModifyButton );
	journalLayout->add( journalRemoveButton );
	journalLayout->add( journalMoveUpButton );
	journalLayout->add( journalMoveDownButton );


	gtk_widget_show( journalToolsTable );

	gtk_box_pack_start( GTK_BOX(verticalBox), journalTitleTable,
							  false, false, 0 );
	gtk_box_pack_start( GTK_BOX(verticalBox), journalScrolledWin,
							  true, true, 0 );
	gtk_box_pack_start( GTK_BOX(verticalBox), journalToolsTable,
							  false, false, 0 );

	gtk_widget_show( verticalBox );

	layout.add( verticalBox,
					(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK) );
}

ToolsetObjectEdit::~ToolsetObjectEdit()
{
	uninstallCurrentTool();
	uninstallCurrentToolkit();
	uninstallCurrentPainter();


	gtk_widget_unref(verticalBox);

	gtk_widget_unref(objectsLabel);
	gtk_widget_unref(objectsTitleTable);

	objects->putbackListManager(myListManager);
	
	gtk_widget_unref(objectCreateButton);
	gtk_widget_unref(objectEditButton);
	gtk_widget_unref(objectDeleteButton);
	gtk_widget_unref(objectShowButton);
	gtk_widget_unref(objectHideButton);
	gtk_widget_unref(objectToolsTable);
	
	
	gtk_widget_unref(journalSeparator);
	gtk_widget_unref(journalLabel);
	gtk_widget_unref(journalTitleTable);
	
	gtk_widget_unref(journalCListWidget);
	gtk_widget_unref(journalScrolledWin);

	gtk_widget_unref(journalModifyButton);
	gtk_widget_unref(journalRemoveButton);
	gtk_widget_unref(journalMoveUpButton);
	gtk_widget_unref(journalMoveDownButton);
	gtk_widget_unref(journalUpBox);
	gtk_widget_unref(journalDownBox);
	gtk_widget_unref(journalUpArrow);
	gtk_widget_unref(journalDownArrow);
	gtk_widget_unref(journalToolsTable);


	delete backgroundObjects;
	delete threadMonitor;
}


void ToolsetObjectEdit::addToolkit(const ObjectType *type, MToolkit *k)
{
	if ( toolkits.count( type->getId() ) == 0 )
	{
		toolkits[ type->getId() ] = k;
	}
}

void ToolsetObjectEdit::addProcess(JournalProcess *proc)
{
	if ( currentObject.journ != NULL )
	{
		string description = proc->getDescription();
		char *text = strdup( description.c_str() );


		int insertPosition = journalSelectedProc + 1;


		ignoreJournalListSelectEvents = true;
		gtk_clist_insert( GTK_CLIST(journalCListWidget),
								processToRow(insertPosition), &text );
		ignoreJournalListSelectEvents = false;

		free(text);

		currentObject.journ->insert( insertPosition, proc );

		gtk_clist_select_row( GTK_CLIST(journalCListWidget),
									 processToRow(insertPosition), 0 );

		//the journal list has changed
		journalCListChanged();

		modifySelectedProcess();
	}
}

void ToolsetObjectEdit::modifySelectedProcess()
{
	if ( currentObject.journ != NULL )
	{
		if ( !isAValidProcessSelected() )
		{
			return;
		}
		
		//if the integrity of the journal list is not valid at this point, stop
		if ( checkIntegrityAt( journalSelectedProc ) )
		{
			if ( gSculptConfigIsMultiThreaded() )
			{
				JournalTaskProgress *progress =
							currentObject.journ->computeObjectThreaded(
															journalSelectedProc - 1 );
				threadMonitor->start( progress,
							GTK_SIGNAL_FUNC(modifySelectedProcComputedBeforeStatic),
							this );
			}
			else
			{
				JournalObject *before =	currentObject.journ->getObjectUnthreaded(
														journalSelectedProc - 1 );
				modifySelectedProcComputedBefore(before);
			}
		}
	}	
}

void ToolsetObjectEdit::modifySelectedProcComputedBeforeStatic(
									JournalTaskProgress *thread, ToolsetObjectEdit *klass)
{
	klass->modifySelectedProcComputedBefore(
					klass->currentObject.journ->getThreadedComputationResult() );
}

void ToolsetObjectEdit::modifySelectedProcComputedBefore(
															JournalObject *before)
{
	JournalProcess *proc =
		currentObject.journ->getProcess( journalSelectedProc );
	const ObjectType *procType = proc->getType();

	ProcessTool *tool = (ProcessTool*)toolMan->takeTool( procType );
	installNewTool(tool, proc, before);


	uninstallCurrentPainter();

	if ( !currentTool.tool->hasOwnPainter() )
	{
		JournalObject *after =
			currentObject.journ->getObjectUnthreaded( journalSelectedProc );

		installNewCurrentPainter(after);
	}

	redrawView();		//current painter has changed
}

void ToolsetObjectEdit::removeSelectedProcess()
{
	if ( currentObject.journ != NULL && isAValidProcessSelected() )
	{
		ignoreJournalListSelectEvents = true;
		gtk_clist_remove( GTK_CLIST(journalCListWidget),
								processToRow(journalSelectedProc) );

		ignoreJournalListSelectEvents = false;

		currentObject.journ->remove( journalSelectedProc );

		findIntegrityBreak();

		journalSelectedProc--;
		if (journalSelectedProc < -1)
		{
			journalSelectedProc = -1;
		}

		gtk_clist_select_row( GTK_CLIST(journalCListWidget),
									 processToRow(journalSelectedProc), 0 );

		//the journal list has changed
		journalCListChanged();
	}
}

void ToolsetObjectEdit::swapSelectedProcessWith(int index)
{
	if ( currentObject.journ != NULL && isAValidProcessSelected() &&
		  index >= 0 && index < currentObject.journ->size() )
	{
		ignoreJournalListSelectEvents = true;
		int row1 = processToRow(journalSelectedProc);
		int row2 = processToRow(index);
		gpointer data1 = gtk_clist_get_row_data( GTK_CLIST(journalCListWidget),
															  row1 );
		gpointer data2 = gtk_clist_get_row_data( GTK_CLIST(journalCListWidget),
															  row2 );
		gtk_clist_set_row_data( GTK_CLIST(journalCListWidget), row1, data2 );
		gtk_clist_set_row_data( GTK_CLIST(journalCListWidget), row2, data1 );
		ignoreJournalListSelectEvents = false;

		currentObject.journ->swap( journalSelectedProc, index );

		findIntegrityBreak();

		journalSelectedProc = index;

		gtk_clist_select_row( GTK_CLIST(journalCListWidget),
									 processToRow(journalSelectedProc), 0 );

		//the journal list has changed
		journalCListChanged();
	}
}



int ToolsetObjectEdit::rowToProcess(int rowNumber)
{
	return rowNumber - 1;
}

int ToolsetObjectEdit::processToRow(int processNumber)
{
	return processNumber + 1;
}

bool ToolsetObjectEdit::isAValidProcessSelected() const
{
	if ( currentObject.journ != NULL )
	{
		return (journalSelectedProc >= 0) &&
				 (journalSelectedProc < currentObject.journ->size());
	}
	else
	{
		return false;
	}
}






void ToolsetObjectEdit::installNewTool(ProcessTool *tool, JournalProcess *proc,
													JournalObject *obj)
{
	currentTool.tool = tool;
	if ( currentTool.tool != NULL )
	{
		currentTool.journalObject = obj;
		currentTool.tool->attachProcess(proc, obj);
		currentTool.tool->install( this, toolBox );
	}
}

void ToolsetObjectEdit::uninstallCurrentTool()
{
	if ( currentTool.tool != NULL )
	{
		toolMan->returnTool( currentTool.tool->getProcessType() );
		currentTool.tool->uninstall();
		currentTool.tool->detachProcess();
		
		delete currentTool.journalObject;		//delete the journal object
	}
	currentTool.tool = NULL;
}

void ToolsetObjectEdit::installNewToolkit(MToolkit *k)
{
	currentToolkit = k;

	if (currentToolkit != NULL)
	{
		currentToolkit->install();
	}
}

void ToolsetObjectEdit::uninstallCurrentToolkit()
{
	if (currentToolkit != NULL)
	{
		currentToolkit->uninstall();
	}
	currentToolkit = NULL;
}

MToolkit* ToolsetObjectEdit::getToolkitForType(const ObjectType *type)
{
	if ( toolkits.count( type->getId() ) )
	{
		return toolkits[ type->getId() ];
	}
	else
	{
		return NULL;
	}
}


void ToolsetObjectEdit::uninstallCurrentPainter()
{
	if ( currentObject.painter != NULL )
	{
		view->removeEventHandler( currentObject.painter );
		delete currentObject.painter->getObject();	//delete the journal object
		delete currentObject.painter;
		currentObject.painter = NULL;
	}
}

void ToolsetObjectEdit::installNewCurrentPainter(JournalObject *obj)
{
	currentObject.painter = painterManager->createPainter( obj->getType() );

	if ( currentObject.painter != NULL )
	{
		currentObject.painter->setObject(obj);
		view->addEventHandler( currentObject.painter );
	}
}

void ToolsetObjectEdit::redrawView()
{
	view->redraw();
}




void ToolsetObjectEdit::findIntegrityBreak()
{
	journalIntegBreak = currentObject.journ->findIntegrityBreak();
}

void ToolsetObjectEdit::journalCListChanged()
{
	if ( currentObject.journ != NULL )
	{
		//compute the location of the first integrity break
		findIntegrityBreak();

		journalUpdateList();
	}
}

bool ToolsetObjectEdit::checkIntegrityAt(int proc)
{
	//check the integrity of the list at a certain location
	if ( currentObject.journ != NULL  &&  journalIntegBreak != -1 )
	{
		return proc < journalIntegBreak;
	}
	else
	{
		return true;
	}
}

void ToolsetObjectEdit::journalClearList()
{
	gtk_clist_clear( GTK_CLIST(journalCListWidget) );
}

void ToolsetObjectEdit::journalUpdateList()
{
	Journal *o = currentObject.journ;

	//build the journal list widget
	gtk_clist_freeze( GTK_CLIST(journalCListWidget) );
	gtk_clist_clear( GTK_CLIST(journalCListWidget) );

	if ( o != NULL )
	{
		char *startText = "***";
		gtk_clist_append( GTK_CLIST(journalCListWidget), &startText );

		for (int i = 0; i < o->size(); i++)
		{
			JournalProcess *proc = o->getProcess(i);
			string description = proc->getDescription();

			//if an integrity break is here, show this to the user
			if ( i == journalIntegBreak )
			{
				description = "## " + description + " ##\0";
			}
			
			char *text = strdup( description.c_str() );

			gtk_clist_append( GTK_CLIST(journalCListWidget), &text );

			free(text);
		}
	}

	gtk_clist_thaw( GTK_CLIST(journalCListWidget) );
}



void ToolsetObjectEdit::setCurrentObject(const string& objectName)
{
	//get a pointer to the journal of the new object
	Journal *o = objects->getObject(objectName);

	//check if its NULL
	if ( o == NULL )
	{
		journalClearList();
		return;
	}


	//uninstall the current tool
	uninstallCurrentTool();


	//update and reactivate the previous background object, if the current
	//object has changed
	if ( objectName != currentObject.name )
	{
		backgroundObjects->update( currentObject.name );
		backgroundObjects->activate( currentObject.name );
	}

	//set the members of the current object struct
	currentObject.journ = o;
	currentObject.name = objectName;

	//update the list
	journalUpdateList();

	//deactivate the new background object
	backgroundObjects->deactivate( objectName );
	
	//set the row number to the end of the list
	journalSelectedProc = o->size();


	//the journal list has changed
	journalCListChanged();


	//select the row
	gtk_clist_select_row( GTK_CLIST(journalCListWidget),
								 journalSelectedProc, 0 );
	//**** the above results in the row select call back being called,
	//which in turn calls setObjectType() (below) which sets the current
	//toolkit
}

void ToolsetObjectEdit::currentObjectDelete()
{
	journalClearList();

	//uninstall the current tool and toolkit
	uninstallCurrentTool();
	uninstallCurrentToolkit();
	uninstallCurrentPainter();
	redrawView();						//painter has changed


	currentObject.name = "";
	currentObject.journ = NULL;
}


void ToolsetObjectEdit::setObjectType(const ObjectType *t)
{
	string typeName;

	if (t != NULL)
	{
		MToolkit *k = getToolkitForType(t);

		uninstallCurrentToolkit();
		installNewToolkit(k);
	}
	else
	{
		uninstallCurrentToolkit();
	}
}


void ToolsetObjectEdit::objectCreateClicked( GtkWidget *widget,
															ToolsetObjectEdit *klass )
{
	//response function deletes the dialog box
	new NameDialog(klass, "Create Object");
}

void ToolsetObjectEdit::objectEditClicked( GtkWidget *widget,
														 ToolsetObjectEdit *klass )
{
	klass->setCurrentObject(
		klass->objects->getSelectionName( klass->myListManager ) );
}

void ToolsetObjectEdit::objectDeleteClicked( GtkWidget *widget,
															ToolsetObjectEdit *klass)
{
	string objName = klass->objects->getSelectionName( klass->myListManager );

	if ( klass->objects->exists(objName) )
	{
		UniqueId objectId = klass->objects->getUniqueId(objName);
		if ( klass->objects->hasDependants(objectId) )
		{
			vector<string> msgs;
			set<string> dependantNames;

			if ( klass->objects->getDependantNames( objectId, dependantNames ) )
			{

				msgs.push_back("Cannot delete object, as the following objects depend on it:");

				for (set<string>::const_iterator iter = dependantNames.begin();
					  iter != dependantNames.end();
					  ++iter)
				{
					msgs.push_back( *iter );
				}


				messageBox("Cannot delete", msgs);
			}
			else
			{
				msgs.push_back("Cannot get dependants list for object" + objName);
				messageBox("gSculpt System Error", msgs);
			}
		}
		else
		{
			klass->objects->remove(objName);
			klass->backgroundObjects->remove(objName);

			if (objName == klass->currentObject.name)
			{
				klass->currentObjectDelete();
			}

			klass->journalCListChanged();
		}
	}
}

void ToolsetObjectEdit::objectShowClicked( GtkWidget *widget,
														 ToolsetObjectEdit *klass )
{
	string name = klass->objects->getSelectionName( klass->myListManager );
	Journal *h = klass->objects->getSelection( klass->myListManager );

	if (h != NULL)
	{
		//only active if the object being shown is not currently
		//being edited
		klass->backgroundObjects->add( name, h, 
												 name != klass->currentObject.name );
	}
}

void ToolsetObjectEdit::objectHideClicked( GtkWidget *widget,
														 ToolsetObjectEdit *klass )
{
	klass->backgroundObjects->remove(
		klass->objects->getSelectionName( klass->myListManager ) );
}



void ToolsetObjectEdit::journalCListRowSelected(GtkWidget *widget, gint row,
																gint col, GdkEventButton *event,
																ToolsetObjectEdit *klass)
{
	if ( !klass->ignoreJournalListSelectEvents )
	{
		klass->journalSelectedProc = rowToProcess(row);

		klass->uninstallCurrentTool();
		klass->uninstallCurrentPainter();
		
		if ( klass->currentObject.journ != NULL )
		{
			int processNumber = rowToProcess(row);
			if ( klass->checkIntegrityAt( processNumber ) )
			{
				if ( gSculptConfigIsMultiThreaded() )
				{
					JournalTaskProgress *progress =
						klass->currentObject.journ->computeObjectThreaded(
																	rowToProcess(row) );
					klass->threadMonitor->start( progress,
						GTK_SIGNAL_FUNC(journalCListRowSelectedThreadFinishedStatic),
						klass );
				}
				else
				{
					JournalObject *object =
						klass->currentObject.journ->getObjectUnthreaded(
																		rowToProcess(row) );
					klass->journalCListRowSelectedThreadFinished(object);
				}
			}
			else
			{
				klass->setObjectType( NULL );
			}
		}

		klass->redrawView();			//painter has changed
	}
}

void ToolsetObjectEdit::journalCListRowSelectedThreadFinishedStatic(
												JournalTaskProgress *thread,
												ToolsetObjectEdit *klass)
{
	JournalObject *object =
			klass->currentObject.journ->getThreadedComputationResult();

	klass->journalCListRowSelectedThreadFinished(object);
}


void ToolsetObjectEdit::journalCListRowSelectedThreadFinished(
																JournalObject *object)
{
	const ObjectType *type = object->getType();
			
	setObjectType( type );
	installNewCurrentPainter( object );
}
				

void ToolsetObjectEdit::journalModifyClicked( GtkWidget *widget,
															 ToolsetObjectEdit *klass )
{
	klass->modifySelectedProcess();
}

void ToolsetObjectEdit::journalRemoveClicked( GtkWidget *widget,
															 ToolsetObjectEdit *klass )
{
	klass->removeSelectedProcess();
}


void ToolsetObjectEdit::journalMoveUpClicked( GtkWidget *widget,
															 ToolsetObjectEdit *klass )
{
	klass->swapSelectedProcessWith( klass->journalSelectedProc - 1 );
}

void ToolsetObjectEdit::journalMoveDownClicked( GtkWidget *widget,
																ToolsetObjectEdit *klass )
{
	klass->swapSelectedProcessWith( klass->journalSelectedProc + 1 );
}





void ToolsetObjectEdit::okPressed(NameDialog *d, const string& name)
{
	delete d;
	if (name != "")
	{
		objects->add(name);
	}
	else
	{
		messageBox("Object was not given a name", "Please enter a name");
	}
}

void ToolsetObjectEdit::cancelPressed(NameDialog *d)
{
	delete d;
}




void ToolsetObjectEdit::toolChanged()
{
	if ( !currentTool.tool->hasOwnPainter() )
	{
		JournalObject *obj =
			currentObject.journ->getObjectUnthreaded( journalSelectedProc );

		uninstallCurrentPainter();
		installNewCurrentPainter(obj);
		redrawView();							//painter has changed
	}
}

void ToolsetObjectEdit::toolDone()
{
	toolBox->setCurrentToolset( getWidget() );


	gtk_clist_select_row( GTK_CLIST(journalCListWidget),
								 journalSelectedProc + 1, 0 );
}





#endif
