/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by:					Geoffrey French
	application:		gSculpt
	last revision:		24/Jun/2000
*/
#ifndef __TOOLSETOBJECTEDIT_H
#define __TOOLSETOBJECTEDIT_H

#include <map>
#include <string>

#include <gtk/gtkwidget.h>

#include "MTool.h"
#include "Toolset.h"
#include "MToolkit.h"
#include "GridLayout.h"
#include "Toolbox.h"
#include "ObjectCollection.h"
#include "Journal.h"
#include "ListManager.h"
#include "NameDialog.h"
#include "ObjPainterManager.h"
#include "BackgroundObjManager.h"
#include "JournalEditor.h"
#include "ObjectPainter.h"
#include "ProcessToolManager.h"
#include "StatusBar.h"
#include "JournalTaskProgress.h"
#include "JournalTaskMonitor.h"




class ToolsetObjectEdit : public Toolset, private NameDialogListener,
								  private ToolListener, public JournalEditor
{
private:
	ObjectCollection *objects;
	ListManager *myListManager;
	
	GtkWidget *verticalBox;

	GtkWidget *objectsLabel, *objectsTitleTable;

	GtkWidget *objectToolsTable;
	GridLayout *objectsLayout;
	GtkWidget *objectCreateButton, *objectEditButton,
				 *objectDeleteButton, *objectShowButton,
				 *objectHideButton;

	ObjPainterManager *painterManager;


	struct {
		Journal *journ;
		string name;
		ObjectPainter *painter;
	} currentObject;
	


	GtkWidget *journalSeparator, *journalLabel, *journalTitleTable;

	GtkWidget *journalCListWidget;
	GtkWidget *journalScrolledWin;

	GtkWidget *journalToolsTable;
	GridLayout *journalLayout;
	
	GtkWidget *journalModifyButton, *journalRemoveButton,
				 *journalMoveUpButton, *journalMoveDownButton;
	GtkWidget *journalUpBox, *journalDownBox,
				 *journalUpArrow, *journalDownArrow;





	ProcessToolManager *toolMan;
	StatusBar *statusBar;
	JournalTaskMonitor *threadMonitor;



	int journalSelectedProc;
	bool ignoreJournalListSelectEvents;
	int journalIntegBreak;



	MToolkit *currentToolkit;
	
	struct {
		ProcessTool *tool;
		JournalObject *journalObject;
	} currentTool;


	BackgroundObjManager *backgroundObjects;

		

	map<Typeid, MToolkit*> toolkits;


	CadView *view;





public:
	ToolsetObjectEdit(Toolbox *tbx, ObjectCollection *objColl,
							ObjPainterManager *pManager,
							CadView *vw,
							ProcessToolManager *toolManager,
							StatusBar *statBar);
	virtual ~ToolsetObjectEdit();


	void addToolkit(const ObjectType *type, MToolkit *k);

	
	virtual void addProcess(JournalProcess *proc);
	void modifySelectedProcess();
	static void modifySelectedProcComputedBeforeStatic(JournalTaskProgress *thread,
																		ToolsetObjectEdit *klass);
	void modifySelectedProcComputedBefore(JournalObject *before);
	void removeSelectedProcess();
	void swapSelectedProcessWith(int index);

private:
	static int rowToProcess(int rowNumber);
	static int processToRow(int processNumber);
	bool isAValidProcessSelected() const;


	void installNewTool(ProcessTool *tool, JournalProcess *proc,
							  JournalObject *obj);
	void uninstallCurrentTool();
	void installNewToolkit(MToolkit *k);
	void uninstallCurrentToolkit();
	MToolkit* getToolkitForType(const ObjectType *type);


	void uninstallCurrentPainter();
	void installNewCurrentPainter(JournalObject *obj);
	void redrawView();



	void findIntegrityBreak();
	void journalCListChanged();
	bool checkIntegrityAt(int proc);
	void journalClearList();
	void journalUpdateList();

	
	void setCurrentObject(const string& objectName);
	void currentObjectDelete();
	void setObjectType(const ObjectType *t);

	static void objectCreateClicked( GtkWidget *widget,
												ToolsetObjectEdit *klass );
	static void objectEditClicked( GtkWidget *widget,
											 ToolsetObjectEdit *klass );
	static void objectDeleteClicked( GtkWidget *widget,
												ToolsetObjectEdit *klass );
	static void objectShowClicked( GtkWidget *widget,
											 ToolsetObjectEdit *klass );
	static void objectHideClicked( GtkWidget *widget,
											 ToolsetObjectEdit *klass );

	static void journalCListRowSelected(GtkWidget *widget, gint row,
													gint col, GdkEventButton *event,
													ToolsetObjectEdit *klass);
	static void journalCListRowSelectedThreadFinishedStatic(
												JournalTaskProgress *thread,
												ToolsetObjectEdit *klass );
	void journalCListRowSelectedThreadFinished(JournalObject *object);
	static void journalModifyClicked( GtkWidget *widget,
												 ToolsetObjectEdit *klass );
	static void journalRemoveClicked( GtkWidget *widget,
												 ToolsetObjectEdit *klass );
	static void journalMoveUpClicked( GtkWidget *widget,
												 ToolsetObjectEdit *klass );
	static void journalMoveDownClicked( GtkWidget *widget,
													ToolsetObjectEdit *klass );


	virtual void okPressed(NameDialog *d, const string& name);
	virtual void cancelPressed(NameDialog *d);


	virtual void toolChanged();
	virtual void toolDone();
};




#endif
