/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by :					Geoffrey French
	application :		gSculpt
	last revision :	30/Jun/2000
	title :				CAD Polyline
*/
#ifndef CPOLYLINEH
#define CPOLYLINEH

#include <list>

#include "array.h"

#include "CModel.h"

#include "CVertex.h"
#include "CadArea.h"
#include "CVertexContainer.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"

#include "Point3.h"




/*
*******************************************************************************
												CPolyline
*******************************************************************************
*/

class CPolyline : public CModel
{
public:
	typedef CVertex VertexType;
	
private:
	CVertexContainer v;
	bool poly_closed;
	CVertexContainer::VertexBackup xformBak, selectBak;

public:
	CPolyline();
	//CPolyline(const CPolyline& p);
	virtual ~CPolyline();


	//CPolyline& operator=(const CPolyline& p);

	virtual void read(FILE *f);
	virtual void write(FILE *f) const;
	

	CVertex* addVertex(const Point3& location, bool marked = true);
	CVertex* addVertexAt(int position, const Point3& location,
								bool marked = true);
	void removeVertex(CVertex* vtx);
	void swapVertices(CVertex *a, CVertex *b);

	void reserveVertices(int s);

	void close();
	void open();
	void setClosed(bool c);
	bool isClosed() const;

	void copyFrom(const CPolyline *p);

	int getNumberOfVertices() const;
	
public:
	CVertexContainer& vertices();
	const CVertexContainer& vertices() const;

	virtual void transform(const Matrix4& m, bool selectedOnly = true);
	virtual void transform(const Transformation *x, bool selectedOnly = true);
	virtual void transformBackup(bool selectedOnly = true);
	virtual void transformRestore();

	virtual void select(SelectPredicate predicate, const SelectRegion *region);
	virtual void selectBackup();
	virtual void selectRestore();

	virtual BBox getBBox(bool selectedOnly = true) const;
	virtual BBox getBBox(const Matrix4& m, bool selectedOnly = true) const;
	virtual BBox getBBox(const Transformation *x, bool selectedOnly = true) const;
};


#endif
