/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __POLYLINEPAINTER_CPP
#define __POLYLINEPAINTER_CPP

#include "PolylinePainter.h"

#include "ModelDraw.h"




PolylinePainter::PolylinePainter(CadArea *c) : ObjectPainter(c)
{
	pl = NULL;
}


void PolylinePainter::setPolyline(CPolyline *p)
{
	pl = p;
}

void PolylinePainter::setObject(JournalObject *o)
{
	ObjectPainter::setObject(o);
	JournalPolyline *p = (JournalPolyline*)o;
	pl = p->getPoly();
}


ObjectPainter* PolylinePainter::clone()
{
	return new PolylinePainter(cadArea);
}


void PolylinePainter::paint()
{
	if (pl != NULL)
	{
		drawPoly();
	}
}

void PolylinePainter::drawPoly()
{
	CVertexContainer &v = pl->vertices();
	CVertexContainer::iterator prev, cur;
	IPoint2 p;

	if (v.size() == 0)
	{
		return;
	}

	for (CVertexContainer::iterator iter = v.begin();
		  iter != v.end();
		  iter++)
	{
		(*iter).update( cadArea->getWorldScreenTransformation(),
							 cadArea->getViewFrustrum(), cadArea->getFrame() );
	}

	//draw polyline

	if ( pl->isClosed() )
	{
		prev = v.end();
		--prev;
		cur = v.begin();
	}
	else
	{
		cur = prev = v.begin();
		++cur;
	}

	while (cur != v.end())
	{
		drawEdge( &(*prev), &(*cur), cadArea );
		prev = cur;
		++cur;
	}


	if ( !backgroundFlag )
	{
		for (CVertexContainer::const_iterator iter = v.begin();
			  iter != v.end();
			  iter++)
		{
			if ( !(*iter).isVisible() )
			{
				continue;
			}
		
			p = IPoint2( (*iter).getScreen() );
			if ( (*iter).isMarked() )
			{
				modelDrawMarkedVertex( cadArea, p );
			}
			else
			{
				modelDrawUnmarkedVertex( cadArea, p );
			}
		}
	}
}

void PolylinePainter::drawEdge(CVertex *s, CVertex *e, CadArea *cadArea)
{
	ILine2 sl;
	Line3 wl;
	IPoint2 p1, p2;


	if ((s->getVisibilityData() != 0) || (e->getVisibilityData() != 0))
	{
		wl = Line3( s->getWorld(), e->getWorld() );
		if ( !cadArea->getViewFrustrum().clip(wl) )
		{
			return;
		}
		else
		{
			p1 = cadArea->transformWorldScreenInt2( wl.s );
			p2 = cadArea->transformWorldScreenInt2( wl.e );
			sl = ILine2( p1, p2 );
		}
	}
	else
	{
		sl = ILine2( IPoint2( s->getScreen() ),
						 IPoint2( e->getScreen() ) );
	}

	if (backgroundFlag)
	{
		modelDrawEdgeBackground(cadArea, sl);
	}
	else
	{
		modelDrawEdge(cadArea, sl);
	}
}



#endif
