/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLGROUPPOLYLINE_CPP
#define __TOOLGROUPPOLYLINE_CPP

#include "ToolGroupPolyline.h"

#include <gtk/gtklabel.h>



ToolGroupPolyline::ToolGroupPolyline(ProcessToolManager *toolManager,
												 JournalEditor *journEditor,
												 CadView *view,
												 ObjPainterManager *painterManager)
												 : MToolGroup(toolManager)
{
	titleLabel = gtk_label_new("Polyline");
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);

	journalEditor = journEditor;

	drawButton = addToolButton( "Draw", GTK_SIGNAL_FUNC(drawClicked) );
	polygonButton = addToolButton( "Polygon", GTK_SIGNAL_FUNC(polygonClicked) );
	subdivideButton = addToolButton( "Subdiv", GTK_SIGNAL_FUNC(subdivideClicked) );
	deleteButton = addToolButton( "Delete", GTK_SIGNAL_FUNC(deleteClicked) );
	reorderButton = addToolButton( "Reorder", GTK_SIGNAL_FUNC(reorderClicked) );

	toolDraw = new ToolPolyDraw( view );
	toolPolygon = new ToolPolyPolygon( view );
	toolSubdivide = new ToolPolySubdivide( view );
	toolDelete = new ToolPolyDelete( view );
	toolReorder = new ToolPolyReorder( view );

	toolMan->addTool( toolDraw->getProcessType(), toolDraw );
	toolMan->addTool( toolPolygon->getProcessType(), toolPolygon );
	toolMan->addTool( toolSubdivide->getProcessType(), toolSubdivide );
	toolMan->addTool( toolDelete->getProcessType(), toolDelete );
	toolMan->addTool( toolReorder->getProcessType(), toolReorder );
}

ToolGroupPolyline::~ToolGroupPolyline()
{
	gtk_widget_unref( titleLabel );
	
	toolMan->eraseTool( toolDraw->getProcessType() );
	toolMan->eraseTool( toolPolygon->getProcessType() );
	toolMan->eraseTool( toolSubdivide->getProcessType() );
	toolMan->eraseTool( toolDelete->getProcessType() );
	toolMan->eraseTool( toolReorder->getProcessType() );
	delete toolDraw;
	delete toolPolygon;
	delete toolSubdivide;
	delete toolDelete;
	delete toolReorder;
}




void ToolGroupPolyline::addToLayout(GridLayout *layout)
{
	MToolGroup::addToLayout( layout );

	layout->addAsLine( titleLabel );

	layout->add( drawButton );
	layout->add( polygonButton );
	layout->add( subdivideButton );
	layout->add( deleteButton );
	layout->add( reorderButton );
}	




void ToolGroupPolyline::drawClicked(GtkWidget *widget,
												ToolGroupPolyline *klass)
{
	ProcPolyDraw *p = new ProcPolyDraw();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupPolyline::polygonClicked(GtkWidget *widget,
													ToolGroupPolyline *klass)
{
	ProcPolyPolygon *p = new ProcPolyPolygon();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupPolyline::subdivideClicked(GtkWidget *widget,
													  ToolGroupPolyline *klass)
{
	ProcPolySubdivide *p = new ProcPolySubdivide();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupPolyline::deleteClicked(GtkWidget *widget,
												  ToolGroupPolyline *klass)
{
	ProcPolyDelete *p = new ProcPolyDelete();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupPolyline::reorderClicked(GtkWidget *widget,
													ToolGroupPolyline *klass)
{
	ProcPolyReorder *p = new ProcPolyReorder();
	klass->journalEditor->addProcess(p);
	delete p;
}



#endif
