/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYDELETE_CPP
#define __TOOLPOLYDELETE_CPP

#include <strstream>
#include <math.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkcheckbutton.h>


#include "ToolPolyDelete.h"

#include "JournalPolyline.h"

#include "Point3.h"





void* __create_ProcPolyDelete()
{
	return (void*)( new ProcPolyDelete() );
}

ObjectType type_ProcPolyDelete( "ProcPolyDelete", "JournalProcess",
										  __create_ProcPolyDelete );




ProcPolyDelete::ProcPolyDelete() : JournalProcess()
{
}

ProcPolyDelete::~ProcPolyDelete()
{
}



void ProcPolyDelete::read(FILE *f)
{
	JournalProcess::read(f);
}

void ProcPolyDelete::write(FILE *f)
{
	JournalProcess::write(f);
}





string ProcPolyDelete::getDescription() const
{
	return "Polyline Delete";
}

const ObjectType* ProcPolyDelete::getType() const
{
	return &type_ProcPolyDelete;
}

const ObjectType* ProcPolyDelete::getTypeStatic()
{
	return &type_ProcPolyDelete;
}



const ObjectType* ProcPolyDelete::getInputType() const
{
	return JournalPolyline::getTypeStatic();
}



JournalProcess* ProcPolyDelete::clone() const
{
	return new ProcPolyDelete();
}


const ObjectType* ProcPolyDelete::_getOutputType(
									const ObjectType* inputType) const
{
	return JournalPolyline::getTypeStatic();
}

JournalObject* ProcPolyDelete::_process(JournalObject *o,
													 JournalTaskProgress *progress)
{
	JournalPolyline *hPolyline = (JournalPolyline*)o;
	CPolyline *poly = hPolyline->getPoly();

	CVertexContainer &vertices = poly->vertices();


	for (int i = 0; i < vertices.size(); i++)
	{
		while ( vertices[i].isMarked() )
		{
			vertices.erase(i);

			if ( i >= vertices.size() )
			{
				break;
			}
		}
	}

	
	return o;
}







//**************************************************


ToolPolyDelete::ToolPolyDelete(CadView *v)
			: ProcessTool( v, "Delete", "---Polyline Delete---", 1 )
{
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolPolyDelete::~ToolPolyDelete()
{
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolPolyDelete::getProcessType() const
{
	return ProcPolyDelete::getTypeStatic();
}




void ToolPolyDelete::doneClicked(GtkWidget *widget, ToolPolyDelete *klass)
{
	klass->done();
}



#endif

