/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYDRAW_CPP
#define __TOOLPOLYDRAW_CPP

#include <strstream>
#include <math.h>

#include <vector>

#include <strstream>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtkradiobutton.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkclist.h>


#include "ToolPolyDraw.h"

#include "JournalPolyline.h"

#include "Point3.h"





void* __create_ProcPolyDraw()
{
	return (void*)( new ProcPolyDraw() );
}

ObjectType type_ProcPolyDraw( "ProcPolyDraw", "JournalProcess",
										  __create_ProcPolyDraw );




ProcPolyDraw::ProcPolyDraw() : JournalProcess()
{
	selectedFlag = true;
	closedFlag = true;
}

ProcPolyDraw::ProcPolyDraw(const array<Point3>& verts, bool selected, bool closed)
									: JournalProcess()
{
	vertices = verts;
	selectedFlag = selected;
	closedFlag = closed;
}

ProcPolyDraw::~ProcPolyDraw()
{
}



void ProcPolyDraw::read(FILE *f)
{
	JournalProcess::read(f);

	int numVertices;
	fread(&numVertices, sizeof(int), 1, f);
	vertices.clear();
	vertices.reserve(numVertices);

	for (int i = 0; i < numVertices; i++)
	{
		Point3 point;
		fread(&point, sizeof(Point3), 1, f);
		vertices.push_back(point);
	}

	fread(&selectedFlag, sizeof(bool), 1, f);
	fread(&closedFlag, sizeof(bool), 1, f);
}

void ProcPolyDraw::write(FILE *f)
{
	JournalProcess::write(f);

	int numVertices = vertices.size();
	fwrite(&numVertices, sizeof(int), 1, f);

	for (int i = 0; i < numVertices; i++)
	{
		Point3 point = vertices[i];
		fwrite(&point, sizeof(Point3), 1, f);
	}

	fwrite(&selectedFlag, sizeof(bool), 1, f);
	fwrite(&closedFlag, sizeof(bool), 1, f);
}





string ProcPolyDraw::getDescription() const
{
	return "Polyline Draw";
}

const ObjectType* ProcPolyDraw::getType() const
{
	return &type_ProcPolyDraw;
}

const ObjectType* ProcPolyDraw::getTypeStatic()
{
	return &type_ProcPolyDraw;
}



const ObjectType* ProcPolyDraw::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* ProcPolyDraw::clone() const
{
	return new ProcPolyDraw( vertices, selectedFlag, closedFlag );
}




const array<Point3>& ProcPolyDraw::getVertices() const
{
	return vertices;
}

const Point3& ProcPolyDraw::getVertex(int index) const
{
	return vertices[index];
}

int ProcPolyDraw::getNumberOfVertices() const
{
	return vertices.size();
}

bool ProcPolyDraw::getVerticesSelected() const
{
	return selectedFlag;
}

bool ProcPolyDraw::getClosed() const
{
	return closedFlag;
}


void ProcPolyDraw::setVertices(const array<Point3>& verts)
{
	vertices = verts;
	modified();
}

void ProcPolyDraw::insertVertex(int index, const Point3& position)
{
	vertices.insert(index);
	vertices[index] = position;
	modified();
}

void ProcPolyDraw::eraseVertex(int index)
{
	vertices.erase(index);
	modified();
}

void ProcPolyDraw::setVertex(int index, const Point3& position)
{
	vertices[index] = position;
	modified();
}

void ProcPolyDraw::setVerticesSelected(bool selected)
{
	selectedFlag = selected;
	modified();
}

void ProcPolyDraw::setClosed(bool closed)
{
	closedFlag = closed;
	modified();
}




const ObjectType* ProcPolyDraw::_getOutputType(const ObjectType* inputType) const
{
	return JournalPolyline::getTypeStatic();
}

JournalObject* ProcPolyDraw::_process(JournalObject *o,
												  JournalTaskProgress *progress)
{
	//determine if its already a polyline
	bool isAlreadyAPolyline =
		o->getType()->isSubclassOf( JournalPolyline::getTypeStatic() );


	JournalPolyline *hPoly;
	if (isAlreadyAPolyline)
	{
		//yes: just use current object
		hPoly = (JournalPolyline*)o;
	}
	else
	{
		//no: delete the supplied object and create a new one
		delete o;
		hPoly = new JournalPolyline();
	}


	//get the polyline object
	CPolyline *poly = hPoly->getPoly();


	poly->reserveVertices( vertices.size() + poly->getNumberOfVertices() );


	//add vertices to polyline
	for (int i = 0; i < vertices.size(); i++)
	{
		poly->addVertex( vertices[i], selectedFlag );
	}


	poly->setClosed(closedFlag);



	return hPoly;
}








//**************************************************


ToolPolyDraw::ToolPolyDraw(CadView *v)
									: ProcessTool(v, "Draw", "Polyline Draw", 2 ),
									  CadClickAndDragEventHandler( v->getCadArea() )
{
	ignoreCheckEvents = false;
	currentFunction = DRAW;
	currentlySelectedRow = 0;
	validRowSelectedFlag = false;




	functionLabel = gtk_label_new("Function:");
	gtk_widget_ref(functionLabel);
	gtk_widget_show(functionLabel);

	
	GSList *group = (GSList*)NULL;
	funcDrawRadio = gtk_radio_button_new_with_label(group, "Draw");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(funcDrawRadio) );
	funcSetPosRadio = gtk_radio_button_new_with_label(group, "Set Position");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(funcSetPosRadio) );
	funcTranslateRadio = gtk_radio_button_new_with_label(group, "Translate");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(funcTranslateRadio) );
	funcNoneRadio = gtk_radio_button_new_with_label(group, "None");

	gtk_widget_ref(funcDrawRadio);
	gtk_widget_ref(funcSetPosRadio);
	gtk_widget_ref(funcTranslateRadio);
	gtk_widget_ref(funcNoneRadio);
	gtk_widget_show(funcDrawRadio);
	gtk_widget_show(funcSetPosRadio);
	gtk_widget_show(funcTranslateRadio);
	gtk_widget_show(funcNoneRadio);

	gtk_signal_connect( GTK_OBJECT(funcDrawRadio), "clicked",
							  GTK_SIGNAL_FUNC(funcDrawClicked), this );
	gtk_signal_connect( GTK_OBJECT(funcSetPosRadio), "clicked",
							  GTK_SIGNAL_FUNC(funcSetPosClicked), this );
	gtk_signal_connect( GTK_OBJECT(funcTranslateRadio), "clicked",
							  GTK_SIGNAL_FUNC(funcTranslateClicked), this );
	gtk_signal_connect( GTK_OBJECT(funcNoneRadio), "clicked",
							  GTK_SIGNAL_FUNC(funcNoneClicked), this );
	



	selectedCheck = gtk_check_button_new_with_label("Select Vertices");
	gtk_widget_ref(selectedCheck);
	gtk_widget_show(selectedCheck);

	gtk_signal_connect( GTK_OBJECT(selectedCheck), "clicked",
							  GTK_SIGNAL_FUNC(selectedClicked), this );



	closedCheck = gtk_check_button_new_with_label("Closed");
	gtk_widget_ref(closedCheck);
	gtk_widget_show(closedCheck);

	gtk_signal_connect( GTK_OBJECT(closedCheck), "clicked",
							  GTK_SIGNAL_FUNC(closedClicked), this );



	snapLabel = gtk_label_new("Snap:");
	gtk_widget_ref(snapLabel);
	gtk_widget_show(snapLabel);


	xSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.001, 0.1, 0.3, 4,
											 NULL, NULL );
	ySnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.001, 0.1, 0.3, 4,
											 NULL, NULL );
	zSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.001, 0.1, 0.3, 4,
											 NULL, NULL );
	gtk_widget_show( xSnapEntry->getWidget() );
	gtk_widget_show( ySnapEntry->getWidget() );
	gtk_widget_show( zSnapEntry->getWidget() );




	listLabel = gtk_label_new("Vertices:");
	gtk_widget_ref(listLabel);
	gtk_widget_show(listLabel);


	char *listColumnTitles[] = { "Positions" };
	vertexCList = gtk_clist_new_with_titles( 1, listColumnTitles );
	gtk_widget_ref(vertexCList);
	gtk_widget_show(vertexCList);

	gtk_signal_connect( GTK_OBJECT(vertexCList), "select_row",
							  GTK_SIGNAL_FUNC(listRowSelected), this );

	vertexScrolledWin = gtk_scrolled_window_new( NULL, NULL );
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(vertexScrolledWin),
											  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_widget_ref(vertexScrolledWin);
	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(vertexScrolledWin), vertexCList );	
	gtk_widget_show(vertexScrolledWin);
											  



	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);
	
	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.addAsLine(functionLabel);
	layout.add(funcDrawRadio);
	layout.add(funcSetPosRadio);
	layout.add(funcTranslateRadio);
	layout.add(funcNoneRadio);
	layout.lineSpace(1);
	layout.addAsLine(selectedCheck);
	layout.addAsLine(closedCheck);
	layout.lineSpace(1);
	layout.addAsLine(snapLabel);
	layout.add( xSnapEntry->getWidget() );
	layout.add( ySnapEntry->getWidget() );
	layout.add( zSnapEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(listLabel);
	layout.addAsBlock( vertexScrolledWin, ISize2(2, 7) );
	layout.lineSpace(1);
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolPolyDraw::~ToolPolyDraw()
{
	delete xSnapEntry;
	delete ySnapEntry;
	delete zSnapEntry;
	
	gtk_widget_unref(functionLabel);
	gtk_widget_unref(funcDrawRadio);
	gtk_widget_unref(funcSetPosRadio);
	gtk_widget_unref(funcTranslateRadio);
	gtk_widget_unref(funcNoneRadio);
	gtk_widget_unref(listLabel);
	gtk_widget_unref(vertexCList);
	gtk_widget_unref(vertexScrolledWin);
	gtk_widget_unref(selectedCheck);
	gtk_widget_unref(closedCheck);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolPolyDraw::getProcessType() const
{
	return ProcPolyDraw::getTypeStatic();
}





void ToolPolyDraw::initialise()
{
	ProcessTool::initialise();

	if ( hasProcess() )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;


		ignoreCheckEvents = true;
		bool selectedFlag = proc->getVerticesSelected();
		bool closed = proc->getClosed();
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(selectedCheck), selectedFlag );
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(closedCheck), closed );
		ignoreCheckEvents = false;


		gtk_clist_freeze( GTK_CLIST(vertexCList) );
		gtk_clist_clear( GTK_CLIST(vertexCList) );


		for (int i = 0; i < proc->getNumberOfVertices(); i++)
		{
			Point3 point = proc->getVertex(i);
			ostrstream stream;

			stream << "[ " << point.x << ", " << point.y <<
						 ", " << point.z << " ]" << (char)0x00;

			char *text = strdup( stream.str() );
		
			gtk_clist_append( GTK_CLIST(vertexCList), &text );

			free(text);
		}

		char *endText = "***";
		gtk_clist_append( GTK_CLIST(vertexCList), &endText );
	
	
		if ( proc->getNumberOfVertices() == 0 )
		{
			gtk_clist_select_row( GTK_CLIST(vertexCList), 0, 0 );
		}
		else
		{
			gtk_clist_select_row( GTK_CLIST(vertexCList),
										 proc->getNumberOfVertices() - 1, 0 );
		}
								 
		gtk_clist_thaw( GTK_CLIST(vertexCList) );
	}
}






void ToolPolyDraw::start()
{
	CadClickAndDragEventHandler::start();

	if ( currentFunction == TRANSLATE && validRowSelectedFlag )
	{
		//get and store the initial translation
		if ( hasProcess() )
		{
			ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;
			translation.initialPosition = proc->getVertex(currentlySelectedRow);
		}

		updateSnapper();
	}
}

void ToolPolyDraw::finish(bool terminated)
{
	CadClickAndDragEventHandler::finish(terminated);

	if ( hasProcess() )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;

		if ( currentFunction == TRANSLATE && validRowSelectedFlag )
		{
			if (terminated)
			{
				proc->setVertex( currentlySelectedRow, translation.initialPosition );
				setRow( currentlySelectedRow, translation.initialPosition );
				setFunctionFinished();
				modified();
			}
		}
	}
}

void ToolPolyDraw::function()
{
	CadClickAndDragEventHandler::function();
	
	if ( currentFunction == TRANSLATE && validRowSelectedFlag )
	{
		if ( hasProcess() )
		{
			ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;

			Vector3 xlat = currentCursorPosition.getWorld() -
								startCursorPosition.getWorld();
	
			xlat = snapper.snap(xlat);

			Point3 newPosition = translation.initialPosition + xlat;

			proc->setVertex( currentlySelectedRow, newPosition );
			setRow( currentlySelectedRow, newPosition );

			modified();
		}
	}
}





void ToolPolyDraw::evPrimaryButtonDown()
{
	CadClickAndDragEventHandler::evPrimaryButtonDown();

	if ( currentFunction == SETPOSITION )
	{
		setFunctionFinished();
	}
	else if ( currentFunction == DRAW )
	{
		insert( snapper.snap( currentCursorPosition.getWorld() ) );
		selectNextRow();
		modified();
	}
}

void ToolPolyDraw::evTertiaryButtonDown()
{
	CadClickAndDragEventHandler::evTertiaryButtonDown();

	if ( currentFunction == DRAW )
	{
		remove();
		modified();
	}

	if ( currentFunction != TRANSLATE )
	{
		setFunctionFinished();
	}
}

void ToolPolyDraw::evMotion(const IPoint2& p)
{
	CadClickAndDragEventHandler::evMotion(p);
	motion();
	modified();
}

void ToolPolyDraw::evEnter(const IPoint2& p)
{
	CadClickAndDragEventHandler::evEnter(p);

	mouseInWorkArea = true;

	if ( currentFunction == DRAW )
	{
		updateSnapper();
		insert( snapper.snap( currentCursorPosition.getWorld() ) );
	}
	else if ( currentFunction == SETPOSITION )
	{
		updateSnapper();
	}
	
	motion();
	modified();
}

void ToolPolyDraw::evLeave(const IPoint2& p)
{
	CadClickAndDragEventHandler::evLeave(p);

	mouseInWorkArea = false;
	motion();

	if ( currentFunction == DRAW )
	{
		remove();
	}
	modified();
}



void ToolPolyDraw::updateSnapper()
{
	//get granularity from numeric entries
	Vector3 granularity = Vector3( xSnapEntry->getValue(),
											 ySnapEntry->getValue(),
											 zSnapEntry->getValue() );

	snapper = VectorSnap(granularity);
}

void ToolPolyDraw::setRow(int row, const Point3& point)
{
	ostrstream stream;

	stream << "[ " << point.x << ", " << point.y <<
				 ", " << point.z << " ]" << (char)0x00;

	char *text = strdup( stream.str() );
		

	gtk_clist_set_text( GTK_CLIST(vertexCList), row, 0, text );


	free(text);
}

void ToolPolyDraw::selectRow(int row)
{
	gtk_clist_select_row( GTK_CLIST(vertexCList), row, 0 );
}





void ToolPolyDraw::insert(const Point3& pos)
{
	if ( hasProcess() )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;
		proc->insertVertex( currentlySelectedRow, pos );

		char *initialText = "";
		gtk_clist_insert( GTK_CLIST(vertexCList),
								currentlySelectedRow, &initialText );
		setRow( currentlySelectedRow, pos );
	}
}

void ToolPolyDraw::remove()
{
	if ( hasProcess() )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;
		proc->eraseVertex( currentlySelectedRow );

		gtk_clist_remove( GTK_CLIST(vertexCList),
								currentlySelectedRow );

		validRowSelectedFlag = false;
	}
}

void ToolPolyDraw::selectNextRow()
{
	selectRow( currentlySelectedRow + 1 );
}

void ToolPolyDraw::motion()
{
	if ( currentFunction == DRAW || currentFunction == SETPOSITION &&
		  mouseInWorkArea && validRowSelectedFlag )
	{
		if ( hasProcess() )
		{
			ProcPolyDraw *proc = (ProcPolyDraw*)journalProcess;

			Point3 newPosition = currentCursorPosition.getWorld();
	
			newPosition = snapper.snap(newPosition);

			proc->setVertex( currentlySelectedRow, newPosition );
			setRow( currentlySelectedRow, newPosition );
		}
	}
}




void ToolPolyDraw::setFunctionFinished()
{
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(funcNoneRadio), true );
}



CAEventHandler* ToolPolyDraw::getEventHandler()
{
	return (CAEventHandler*)this;
}





void ToolPolyDraw::funcDrawClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	klass->currentFunction = DRAW;
	klass->updateSnapper();
}

void ToolPolyDraw::funcSetPosClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	klass->currentFunction = SETPOSITION;
	klass->updateSnapper();
}

void ToolPolyDraw::funcTranslateClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	klass->currentFunction = TRANSLATE;
	klass->updateSnapper();
}

void ToolPolyDraw::funcNoneClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	klass->currentFunction = FINISHED;
	klass->updateSnapper();
}


void ToolPolyDraw::listRowSelected(GtkWidget *widget, gint row, gint col,
											  GdkEventButton *event, ToolPolyDraw *klass)
{
	klass->currentlySelectedRow = row;

	if ( klass->hasProcess() )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)klass->journalProcess;
		
		klass->validRowSelectedFlag =
			klass->currentlySelectedRow < proc->getNumberOfVertices();
	}
}


void ToolPolyDraw::selectedClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	bool selectedFlag =
		gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(klass->selectedCheck) ) != 0;
	
	if ( klass->hasProcess() && !klass->ignoreCheckEvents )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)klass->journalProcess;
		
		proc->setVerticesSelected(selectedFlag);
		klass->modified();
	}
}

void ToolPolyDraw::closedClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	bool closed =
		gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(klass->closedCheck) ) != 0;
	
	if ( klass->hasProcess() && !klass->ignoreCheckEvents )
	{
		ProcPolyDraw *proc = (ProcPolyDraw*)klass->journalProcess;
		
		proc->setClosed(closed);
		klass->modified();
	}
}


void ToolPolyDraw::doneClicked(GtkWidget *widget, ToolPolyDraw *klass)
{
	klass->done();
}



#endif

