/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYDRAW_H
#define __TOOLPOLYDRAW_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>
#include "array.h"


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "NumericEntry.h"
#include "CPolyline.h"
#include "VectorSnap.h"
#include "CadEventHandlers.h"



class ProcPolyDraw : public JournalProcess
{
private:
	array<Point3> vertices;
	bool selectedFlag, closedFlag;
	
public:
	ProcPolyDraw();
	ProcPolyDraw(const array<Point3>& verts, bool selected, bool closed);
	virtual ~ProcPolyDraw();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;


	const array<Point3>& getVertices() const;
	const Point3& getVertex(int index) const;
	int getNumberOfVertices() const;
	bool getVerticesSelected() const;
	bool getClosed() const;
	void setVertices(const array<Point3>& verts);
	void insertVertex(int index, const Point3& position);
	void eraseVertex(int index);
	void setVertex(int index, const Point3& position);
	void setVerticesSelected(bool selected);
	void setClosed(bool closed);
	

protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
};






class ToolPolyDraw : public ProcessTool,
							public CadClickAndDragEventHandler
{
private:
	typedef enum {
		DRAW,
		SETPOSITION,
		TRANSLATE,
		FINISHED
	} Function;
	
	GtkWidget *functionLabel;
	GtkWidget *funcDrawRadio, *funcSetPosRadio, *funcTranslateRadio,
				 *funcNoneRadio;

	GtkWidget *selectedCheck, *closedCheck;

	GtkWidget *snapLabel;
	NumericEntry *xSnapEntry, *ySnapEntry, *zSnapEntry;
	
	GtkWidget *listLabel;
	GtkWidget *vertexCList, *vertexScrolledWin;

	GtkWidget *doneButton;


	Function currentFunction;
	int currentlySelectedRow;
	bool ignoreCheckEvents;
	VectorSnap snapper;
	bool validRowSelectedFlag;

	bool mouseInWorkArea;
	


	struct {
		Point3 initialPosition;
	} translation;


public:
	ToolPolyDraw(CadView *v);
	~ToolPolyDraw();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();


	virtual void evPrimaryButtonDown();
	virtual void evTertiaryButtonDown();
	virtual void evMotion(const IPoint2& p);
	virtual void evEnter(const IPoint2& p);
	virtual void evLeave(const IPoint2& p);

	void updateSnapper();
	void setRow(int row, const Point3& point);
	void selectRow(int row);
	
	void insert(const Point3& pos);
	void remove();
	void selectNextRow();
	void motion();


	void setFunctionFinished();


	virtual CAEventHandler* getEventHandler();


private:
	static void funcDrawClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void funcSetPosClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void funcTranslateClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void funcNoneClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void listRowSelected(GtkWidget *widget, gint row, gint col,
										 GdkEventButton *event, ToolPolyDraw *klass);
	static void selectedClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void closedClicked(GtkWidget *widget, ToolPolyDraw *klass);
	static void doneClicked(GtkWidget *widget, ToolPolyDraw *klass);
};


#endif
