/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYPOLYGON_CPP
#define __TOOLPOLYPOLYGON_CPP

#include <strstream>
#include <math.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkcheckbutton.h>

#include "ToolPolyPolygon.h"

#include "JournalPolyline.h"
#include "CPolyline.h"

#include "Point3.h"


//NOTE:
//an arc angle of > 2*PI means 2*PI; this is because a full 360 degree
//arc can be compute specially


void* __create_ProcPolyPolygon()
{
	return (void*)( new ProcPolyPolygon() );
}

ObjectType type_ProcPolyPolygon("ProcPolyPolygon", "JournalProcess",
											 __create_ProcPolyPolygon);



ProcPolyPolygon::ProcPolyPolygon() : JournalProcess()
{
	numVertices = 16;
	radius = 50.0;
	arcAngle = 360.0;
	selectedFlag = true;
	alignEdgesWithAxisFlag = false;
	centreVertexFlag = false;
	closedFlag = true;
}

ProcPolyPolygon::ProcPolyPolygon(int numVerts, double rad,
											double arcSegmentAngle,
											bool selected, bool alignEdges,
											bool centreFlag, bool closeFlag)
											: JournalProcess()
{
	numVertices = numVerts;
	radius = rad;
	arcAngle = arcSegmentAngle;
	selectedFlag = selected;
	alignEdgesWithAxisFlag = alignEdges;
	centreVertexFlag = centreFlag;
	closedFlag = closeFlag;
}

ProcPolyPolygon::~ProcPolyPolygon()
{
}




void ProcPolyPolygon::read(FILE *f)
{
	JournalProcess::read(f);

	fread(&numVertices, sizeof(int), 1, f);
	fread(&radius, sizeof(double), 1, f);
	fread(&arcAngle, sizeof(double), 1, f);
	fread(&selectedFlag, sizeof(bool), 1, f);
	fread(&alignEdgesWithAxisFlag, sizeof(bool), 1, f);
	fread(&centreVertexFlag, sizeof(bool), 1, f);
	fread(&closedFlag, sizeof(bool), 1, f);
}

void ProcPolyPolygon::write(FILE *f)
{
	JournalProcess::write(f);

	fwrite(&numVertices, sizeof(int), 1, f);
	fwrite(&radius, sizeof(double), 1, f);
	fwrite(&arcAngle, sizeof(double), 1, f);
	fwrite(&selectedFlag, sizeof(bool), 1, f);
	fwrite(&alignEdgesWithAxisFlag, sizeof(bool), 1, f);
	fwrite(&centreVertexFlag, sizeof(bool), 1, f);
	fwrite(&closedFlag, sizeof(bool), 1, f);
}






string ProcPolyPolygon::getDescription() const
{
	return "Polygon";
}

const ObjectType* ProcPolyPolygon::getType() const
{
	return &type_ProcPolyPolygon;
}

const ObjectType* ProcPolyPolygon::getTypeStatic()
{
	return &type_ProcPolyPolygon;
}



const ObjectType* ProcPolyPolygon::getInputType() const
{
	return JournalObject::getTypeStatic();
}



JournalProcess* ProcPolyPolygon::clone() const
{
	return new ProcPolyPolygon( numVertices, radius, arcAngle,
										 selectedFlag, alignEdgesWithAxisFlag,
										 centreVertexFlag, closedFlag );
}




int ProcPolyPolygon::getNumVertices() const
{
	return numVertices;
}

double ProcPolyPolygon::getRadius() const
{
	return radius;
}

double ProcPolyPolygon::getArcAngle() const
{
	return arcAngle;
}

bool ProcPolyPolygon::getVerticesSelected() const
{
	return selectedFlag;
}

bool ProcPolyPolygon::getAlignEdgesWithAxis() const
{
	return alignEdgesWithAxisFlag;
}

bool ProcPolyPolygon::getCentreVertex() const
{
	return centreVertexFlag;
}

bool ProcPolyPolygon::getClosed() const
{
	return closedFlag;
}


void ProcPolyPolygon::setNumVertices(int n)
{
	numVertices = n;
	modified();
}

void ProcPolyPolygon::setRadius(double r)
{
	radius = r;
	modified();
}

void ProcPolyPolygon::setArcAngle(double angle)
{
	arcAngle = angle;
	modified();
}

void ProcPolyPolygon::setVerticesSelected(bool selected)
{
	selectedFlag = selected;
	modified();
}

void ProcPolyPolygon::setAlignEdgesWithAxis(bool alignEdges)
{
	alignEdgesWithAxisFlag = alignEdges;
	modified();
}

void ProcPolyPolygon::setCentreVertex(bool centreVertex)
{
	centreVertexFlag = centreVertex;
	modified();
}

void ProcPolyPolygon::setClosed(bool closed)
{
	closedFlag = closed;
	modified();
}




const ObjectType* ProcPolyPolygon::_getOutputType(
									const ObjectType* inputType) const
{
	return JournalPolyline::getTypeStatic();
}

JournalObject* ProcPolyPolygon::_process(JournalObject *o,
													  JournalTaskProgress *progress)
{
	//determine if its already a polyline
	bool isAlreadyAPolyline =
		o->getType()->isSubclassOf( JournalPolyline::getTypeStatic() );


	JournalPolyline *hPoly;
	if (isAlreadyAPolyline)
	{
		//yes: just use current object
		hPoly = (JournalPolyline*)o;
	}
	else
	{
		//no: delete the supplied object and create a new one
		delete o;
		hPoly = new JournalPolyline();
	}


	//get the polyline object
	CPolyline *poly = hPoly->getPoly();


	bool fullCircleFlag = arcAngle >= 360.0;


	double arc = DEGTORAD(arcAngle);
	if ( fullCircleFlag )
	{
		arc = M_PI * 2.0;
	}

	double angle = 0.0;
	double angleIncrement;

	if ( fullCircleFlag )
	{
		angleIncrement = -arc / (double)numVertices;
	}
	else
	{
		angleIncrement = -arc / (double)(numVertices - 1);
	}

	
	if (alignEdgesWithAxisFlag)
	{
		angle = angleIncrement * 0.5;
	}

	poly->reserveVertices( numVertices + poly->getNumberOfVertices() );

	for (int i = 0; i < numVertices; i++)
	{
		poly->addVertex( Point3( sin(angle) * radius, cos(angle) * radius, 0.0 ),
							  selectedFlag );
		angle += angleIncrement;
	}

	if ( centreVertexFlag && ( arcAngle < 360.0 ) )
	{
		poly->addVertex( Point3(), selectedFlag );
	}


	poly->setClosed(closedFlag);
	
		
	return (JournalObject*)hPoly;
}




//**************************************************


ToolPolyPolygon::ToolPolyPolygon(CadView *v)
											: ProcessTool( v, "Polygon", "Polygon", 2 )
{
	selectedCheck = gtk_check_button_new_with_label("Selected");
	gtk_widget_ref(selectedCheck);
	gtk_widget_show(selectedCheck);

	gtk_signal_connect( GTK_OBJECT(selectedCheck), "clicked",
							  GTK_SIGNAL_FUNC(selectedClicked), this );



	alignEdgesCheck = gtk_check_button_new_with_label("Align Edges");
	gtk_widget_ref(alignEdgesCheck);
	gtk_widget_show(alignEdgesCheck);

	gtk_signal_connect( GTK_OBJECT(alignEdgesCheck), "clicked",
							  GTK_SIGNAL_FUNC(alignEdgesClicked), this );



	centreVertexCheck = gtk_check_button_new_with_label("Centre Vertex");
	gtk_widget_ref(centreVertexCheck);
	gtk_widget_show(centreVertexCheck);

	gtk_signal_connect( GTK_OBJECT(centreVertexCheck), "clicked",
							  GTK_SIGNAL_FUNC(centreVertexClicked), this );



	closedCheck = gtk_check_button_new_with_label("Closed");
	gtk_widget_ref(closedCheck);
	gtk_widget_show(closedCheck);

	gtk_signal_connect( GTK_OBJECT(closedCheck), "clicked",
							  GTK_SIGNAL_FUNC(closedClicked), this );



	verticesLab = gtk_label_new("Vertices:");
	gtk_widget_ref(verticesLab);
	gtk_widget_show(verticesLab);


	verticesEntry = new NumericEntry( 16.0, 1.0, 1048576.0, 1.0, 32.0, 0.5, 0,
												 GTK_SIGNAL_FUNC(verticesChanged), this );
	gtk_widget_show( verticesEntry->getWidget() );




	radiusLab = gtk_label_new("Radius:");
	gtk_widget_ref(radiusLab);
	gtk_widget_show(radiusLab);


	radiusEntry = new NumericEntry( 100.0, 0.0, 1000000000.0, 1.0, 50.0, 0.5, 4,
											  GTK_SIGNAL_FUNC(radiusChanged), this );
	gtk_widget_show( radiusEntry->getWidget() );




	arcAngleLabel = gtk_label_new("Arc:");
	gtk_widget_ref(arcAngleLabel);
	gtk_widget_show(arcAngleLabel);


	arcAngleEntry = new NumericEntry( 360.0, 0.0, 360.0, 0.01, 1.0, 0.3, 4,
												 GTK_SIGNAL_FUNC(arcAngleChanged), this );
	gtk_widget_show( arcAngleEntry->getWidget() );




	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );


	layout.addAsLine(selectedCheck);
	layout.addAsLine(alignEdgesCheck);
	layout.addAsLine(centreVertexCheck);
	layout.addAsLine(closedCheck);
	layout.add(verticesLab);
	layout.add( verticesEntry->getWidget() );
	layout.add(radiusLab);
	layout.add( radiusEntry->getWidget() );
	layout.add(arcAngleLabel);
	layout.add( arcAngleEntry->getWidget() );
	layout.lineSpace();
	layout.addAsLine(doneButton);
}

ToolPolyPolygon::~ToolPolyPolygon()
{
	delete verticesEntry;
	delete radiusEntry;
	delete arcAngleEntry;
	
	gtk_widget_unref(selectedCheck);
	gtk_widget_unref(alignEdgesCheck);
	gtk_widget_unref(centreVertexCheck);
	gtk_widget_unref(closedCheck);
	gtk_widget_unref(verticesLab);
	gtk_widget_unref(radiusLab);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolPolyPolygon::getProcessType() const
{
	return ProcPolyPolygon::getTypeStatic();
}



void ToolPolyPolygon::initialise()
{
	ProcessTool::initialise();

	if ( hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)journalProcess;


		bool selected = proc->getVerticesSelected();
		bool alignEdges = proc->getAlignEdgesWithAxis();
		bool centreVertex = proc->getCentreVertex();
		bool closed = proc->getClosed();

		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(selectedCheck),
												selected );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(alignEdgesCheck),
												alignEdges );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(centreVertexCheck),
												centreVertex );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(closedCheck),
												closed );
		verticesEntry->setValue( proc->getNumVertices() );
		radiusEntry->setValue( proc->getRadius() );
		arcAngleEntry->setValue( proc->getArcAngle() );
	}
}



void ToolPolyPolygon::selectedClicked(GtkWidget *widget,
												  ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
		
		bool selected = gtk_toggle_button_get_active(
									GTK_TOGGLE_BUTTON( klass->selectedCheck ) );

		proc->setVerticesSelected(selected);
		klass->modified();
	}
}													

void ToolPolyPolygon::alignEdgesClicked(GtkWidget *widget,
													 ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
		
		bool alignEdges = gtk_toggle_button_get_active(
									GTK_TOGGLE_BUTTON( klass->alignEdgesCheck ) );

		proc->setAlignEdgesWithAxis(alignEdges);
		klass->modified();
	}
}													

void ToolPolyPolygon::centreVertexClicked(GtkWidget *widget,
														ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
		
		bool centreVertex = gtk_toggle_button_get_active(
									GTK_TOGGLE_BUTTON( klass->centreVertexCheck ) );

		proc->setCentreVertex(centreVertex);
		klass->modified();
	}
}													

void ToolPolyPolygon::closedClicked(GtkWidget *widget,
												ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
		
		bool closed = gtk_toggle_button_get_active(
							GTK_TOGGLE_BUTTON( klass->closedCheck ) );

		proc->setClosed(closed);
		klass->modified();
	}
}													

void ToolPolyPolygon::verticesChanged(NumericEntry *widget,
												  ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
		
		proc->setNumVertices( klass->verticesEntry->getValueAsInt() );
		klass->modified();
	}
}

void ToolPolyPolygon::radiusChanged(NumericEntry *widget,
												ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
	
		proc->setRadius( klass->radiusEntry->getValue() );
		klass->modified();
	}
}

void ToolPolyPolygon::arcAngleChanged(NumericEntry *widget,
												  ToolPolyPolygon *klass)
{
	if ( klass->hasProcess() )
	{
		ProcPolyPolygon *proc = (ProcPolyPolygon*)klass->journalProcess;
	
		proc->setArcAngle( klass->arcAngleEntry->getValue() );
		klass->modified();
	}
}

void ToolPolyPolygon::doneClicked(GtkWidget *widget, ToolPolyPolygon *klass)
{
	klass->done();
}



#endif

