/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYPOLYGON_H
#define __TOOLPOLYPOLYGON_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "NumericEntry.h"



//NOTE:
//an arc angle of > 2*PI means 2*PI; this is because a full 360 degree
//arc can be compute specially


class ProcPolyPolygon : public JournalProcess
{
private:
	int numVertices;
	double radius, arcAngle;
	bool selectedFlag;
	bool alignEdgesWithAxisFlag, centreVertexFlag, closedFlag;
	
public:
	ProcPolyPolygon();
	ProcPolyPolygon(int numVerts, double rad, double arcSegmentAngle,
						 bool selected, bool alignEdges, bool centreVertex,
						 bool closeFlag);
	virtual ~ProcPolyPolygon();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;

	int getNumVertices() const;
	double getRadius() const;
	double getArcAngle() const;
	bool getVerticesSelected() const;
	bool getAlignEdgesWithAxis() const;
	bool getCentreVertex() const;
	bool getClosed() const;
	void setNumVertices(int n);
	void setRadius(double r);
	void setArcAngle(double angle);
	void setVerticesSelected(bool selected);
	void setAlignEdgesWithAxis(bool alignEdges);
	void setCentreVertex(bool centreVertex);
	void setClosed(bool closed);

protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
};




class ToolPolyPolygon : public ProcessTool
{
private:
	GtkWidget *selectedCheck, *alignEdgesCheck, *centreVertexCheck, *closedCheck;
	GtkWidget *verticesLab;
	NumericEntry *verticesEntry;
	GtkWidget *radiusLab;
	NumericEntry *radiusEntry;
	GtkWidget *arcAngleLabel;
	NumericEntry *arcAngleEntry;
	GtkWidget *doneButton;

public:
	ToolPolyPolygon(CadView *v);
	~ToolPolyPolygon();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();


private:
	static void selectedClicked(GtkWidget *widget, ToolPolyPolygon *klass);
	static void alignEdgesClicked(GtkWidget *widget, ToolPolyPolygon *klass);
	static void centreVertexClicked(GtkWidget *widget, ToolPolyPolygon *klass);
	static void closedClicked(GtkWidget *widget, ToolPolyPolygon *klass);
	static void verticesChanged(NumericEntry *widget, ToolPolyPolygon *klass);
	static void radiusChanged(NumericEntry *widget, ToolPolyPolygon *klass);	
	static void arcAngleChanged(NumericEntry *widget, ToolPolyPolygon *klass);	
	static void doneClicked(GtkWidget *widget, ToolPolyPolygon *klass);
};


#endif
