/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYREORDER_H
#define __TOOLPOLYREORDER_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "CPolyline.h"



class ProcPolyReorder : public JournalProcess
{
public:
	typedef enum {
		SWAP_CONSECUTIVE,
		SWAP_HALVES,
		REVERSE,
		LINK_AB,
		LINK_BA
	} ReorderOperation;

protected:
	ReorderOperation operation;

public:
	ProcPolyReorder();
	ProcPolyReorder(ReorderOperation op);
	virtual ~ProcPolyReorder();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;


	ReorderOperation getOperation() const;
	void setOperation(ReorderOperation op);
	

protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);


	void swapConsecutive(vector<CPolyline::VertexType>& vertices) const;
	void swapHalves(vector<CPolyline::VertexType>& vertices) const;
	void reverse(vector<CPolyline::VertexType>& vertices) const;
	void linkAB(CVertexContainer& vertices) const;
	void linkBA(CVertexContainer& vertices) const;

	bool findIndices(const CVertexContainer& vertices,
						  int &a, int &b) const;
	void link(CVertexContainer& vertices, int a, int b) const;
};




class ToolPolyReorder : public ProcessTool
{
private:
	GtkWidget *operationLabel;
	GtkWidget *swapConsecRadio, *swapHalvesRadio, *reverseRadio,
				 *linkABRadio, *linkBARadio;
	
	GtkWidget *doneButton;


	bool ignoreRadioEvents;

public:
	ToolPolyReorder(CadView *v);
	~ToolPolyReorder();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();


private:
	static void swapConsecClicked(GtkWidget *widget, ToolPolyReorder *klass);
	static void swapHalvesClicked(GtkWidget *widget, ToolPolyReorder *klass);
	static void reverseClicked(GtkWidget *widget, ToolPolyReorder *klass);
	static void linkABClicked(GtkWidget *widget, ToolPolyReorder *klass);
	static void linkBAClicked(GtkWidget *widget, ToolPolyReorder *klass);
	static void doneClicked(GtkWidget *widget, ToolPolyReorder *klass);
};


#endif
