/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLPOLYSUBDIVIDE_H
#define __TOOLPOLYSUBDIVIDE_H

#include <gtk/gtkwidget.h>
#include <gtk/gtkobject.h>

#include <string>


#include "JournalProcess.h"
#include "ProcessTool.h"
#include "NumericEntry.h"
#include "CPolyline.h"



class ProcPolySubdivide : public JournalProcess
{
public:
	typedef enum {
		SELECTSTATUS_SELECTED,
		SELECTSTATUS_UNSELECTED,
		SELECTSTATUS_ONLYSELECTED
	} SelectStatus;

	typedef enum {
		CURVETYPE_LINEAR,
		CURVETYPE_CATMULROM
	} CurveType;
	
private:
	int numberToInsert;
	SelectStatus selectStatus;
	CurveType curveType;
	
public:
	ProcPolySubdivide();
	ProcPolySubdivide(int numberOfVerticesToInsert, SelectStatus selStatus,
							CurveType curveTp);
	virtual ~ProcPolySubdivide();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;


	int getNumberOfVerticesToInsert() const;
	SelectStatus getSelectStatus() const;
	CurveType getCurveType() const;
	void setNumberOfVerticesToInsert(int n);
	void setSelectStatus(SelectStatus selStatus);
	void setCurveType(CurveType curveTp);
	

protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);

private:
	void insert(CPolyline *poly, int position,
					vector<int>& verticesAdded,
					const vector<Point3>& vertexPositions,
					int n) const;
};




class ToolPolySubdivide : public ProcessTool
{
private:
	GtkWidget *selectStatusLabel;
	GtkWidget *selectedRadio, *unselectedRadio, *onlySelectedRadio;

	GtkWidget *curveTypeLabel;
	GtkWidget *linearRadio, *catmulromRadio;

	GtkWidget *numLabel;
	NumericEntry *numEntry;
	
	GtkWidget *doneButton;

	bool ignoreRadioEvents;

public:
	ToolPolySubdivide(CadView *v);
	~ToolPolySubdivide();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();


private:
	static void selectedClicked(GtkWidget *widget, ToolPolySubdivide *klass);
	static void unselectedClicked(GtkWidget *widget, ToolPolySubdivide *klass);
	static void onlySelectedClicked(GtkWidget *widget, ToolPolySubdivide *klass);
	static void linearClicked(GtkWidget *widget, ToolPolySubdivide *klass);
	static void catmumromClicked(GtkWidget *widget, ToolPolySubdivide *klass);
	static void numChanged(NumericEntry *widget, ToolPolySubdivide *klass);
	static void doneClicked(GtkWidget *widget, ToolPolySubdivide *klass);
};


#endif
