/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLGROUPSELECT_CPP
#define __TOOLGROUPSELECT_CPP

#include "ToolGroupSelect.h"

#include <gtk/gtklabel.h>



ToolGroupSelect::ToolGroupSelect(ProcessToolManager *toolManager,
											JournalEditor *journEditor,
											CadView *view,
											ObjPainterManager *painterManager)
											: MToolGroup(toolManager)
{
	titleLabel = gtk_label_new("Select");
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);

	journalEditor = journEditor;

	rectButton = addToolButton( "Rect", GTK_SIGNAL_FUNC(rectClicked) );
	circButton = addToolButton( "Circular", GTK_SIGNAL_FUNC(circClicked) );
	selButton = addToolButton( "Select", GTK_SIGNAL_FUNC(selClicked) );
	invButton = addToolButton( "Invert", GTK_SIGNAL_FUNC(invClicked) );
	unselButton = addToolButton( "Unselect", GTK_SIGNAL_FUNC(unselClicked) );

	toolRect = new ToolSelectRectangular(view, painterManager);
	toolCirc = new ToolSelectCircular(view, painterManager);
	toolSelectAll = new ToolSelectAll(view, painterManager);

	toolMan->addTool(toolRect->getProcessType(), toolRect);
	toolMan->addTool(toolCirc->getProcessType(), toolCirc);
	toolMan->addTool(toolSelectAll->getProcessType(), toolSelectAll);
}

ToolGroupSelect::~ToolGroupSelect()
{
	gtk_widget_unref(titleLabel);
	
	toolMan->eraseTool(toolRect->getProcessType());
	toolMan->eraseTool(toolCirc->getProcessType());
	toolMan->eraseTool(toolSelectAll->getProcessType());
	delete toolRect;
	delete toolCirc;
	delete toolSelectAll;
}



void ToolGroupSelect::addToLayout(GridLayout *layout)
{
	MToolGroup::addToLayout(layout);

	layout->addAsLine(titleLabel);

	layout->add(rectButton);
	layout->add(circButton);
	layout->add(selButton);
	layout->add(invButton);
	layout->add(unselButton);
}



void ToolGroupSelect::rectClicked(GtkWidget *widget, ToolGroupSelect *klass)
{
	ProcSelectRectangular *p = new ProcSelectRectangular();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupSelect::circClicked(GtkWidget *widget, ToolGroupSelect *klass)
{
	ProcSelectCircular *p = new ProcSelectCircular();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupSelect::selClicked(GtkWidget *widget, ToolGroupSelect *klass)
{
	ProcSelectAll *p = new ProcSelectAll(SELECTPREDICATE_SELECT);
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupSelect::invClicked(GtkWidget *widget, ToolGroupSelect *klass)
{
	ProcSelectAll *p = new ProcSelectAll(SELECTPREDICATE_INVERT);
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupSelect::unselClicked(GtkWidget *widget, ToolGroupSelect *klass)
{
	ProcSelectAll *p = new ProcSelectAll(SELECTPREDICATE_UNSELECT);
	klass->journalEditor->addProcess(p);
	delete p;
}




#endif
