/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSELECTALL_CPP
#define __TOOLSELECTALL_CPP

#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtklabel.h>

#include <strstream>

#include "ToolSelectAll.h"

#include "JournalModel.h"





void* __create_ProcSelectAll()
{
	return (void*)( new ProcSelectAll() );
}

ObjectType type_ProcSelectAll( "ProcSelectAll", "ProcSelectBase",
										 __create_ProcSelectAll );





ProcSelectAll::ProcSelectAll() : ProcSelectBase()
{
}

ProcSelectAll::ProcSelectAll(SelectPredicate pred) : ProcSelectBase(pred)
{
}


ProcSelectAll::~ProcSelectAll()
{
}



void ProcSelectAll::read(FILE *f)
{
	ProcSelectBase::read(f);
}

void ProcSelectAll::write(FILE *f)
{
	ProcSelectBase::write(f);
}





string ProcSelectAll::getDescription() const
{
	switch(selectPredicate)
	{
	case SELECTPREDICATE_OR:
		return "Select All";
	case SELECTPREDICATE_SELECT:
		return "Select All";
	case SELECTPREDICATE_AND:
		return "Select None";
	case SELECTPREDICATE_INVERT:
		return "Invert Selection";
	case SELECTPREDICATE_UNSELECT:
		return "Unselect All";
	default:
		return "Select None";
	}
}

const ObjectType* ProcSelectAll::getType() const
{
	return &type_ProcSelectAll;
}

const ObjectType* ProcSelectAll::getTypeStatic()
{
	return &type_ProcSelectAll;
}



JournalProcess* ProcSelectAll::clone() const
{
	return new ProcSelectAll(selectPredicate);
}



void ProcSelectAll::selectInModel(JournalModel *model) const
{
	model->getModel()->select( selectPredicate, NULL );
}







ToolSelectAll::ToolSelectAll(CadView *v,
									  ObjPainterManager *pman)
									  : ToolSelectModel( v, pman, "Select",
																"Select All", 1 )
{
	functionLabel = gtk_label_new("Uninitialised");
	gtk_widget_ref(functionLabel);
	gtk_widget_show(functionLabel);
	
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);


	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
	
		
	gtk_widget_show(doneButton);
	
	layout.addAsLine(functionLabel);
	layout.lineSpace(1);
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolSelectAll::~ToolSelectAll()
{
	gtk_widget_unref(functionLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolSelectAll::getProcessType() const
{
	return ProcSelectAll::getTypeStatic();
}


void ToolSelectAll::initialise()
{
	ToolSelectModel::initialise();

	if ( hasProcess() )
	{
		ProcSelectAll *proc = (ProcSelectAll*)journalProcess;
		
		SelectPredicate pred = proc->getPredicate();

		switch(pred)
		{
		case SELECTPREDICATE_OR:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									  "Function: OR" );
			break;
		case SELECTPREDICATE_SELECT:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									  "Function: Select All" );
			break;
		case SELECTPREDICATE_AND:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									 "Function: AND" );
			break;
		case SELECTPREDICATE_INVERT:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									  "Function: Invert Selection" );
			break;
		case SELECTPREDICATE_UNSELECT:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									  "Function: Unselect All" );
			break;
		default:
			gtk_label_set_text( GTK_LABEL(functionLabel),
									  "Function: Unknown" );
			break;
		}
	}
}



void ToolSelectAll::doneClicked(GtkWidget *widget, ToolSelectAll *klass)
{
	klass->done();
}













#endif

