/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSELECTBASE_CPP
#define __TOOLSELECTBASE_CPP

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkradiobutton.h>

#include <strstream>

#include "ToolSelectBase.h"






void* __create_ProcSelectBase()
{
	return (void*)( new ProcSelectBase() );
}

ObjectType type_ProcSelectBase( "ProcSelectBase", "JournalProcess",
										  __create_ProcSelectBase );






ProcSelectBase::ProcSelectBase() : JournalProcess()
{
	selectPredicate = SELECTPREDICATE_OR;
}

ProcSelectBase::ProcSelectBase(SelectPredicate pred) : JournalProcess()
{
	selectPredicate = pred;
}


ProcSelectBase::~ProcSelectBase()
{
}



void ProcSelectBase::read(FILE *f)
{
	JournalProcess::read(f);

	fread(&selectPredicate, sizeof(SelectPredicate), 1, f);
}

void ProcSelectBase::write(FILE *f)
{
	JournalProcess::write(f);

	fwrite(&selectPredicate, sizeof(SelectPredicate), 1, f);
}





string ProcSelectBase::getDescription() const
{
	return "Select Base Class Process";
}

const ObjectType* ProcSelectBase::getType() const
{
	return &type_ProcSelectBase;
}

const ObjectType* ProcSelectBase::getTypeStatic()
{
	return &type_ProcSelectBase;
}



const ObjectType* ProcSelectBase::getInputType() const
{
	return JournalModel::getTypeStatic();
}



JournalProcess* ProcSelectBase::clone() const
{
	return new ProcSelectBase(selectPredicate);
}



void ProcSelectBase::setPredicate(SelectPredicate pred)
{
	selectPredicate = pred;
	modified();
}

SelectPredicate ProcSelectBase::getPredicate() const
{
	return selectPredicate;
}



const ObjectType* ProcSelectBase::_getOutputType(const ObjectType* inputType) const
{
	return inputType;
}

JournalObject* ProcSelectBase::_process(JournalObject *o,
													 JournalTaskProgress *progress)
{
	JournalModel *hmodel = (JournalModel*)o;
	selectInModel( hmodel );
	return (JournalObject*)hmodel;
}

void ProcSelectBase::_processInto(JournalObject *o, JournalObject *dst,
											 JournalTaskProgress *progress)
{
	dst->copyFrom(o);
	JournalModel *hmodel = (JournalModel*)dst;
	selectInModel( hmodel );
}


void ProcSelectBase::selectInModel(JournalModel *model) const
{
}







ToolSelectBase::ToolSelectBase( CadView *v,
										  ObjPainterManager *pman,
										  const string& funcName, const string& optTitle,
										  int tableWidth = 1 )
				: ToolSelectModel( v, pman, funcName, optTitle, tableWidth ),
				  CadClickAndDragEventHandler( v->getCadArea() )
{
	functionLabel = gtk_label_new("Function:");
	gtk_widget_ref(functionLabel);

	gtk_widget_show(functionLabel);
	layout.addAsLine(functionLabel);

	
	GSList *radioButtonGroup = (GSList*)NULL;

	for (int i = 0; i < getNumberOfSelectPredicates(); i++)
	{
		SelectPredicate predicate = (SelectPredicate)i;
		
		//fill in the response data structure
		responseData[i].klass = this;
		responseData[i].predicate = predicate;
		
		//create the radio button widget
		functionRadios[i] =
			gtk_radio_button_new_with_label( radioButtonGroup,
														getNameOfSelectPredicate(predicate) );
		//create the group for all except the last one
		if (i != 4)
		{
			radioButtonGroup =
				gtk_radio_button_group( GTK_RADIO_BUTTON(functionRadios[i]) );
		}

		gtk_widget_ref( functionRadios[i] );		//ref the widget

		gtk_signal_connect( GTK_OBJECT(functionRadios[i]), "clicked",
								  GTK_SIGNAL_FUNC(functionRadioClicked),
								  &responseData[i] );

		gtk_widget_show( functionRadios[i] );

		layout.add( functionRadios[i] );
	}
	
	layout.lineSpace(1);
}

ToolSelectBase::~ToolSelectBase()
{
	gtk_widget_unref(functionLabel);

	for (int i = 0; i < 5; i++)
	{
		gtk_widget_unref( functionRadios[i] );
	}
}



const ObjectType* ToolSelectBase::getProcessType() const
{
	return ProcSelectBase::getTypeStatic();
}




void ToolSelectBase::initialise()
{
	ToolSelectModel::initialise();

	if ( hasProcess() )
	{
		ProcSelectBase *proc = (ProcSelectBase*)journalProcess;
		SelectPredicate pred = proc->getPredicate();
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON( functionRadios[ (int)pred ] ), true );
	}
}


void ToolSelectBase::functionRadioClicked( GtkWidget *widget,
														 RadioButtonResponseData *data )
{
	ToolSelectBase *klass = data->klass;
	if ( klass->hasProcess() )
	{
		ProcSelectBase *proc = (ProcSelectBase*)klass->journalProcess;

		SelectPredicate pred = data->predicate;
		proc->setPredicate( pred );

		klass->modified();
	}		
}




CAEventHandler* ToolSelectBase::getEventHandler()
{
	return this;
}



#endif

