/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSELECTBASE_H
#define __TOOLSELECTBASE_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "JournalProcess.h"
#include "JournalTaskProgress.h"
#include "CadEventHandlers.h"
#include "SelectPredicate.h"
#include "ToolSelectModel.h"




class ProcSelectBase : public JournalProcess
{
protected:
	SelectPredicate selectPredicate;
	
public:
	ProcSelectBase();
	ProcSelectBase(SelectPredicate pred);
	virtual ~ProcSelectBase();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual const ObjectType* getInputType() const;

	virtual JournalProcess* clone() const;



	void setPredicate(SelectPredicate pred);
	SelectPredicate getPredicate() const;


protected:
	virtual const ObjectType* _getOutputType(const ObjectType* inputType) const;
	virtual JournalObject* _process(JournalObject *o,
											  JournalTaskProgress *progress);
	virtual void _processInto(JournalObject *o, JournalObject *dst,
									  JournalTaskProgress *progress);

	virtual void selectInModel(JournalModel *model) const;
};



class ToolSelectBase : public ToolSelectModel, public CadClickAndDragEventHandler
{
private:
	struct RadioButtonResponseData {
		ToolSelectBase *klass;
		SelectPredicate predicate;
	};
	
	GtkWidget *functionLabel;

	GtkWidget *functionRadios[5];
	RadioButtonResponseData responseData[5];


	SelectPredicate *predicate;

public:
	ToolSelectBase( CadView *v, ObjPainterManager *pman,
						 const string& funcName, const string& optTitle,
						 int tableWidth = 1 );
	virtual ~ToolSelectBase();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();
	
private:
	static void functionRadioClicked(GtkWidget *widget,
												RadioButtonResponseData *data);


protected:
	virtual CAEventHandler* getEventHandler();
};



#endif
