/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSELECTCIRCULAR_CPP
#define __TOOLSELECTCIRCULAR_CPP

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkradiobutton.h>

#include <strstream>

#include "ToolSelectCircular.h"

#include "JournalModel.h"
#include "Matrix4.h"
#include "ModelDraw.h"







CircularSelectRegion::CircularSelectRegion() : SelectRegion()
{
	//make sure the rectangle contains NOTHING
	centre = Point3();
	radiusSquared = -1.0;
}

CircularSelectRegion::CircularSelectRegion(const Matrix4& worldToScreen,
														 const Point3& circleCentre,
														 double circleRadius)
														 : SelectRegion(worldToScreen)
{
	centre = circleCentre;
	radiusSquared = circleRadius * circleRadius;
}


bool CircularSelectRegion::isSelected2D(const Point3& point) const
{
	Vector3 centreToPoint = point - centre;

	return ( ( centreToPoint.x * centreToPoint.x ) +
				( centreToPoint.y * centreToPoint.y ) ) <= radiusSquared;
}





void* __create_ProcSelectCircular()
{
	return (void*)( new ProcSelectCircular() );
}

ObjectType type_ProcSelectCircular( "ProcSelectCircular",
												"ProcSelectBase",
												__create_ProcSelectCircular );






ProcSelectCircular::ProcSelectCircular() : ProcSelectBase()
{
}

ProcSelectCircular::ProcSelectCircular( SelectPredicate pred,
													 const CircularSelectRegion& region )
													 : ProcSelectBase(pred)
{
	selectRegion = region;
}
	

ProcSelectCircular::~ProcSelectCircular()
{
}



void ProcSelectCircular::read(FILE *f)
{
	ProcSelectBase::read(f);

	fread(&selectRegion, sizeof(CircularSelectRegion), 1, f);
}

void ProcSelectCircular::write(FILE *f)
{
	ProcSelectBase::write(f);

	fwrite(&selectRegion, sizeof(CircularSelectRegion), 1, f);
}





string ProcSelectCircular::getDescription() const
{
	return "Circular Select";
}

const ObjectType* ProcSelectCircular::getType() const
{
	return &type_ProcSelectCircular;
}

const ObjectType* ProcSelectCircular::getTypeStatic()
{
	return &type_ProcSelectCircular;
}



JournalProcess* ProcSelectCircular::clone() const
{
	return new ProcSelectCircular(selectPredicate, selectRegion);
}



void ProcSelectCircular::setSelectRegion(const CircularSelectRegion& region)
{
	selectRegion = region;
	modified();
}

const CircularSelectRegion& ProcSelectCircular::getSelectRegion() const
{
	return selectRegion;
}



void ProcSelectCircular::selectInModel(JournalModel *model) const
{
	model->getModel()->select( selectPredicate, &selectRegion );
}







ToolSelectCircular::ToolSelectCircular( CadView *v,
													 ObjPainterManager *pman )
									: ToolSelectBase( v, pman, "Circ",
															"Circular Select" )
{
	drawRegionFlag = false;
	
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);

	gtk_widget_show(doneButton);


	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );
	

	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolSelectCircular::~ToolSelectCircular()
{
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolSelectCircular::getProcessType() const
{
	return ProcSelectCircular::getTypeStatic();
}



void ToolSelectCircular::start()
{
	ToolSelectBase::start();

	if ( hasProcess() )
	{
		ProcSelectCircular *proc = (ProcSelectCircular*)journalProcess;

		initialSelectRegion = proc->getSelectRegion();

		circleCentre = startCursorPosition.getScreenInt();
		drawRegionFlag = true;
	}
}

void ToolSelectCircular::finish(bool terminated)
{
	ToolSelectBase::finish(terminated);

	if ( hasProcess() )
	{
		ProcSelectCircular *proc = (ProcSelectCircular*)journalProcess;
		drawRegionFlag = false;

		if (terminated)
		{
			proc->setSelectRegion( initialSelectRegion );
		modified();
		}
		else
		{
		updateView();
		}
	}
}

void ToolSelectCircular::function()
{
	ToolSelectBase::function();

	if ( hasProcess() )
	{
		ProcSelectCircular *proc = (ProcSelectCircular*)journalProcess;

		Point3 startPoint = startCursorPosition.getScreenFloat();
		Point3 currentPoint = currentCursorPosition.getScreenFloat();

		double distance = (currentPoint - startPoint).length();
		circleRadius = (int)(distance + 0.5);

		CircularSelectRegion region =
			CircularSelectRegion( view->getCadArea()->getWorldScreenMatrix(),
										 startPoint, distance );

		proc->setSelectRegion( region );

		modified();
	}
}


void ToolSelectCircular::paint()
{
	if (drawRegionFlag)
	{
		modelDrawSelectCircle( view->getCadArea(), circleCentre, circleRadius );
	}
}






void ToolSelectCircular::doneClicked( GtkWidget *widget,
												  ToolSelectCircular *klass )
{
	klass->done();
}


#endif

