/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSELECTCIRCULAR_H
#define __TOOLSELECTCIRCULAR_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "JournalProcess.h"
#include "ToolSelectBase.h"
#include "SelectRegion.h"
#include "BBox.h"



class CircularSelectRegion : public SelectRegion
{
private:
	Point3 centre;
	double radiusSquared;

public:
	CircularSelectRegion();
	CircularSelectRegion(const Matrix4& worldToScreen,
								const Point3& circleCentre,
								double circleRadius);

protected:
	virtual bool isSelected2D(const Point3& point) const;
};




class ProcSelectCircular : public ProcSelectBase
{
protected:
	CircularSelectRegion selectRegion;

public:
	ProcSelectCircular();
	ProcSelectCircular(SelectPredicate pred,
								 const CircularSelectRegion& region);
	virtual ~ProcSelectCircular();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setSelectRegion(const CircularSelectRegion& region);
	const CircularSelectRegion& getSelectRegion() const;


protected:
	virtual void selectInModel(JournalModel *model) const;
};






class ToolSelectCircular : public ToolSelectBase
{
private:
	GtkWidget *doneButton;

	CircularSelectRegion initialSelectRegion;
	
	bool drawRegionFlag;
	IPoint2 circleCentre;
	int circleRadius;

public:
	ToolSelectCircular( CadView *v, ObjPainterManager *pman );
	virtual ~ToolSelectCircular();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	virtual void paint();


private:
	static void doneClicked(GtkWidget *widget, ToolSelectCircular *klass);
};


#endif
