/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADTOOLAXISCHOOSER_CPP
#define __CADTOOLAXISCHOOSER_CPP

#include <gtk/gtkmarshal.h>

#include <gtk/gtkradiobutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>

#include "CadToolAxisChooser.h"



void CadToolAxisChooser::emitSignal(GtkWidget *widget)
{
	if ( sigFunc != (GtkSignalFunc)0 )
	{
		gtk_marshal_NONE__NONE( (GtkObject*)widget, sigFunc, sigFuncData,
										(GtkArg*)NULL );
	}
}


void CadToolAxisChooser::radioXClicked(GtkWidget *widget,
													CadToolAxisChooser *klass)
{
	klass->axis = AXIS_X;
	klass->emitSignal(widget);	
}

void CadToolAxisChooser::radioYClicked(GtkWidget *widget,
													CadToolAxisChooser *klass)
{
	klass->axis = AXIS_Y;
	klass->emitSignal(widget);	
}

void CadToolAxisChooser::radioZClicked(GtkWidget *widget,
													CadToolAxisChooser *klass)
{
	klass->axis = AXIS_Z;
	klass->emitSignal(widget);	
}




CadToolAxisChooser::CadToolAxisChooser(Axis initialChoice)
{
	constructorCommon(initialChoice, AXIS);
	disconnect();
}

CadToolAxisChooser::CadToolAxisChooser(GtkSignalFunc func, gpointer data,
													Axis initialChoice,
													LabelTexts labelTexts)
{
	disconnect();					//building widgets can generate stray signals:
										//make sure we dont try to propagate them
	constructorCommon(initialChoice, labelTexts);
	connect(func, data);
}

CadToolAxisChooser::~CadToolAxisChooser()
{
	gtk_widget_unref(titleLabel);
	for (int i = 0; i < 3; i++)
	{
		gtk_widget_unref(radioButtons[i]);
	}
}


void CadToolAxisChooser::constructorCommon( Axis initialChoice,
														  LabelTexts labelTexts )
{
	const char *titleTexts[] = { "Axis:", "Plane:" };
	const char *radioButtonTextsAxis[] =
		{ "X-Axis", "Y-Axis", "Z-Axis" };
	const char *radioButtonTextsPlane[] =
		{ "X-Plane", "Y-Plane", "Z-Plane" };

	const char **radioButtonTexts;

	if (labelTexts == AXIS)
	{
		radioButtonTexts = radioButtonTextsAxis;
	}
	else
	{
		radioButtonTexts = radioButtonTextsPlane;
	}		
		

	titleLabel = gtk_label_new( titleTexts[labelTexts] );
	gtk_widget_ref(titleLabel);


	GtkSignalFunc radioButtonFunctions[] =
		{ GTK_SIGNAL_FUNC(radioXClicked),
		  GTK_SIGNAL_FUNC(radioYClicked),
		  GTK_SIGNAL_FUNC(radioZClicked) };


	GSList *radioButtonGroup = (GSList*)NULL;

	for (int i = 0; i < 3; i++)
	{
		//create the radio button widget
		radioButtons[i] =
			gtk_radio_button_new_with_label( radioButtonGroup,
														radioButtonTexts[i] );
		//create the group for all except the last one
		if (i != 2)
		{
			radioButtonGroup =
				gtk_radio_button_group( GTK_RADIO_BUTTON(radioButtons[i]) );
		}

		gtk_widget_ref( radioButtons[i] );		//ref the widget

		gtk_signal_connect( GTK_OBJECT(radioButtons[i]), "clicked",
								  GTK_SIGNAL_FUNC(radioButtonFunctions[i]), this );

	}

	setChoice(initialChoice);
}



void CadToolAxisChooser::connect(GtkSignalFunc func, gpointer data)
{
	sigFunc = func;
	sigFuncData = data;
}

void CadToolAxisChooser::disconnect()
{
	sigFunc = (GtkSignalFunc)0;
	sigFuncData = NULL;
}

void CadToolAxisChooser::addToLayout(GridLayout& layout)
{
	gtk_widget_show(titleLabel);

	layout.addAsLine(titleLabel);

	for (int i = 0; i < 3; i++)
	{
		gtk_widget_show(radioButtons[i]);
		layout.add(radioButtons[i]);
	}


	layout.lineSpace(1);
}


void CadToolAxisChooser::setChoice(Axis choice)
{
	switch(choice)
	{
	case AXIS_X:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioButtons[0]), true );
		break;
	case AXIS_Y:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioButtons[1]), true );
		break;
	case AXIS_Z:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioButtons[2]), true );
		break;
	default:
		break;
	}
}

Axis CadToolAxisChooser::getChoice() const
{
	return axis;
}


GtkWidget* CadToolAxisChooser::getWidget(int widgetNumber)
{
	return radioButtons[widgetNumber];
}


#endif
