/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADTOOLCENTRECHOOSER_CPP
#define __CADTOOLCENTRECHOOSER_CPP

#include <gtk/gtkmarshal.h>

#include <gtk/gtkradiobutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>

#include "CadToolCentreChooser.h"



CadToolCentreChooser::CadToolCentreChooser()
{
	constructorCommon();
	disconnect();
}

CadToolCentreChooser::CadToolCentreChooser(GtkSignalFunc func, gpointer data)
{
	disconnect();					//building widgets can generate stray signals:
										//make sure we dont try to propagate them
	constructorCommon();
	connect(func, data);
}

CadToolCentreChooser::~CadToolCentreChooser()
{
	gtk_widget_unref(titleLabel);
	gtk_widget_unref(radioCentre);
	gtk_widget_unref(radioOrigin);
}


void CadToolCentreChooser::constructorCommon()
{
	titleLabel = gtk_label_new("Centre:");
	gtk_widget_ref(titleLabel);

	GSList *group = (GSList*)NULL;
	radioCentre = gtk_radio_button_new_with_label(group, "Median");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(radioCentre) );
	radioOrigin = gtk_radio_button_new_with_label(group, "Origin");

	gtk_widget_ref(radioCentre);
	gtk_widget_ref(radioOrigin);

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioCentre), true );

	gtk_signal_connect( GTK_OBJECT(radioCentre), "clicked",
							  GTK_SIGNAL_FUNC(centreClicked), this );
	gtk_signal_connect( GTK_OBJECT(radioOrigin), "clicked",
							  GTK_SIGNAL_FUNC(originClicked), this );

	centre = MEDIAN;
}



void CadToolCentreChooser::connect(GtkSignalFunc func, gpointer data)
{
	sigFunc = func;
	sigFuncData = data;
}

void CadToolCentreChooser::disconnect()
{
	sigFunc = (GtkSignalFunc)0;
	sigFuncData = NULL;
}

void CadToolCentreChooser::addToLayout(GridLayout& layout)
{
	gtk_widget_show(titleLabel);
	gtk_widget_show(radioCentre);
	gtk_widget_show(radioOrigin);

	layout.addAsLine(titleLabel);
	layout.add(radioCentre);
	layout.add(radioOrigin);
	layout.lineSpace(1);
}


void CadToolCentreChooser::setChoice(Centre cen)
{
	centre = cen;

	if (centre == MEDIAN)
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioCentre), true );
	}
	else if (centre == ORIGIN)
	{
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radioOrigin), true );
	}
}

Centre CadToolCentreChooser::getChoice() const
{
	return centre;
}


GtkWidget* CadToolCentreChooser::getWidget(Centre cen)
{
	if (cen == MEDIAN)
	{
		return radioCentre;
	}
	else if (cen == ORIGIN)
	{
		return radioOrigin;
	}

	return (GtkWidget*)NULL;
}



void CadToolCentreChooser::emitSignal(GtkWidget *widget)
{
	if ( sigFunc != (GtkSignalFunc)0 )
	{
		gtk_marshal_NONE__NONE( (GtkObject*)widget, sigFunc, sigFuncData,
										(GtkArg*)NULL );
	}
}

void CadToolCentreChooser::centreClicked(GtkWidget *widget,
													  CadToolCentreChooser *klass)
{
	klass->centre = MEDIAN;
	klass->emitSignal(widget);
}

void CadToolCentreChooser::originClicked(GtkWidget *widget,
													  CadToolCentreChooser *klass)
{
	klass->centre = ORIGIN;
	klass->emitSignal(widget);
}





#endif
