/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLAXIS_CPP
#define __TOOLAXIS_CPP

#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolAxis.h"






void* __create_ProcAxis()
{
	return (void*)( new ProcAxis() );
}

ObjectType type_ProcAxis( "ProcAxis", "ProcXform", __create_ProcAxis );






ProcAxis::ProcAxis() : JournalProcess()
{
	axis = AXIS_Z;
}

ProcAxis::ProcAxis(Axis a) : JournalProcess()
{
	axis = a;
}

ProcAxis::~ProcAxis()
{
}



void ProcAxis::read(FILE *f)
{
	JournalProcess::read(f);

	fread(&axis, sizeof(Axis), 1, f);
}

void ProcAxis::write(FILE *f)
{
	JournalProcess::write(f);

	fwrite(&axis, sizeof(Axis), 1, f);
}





string ProcAxis::getDescription() const
{
	return "Axis";
}

const ObjectType* ProcAxis::getType() const
{
	return &type_ProcAxis;
}

const ObjectType* ProcAxis::getTypeStatic()
{
	return &type_ProcAxis;
}



JournalProcess* ProcAxis::clone() const
{
	return new ProcAxis(axis);
}




void ProcAxis::setAxis(Axis a)
{
	axis = a;
	modified();
}

Axis ProcAxis::getAxis() const
{
	return axis;
}



Axes3 ProcAxis::getAxes() const
{
	return Axes3(axis);
}





ToolAxis::ToolAxis(CadView *v, const string& funcName, const string& optTitle,
						 int tableWidth,
						 CadToolAxisChooser::LabelTexts labelTexts)
						 : ProcessTool( v, funcName, optTitle, tableWidth )
{
	axisChooser = new CadToolAxisChooser( GTK_SIGNAL_FUNC(axisChanged),
													  this, AXIS_Z, labelTexts );

	axisChooser->addToLayout(layout);
}

ToolAxis::~ToolAxis()
{
	delete axisChooser;
}



const ObjectType* ToolAxis::getProcessType() const
{
	return ProcAxis::getTypeStatic();
}



void ToolAxis::initialise()
{
	ProcessTool::initialise();

	if ( hasProcess() )
	{
		ProcAxis *proc = (ProcAxis*)journalProcess;

		axisChooser->setChoice( proc->getAxis() );
	}
}


void ToolAxis::axisChanged(GtkWidget *widget, ToolAxis *klass)
{
	if ( klass->hasProcess() )
	{
		ProcAxis *proc = (ProcAxis*)klass->journalProcess;
		proc->setAxis( klass->axisChooser->getChoice() );
		klass->modified();
	}
}



#endif

