/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/************************
 Licensed under the terms
 and conditions described
 in the file 'COPYING'
 ************************/
#ifndef CADTOOLAXESCPP
#define CADTOOLAXESCPP

#include "CadToolAxes.h"




CadToolAxes::CadToolAxes(AxisNumber defaultAxis) : CadToolSection()
{
	cta_label = gtk_label_new("Transformation Axis:");
	cta_world = gtk_label_new("World:");
	cta_screen = gtk_label_new("Screen:");

	gtk_widget_ref(cta_label);
	gtk_widget_ref(cta_world);
	gtk_widget_ref(cta_screen);

	GSList *group = (GSList*)NULL;
	cta_wld_x = gtk_radio_button_new_with_label(group, "X Axis");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(cta_wld_x) );
	cta_wld_y = gtk_radio_button_new_with_label(group, "Y Axis");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(cta_wld_y) );
	cta_wld_z = gtk_radio_button_new_with_label(group, "Z Axis");
	
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(cta_wld_z) );
	cta_scr_x = gtk_radio_button_new_with_label(group, "Horizontal");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(cta_scr_x) );
	cta_scr_y = gtk_radio_button_new_with_label(group, "Vertical");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(cta_scr_y) );
	cta_scr_z = gtk_radio_button_new_with_label(group, "Depth");

	gtk_widget_ref(cta_wld_x);
	gtk_widget_ref(cta_wld_y);
	gtk_widget_ref(cta_wld_z);
	gtk_widget_ref(cta_scr_x);
	gtk_widget_ref(cta_scr_y);
	gtk_widget_ref(cta_scr_z);


	cta_num = defaultAxis;
	switch(cta_num)
	{
	case WORLDX:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_wld_x), true );
		world_x_clicked(NULL, this);
		break;
	case WORLDY:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_wld_y), true );
		world_y_clicked(NULL, this);
		break;
	case WORLDZ:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_wld_z), true );
		world_z_clicked(NULL, this);
		break;
	case SCREENX:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_scr_x), true );
		screen_x_clicked(NULL, this);
		break;
	case SCREENY:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_scr_y), true );
		screen_y_clicked(NULL, this);
		break;
	case SCREENZ:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_scr_z), true );
		screen_z_clicked(NULL, this);
		break;
	default:
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cta_wld_x), true );
		world_x_clicked(NULL, this);
		break;
	}


	gtk_signal_connect( GTK_OBJECT(cta_wld_x), "clicked",
							  GTK_SIGNAL_FUNC(world_x_clicked), this );
	gtk_signal_connect( GTK_OBJECT(cta_wld_y), "clicked",
							  GTK_SIGNAL_FUNC(world_y_clicked), this );
	gtk_signal_connect( GTK_OBJECT(cta_wld_z), "clicked",
							  GTK_SIGNAL_FUNC(world_z_clicked), this );
	gtk_signal_connect( GTK_OBJECT(cta_scr_x), "clicked",
							  GTK_SIGNAL_FUNC(screen_x_clicked), this );
	gtk_signal_connect( GTK_OBJECT(cta_scr_y), "clicked",
							  GTK_SIGNAL_FUNC(screen_y_clicked), this );
	gtk_signal_connect( GTK_OBJECT(cta_scr_z), "clicked",
							  GTK_SIGNAL_FUNC(screen_z_clicked), this );
}

CadToolAxes::~CadToolAxes()
{
	gtk_widget_unref(cta_label);
	gtk_widget_unref(cta_world);
	gtk_widget_unref(cta_screen);
	gtk_widget_unref(cta_wld_x);
	gtk_widget_unref(cta_wld_y);
	gtk_widget_unref(cta_wld_z);
	gtk_widget_unref(cta_scr_x);
	gtk_widget_unref(cta_scr_y);
	gtk_widget_unref(cta_scr_z);
}


void CadToolAxes::addToGrid(GridLayout& gr)
{
	gr.expand( ISize2(2, 5) );

	gtk_widget_show(cta_label);
	gtk_widget_show(cta_world);
	gtk_widget_show(cta_screen);
	gtk_widget_show(cta_wld_x);
	gtk_widget_show(cta_wld_y);
	gtk_widget_show(cta_wld_z);
	gtk_widget_show(cta_scr_x);
	gtk_widget_show(cta_scr_y);
	gtk_widget_show(cta_scr_z);

	gr.addAsLine(cta_label);
	gr.add(cta_world);
	gr.add(cta_screen);
	gr.freshLine();
	gr.add(cta_wld_x);
	gr.add(cta_scr_x);
	gr.freshLine();
	gr.add(cta_wld_y);
	gr.add(cta_scr_y);
	gr.freshLine();
	gr.add(cta_wld_z);
	gr.add(cta_scr_z);
	gr.freshLine();
	gr.line(1);	
}



void CadToolAxes::world_x_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(0.0, 0.0, -1.0),
										 Vector3(0.0, 1.0, 0.0),
										 Vector3(1.0, 0.0, 0.0) );
	klass->cta_inScreenSpace = false;
	klass->cta_num = WORLDX;
}

void CadToolAxes::world_y_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(1.0, 0.0, 0.0),
										 Vector3(0.0, 0.0, -1.0),
										 Vector3(0.0, 1.0, 0.0) );
	klass->cta_inScreenSpace = false;
	klass->cta_num = WORLDY;
}

void CadToolAxes::world_z_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(1.0, 0.0, 0.0),
										 Vector3(0.0, 1.0, 0.0),
										 Vector3(0.0, 0.0, 1.0) );
	klass->cta_inScreenSpace = false;
	klass->cta_num = WORLDZ;
}

void CadToolAxes::screen_x_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(0.0, 0.0, -1.0),
										 Vector3(0.0, 1.0, 0.0),
										 Vector3(1.0, 0.0, 0.0) );
	klass->cta_inScreenSpace = true;
	klass->cta_num = SCREENX;
}

void CadToolAxes::screen_y_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(1.0, 0.0, 0.0),
										 Vector3(0.0, 0.0, -1.0),
										 Vector3(0.0, 1.0, 0.0) );
	klass->cta_inScreenSpace = true;
	klass->cta_num = SCREENY;
}

void CadToolAxes::screen_z_clicked(GtkWidget *widget, CadToolAxes *klass)
{
	klass->cta_rawAxes = Axes3( Vector3(1.0, 0.0, 0.0),
										 Vector3(0.0, 1.0, 0.0),
										 Vector3(0.0, 0.0, 1.0) );
	klass->cta_inScreenSpace = true;
	klass->cta_num = SCREENZ;
}


Axes3 CadToolAxes::getAxes(CadArea *ca) const
{
	if (cta_inScreenSpace)
	{
		return cta_rawAxes * ca->getScreenWorldMatrix();
	}
	else
	{
		return cta_rawAxes;
	}
}

CadToolAxes::AxisNumber CadToolAxes::getAxisNumber() const
{
	return cta_num;
}




#endif
