/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/************************
 Licensed under the terms
 and conditions described
 in the file 'COPYING'
 ************************/
#ifndef CADTOOLCENTRECPP
#define CADTOOLCENTRECPP

#include "CadToolCentre.h"



CadToolCentre::CadToolCentre() : CadToolSection()
{
	ctc_label = gtk_label_new("Transformation Centre:");
	gtk_widget_ref(ctc_label);

	GSList *group = (GSList*)NULL;
	ctc_centre = gtk_radio_button_new_with_label(group, "Object Centre");
	group = gtk_radio_button_group( GTK_RADIO_BUTTON(ctc_centre) );
	ctc_origin = gtk_radio_button_new_with_label(group, "Origin");

	gtk_widget_ref(ctc_centre);
	gtk_widget_ref(ctc_origin);

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ctc_centre), true );

	gtk_signal_connect( GTK_OBJECT(ctc_centre), "clicked",
							  GTK_SIGNAL_FUNC(centre_clicked), this );
	gtk_signal_connect( GTK_OBJECT(ctc_origin), "clicked",
							  GTK_SIGNAL_FUNC(origin_clicked), this );

	ctc_centreFlag = true;
}

CadToolCentre::~CadToolCentre()
{
	gtk_widget_unref(ctc_label);
	gtk_widget_unref(ctc_centre);
	gtk_widget_unref(ctc_origin);
}


void CadToolCentre::addToGrid(GridLayout& gr)
{
	gr.expand( ISize2(2, 3) );

	gtk_widget_show(ctc_label);
	gtk_widget_show(ctc_centre);
	gtk_widget_show(ctc_origin);

	gr.addAsLine(ctc_label);
	gr.add(ctc_centre);
	gr.add(ctc_origin);
	gr.lineSpace(1);
}


void CadToolCentre::centre_clicked(GtkWidget *widget, CadToolCentre *klass)
{
	klass->ctc_centreFlag = true;
}

void CadToolCentre::origin_clicked(GtkWidget *widget, CadToolCentre *klass)
{
	klass->ctc_centreFlag = false;
}


bool CadToolCentre::centreIsOrigin() const
{
	return !ctc_centreFlag;
}

bool CadToolCentre::centreIsObjectCentre() const
{
	return ctc_centreFlag;
}

Point3 CadToolCentre::getCentre(const CVertexList& l) const
{
	if (ctc_centreFlag)
	{
		return l.boundingBox().getCentre();
	}
	else
	{
		return Point3();
	}
}


#endif
