/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __PROCESSTOOLMANAGER_CPP
#define __PROCESSTOOLMANAGER_CPP

#include "ProcessToolManager.h"



ProcessToolManagerToolInUseException::
	ProcessToolManagerToolInUseException(const ObjectType *type)
{
	t = type;
}




ProcessToolManager::ProcessToolManager()
{
}



void ProcessToolManager::addTool(const ObjectType *type, ProcessTool *tool)
{
	Entry e;

	e.tool = tool;
	e.lockFlag = false;

	tools[ type->getId() ] = e;
}

void ProcessToolManager::eraseTool(const ObjectType *type)
{
	Typeid tid = type->getId();
	
	if ( tools.count(tid) > 0 )
	{
		if ( tools[tid].lockFlag )
		{
			throw ProcessToolManagerToolInUseException(type);
		}
		else
		{
			tools.erase(tid);
		}
	}
}	


ProcessTool* ProcessToolManager::takeTool(const ObjectType *type)
{
	Typeid tid = type->getId();
	
	if ( tools.count(tid) > 0 )
	{
		Entry &e = tools[tid];

		if ( e.lockFlag )
		{
			throw ProcessToolManagerToolInUseException(type);
		}
		else
		{
			e.lockFlag = true;
			return e.tool;
		}
	}
	else
	{
		return NULL;
	}
}

void ProcessToolManager::returnTool(const ObjectType *type)
{
	if ( tools.count( type->getId() ) )
	{
		tools[ type->getId() ].lockFlag = false;
	}
}




#endif
