/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOL_CPP
#define __TOOL_CPP

#include <gtk/gtktable.h>
#include <gtk/gtklabel.h>


#include "Tool.h"


void ToolListener::toolChanged()
{
}

void ToolListener::toolDone()
{
}




/*
*******************************************************************************
												Tool
*******************************************************************************
*/

Tool::Tool(const string& funcName, const string& optTitle, int tableWidth)
{
	functionName = funcName;
	
	toolTable = gtk_table_new(1, tableWidth, true);
	gtk_widget_ref(toolTable);

	titleLabel = gtk_label_new( optTitle.c_str() );
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);


	layout = GridLayout( GTK_TABLE(toolTable),
								(GtkAttachOptions)(GTK_FILL | GTK_SHRINK | GTK_EXPAND),
								(GtkAttachOptions)(GTK_SHRINK) );

	layout.addAsLine(titleLabel);
	layout.line(1);



	listener = NULL;
}

Tool::~Tool()
{
	gtk_widget_unref(toolTable);
	gtk_widget_unref(titleLabel);
}


void Tool::install(ToolListener *l, Toolbox *toolBox)
{
	currentToolbox = toolBox;
	addToToolbox(toolBox);
	listener = l;
	initialise();
}

void Tool::uninstall()
{
	removeFromToolbox(currentToolbox);
	shutdown();
	listener = NULL;
	currentToolbox = NULL;
}


void Tool::initialise()
{
}

void Tool::shutdown()
{
}


void Tool::modified()
{
	if (listener != NULL)
	{
		listener->toolChanged();
	}
}

void Tool::done()
{
	if (listener != NULL)
	{
		listener->toolDone();
	}
}


void Tool::addToToolbox(Toolbox *toolbox)
{
	gtk_widget_show(toolTable);
	toolbox->addToolset(toolTable, functionName, true);
}

void Tool::removeFromToolbox(Toolbox *toolbox)
{
	toolbox->removeToolset(toolTable);
	gtk_widget_hide(toolTable);
}




#endif
