/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLSET_CPP
#define __TOOLSET_CPP

#include <gtk/gtktable.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include "Toolset.h"



void Toolset::constructorShared(const string& name)
{
	toolsetLabel = gtk_label_new( name.c_str() );
	gtk_widget_ref(toolsetLabel);
	gtk_widget_show(toolsetLabel);

	toolTable = gtk_table_new(1, 3, true);
	gtk_widget_ref(toolTable);
	gtk_widget_show(toolTable);
	layout = GridLayout( GTK_TABLE(toolTable) );

	layout.addAsLine(toolsetLabel);
	layout.lineSpace();
}



Toolset::Toolset()
{
	toolBox = NULL;
	toolsetLabel = NULL;
	toolTable = NULL;
}

Toolset::Toolset(Toolbox *tbx)
{
	toolBox = tbx;
	constructorShared();
}

Toolset::Toolset(const string& name, Toolbox *tbx)
{
	toolBox = tbx;
	constructorShared(name);
}

Toolset::~Toolset()
{
	if (toolsetLabel != NULL)
	{
		gtk_widget_unref(toolsetLabel);
	}
	if (toolTable != NULL)
	{
		gtk_widget_unref(toolTable);
	}

	for (int i = 0; i < (int)toolButtons.size(); i++)
	{
		gtk_widget_unref( toolButtons[i] );
	}
}


GtkWidget* Toolset::getWidget()
{
	return toolTable;
}


GtkWidget* Toolset::addToolButton(const string& buttonText,
											 GtkSignalFunc buttonClickedResponse)
{
	GtkWidget *button = gtk_button_new_with_label( buttonText.c_str() );
	gtk_widget_ref(button);
	gtk_signal_connect( GTK_OBJECT(button), "clicked",
							  GTK_SIGNAL_FUNC(buttonClickedResponse), this );
	gtk_widget_show(button);

	layout.add(button);

	toolButtons.push_back(button);

	return button;
}


#endif
