/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TYPEMANAGER_CPP
#define __TYPEMANAGER_CPP

#include "TypeManager.h"









TypeManager *__global_type_manager__ = NULL;


void initialiseTypeManager()
{
	if ( __global_type_manager__ == NULL )
	{
		__global_type_manager__ = new TypeManager();
	}
}

void shutdownTypeManager()
{
	delete __global_type_manager__;
}

TypeManager& getTypeManager()
{
	initialiseTypeManager();
	return *__global_type_manager__;
}





ObjectType::ObjectType(const string& type, const string& parentType,
							  CreateFunction createFunc)
{
	parentTypeName = parentType;
	typeName = type;
	parentTid = tid = -1;
	creator = createFunc;
	getTypeManager().addType(this);
}


const string& ObjectType::getName() const
{
	return typeName;
}

const string& ObjectType::getParentName() const
{
	return parentTypeName;
}


Typeid ObjectType::getId() const
{
	return tid;
}

Typeid ObjectType::getParentId() const
{
	return parentTid;
}


void* ObjectType::create() const
{
	return (*creator)();
}


const ObjectType* ObjectType::getParent() const
{
	return __global_type_manager__->getParentType(this);
}


bool ObjectType::isInitialised() const
{
	return tid != -1;
}


bool ObjectType::isSubclassOf(const ObjectType* t) const
{
	return getTypeManager().isSubclass(this, t);
}











TypeManager::TypeManager()
{
}


void TypeManager::addType(ObjectType* type)
{
	if ( type->isInitialised() )
	{
		return;
	}

	addToList(type);
	registerType(type);


	type->parentTid = -1;
	if ( type->parentTypeName != "" )
	{
		ObjectType *p = getParent(type);

		if (p != NULL)
		{
			type->parentTid = p->tid;
		}
		else
		{
			addToUnparented(type);
		}
	}

	attachChildren(type);	
}

const ObjectType* TypeManager::getParentType(const ObjectType* type) const
{
	if (type->parentTid != -1)
	{
		return types[ type->parentTid ];
	}
	else
	{
		return NULL;
	}
}

bool TypeManager::isSubclass(const ObjectType* t1, const ObjectType* t2) const
{
	Typeid tid;
	tid = t1->tid;

	while (tid != -1)
	{
		if (tid == t2->tid)
		{
			return true;
		}

		tid = types[tid]->parentTid;
	}

	return false;
}


void* TypeManager::create(const string& name)
{
	if ( nameToType.count(name) > 0 )
	{
		return nameToType[name]->create();
	}
	else
	{
		return NULL;
	}
}



void TypeManager::addToList(ObjectType *type)
{
	type->tid = types.size();
	types.push_back(type);
}

void TypeManager::registerType(ObjectType *type)
{
	nameToType[ type->typeName ] = type;
}

ObjectType* TypeManager::getParent(ObjectType *type)
{
	if ( nameToType.count( type->parentTypeName ) > 0 )
	{
		return nameToType[ type->parentTypeName ];
	}
	else
	{
		return NULL;
	}
}

void TypeManager::addToUnparented(ObjectType *type)
{
	unparented[ type->parentTypeName ].push_back( type );
}

void TypeManager::attachChildren(ObjectType *type)
{
	if ( unparented.count( type->typeName ) )
	{
		list<ObjectType*> &l = unparented[ type->typeName ];
		
		for ( list<ObjectType*>::iterator iter = l.begin();
				iter != l.end();
				++iter )
		{
			(*iter)->parentTid = type->tid;
		}
		unparented.erase( type->typeName );
	}
}









#endif
