/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TYPEMANAGER_H
#define __TYPEMANAGER_H

#include <string>
#include <vector>
#include <map>
#include <list>





typedef int Typeid;




class TypeManager;

void initialiseTypeManager();
void shutdownTypeManager();
TypeManager& getTypeManager();




class ObjectType
{
public:
	typedef void* (*CreateFunction)();
	
private:
	string parentTypeName, typeName;
	Typeid parentTid, tid;
	CreateFunction creator;
	
public:
	ObjectType(const string& type, const string& parentType,
				  CreateFunction createFunc);

	const string& getName() const;
	const string& getParentName() const;

	Typeid getId() const;
	Typeid getParentId() const;

	void* create() const;	


	const ObjectType* getParent() const;
	

	bool isInitialised() const;


	bool isSubclassOf(const ObjectType* t) const;


	friend class TypeManager;
};



class TypeManager
{
private:
	vector<ObjectType*> types;
	map<string, ObjectType*> nameToType;
	map<string, list<ObjectType*> > unparented;

public:
	TypeManager();

	void addType(ObjectType* type);
	

	const ObjectType* getParentType(const ObjectType* type) const;

	bool isSubclass(const ObjectType* t1,
						 const ObjectType* t2) const;

	void* create(const string& name);

private:
	void addToList(ObjectType *type);
	void registerType(ObjectType *type);
	ObjectType* getParent(ObjectType *type);
	void addToUnparented(ObjectType *type);
	void attachChildren(ObjectType *type);
};





#endif
