/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CVERTEX_CPP
#define __CVERTEX_CPP

#include "CVertex.h"





CVertex::CVertex()
{
	markFlag = showFlag = false;
	fr = 0;
	visData = 0;
}

CVertex::CVertex(const Point3& v)
{
	wld = scr = v;
	markFlag = false;
	showFlag = true;
	fr = 0;
	visData = 0;
}



void CVertex::read(FILE *f)
{
	wld.read(f);
	fread( &markFlag, sizeof(bool), 1, f );

	scr = wld;
	showFlag = true;
	fr = 0;
	visData = 0;
}

void CVertex::write(FILE *f) const
{
	wld.write(f);
	fwrite( &markFlag, sizeof(bool), 1, f );
}




void CVertex::mark()
{
	markFlag = true;
}

void CVertex::unmark()
{
	markFlag = false;
}

bool CVertex::isMarked() const
{
	return markFlag;
}


void CVertex::show()
{
	showFlag = true;
}

void CVertex::hide()
{
	showFlag = false;
}

bool CVertex::isShown() const
{
	return showFlag;
}


bool CVertex::checkVisibilityBit(int b) const
{
	return visData & (1 << b);
}

long CVertex::getVisibilityData() const
{
	return visData;
}

bool CVertex::isVisible() const
{
	return visData == 0;
}


void CVertex::setWorld(const Point3& v)
{
	wld = scr = v;
	fr = 0;
}

Point3 CVertex::getWorld() const
{
	return wld;
}

Point3 CVertex::getScreen() const
{
	return scr;
}



void CVertex::copyFrom(const CVertex *v)
{
	wld = v->wld;
	markFlag = v->markFlag;
}


void CVertex::transform(const Matrix4& m)
{
	scr = wld = wld * m;
	fr = 0;
}

void CVertex::transform(const Transformation *x)
{
	scr = wld = x->transformPoint(wld);
	fr = 0;
}


void CVertex::select(SelectPredicate predicate, const SelectRegion *region)
{
	bool alreadySelected = markFlag;
	bool inRegion;

	if ( region != NULL )
	{
		inRegion = region->isSelected(wld);
	}
	else
	{
		inRegion = true;
	}
	
	bool newSelectState =
		selectPredicateIsSelected( predicate, alreadySelected, inRegion );

	if ( newSelectState )
	{
		mark();
	}
	else
	{
		unmark();
	}
}


void CVertex::update(const Transformation *w2s, const ViewFrustrum& frus,
							const Frame& currentFrame)
{
	if (fr != currentFrame)
	{
		scr = w2s->transformPoint(wld);
		visData = frus.pointBitList(wld);
		fr = currentFrame;
	}
}



CVertex CVertex::backup() const
{
	CVertex v;
	v.wld = wld;
	v.markFlag = markFlag;
	
	return v;
}

void CVertex::restore(const CVertex& bak)
{
	setWorld( bak.wld );
	if ( bak.markFlag )
	{
		mark();
	}
	else
	{
		unmark();
	}	
}

	


#endif
