/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by:					Geoffrey French
	application:		gSculpt
	last revision:		16/Apr/2000
*/
#ifndef __CVERTEX_H
#define __CVERTEX_H

#include <stdio.h>

#include "Point3.h"
#include "Transformation.h"
#include "Frame.h"
#include "ViewFrustrum.h"
#include "Matrix4.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"




class CVertex
{
private:
	Point3 wld, scr;
	bool markFlag, showFlag;
	long visData;
	Frame fr;

public:
	CVertex();
	CVertex(const Point3& v);


	void read(FILE *f);
	void write(FILE *f) const;



	//mark, unmark, check marked status
	void mark();
	void unmark();
	bool isMarked() const;

	//show, hide, check show/hide status
	void show();
	void hide();
	bool isShown() const;

	//set/clear/check visibility bits
	bool checkVisibilityBit(int b) const;
	long getVisibilityData() const;
	bool isVisible() const;

	//set world, get world
	void setWorld(const Point3& v);
	Point3 getWorld() const;
	Point3 getScreen() const;


	void copyFrom(const CVertex *v);

	

	//transform vertex (transform world coord)
	void transform(const Matrix4& m);
	void transform(const Transformation *x);

	//select vertex
	void select(SelectPredicate predicate, const SelectRegion *region);

	
	//update the screen coordinate, and visibility data:
	//transforms world coord to get screen coord using w2s if necessary
	void update(const Transformation *w2s, const ViewFrustrum& frus,
					const Frame& currentFrame);


	CVertex backup() const;
	void restore(const CVertex& bak);
};



#endif
