/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
/*	by :					Geoffrey French
	application :		gSculpt
	last revision :	09/Jul/2000
	title :				Vertex Container
	puspose :			Contain a list of CAD vertices for use in 3D models,
							polylines, etc.
*/
#ifndef __CVERTEXCONTAINER_H
#define __CVERTEXCONTAINER_H

#include <stdio.h>

#include <vector>

#include "array.h"

#include "CVertex.h"
#include "BBox.h"
#include "Matrix4.h"
#include "Transformation.h"
#include "SelectPredicate.h"
#include "SelectRegion.h"


/*
*******************************************************************************
										CVertexContainer
*******************************************************************************
*/


class CVertexContainer : public array<CVertex>
{
public:
	typedef CVertex VertexType;

	
	class VertexBackup
	{
	private:
		struct Entry {
			int index;
			CVertex data;
		};
		
		typedef vector<Entry> BackupData;
	
		BackupData bakData;


		friend class CVertexContainer;
	};

	

	CVertexContainer();


	void read(FILE *f);
	void write(FILE *f) const;


	CVertex& get(int i);
	const CVertex& get(int i) const;


	void copyFrom(const CVertexContainer *c);


	void transform(const Matrix4& m, bool selectedOnly);
	void transform(const Transformation *x, bool selectedOnly);

	void select(SelectPredicate predicate, const SelectRegion* region);

	VertexBackup backup(bool selectedOnly) const;
	void restore(const VertexBackup& b);


	BBox getBBox(bool selectedOnly = true) const;
	BBox getBBox(const Matrix4& m, bool selectedOnly = true) const;
	BBox getBBox(const Transformation *x, bool selectedOnly = true) const;
};



#endif
