/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __GRIDLAYOUT_CPP
#define __GRIDLAYOUT_CPP

#include <gtk/gtkcontainer.h>

#include "GridLayout.h"



void GridLayout::resize()
{
	int h = pos.y + 1;

	if (pos.x == 0)
	{
		h--;
	}

	gtk_table_resize( tab, h > tab->nrows ? h : tab->nrows, tab->ncols );
}


GridLayout::GridLayout()
{
	tab = (GtkTable*)NULL;
	pos = IPoint2();
}

GridLayout::GridLayout(GtkTable *t, GtkAttachOptions xat,
							  GtkAttachOptions yat)
{
	tab = t;
	pos = IPoint2();
	xAttachOpt = xat;
	yAttachOpt = yat;
}

GtkTable* GridLayout::getTableWidget()
{
	return tab;
}



GtkAttachOptions GridLayout::getXAttachOptions() const
{
	return xAttachOpt;
}

GtkAttachOptions GridLayout::getYAttachOptions() const
{
	return yAttachOpt;
}

void GridLayout::setAttachOptions( GtkAttachOptions xat,
											  GtkAttachOptions yat )
{
	xAttachOpt = xat;
	yAttachOpt = yat;
}



IPoint2 GridLayout::getPosition() const
{
	return pos;
}

void GridLayout::setPosition(const IPoint2& p)
{
	pos = p;
	resize();
}

ISize2 GridLayout::getSize() const
{
	return ISize2(tab->ncols, tab->nrows);
}

void GridLayout::expand(const ISize2& s)
{
	ISize2 x = getSize();

	if (x.w < s.w)
	{
		x.w = s.w;
	}

	if (x.h < s.h)
	{
		x.h = s.h;
	}

	gtk_table_resize( tab, x.h, x.w );
}

void GridLayout::line(int lines)
{
	pos.x = 0;
	pos.y += lines;
	resize();
}

void GridLayout::lineSpace(int lines)
{
	if (pos.x == 0)
	{
		line(lines);
	}
	else
	{
		line(lines + 1);
	}	
}

void GridLayout::nextLine()
{
	line(1);
}

void GridLayout::newLine()
{
	lineSpace(0);
}

void GridLayout::space(int nspaces)
{
	pos.x += nspaces;
	if (pos.x >= tab->ncols)
	{
		pos.x = 0;
		pos.y++;
	}
	resize();
}

void GridLayout::add(GtkWidget *w, int width)
{
	add( w, xAttachOpt, yAttachOpt, width );
}

void GridLayout::add( GtkWidget *w, GtkAttachOptions xat,
							 GtkAttachOptions yat, int width )
{
	if ((pos.x != 0) && (width > (tab->ncols - pos.x)))
	{
		line();
	}

	if ( w != NULL )
	{
		gtk_table_attach(tab, w,
							  pos.x, pos.x + width, pos.y, pos.y + 1,
							  xat, yat,
							  0, 0);
	}

	space(width);
}

void GridLayout::addAsLine(GtkWidget *w)
{
	add(w, tab->ncols);
}

void GridLayout::addAsLine( GtkWidget *w, GtkAttachOptions xat,
									 GtkAttachOptions yat )
{
	add(w, xat, yat, tab->ncols);
}

void GridLayout::addAsBlock(GtkWidget *w, const ISize2& s)
{
	addAsBlock( w, s, xAttachOpt, yAttachOpt );
}

void GridLayout::addAsBlock( GtkWidget *w, const ISize2& s,
									  GtkAttachOptions xat, GtkAttachOptions yat )
{
	if ( ( pos.x != 0 ) && ( s.w > ( tab->ncols - pos.x ) ) )
	{
		line();
	}

	if ( w != NULL )
	{
		gtk_table_attach(tab, w,
							  pos.x, pos.x + s.w, pos.y, pos.y + s.h,
							  xat, yat,
							  0, 0);
	}

	pos.y += s.h - 1;
	space(s.w);
}

void GridLayout::remove(GtkWidget *w)
{
	gtk_container_remove( GTK_CONTAINER(tab), w );
}



#endif
