/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __LISTMANAGER_CPP
#define __LISTMANAGER_CPP

#include <gtk/gtkclist.h>
#include <gtk/gtkscrolledwindow.h>

#include "ListManager.h"




ListManager::ListManager(const string& columnTitle)
{
	char *titles[] = { strdup( columnTitle.c_str() ) };
	
	gtkLs = gtk_clist_new_with_titles(1, titles);
	gtk_widget_ref(gtkLs);
	gtk_widget_show(gtkLs);
	gtk_signal_connect( GTK_OBJECT(gtkLs), "select_row",
							  GTK_SIGNAL_FUNC(select_row), this );

	scrolledWindow = gtk_scrolled_window_new( NULL, NULL );
	gtk_widget_ref(scrolledWindow);
	gtk_scrolled_window_set_policy(
		GTK_SCROLLED_WINDOW(scrolledWindow),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(scrolledWindow),
		gtkLs );


	currentSelection = "";

	free(titles[0]);
}

ListManager::~ListManager()
{
	gtk_widget_unref(gtkLs);
	gtk_widget_unref(scrolledWindow);
}



bool ListManager::exists(const string& n) const
{
	return names.count(n) > 0;
}

void ListManager::add(const string& n)
{
	names.insert(n);
	updateList();
}

void ListManager::remove(const string& n)
{
	names.erase(n);
	updateList();
	if (currentSelection == n)
	{
		currentSelection = "";
	}
}

void ListManager::clear()
{
	names.clear();
	updateList();
	currentSelection = "";
}

string ListManager::getSelection() const
{
	return currentSelection;
}

GtkWidget* ListManager::getWidget()
{
	return scrolledWindow;
}



void ListManager::updateList()
{
	char *text;

	gtk_clist_freeze( GTK_CLIST(gtkLs) );	
	gtk_clist_clear( GTK_CLIST(gtkLs) );

	for (set<string>::const_iterator iter = names.begin();
		  iter != names.end();
		  ++iter)
	{
		text = strdup( (*iter).c_str() );
		gtk_clist_append( GTK_CLIST(gtkLs), &text );
		free( text );
	}

	gtk_clist_thaw( GTK_CLIST(gtkLs) );
}


void ListManager::select_row(GtkWidget *widget, gint row, gint col,
										GdkEventButton *event, ListManager *klass)

{
	gchar *text;

	gtk_clist_get_text( GTK_CLIST(klass->gtkLs), row, col, &text );
	
	klass->currentSelection = text;
}




#endif
