/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __NAMEDIALOG_CPP
#define __NAMEDIALOG_CPP

#include "NameDialog.h"

#include <gtk/gtkwindow.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktable.h>
#include <gtk/gtksignal.h>




void NameDialogListener::okPressed(NameDialog *d, const string& name)
{
}

void NameDialogListener::cancelPressed(NameDialog *d)
{
}




NameDialog::NameDialog(const string& title, const string& prompt)
{
	listener = NULL;
	constructorCommon(title, prompt);
}

NameDialog::NameDialog(OkCallback ocb, CancelCallback ccb,
							  const string& title, const string& prompt)
{
	listener = NULL;
	okCall = ocb;
	canCall = ccb;
	constructorCommon(title, prompt);
}

NameDialog::NameDialog(NameDialogListener *l,
							  const string& title = "Name",
							  const string& prompt = "name:")
{
	listener = l;
	constructorCommon(title, prompt);
}	

NameDialog::~NameDialog()
{
	gtk_widget_unref(promptLabel);
	gtk_widget_unref(entry);
	gtk_widget_unref(ok);
	gtk_widget_unref(cancel);
	gtk_widget_unref(table);
	gtk_widget_unref(win);
}


void NameDialog::constructorCommon(const string& title, const string& prompt)
{
	win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_ref(win);
	gtk_window_set_title( GTK_WINDOW(win), title.c_str() );

	table = gtk_table_new(2, 3, 0);
	gtk_widget_ref(table);

	promptLabel = gtk_label_new( prompt.c_str() );
	gtk_widget_ref(promptLabel);

	entry = gtk_entry_new();
	gtk_widget_ref(entry);

	ok = gtk_button_new_with_label("OK");
	gtk_widget_ref(ok);

	cancel = gtk_button_new_with_label("Cancel");
	gtk_widget_ref(cancel);	


	gtk_signal_connect( GTK_OBJECT(ok), "clicked",
							  GTK_SIGNAL_FUNC(ok_response), this );
	gtk_signal_connect( GTK_OBJECT(cancel), "clicked",
							  GTK_SIGNAL_FUNC(can_response), this );
	gtk_signal_connect( GTK_OBJECT(win), "destroy",
							  GTK_SIGNAL_FUNC(win_destroy), this );

	GtkAttachOptions ao = (GtkAttachOptions)GTK_FILL;

	gtk_table_attach( GTK_TABLE(table), promptLabel, 0, 1, 0, 1, ao, ao, 10, 20 );
	gtk_table_attach( GTK_TABLE(table), entry, 1, 3, 0, 1, ao, ao, 10, 20 );
	gtk_table_attach( GTK_TABLE(table), ok, 0, 2, 1, 2, ao, ao, 10, 20 );
	gtk_table_attach( GTK_TABLE(table), cancel, 2, 3, 1, 2, ao, ao, 10, 20 );

	gtk_container_add( GTK_CONTAINER(win), table );

	gtk_widget_show(promptLabel);
	gtk_widget_show(entry);
	gtk_widget_show(ok);
	gtk_widget_show(cancel);
	gtk_widget_show(table);
	gtk_widget_show(win);

	okSelected = false;
}


void NameDialog::ok_response(GtkWidget *widget, NameDialog *klass)
{
	klass->okSelected = true;
	gtk_widget_destroy( GTK_WIDGET(klass->win) );
	klass->okCall.call( klass, gtk_entry_get_text( GTK_ENTRY(klass->entry) ) );
	if ( klass->listener != NULL )
	{
		klass->listener->okPressed( klass,
											 gtk_entry_get_text( GTK_ENTRY(klass->entry) ) );
	}
}

void NameDialog::can_response(GtkWidget *widget, NameDialog *klass)
{
	klass->okSelected = false;
	gtk_widget_destroy( GTK_WIDGET(klass->win) );
}

void NameDialog::win_destroy(GtkWidget *widget, NameDialog *klass)
{
	if (!klass->okSelected)
	{
		klass->canCall.call( klass );
		if ( klass->listener != NULL )
		{
			klass->listener->cancelPressed( klass );
		}
		
	}
}



#endif
