/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __NUMERICENTRY_CPP
#define __NUMERICENTRY_CPP

#include <gtk/gtkmarshal.h>

#include <gtk/gtkspinbutton.h>
#include <gtk/gtkadjustment.h>
#include <gtk/gtksignal.h>

#include "NumericEntry.h"




NumericEntry::NumericEntry(double value, double lower, double upper,
									double stepInc, double pageInc, double climbRate,
									int digits)
{
	constructorCommon(value, lower, upper, stepInc, pageInc, climbRate, digits);
}

NumericEntry::NumericEntry(double value, double lower, double upper,
									double stepInc, double pageInc, double climbRate,
									int digits, GtkSignalFunc responseFunc,
									gpointer responseData)
{
	constructorCommon(value, lower, upper, stepInc, pageInc, climbRate, digits);
	
	connect( responseFunc, responseData );
}

NumericEntry::~NumericEntry()
{
	gtk_object_unref(adj);
	gtk_widget_unref(spinbt);
}



void NumericEntry::constructorCommon(double value, double lower, double upper,
												 double stepInc, double pageInc,
												 double climbRate, int digits)
{
	blockChangeEvents = true;
	adj = gtk_adjustment_new( value, lower, upper, stepInc, pageInc, 0.0 );
	gtk_object_ref(adj);

	spinbt = gtk_spin_button_new( GTK_ADJUSTMENT(adj), climbRate, digits );
	gtk_widget_ref(spinbt);


	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON(spinbt), true );


	gtk_signal_connect( GTK_OBJECT(adj), "value_changed",
							  GTK_SIGNAL_FUNC(valueChangedResponse), this );
	blockChangeEvents = false;
}



GtkWidget* NumericEntry::getWidget()
{
	return spinbt;
}



void NumericEntry::wrap()
{
	gtk_spin_button_set_wrap( GTK_SPIN_BUTTON(spinbt), true );
}

void NumericEntry::nowrap()
{
	gtk_spin_button_set_wrap( GTK_SPIN_BUTTON(spinbt), false );
}


void NumericEntry::snap()
{
	gtk_spin_button_set_snap_to_ticks( GTK_SPIN_BUTTON(spinbt), true );
}

void NumericEntry::nosnap()
{
	gtk_spin_button_set_snap_to_ticks( GTK_SPIN_BUTTON(spinbt), false );
}



void NumericEntry::connect(GtkSignalFunc responseFunc, gpointer responseData)
{
	changeResponseFunction = responseFunc;
	changeResponseData = responseData;
}

void NumericEntry::disconnect()
{
	changeResponseFunction = (GtkSignalFunc)NULL;
}



void NumericEntry::setValue(double v)
{
	blockChangeEvents = true;
	gtk_spin_button_set_value( GTK_SPIN_BUTTON(spinbt), v );
	blockChangeEvents = false;
}


double NumericEntry::getValue() const
{
	return gtk_spin_button_get_value_as_float( GTK_SPIN_BUTTON(spinbt) );
}

int NumericEntry::getValueAsInt() const
{
	return gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(spinbt) );
}



void NumericEntry::valueChangedResponse(GtkObject *obj, NumericEntry *klass)
{
	GtkSignalFunc f = klass->changeResponseFunction;

	if ( f != NULL && !klass->blockChangeEvents )
	{
		gtk_marshal_NONE__NONE( (GtkObject*)klass, f, klass->changeResponseData,
										(GtkArg*)NULL );
	}
}



#endif

