/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef CAEH_GRIDCPP
#define CAEH_GRIDCPP

#include "CAEH_grid.h"

#include <strstream>



CAEH_grid::CAEH_grid(CadArea *c, const CGrid& gr)
						  : CAEventHandler(c)
{
	g = gr;
	gridChanged = true;
	viewFrame = 0;
}

CAEH_grid::~CAEH_grid()
{
}


void CAEH_grid::setGrid(const CGrid& gr)
{
	g = gr;
	gridChanged = true;
}

void CAEH_grid::paint()
{
	if (gridChanged || viewFrame != cadArea->getFrame())
	{
		generateList();
		gridChanged = false;
		viewFrame = cadArea->getFrame();
	}

	for (int i = 0; i < lineList.size(); i++)
	{
		cadArea->setForeground( lineList[i].col );
		for (int j = 0; j < lineList[i].hlines.size(); j++)
		{
			cadArea->drawLine( lineList[i].hlines[j] );
			ostrstream stream;
			if ( cadArea->isScreenXInverted() )
			{
				stream << -lineList[i].hvalues[j] << (char)0x00;
			}
			else
			{
				stream << lineList[i].hvalues[j] << (char)0x00;
			}
			cadArea->drawString( lineList[i].hlines[j].p1 + IVector2(2, 2),
										stream.str() );
		}
		for (int j = 0; j < lineList[i].vlines.size(); j++)
		{
			cadArea->drawLine( lineList[i].vlines[j] );
			ostrstream stream;
			if ( cadArea->isScreenYInverted() )
			{
				stream << -lineList[i].vvalues[j] << (char)0x00;
			}
			else
			{
				stream << lineList[i].vvalues[j] << (char)0x00;
			}
			cadArea->drawString( lineList[i].vlines[j].p1 + IVector2(2, 2),
										stream.str() );
		}
	}
}

void CAEH_grid::generateList()
{
	array<Line3> hlines;
	array<Line3> vlines;

	array<Line3>::const_iterator iter;

	Point3 v1, v2;
	IPoint2 p1, p2;

	v1 = Point3(0.0, 0.0, 0.0);
	v2 = Point3( (double)(cadArea->getSize().w - 1),
					 (double)(cadArea->getSize().h - 1), 0.0 );

	g.setCorners( cadArea->transformScreenWorld_nodirection(v1),
					  cadArea->transformScreenWorld_nodirection(v2) );

	lineList.clear();

	for (int i = g.nLevels() - 1; i >= 0; i--)
	{
		int j = lineList.push_back(LevelLineList());
		lineList[j].col = g.getGridColour( j );
		
	 	hlines = g.generateHLines(i);
	 	vlines = g.generateVLines(i);

	 	iter = hlines.begin();
	 	while ( iter != hlines.end() )
	 	{
	 		p1 = cadArea->transformWorldScreen_nodirectionInt2( (*iter).s );
	 		p2 = cadArea->transformWorldScreen_nodirectionInt2( (*iter).e );
	 		lineList[j].hlines.push_back( ILine2( p1, p2 ) );
	 		lineList[j].hvalues.push_back( (*iter).s.x );
	 		iter++;
	 	}

	 	iter = vlines.begin();
	 	while ( iter != vlines.end() )
	 	{
	 		p1 = cadArea->transformWorldScreen_nodirectionInt2( (*iter).s );
	 		p2 = cadArea->transformWorldScreen_nodirectionInt2( (*iter).e );
	 		lineList[j].vlines.push_back( ILine2( p1, p2 ) );
	 		lineList[j].vvalues.push_back( (*iter).s.y );
	 		iter++;
	 	}
	}
}



#endif
