/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADVIEW_CPP
#define __CADVIEW_CPP

#include "CadView.h"









CadViewDragZoomEventHandler::CadViewDragZoomEventHandler(CadArea *c)
																			: CAEventHandler(c)
{
	runningMode = MODE_NONE;
	zoompp = pow(2.0, 0.01);		//magnify by 2x for every 100 pixels
}

CadViewDragZoomEventHandler::CadViewDragZoomEventHandler(CadArea *c,
																			double zoomPerPixel)
																			: CAEventHandler(c)
{
	runningMode = MODE_NONE;
	zoompp = zoomPerPixel;
}



void CadViewDragZoomEventHandler::evButtonDown(int button)
{
	if ( runningMode == MODE_NONE && button == 1 )
	{
		int state = cadArea->getPointerStatus().getModifierKeyStates();

		if ( state & GDK_SHIFT_MASK )
		{
			runningMode = MODE_ZOOM;
		}
		else if ( state & GDK_CONTROL_MASK )
		{
			runningMode = MODE_DRAG;
		}

		previousPosition = cadArea->getPointerStatus().getPosition();
	}
}

void CadViewDragZoomEventHandler::evButtonUp(int button)
{
	if ( button == 1 )
	{
		runningMode = MODE_NONE;
	}
}


void CadViewDragZoomEventHandler::evMotion(const IPoint2& p)
{
	if ( runningMode == MODE_DRAG )
	{
		Point3 prev, cur;
		Vector3 movement;
	
		prev = cadArea->transformScreenWorld(
			Point3(previousPosition.x, previousPosition.y, 0.0) );
		cur = cadArea->transformScreenWorld(
						Point3(cadArea->getPointerStatus().getPosition().x,
								 cadArea->getPointerStatus().getPosition().y,
								 0.0) );
		movement = cur - prev;
		cadArea->drag(movement);
		previousPosition = cadArea->getPointerStatus().getPosition();
		cadArea->redraw();
	}
	else if ( runningMode == MODE_ZOOM )
	{
		int pixels = ( cadArea->getPointerStatus().getPosition()
							- previousPosition ).y;
		cadArea->zoom( pow( zoompp, (double)pixels ) );
		previousPosition = cadArea->getPointerStatus().getPosition();
		cadArea->redraw();
	}
}









CadView::CadView()
{
	cadArea = new CadArea();

	gtk_widget_show( cadArea->getWidget() );

	clearEventHandler = new DAEH_clear( (DrawingArea*)cadArea );
	gridEventHandler = new CAEH_grid(cadArea);
	dragZoomEventHandler = new CadViewDragZoomEventHandler(cadArea);
	cadArea->addEventHandler(clearEventHandler);
	cadArea->addEventHandler(gridEventHandler);
	cadArea->addEventHandler(dragZoomEventHandler);

	currentEventHandler = NULL;
}

CadView::~CadView()
{
	delete clearEventHandler;
	delete gridEventHandler;
	delete dragZoomEventHandler;
	delete cadArea;
}


void CadView::addEventHandler(CAEventHandler *e)
{
	cadArea->addEventHandler(e);
}

void CadView::removeEventHandler(CAEventHandler *e)
{
	cadArea->removeEventHandler(e);
}

void CadView::newEventHandler(CAEventHandler *e)
{
	if (currentEventHandler != NULL)
	{
		cadArea->replaceEventHandler(currentEventHandler, e);
	}
	else
	{
		cadArea->addEventHandler(e);
	}
	currentEventHandler = e;
}



GtkWidget* CadView::getWidget()
{
	return cadArea->getWidget();
}

CadArea* CadView::getCadArea()
{
	return cadArea;
}


void CadView::redraw()
{
	cadArea->redraw();
}





#endif
