/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __CADVIEW_H
#define __CADVIEW_H

#include <gtk/gtkwidget.h>

#include "CadArea.h"
#include "DAEH_clear.h"
#include "CAEH_grid.h"




class CadViewDragZoomEventHandler : public CAEventHandler
{
private:
	typedef enum {
		MODE_NONE,
		MODE_DRAG,
		MODE_ZOOM
	} Mode;

	Mode runningMode;
	IPoint2 previousPosition;
	double zoompp;
	
public:
	CadViewDragZoomEventHandler(CadArea *c);
	CadViewDragZoomEventHandler(CadArea *c, double zoomPerPixel);
	
protected:
	virtual void evButtonDown(int button);
	virtual void evButtonUp(int button);
	//pointer movements
	virtual void evMotion(const IPoint2& p);
};


//CadView class
//encapsulates a CadArea, along with event handlers to clear the CadArea and draw
//the grid


class CadView
{
private:
	CadArea *cadArea;
	DAEH_clear *clearEventHandler;
	CAEH_grid *gridEventHandler;
	CadViewDragZoomEventHandler *dragZoomEventHandler;

	CAEventHandler *currentEventHandler;


public:
	CadView();
	virtual ~CadView();

	//add en event handler to the list
	void addEventHandler(CAEventHandler *e);
	//remove an event handler from the list
	void removeEventHandler(CAEventHandler *e);
	//replace the current event handler with e
	void newEventHandler(CAEventHandler *e);

	GtkWidget* getWidget();
	CadArea* getCadArea();

	void redraw();
};




#endif
