/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __DRAWINGAREA_CPP
#define __DRAWINGAREA_CPP

#include <iostream>

#include <gtk/gtkdrawingarea.h>
#include <gtk/gtksignal.h>

#include "DrawingArea.h"



/*
*******************************************************************************
												DAColour
*******************************************************************************
*/												

//default constructor
DAColour::DAColour()
{
	gdkColour.red = gdkColour.green = gdkColour.blue = 0;
	gdkColour.pixel = 0;						//set pixel value to 0
	allocatedFlag = false;					//not allocated
}

//construct from rgb components
DAColour::DAColour(int ir, int ig, int ib)
{
	gdkColour.red = ir;
	gdkColour.green = ig;
	gdkColour.blue = ib;
	gdkColour.pixel = 0;						//not allocated, so pixel value is unknown
	allocatedFlag = false;					//not allocated
}

//(private) construct from GdkColor
DAColour::DAColour(const GdkColor& c)
{
	gdkColour = c;								//assign colour object
	allocatedFlag = false;					//not allocated
}


//get red component
int DAColour::getRed() const
{
	return gdkColour.red;
}

//get green component
int DAColour::getGreen() const
{
	return gdkColour.green;
}

//get blue component
int DAColour::getBlue() const
{
	return gdkColour.blue;
}


GdkColor DAColour::getGdkColour() const
{
	return gdkColour;
}


//allocate the colour
bool DAColour::allocate()
{
	if (!allocatedFlag)						//only try to allocted if it hasn't already
	{												//been allocated
		//allocate using system colour map
		if (gdk_color_alloc(gdk_colormap_get_system(), &gdkColour))
		{
			allocatedFlag = true;			//successful : has been allocated
			return true;
		}
		else
		{
			allocatedFlag = false;			//unsuccessful : has not been allocated
			return false;
		}
	}
	else
	{
		return true;
	}
}

//been allocated?
bool DAColour::isAllocated() const
{
	return allocatedFlag;
}





/*
*******************************************************************************
										EventMouseButton
*******************************************************************************
*/

EventMouseButton::EventMouseButton() : Event()
{
}

EventMouseButton::EventMouseButton(const string& nm, int bt) : Event(nm)
{
	button = bt;
}


int EventMouseButton::getButton() const
{
	return button;
}


/*
*******************************************************************************
										EventMouseButton
*******************************************************************************
*/

EventMouseMotion::EventMouseMotion() : Event()
{
}

EventMouseMotion::EventMouseMotion(const string& nm, const IPoint2& pos)
											 : Event(nm)
{
	position = pos;
}


IPoint2 EventMouseMotion::getPosition() const
{
	return position;
}


/*
*******************************************************************************
											EventKey
*******************************************************************************
*/

EventKey::EventKey() : Event()
{
}

EventKey::EventKey(const string& nm, int keyValue) : Event(nm)
{
	keyVal = keyValue;
}


int EventKey::getKeyValue() const
{
	return keyVal;
}


/*
*******************************************************************************
											EventConfig
*******************************************************************************
*/

EventConfig::EventConfig()
								: Event()
{
}

EventConfig::EventConfig(const string& nm, const ISize2& sz)
								: Event(nm)
{
	size = sz;
}


ISize2 EventConfig::getSize() const
{
	return size;
}

/*
*******************************************************************************
											DAEventHandler
*******************************************************************************
*/

//default constructor
DAEventHandler::DAEventHandler(DrawingArea* drawArea) : EventHandler()
{
	drawingArea = drawArea;
}

//the event response functions
void DAEventHandler::paint()
{
}

void DAEventHandler::evButtonDown(int button)
{
}

void DAEventHandler::evButtonUp(int button)
{
}

void DAEventHandler::evMotion(const IPoint2& p)
{
}

void DAEventHandler::evEnter(const IPoint2& p)
{
}

void DAEventHandler::evLeave(const IPoint2& p)
{
}

void DAEventHandler::evKeyPress(int keyValue)
{
}

void DAEventHandler::evKeyRelease(int keyValue)
{
}

void DAEventHandler::evConfig(const ISize2& s)
{
}



void DAEventHandler::redirectEvent(Event *ev)
{
	if (ev->getName() == "paint")
	{
		paint();
	}
	else if (ev->getName() == "button_down")
	{
		evButtonDown( ((EventMouseButton*)ev)->getButton() );
	}
	else if (ev->getName() == "button_up")
	{
		evButtonUp( ((EventMouseButton*)ev)->getButton() );
	}
	else if (ev->getName() == "mouse_motion")
	{
		evMotion( ((EventMouseMotion*)ev)->getPosition() );
	}
	else if (ev->getName() == "mouse_enter")
	{
		evEnter( ((EventMouseMotion*)ev)->getPosition() );
	}
	else if (ev->getName() == "mouse_leave")
	{
		evLeave( ((EventMouseMotion*)ev)->getPosition() );
	}
	else if (ev->getName() == "key_press")
	{
		evKeyPress( ((EventKey*)ev)->getKeyValue() );
	}
	else if (ev->getName() == "key_release")
	{
		evKeyRelease( ((EventKey*)ev)->getKeyValue() );
	}
	else if (ev->getName() == "configure")
	{
		evConfig( ((EventConfig*)ev)->getSize() );
	}	
}



/*
*******************************************************************************
*/


//constructor
DrawingArea::DrawingArea()
{
	//set the foreground and background colours
	foreground.red = foreground.green = foreground.blue = 0xffff;
	background.red = background.green = background.blue = 0x0;

	//allocate them
	gdk_color_alloc(gdk_colormap_get_system(), &foreground);
	gdk_color_alloc(gdk_colormap_get_system(), &background);

	//create a new GTK drawing area widget
	gtkDrawingAreaWidget = gtk_drawing_area_new();

	//connect the desired signals
	gtk_signal_connect( GTK_OBJECT(gtkDrawingAreaWidget), "configure_event",
							  GTK_SIGNAL_FUNC(configure_event), this);
	gtk_signal_connect( GTK_OBJECT(gtkDrawingAreaWidget), "expose_event",
							  GTK_SIGNAL_FUNC(expose_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "button_press_event",
							  GTK_SIGNAL_FUNC(button_press_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "button_release_event",
							  GTK_SIGNAL_FUNC(button_release_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "motion_notify_event",
							  GTK_SIGNAL_FUNC(motion_notify_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "key_press_event",
							  GTK_SIGNAL_FUNC(key_press_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "key_release_event",
							  GTK_SIGNAL_FUNC(key_release_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "enter_notify_event",
							  GTK_SIGNAL_FUNC(enter_notify_event), this);
	gtk_signal_connect( GTK_OBJECT (gtkDrawingAreaWidget), "leave_notify_event",
							  GTK_SIGNAL_FUNC(leave_notify_event), this);

	//make sure the event get received
	gtk_widget_set_events( gtkDrawingAreaWidget,
								  GDK_EXPOSURE_MASK |
								  GDK_LEAVE_NOTIFY_MASK |
								  GDK_BUTTON_PRESS_MASK |
								  GDK_BUTTON_RELEASE_MASK |
								  GDK_POINTER_MOTION_MASK |
								  GDK_POINTER_MOTION_HINT_MASK |
								  GDK_KEY_PRESS_MASK |
								  GDK_KEY_RELEASE_MASK |
								  GDK_ENTER_NOTIFY_MASK |
								  GDK_LEAVE_NOTIFY_MASK );

	//add a reference count
	gtk_widget_ref(gtkDrawingAreaWidget);

	//assign NULL to the pixmaps, the GC, and the event handler
	backingPixmap = NULL;
	snapPixmap = NULL;
	drawingAreaGC = NULL;

	eventHandlersActiveFlag = true;
}

//destructor
DrawingArea::~DrawingArea()
{
	//kill all existing objects
	if (backingPixmap != NULL)
	{
		gdk_pixmap_unref(backingPixmap);
	}
	if (snapPixmap != NULL)
	{
		gdk_pixmap_unref(snapPixmap);
	}
	if (drawingAreaGC != NULL)
	{
		gdk_gc_unref(drawingAreaGC);
	}

	//unref the widget
	gtk_widget_unref(gtkDrawingAreaWidget);
}




//respond to a configure event
gint DrawingArea::configure_event(GtkWidget *widget, GdkEventConfigure *event,
											 DrawingArea *klass)
{
	//get the widget state, so we know which GC's to use
	int index = GTK_WIDGET_STATE(widget);

	//set the DrawingArea's size
	klass->drawingAreaSize =
		ISize2(widget->allocation.width, widget->allocation.height);

	//kill off all the old pixmaps and GCs
	if (klass->backingPixmap != NULL)
	{
		gdk_pixmap_unref(klass->backingPixmap);
	}
	if (klass->snapPixmap != NULL)
	{
		gdk_pixmap_unref(klass->snapPixmap);
	}

	if (klass->drawingAreaGC != NULL)
	{
		gdk_gc_unref(klass->drawingAreaGC);
	}

	//make new pixmaps
	klass->backingPixmap = gdk_pixmap_new(widget->window,
													  klass->drawingAreaSize.w,
													  klass->drawingAreaSize.h, -1);
	klass->snapPixmap = gdk_pixmap_new(widget->window,
												  klass->drawingAreaSize.w,
												  klass->drawingAreaSize.h, -1);

	//make a new GC
	klass->drawingAreaGC = gdk_gc_new(widget->window);
	//copy the foreground GC of the drawingArea to the new GC
	gdk_gc_copy(klass->drawingAreaGC, widget->style->fg_gc[index]);

	//set the foreground and background colours of the new GC
	gdk_gc_set_foreground(klass->drawingAreaGC, &(klass->foreground));
	gdk_gc_set_background(klass->drawingAreaGC, &(klass->background));

	//generate the events
	klass->ev_config(klass->drawingAreaSize);
	EventConfig ev("configure", klass->drawingAreaSize);
	klass->event(&ev);

	//a redraw will be required
	klass->redraw();

	return TRUE;
}

//respond to an exposure event
gint DrawingArea::expose_event(GtkWidget *widget, GdkEventExpose *event,
										 DrawingArea *klass)
{
	//if the backing pixmap exists
	if (klass->backingPixmap != NULL)
	{
		//copy the appropriate portion to the widget
		gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
							 klass->backingPixmap, event->area.x, event->area.y,
							 event->area.x, event->area.y,
							 event->area.width, event->area.height);
	}
	
	return FALSE;
}

//respond to button press event
gint DrawingArea::button_press_event(GtkWidget *widget, GdkEventButton *event,
												 DrawingArea *klass)
{
	//set the appropriate button flag in the pointer status
	klass->pointerStatus.setButton(event->button);
	klass->pointerStatus.setModifierKeyStates( event->state );
	
	//generate events
	klass->ev_buttonDown(event->button);
	EventMouseButton ev("button_down", event->button);
	klass->event(&ev);
	
	return true;
}

//respond to button release event
gint DrawingArea::button_release_event(GtkWidget *widget, GdkEventButton *event,
													DrawingArea *klass)
{
	//clear the appropriate button flag in the pointer status
	klass->pointerStatus.clearButton(event->button);
	klass->pointerStatus.setModifierKeyStates( event->state );
	
	//generate events
	klass->ev_buttonUp(event->button);
	EventMouseButton ev("button_up", event->button);
	klass->event(&ev);
	
	return true;
}

//respond to pointer motion event
gint DrawingArea::motion_notify_event(GtkWidget *widget, GdkEventMotion *event,
												  DrawingArea *klass)
{
	int x, y;
	GdkModifierType state;

	//get the pointer position and set the pointer status position
	if (event->is_hint)
	{
		gdk_window_get_pointer(event->window, &x, &y, &state);
		klass->pointerStatus.setPosition( IPoint2(x, klass->drawingAreaSize.h - y) );
	}
	else
	{
		klass->pointerStatus.setPosition(
			IPoint2( (int)event->x, klass->drawingAreaSize.h - (int)event->y ) );
	}
	klass->pointerStatus.setModifierKeyStates( event->state );

	//generate events
	klass->ev_motion(klass->pointerStatus.getPosition());
	EventMouseMotion ev("mouse_motion", klass->pointerStatus.getPosition());
	klass->event(&ev);

	return true;
}

gint DrawingArea::enter_notify_event(GtkWidget *widget, GdkEventCrossing *event,
												 DrawingArea *klass)
{
	PointerStatus data;
	int x, y;
	GdkModifierType state;

	//get the pointer position
	gdk_window_get_pointer(event->window, &x, &y, &state);

	//set the pointer status position, and set the inside flag
	klass->pointerStatus.setPosition( IPoint2(x, klass->drawingAreaSize.h - y) );
	klass->pointerStatus.setInside();

	//generate events
	klass->ev_enter(klass->pointerStatus.getPosition());
	EventMouseMotion ev("mouse_enter", klass->pointerStatus.getPosition());
	klass->event(&ev);

	return true;
}

gint DrawingArea::leave_notify_event(GtkWidget *widget, GdkEventCrossing *event,
												 DrawingArea *klass)
{
	PointerStatus data;
	int x, y;
	GdkModifierType state;

	//get the pointer position
	gdk_window_get_pointer(event->window, &x, &y, &state);

	//set the pointer status position, and clear the inside flag
	klass->pointerStatus.setPosition( IPoint2(x, klass->drawingAreaSize.h - y) );
	klass->pointerStatus.clearInside();

	//generate events
	klass->ev_leave(klass->pointerStatus.getPosition());
	EventMouseMotion ev("mouse_leave", klass->pointerStatus.getPosition());
	klass->event(&ev);

	return true;
}

gint DrawingArea::key_press_event(GtkWidget *widget, GdkEventKey *event,
											 DrawingArea *klass)
{
	//generate events
	klass->ev_keyPress(event->keyval);
	EventKey ev("key_press", event->keyval);
	klass->event(&ev);
	
	return true;
}

gint DrawingArea::key_release_event(GtkWidget *widget, GdkEventKey *event,
												DrawingArea *klass)
{
	//generate events
	klass->ev_keyRelease(event->keyval);
	EventKey ev("key_release", event->keyval);
	klass->event(&ev);
	
	return true;
}


void DrawingArea::event(Event *ev)
{
	if (eventHandlersActiveFlag)
	{
		for (list<DAEventHandler*>::iterator iter = eventHandlers.begin();
			  iter != eventHandlers.end();
			  ++iter)
		{
			if ( (*iter)->isActive() )
			{
				(*iter)->event(ev);
			}
		}
	}
}


//DrawingArea event functions
void DrawingArea::ev_paint()
{
}

void DrawingArea::ev_buttonDown(int button)
{
}

void DrawingArea::ev_buttonUp(int button)
{
}

void DrawingArea::ev_motion(const IPoint2& p)
{
}

void DrawingArea::ev_enter(const IPoint2& p)
{
}

void DrawingArea::ev_leave(const IPoint2& p)
{
}

void DrawingArea::ev_keyPress(int keyValue)
{
}

void DrawingArea::ev_keyRelease(int keyValue)
{
}

void DrawingArea::ev_config(const ISize2& sz)
{
}




//get the GTK widget
GtkWidget *DrawingArea::getWidget()
{
	return gtkDrawingAreaWidget;
}

//get the size of the drawing area
ISize2 DrawingArea::getSize() const
{
	return drawingAreaSize;
}

//get the pointer status
const PointerStatus& DrawingArea::getPointerStatus() const
{
	return pointerStatus;
}


//manipulate the event handler list
void DrawingArea::addEventHandler(DAEventHandler *h)
{
	eventHandlers.push_back(h);
}

void DrawingArea::removeEventHandler(DAEventHandler *h)
{
	eventHandlers.remove(h);
}

bool DrawingArea::replaceEventHandler(DAEventHandler *old, DAEventHandler *h)
{
	for (list<DAEventHandler*>::iterator iter = eventHandlers.begin();
		  iter != eventHandlers.end();
		  ++iter)
	{
		if (*iter == old)
		{
			*iter = h;
			return true;
		}
	}
	return false;
}

void DrawingArea::activateEventHandlers()
{
	eventHandlersActiveFlag = true;
}

void DrawingArea::deactivateEventHandlers()
{
	eventHandlersActiveFlag = false;
}

bool DrawingArea::isActiveEventHandlers() const
{
	return eventHandlersActiveFlag;
}




//redraw request
void DrawingArea::redraw()
{
	//generate events
	ev_paint();
	if (eventHandlersActiveFlag)
	{
		Event ev("paint");
		list<DAEventHandler*>::iterator iter;

		for (iter = eventHandlers.begin(); iter != eventHandlers.end(); ++iter)
		{
			DAEventHandler *h;
			h = (*iter);
			if ( h->isActive() )
			{
				h->event(&ev);
			}
		}
	}
	//copy the contents of the backing pixmap to the drawing area
	gdk_draw_pixmap(gtkDrawingAreaWidget->window,
						 gtkDrawingAreaWidget->style->
						 		fg_gc[GTK_WIDGET_STATE(gtkDrawingAreaWidget)],
						 backingPixmap, 0, 0, 0, 0,
						 drawingAreaSize.w, drawingAreaSize.h);
}

//set the foreground colour
void DrawingArea::setForeground(DAColour& fore)
{
	//if the new colour is the same as the current one, ignore the request
	if ( (foreground.red == fore.getRed()) &&
		  (foreground.green == fore.getGreen()) &&
		  (foreground.blue == fore.getBlue()) )
	{
		return;
	}
	//make sure the colour has been allocated
	if (!fore.isAllocated())
	{
		if (!fore.allocate())
		{
			return;
		}
	}
	//set this' foreground colour value
	foreground = fore.getGdkColour();
	//set the GC's foreground colour
	if (drawingAreaGC != NULL)
	{
		gdk_gc_set_foreground(drawingAreaGC, &foreground);
	}
}	

//set the background colour
void DrawingArea::setBackground(DAColour& back)
{
	//if the new colour is the same as the current one, ignore the request
	if ( (background.red == back.getRed()) &&
		  (background.green == back.getGreen()) &&
		  (background.blue == back.getBlue()) )
	{
		return;
	}
	//make sure the colour has been allocated
	if (!back.isAllocated())
	{
		if (!back.allocate())
		{
			return;
		}		
	}
	//set this' background colour value
	background = back.getGdkColour();
	//set the GC's background colour
	if (drawingAreaGC != NULL)
	{
		gdk_gc_set_background(drawingAreaGC, &background);
	}
}

//clear the drawing area to the background colour
void DrawingArea::drawClear()
{
	//set the GC's foreground colour the the background colour
	gdk_gc_set_foreground(drawingAreaGC, &background);
	//draw a rectangle covering the drawind area
	gdk_draw_rectangle(backingPixmap, drawingAreaGC, (gint)true, 0, 0,
							 drawingAreaSize.w, drawingAreaSize.h);
	//set the GC's foregroudn colour back to the foreground colour
	gdk_gc_set_foreground(drawingAreaGC, &foreground);
}

//draw a dot
void DrawingArea::drawDot(const IPoint2& p)
{
	gdk_draw_point(backingPixmap, drawingAreaGC, p.x, drawingAreaSize.h - p.y);
}

//draw a line
void DrawingArea::drawLine(const ILine2& l)
{
	gdk_draw_line(backingPixmap, drawingAreaGC,
					  l.p1.x, drawingAreaSize.h - l.p1.y,
					  l.p2.x, drawingAreaSize.h - l.p2.y);
}

//draw rectangle
void DrawingArea::drawRectangle(const IRect2& r, bool filled)
{
	gdk_draw_rectangle(backingPixmap, drawingAreaGC, (gint)filled,
							 r.x, (drawingAreaSize.h - r.y) - r.h, r.w, r.h);
}

//draw an arc
void DrawingArea::drawArc(const IPoint2& centre, const ISize2& s, int a1, int a2,
								  bool filled)
{
	gdk_draw_arc(backingPixmap, drawingAreaGC, (gint)filled,
					 centre.x, (drawingAreaSize.h - centre.y) - s.h, s.w, s.h, a1, a2);
}

//draw a string
void DrawingArea::drawString(const IPoint2& p, const char *str)
{
	gdk_draw_string(backingPixmap, gtkDrawingAreaWidget->style->font,
						 drawingAreaGC, p.x, drawingAreaSize.h - p.y, str);
}

//get the foreground colour
DAColour DrawingArea::getForeground() const
{
	return DAColour(foreground);
}

//get the background colour
DAColour DrawingArea::getBackground() const
{
	return DAColour(background);
}

//take a snapshot
void DrawingArea::takeSnapshot()
{
	//draw the contents of the backing pixmap onto the snapshot pixmap
	gdk_draw_pixmap(snapPixmap, drawingAreaGC, backingPixmap,
						 0, 0, 0, 0,
						 drawingAreaSize.w, drawingAreaSize.h);
}

//draw the snapshot
void DrawingArea::drawSnap(const IRect2& r)
{
	//draw the contenst of the snapshot pixmap onto the backing pixmap
	gdk_draw_pixmap(backingPixmap, drawingAreaGC, snapPixmap,
						 r.x, drawingAreaSize.h - r.y,
						 r.x, drawingAreaSize.h - r.y,
						 r.w, r.h);
}

void DrawingArea::drawSnap()
{
	drawSnap( IRect2( IPoint2(0, 0), getSize() ) );
}


#endif
