/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef POINTERSTATUSCC
#define POINTERSTATUSCC

#include <gdk/gdk.h>

#include "PointerStatus.h"

/*
*******************************************************************************
*/


//default constructor
PointerStatus::PointerStatus()
{
	//set position to 0, button flags to false, and inside flags to true
	pos = IPoint2();
	buttons = array<bool>();
	buttons.resize(8);
	in = true;
}

//take values as parameters
PointerStatus::PointerStatus(const IPoint2& position, int nb, bool inside)
{
	pos = position;
	in = inside;
	buttons = array<bool>(nb);
}

PointerStatus::PointerStatus(const IPoint2& position, const array<bool>& bt, bool inside)
{
	pos = position;
	in = inside;
	buttons = bt;
}

//set position
void PointerStatus::setPosition(const IPoint2& position)
{
	pos = position;
}

//set button
void PointerStatus::setButton(int bt)
{
	buttons[bt] = true;
}

//clear button
void PointerStatus::clearButton(int bt)
{
	buttons[bt] = false;
}

//set the inside bounds flag
void PointerStatus::setInside()
{
	in = true;
}

//clear the inside bounds flag
void PointerStatus::clearInside()
{
	in = false;
}

//set modifier key states
void PointerStatus::setModifierKeyStates(int s)
{
	modifierKeyStates = s;
}



//get position
IPoint2 PointerStatus::getPosition() const
{
	return pos;
}

//get button status
bool PointerStatus::getButton(int bt) const
{
	return buttons[bt];
}

//get inside status
bool PointerStatus::isInside() const
{
	return in;
}

//get modifier key states
int PointerStatus::getModifierKeyStates() const
{
	return modifierKeyStates;
}

//get status of shift key
bool PointerStatus::isShift() const
{
	return (modifierKeyStates & GDK_SHIFT_MASK) != 0;
}

//get status of control key
bool PointerStatus::isControl() const
{
	return (modifierKeyStates & GDK_CONTROL_MASK) != 0;
}

//get status of alt key
bool PointerStatus::isAlt() const
{
	return (modifierKeyStates & GDK_MOD1_MASK) != 0;
}



#endif
