/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __ATTRACTORTRANSFORMATION_CPP
#define __ATTRACTORTRANSFORMATION_CPP

#include "AttractorTransformation.h"



AttractorTransformation::AttractorTransformation() : Transformation()
{
	strength = 1.0;
	sharpness = 0.05;
	centre = Point3();
	orig = Point3();
	iterations = 1;
	in3D = false;
}

AttractorTransformation::AttractorTransformation(double str, double sh,
																 const Point3& c,
																 int it, bool td)
																 : Transformation()
{
	strength = str;
	sharpness = sh;
	centre = c;
	iterations = it;
	orig = xfPoint( Point3() );
	in3D = td;
}


AttractorTransformation::AttractorTransformation(double str, double sh,
																 const Point3& c,
																 int it, bool td,
																 const Point3& o,
																 const Transformation *x)
																 : Transformation(x)
{
	strength = str;
	sharpness = sh;
	centre = c;
	iterations = it;
	in3D = td;
	orig = o;
}



Transformation* AttractorTransformation::clone() const
{
	return new AttractorTransformation( strength, sharpness, centre,
													iterations, in3D, orig, this );
}


Point3 AttractorTransformation::xfPoint(const Point3& p) const
{
	Point3 x = p;
	double distance, parameter, force;
	Vector3 v;

	for (int i = 0; i < iterations; i++)
	{
		v = x - centre;

		if (!in3D)
		{
			v.z = 0.0;
		}

		distance = v.length();
	
		parameter = (strength * sharpness) + distance;

		if (parameter != 0.0)
		{
			force = strength / parameter;

			x = x + movement(force, distance, v);
		}
	}

	return x;
}

Vector3 AttractorTransformation::xfVector(const Vector3& v, const Point3& at) const
{
	//no need to subtract the result of transforming the origin, because
	//the origin remains unchanged
	return xfPoint( Point3(v) ) - orig;
}

Vector3 AttractorTransformation::movement(double force, double dist,
														const Vector3& centreToPoint) const
{
	double fraction = (dist - force) / dist;
	if (fraction < 0.0)
	{
		fraction = 0.0;
	}

	return -centreToPoint * (1.0 - fraction);
}



#endif

