/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __BEND3DTRANSFORMATION_CPP
#define __BEND3DTRANSFORMATION_CPP

#include "Bend3DTransformation.h"

#include <math.h>

#include "Matrix4.h"



Bend3DTransformation::Bend3DTransformation() : Transformation()
{
	grad = 1.0;
}

Bend3DTransformation::Bend3DTransformation(double dAdD)
														 : Transformation()
{
	grad = dAdD;
}



Bend3DTransformation::Bend3DTransformation(double dAdD,
														 const Transformation *x)
														 : Transformation(x)
{
	grad = dAdD;
}



Transformation* Bend3DTransformation::clone() const
{
	return new Bend3DTransformation( grad, this );
}


Point3 Bend3DTransformation::xfPoint(const Point3& p) const
{
	if ( p.x == 0 && p.y == 0 )
	{
		return p;
	}
	
	double zPlaneDist = sqrt( p.x * p.x + p.y * p.y );
	double oneOverZPlaneDist = 1.0 / zPlaneDist;
	double xFraction = p.x * oneOverZPlaneDist;
	double yFraction = p.y * oneOverZPlaneDist;
	
	double angle = zPlaneDist * grad;
	double radius = p.z;

	return Point3( sin(angle) * radius * xFraction,
						sin(angle) * radius * yFraction,
						cos(angle) * radius );
}

Vector3 Bend3DTransformation::xfVector(const Vector3& v, const Point3& at) const
{
	if ( at.x == 0 && at.y == 0 )
	{
		return v;
	}

	double zPlaneDist = sqrt( at.x * at.x + at.y * at.y );
	double oneOverZPlaneDist = 1.0 / zPlaneDist;
	double xFraction = at.x * oneOverZPlaneDist;
	double yFraction = at.y * oneOverZPlaneDist;
	
	double angle = zPlaneDist * grad;


	Vector3 rotationAxis = Vector3(xFraction, yFraction, 0.0) *
								  Matrix4_rotateZ( M_PI * 0.5 );
	Matrix4 rotation = Matrix4_rotate( rotationAxis, -angle );
	
	return v * rotation;	
}



#endif
