/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __BENDTRANSFORMATION_CPP
#define __BENDTRANSFORMATION_CPP

#include "BendTransformation.h"

#include "Matrix4.h"



BendTransformation::BendTransformation() : Transformation()
{
	grad = 1.0;
}

BendTransformation::BendTransformation(double dAdX)
													: Transformation()
{
	grad = dAdX;
}



BendTransformation::BendTransformation(double dAdX, const Transformation *x)
													: Transformation(x)
{
	grad = dAdX;
}



Transformation* BendTransformation::clone() const
{
	return new BendTransformation( grad, this );
}


Point3 BendTransformation::xfPoint(const Point3& p) const
{
	double angle = p.z * grad;
	double radius = p.x;

	return Point3( cos(angle) * radius, p.y, sin(angle) * radius );
}

Vector3 BendTransformation::xfVector(const Vector3& v, const Point3& at) const
{
	double angle = at.z * grad;

	Matrix4 rotation = Matrix4_rotateY(-angle);

	return v * rotation;	
}



#endif
