/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TAPERTRANSFORMATION_CPP
#define __TAPERTRANSFORMATION_CPP

#include "TaperTransformation.h"



TaperTransformation::TaperTransformation() : Transformation()
{
	zeroU = 1.0;
	zeroV = 1.0;
	dUdD = 0.0;
	dVdD = 0.0;
}

TaperTransformation::TaperTransformation(double d0, double u0, double v0,
													  double d1, double u1, double v1)
													  : Transformation()
{
	double oneOverDD = 1.0 / (d1 - d0);

	dUdD = (u1 - u0) * oneOverDD;
	dVdD = (v1 - v0) * oneOverDD;

	zeroU = u0 - (d0 * dUdD);
	zeroV = v0 - (d0 * dVdD);
}


TaperTransformation::TaperTransformation(double zu, double gu,
													  double zv, double gv,
													  const Transformation *x)
													  : Transformation(x)
{
	zeroU = zu;
	dUdD = gu;
	zeroV = zv;
	dVdD = gv;
}



Transformation* TaperTransformation::clone() const
{
	return new TaperTransformation( zeroU, dUdD,
											  zeroV, dVdD, this );
}


Point3 TaperTransformation::xfPoint(const Point3& p) const
{
	double su = (p.z * dUdD) + zeroU;
	double sv = (p.z * dVdD) + zeroV;
	Matrix4 sc = Matrix4_scale( Vector3( su, sv, 1.0 ) );
	return (p * sc);
}

Vector3 TaperTransformation::xfVector(const Vector3& v, const Point3& at) const
{
	double su = (at.z * dUdD) + zeroU;
	double sv = (at.z * dVdD) + zeroV;
	Matrix4 sc = Matrix4_scale( Vector3( su, sv, 1.0 ) );
	return (v * sc).getNormalised();
}



#endif

