/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TRANSFORMATION_CPP
#define __TRANSFORMATION_CPP

#include "Transformation.h"


#ifndef NULL
#define NULL 0
#endif



/*
*******************************************************************************
										Transformation
*******************************************************************************
*/

Transformation::Transformation()
{
	child = NULL;
}

Transformation::Transformation(const Transformation *c)
{
	if (c->child == NULL)
	{
		child = NULL;
	}
	else
	{
		child = c->child->clone();
	}
}

Transformation::~Transformation()
{
	if (child != NULL)
	{
		delete child;
	}
}


Point3 Transformation::transformPoint(const Point3& p) const
{
	if (child != NULL)
	{
		return child->transformPoint( xfPoint(p) );
	}
	else
	{
		return xfPoint(p);
	}
}

Vector3 Transformation::transformVector(const Vector3& v, const Point3& at) const
{
	if (child != NULL)
	{
		return child->transformVector( xfVector(v, at), at );
	}
	else
	{
		return xfVector(v, at);
	}
}


void Transformation::setChild(const Transformation *x)
{
	if (child != NULL)
	{
		delete child;
	}
	
	child = x->clone();
}

Transformation* Transformation::clone() const
{
	return new Transformation(this);
}


Point3 Transformation::xfPoint(const Point3& p) const
{
	return p;
}

Vector3 Transformation::xfVector(const Vector3& v, const Point3& at) const
{
	return v;
}





#endif
