/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TWISTTRANSFORMATION_CPP
#define __TWISTTRANSFORMATION_CPP

#include "TwistTransformation.h"



TwistTransformation::TwistTransformation() : Transformation()
{
	zeroR = 1.0;
	dRdD = 0.0;
	type = LINEAR;
}

TwistTransformation::TwistTransformation(double d0, double r0,
													  double d1, double r1,
													  TwistType t)
													  : Transformation()
{
	dRdD = (r1 - r0) / (d1 - d0);
	zeroR = r0 - (d0 * dRdD);
	type = t;
}

TwistTransformation::TwistTransformation(double zr, double g, TwistType t,
													  const Transformation *x)
													  : Transformation(x)
{
	zeroR = zr;
	dRdD = g;
	type = t;
}



Transformation* TwistTransformation::clone() const
{
	return new TwistTransformation(zeroR, dRdD, type, this);
}


Point3 TwistTransformation::xfPoint(const Point3& p) const
{
	if (type == LINEAR)
	{
		return p * Matrix4_rotate( Vector3(0.0, 0.0, 1.0),
											p.z * dRdD + zeroR );
	}
	else if (type == RADIAL)
	{
		return p * Matrix4_rotate( Vector3(0.0, 0.0, 1.0),
											abs_polar(p.x, p.y) * dRdD + zeroR );
	}
	else
	{
		return p;
	}
}

Vector3 TwistTransformation::xfVector(const Vector3& v, const Point3& at) const
{
	if (type == LINEAR)
	{
		return v * Matrix4_rotate( Vector3(0.0, 0.0, 1.0),
											at.z * dRdD + zeroR );
	}
	else if (type == RADIAL)
	{
		return v * Matrix4_rotate( Vector3(0.0, 0.0, 1.0),
											abs_polar(at.x, at.y) * dRdD + zeroR );
	}
	else
	{
		return v;
	}
}



#endif

