/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLGROUPXFORM_CPP
#define __TOOLGROUPXFORM_CPP

#include "ToolGroupXform.h"

#include <gtk/gtklabel.h>



ToolGroupXform::ToolGroupXform(ProcessToolManager *toolManager,
										 JournalEditor *journEditor,
										 CadView *view,
										 ObjPainterManager *painterManager)
										 : MToolGroup(toolManager)
{
	titleLabel = gtk_label_new("Transformations");
	gtk_widget_ref(titleLabel);
	gtk_widget_show(titleLabel);

	journalEditor = journEditor;



	xlatButton = addToolButton( "Translate", GTK_SIGNAL_FUNC(xlatClicked) );
	movetoButton = addToolButton( "Move To", GTK_SIGNAL_FUNC(movetoClicked) );
	rotButton = addToolButton( "Rotate", GTK_SIGNAL_FUNC(rotClicked) );
	snapButton = addToolButton( "Snap", GTK_SIGNAL_FUNC(snapClicked) );
	scaleButton = addToolButton( "Scale", GTK_SIGNAL_FUNC(scaleClicked) );
	scale3dButton = addToolButton( "Scale 3D", GTK_SIGNAL_FUNC(scale3dClicked) );
	shearButton = addToolButton( "Shear", GTK_SIGNAL_FUNC(shearClicked) );
	twistButton = addToolButton( "Twist", GTK_SIGNAL_FUNC(twistClicked) );
	taperButton = addToolButton( "Taper", GTK_SIGNAL_FUNC(taperClicked) );
	bendButton = addToolButton( "Bend", GTK_SIGNAL_FUNC(bendClicked) );
	bend3DButton = addToolButton( "Bend 3D", GTK_SIGNAL_FUNC(bend3DClicked) );
	magnetButton = addToolButton( "Magnet", GTK_SIGNAL_FUNC(magnetClicked) );

	toolXlat = new ToolXfTranslate( view, painterManager );
	toolMoveto = new ToolXfMoveTo( view, painterManager );
	toolRot = new ToolXfRotate( view, painterManager );
	toolSnap = new ToolXfSnap( view, painterManager );
	toolScale = new ToolXfScale( view, painterManager );
	toolScale3d = new ToolXfScale3D( view, painterManager );
	toolShear = new ToolXfShear( view, painterManager );
	toolTwist = new ToolXfTwist( view, painterManager );
	toolTaper = new ToolXfTaper( view, painterManager );
	toolBend = new ToolXfBend( view, painterManager );
	toolBend3D = new ToolXfBend3D( view, painterManager );
	toolMagnet = new ToolXfMagnet( view, painterManager );

	toolMan->addTool( toolXlat->getProcessType(), toolXlat );
	toolMan->addTool( toolMoveto->getProcessType(), toolMoveto );
	toolMan->addTool( toolRot->getProcessType(), toolRot );
	toolMan->addTool( toolSnap->getProcessType(), toolSnap );
	toolMan->addTool( toolScale->getProcessType(), toolScale );
	toolMan->addTool( toolScale3d->getProcessType(), toolScale3d );
	toolMan->addTool( toolShear->getProcessType(), toolShear );
	toolMan->addTool( toolTwist->getProcessType(), toolTwist );
	toolMan->addTool( toolTaper->getProcessType(), toolTaper );
	toolMan->addTool( toolBend->getProcessType(), toolBend );
	toolMan->addTool( toolBend3D->getProcessType(), toolBend3D );
	toolMan->addTool( toolMagnet->getProcessType(), toolMagnet );
}

ToolGroupXform::~ToolGroupXform()
{
	gtk_widget_unref(titleLabel);
	
	toolMan->eraseTool( toolXlat->getProcessType() );
	toolMan->eraseTool( toolMoveto->getProcessType() );
	toolMan->eraseTool( toolRot->getProcessType() );
	toolMan->eraseTool( toolSnap->getProcessType() );
	toolMan->eraseTool( toolScale->getProcessType() );
	toolMan->eraseTool( toolScale3d->getProcessType() );
	toolMan->eraseTool( toolShear->getProcessType() );
	toolMan->eraseTool( toolTwist->getProcessType() );
	toolMan->eraseTool( toolTaper->getProcessType() );
	toolMan->eraseTool( toolBend->getProcessType() );
	toolMan->eraseTool( toolBend3D->getProcessType() );
	toolMan->eraseTool( toolMagnet->getProcessType()) ;
	delete toolXlat;
	delete toolMoveto;
	delete toolRot;
	delete toolSnap;
	delete toolScale;
	delete toolScale3d;
	delete toolShear;
	delete toolTwist;
	delete toolTaper;
	delete toolBend;
	delete toolBend3D;
	delete toolMagnet;
}



void ToolGroupXform::addToLayout(GridLayout *layout)
{
	MToolGroup::addToLayout(layout);
	
	layout->addAsLine(titleLabel);
	layout->add(xlatButton);
	layout->add(movetoButton);
	layout->add(rotButton);
	layout->add(snapButton);
	layout->add(scaleButton);
	layout->add(scale3dButton);
	layout->add(shearButton);
	layout->add(twistButton);
	layout->add(taperButton);
	layout->add(bendButton);
	layout->add(bend3DButton);
	layout->add(magnetButton);
}




void ToolGroupXform::xlatClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfTranslate *p = new ProcXfTranslate();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::movetoClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfMoveTo *p = new ProcXfMoveTo();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::rotClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfRotate *p = new ProcXfRotate();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::snapClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfSnap *p = new ProcXfSnap();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::scaleClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfScale *p = new ProcXfScale();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::scale3dClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfScale3D *p = new ProcXfScale3D();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::shearClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfShear *p = new ProcXfShear();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::twistClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfTwist *p = new ProcXfTwist();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::taperClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfTaper *p = new ProcXfTaper();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::bendClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfBend *p = new ProcXfBend();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::bend3DClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfBend3D *p = new ProcXfBend3D();
	klass->journalEditor->addProcess(p);
	delete p;
}

void ToolGroupXform::magnetClicked(GtkWidget *widget, ToolGroupXform *klass)
{
	ProcXfMagnet *p = new ProcXfMagnet();
	klass->journalEditor->addProcess( p );
	delete p;
}



#endif
