/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFBEND_H
#define __TOOLXFBEND_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "ToolXformAxis.h"
#include "NumericEntry.h"



class ProcXfBend : public ProcXformAxis
{
private:
	double direction, bendAngle;
	
public:
	ProcXfBend();
	ProcXfBend(Axis a, double dir, double bend);
	virtual ~ProcXfBend();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setDirection(double dir);
	void setBendAngle(double bend);
	double getDirection() const;
	double getBendAngle() const;

protected:
	virtual void transformModel(JournalModel *model, const Point3& centre,
										 const Matrix4& axesMatrix,
										 const Matrix4& axesMatrixInverse,
										 const BBox& boundingBoxInTransformationSpace) const;
};



class ToolXfBend : public ToolXformAxis
{
private:
	GtkWidget *directionLabel;
	NumericEntry *directionEntry;

	GtkWidget *snapLabel;
	NumericEntry *snapEntry;
	GtkWidget *bendAngleLabel;
	NumericEntry *bendAngleEntry;

	GtkWidget *doneButton;


	ValueSnap snapper;


	double initialBendAngleValue;

public:
	ToolXfBend(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfBend();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	
private:
	static void directionChanged(NumericEntry *widget, ToolXfBend *klass);
	static void bendAngleChanged(NumericEntry *widget, ToolXfBend *klass);
	static void doneClicked(GtkWidget *widget, ToolXfBend *klass);
};



#endif
