/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFBEND3D_H
#define __TOOLXFBEND3D_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "ToolXformAxis.h"
#include "NumericEntry.h"



class ProcXfBend3D : public ProcXformAxis
{
private:
	double bendAngle;
	
public:
	ProcXfBend3D();
	ProcXfBend3D(Axis a, double bend);
	virtual ~ProcXfBend3D();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setBend3DAngle(double bend);
	double getBend3DAngle() const;

protected:
	virtual void transformModel(JournalModel *model, const Point3& centre,
										 const Matrix4& axesMatrix,
										 const Matrix4& axesMatrixInverse,
										 const BBox& boundingBoxInTransformationSpace) const;
};



class ToolXfBend3D : public ToolXformAxis
{
private:
	GtkWidget *snapLabel;
	NumericEntry *snapEntry;

	GtkWidget *bendAngleLabel;
	NumericEntry *bendAngleEntry;

	GtkWidget *doneButton;


	ValueSnap snapper;


	double initialBend3DAngleValue;

public:
	ToolXfBend3D(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfBend3D();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	
private:
	static void bendAngleChanged(NumericEntry *widget, ToolXfBend3D *klass);
	static void doneClicked(GtkWidget *widget, ToolXfBend3D *klass);
};



#endif
