/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFMAGNET_CPP
#define __TOOLXFMAGNET_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>


#include "JournalModel.h"
#include "Matrix4.h"

#include "MagnetTransformation.h"

#include "ToolXfMagnet.h"




void* __create_ProcXfMagnet()
{
	return (void*)( new ProcXfMagnet() );
}

ObjectType type_ProcXfMagnet( "ProcXfMagnet", "ProcXform",
										__create_ProcXfMagnet );






ProcXfMagnet::ProcXfMagnet() : ProcXform()
{
	magnetPosition = Point3();
	strength = 10.0;
	sharpness = 0.1;
	iterations = 1;
	in3dFlag = true;
}

ProcXfMagnet::ProcXfMagnet(const Point3& p, double str, double sharp, int iter,
									bool in3d)
									: ProcXform()
{
	magnetPosition = p;
	strength = str;
	sharpness = sharp;
	iterations = iter;
	in3dFlag = in3d;
}

ProcXfMagnet::~ProcXfMagnet()
{
}



void ProcXfMagnet::read(FILE *f)
{
	ProcXform::read( f );

	magnetPosition.read( f );
	fread( &strength, sizeof(double), 1, f );
	fread( &sharpness, sizeof(double), 1, f );
	fread( &iterations, sizeof(int), 1, f );
	fread( &in3dFlag, sizeof(bool), 1, f );
}

void ProcXfMagnet::write(FILE *f)
{
	ProcXform::write( f );

	magnetPosition.write( f );
	fwrite( &strength, sizeof(double), 1, f );
	fwrite( &sharpness, sizeof(double), 1, f );
	fwrite( &iterations, sizeof(int), 1, f );
	fwrite( &in3dFlag, sizeof(bool), 1, f );
}





string ProcXfMagnet::getDescription() const
{
	return "Magnet";
}

const ObjectType* ProcXfMagnet::getType() const
{
	return &type_ProcXfMagnet;
}

const ObjectType* ProcXfMagnet::getTypeStatic()
{
	return &type_ProcXfMagnet;
}





JournalProcess* ProcXfMagnet::clone() const
{
	return new ProcXfMagnet( magnetPosition, strength, sharpness, iterations,
									 in3dFlag );
}


void ProcXfMagnet::setPosition(const Point3& p)
{
	magnetPosition = p;
	modified();
}

void ProcXfMagnet::setStrength(double str)
{
	strength = str;
	modified();
}

void ProcXfMagnet::setSharpness(double sharp)
{
	sharpness = sharp;
	modified();
}

void ProcXfMagnet::setIterations(int iter)
{
	iterations = iter;
	modified();
}

void ProcXfMagnet::setIn3d(bool in3d)
{
	in3dFlag = in3d;
	modified();
}


const Point3& ProcXfMagnet::getPosition() const
{
	return magnetPosition;
}

double ProcXfMagnet::getStrength() const
{
	return strength;
}

double ProcXfMagnet::getSharpness() const
{
	return sharpness;
}

int ProcXfMagnet::getIterations() const
{
	return iterations;
}

bool ProcXfMagnet::getIn3d() const
{
	return in3dFlag;
}



void ProcXfMagnet::transformModel(JournalModel *model) const
{
	MagnetTransformation xform( strength, sharpness, magnetPosition,
										 iterations, in3dFlag );

	model->getModel()->transform( &xform );
}






ToolXfMagnet::ToolXfMagnet(CadView *v, ObjPainterManager *pman)
									: ToolXformClickAndDrag( v, pman, "Magnet",
																	 "Magnet", 1 )
{
	snapper = VectorSnap( Vector3() );


	//build the widgets

	strengthLabel = gtk_label_new( "Strength:" );
	gtk_widget_ref( strengthLabel );
	gtk_widget_show( strengthLabel );

	sharpnessLabel = gtk_label_new( "Sharpness:" );
	gtk_widget_ref( sharpnessLabel );
	gtk_widget_show( sharpnessLabel );

	iterationsLabel = gtk_label_new( "Iterations:" );
	gtk_widget_ref( iterationsLabel );
	gtk_widget_show( iterationsLabel );


	strengthEntry = new NumericEntry( 10.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.5, 4,
												 GTK_SIGNAL_FUNC( strengthChanged ), this );

	sharpnessEntry = new NumericEntry( 0.1, 0.0, 1000.0, 0.001, 0.05, 0.1, 4,
												  GTK_SIGNAL_FUNC( sharpnessChanged ), this );

	iterationsEntry = new NumericEntry( 10.0, 0, 1000.0, 1.0, 10.0, 0.5, 0,
													GTK_SIGNAL_FUNC( iterationsChanged ), this );
	
	gtk_widget_show( strengthEntry->getWidget() );
	gtk_widget_show( sharpnessEntry->getWidget() );
	gtk_widget_show( iterationsEntry->getWidget() );

	


	//snap
	snapLabel = gtk_label_new("snap to grid:");
	gtk_widget_ref(snapLabel);
	gtk_widget_show(snapLabel);
	xSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	ySnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	zSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	gtk_widget_show( xSnapEntry->getWidget() );
	gtk_widget_show( ySnapEntry->getWidget() );
	gtk_widget_show( zSnapEntry->getWidget() );



	//translation
	posLabel = gtk_label_new( "position:" );
	gtk_widget_ref( posLabel );
	gtk_widget_show( posLabel );
	xPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	yPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	zPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	gtk_widget_show( xPosEntry->getWidget() );
	gtk_widget_show( yPosEntry->getWidget() );
	gtk_widget_show( zPosEntry->getWidget() );


	//done button
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);



	//connect done button signal
	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
	
		
	
	//add to the layout
	layout.add( strengthLabel );
	layout.add( strengthEntry->getWidget() );
	layout.newLine();
	layout.add( sharpnessLabel );
	layout.add( sharpnessEntry->getWidget() );
	layout.newLine();
	layout.add( iterationsLabel );
	layout.add( iterationsEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(snapLabel);
	layout.add( xSnapEntry->getWidget() );
	layout.add( ySnapEntry->getWidget() );
	layout.add( zSnapEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine( posLabel );
	layout.add( xPosEntry->getWidget() );
	layout.add( yPosEntry->getWidget() );
	layout.add( zPosEntry->getWidget() );
	layout.lineSpace(1);
	layout.add(doneButton);
	layout.newLine();
}

ToolXfMagnet::~ToolXfMagnet()
{
	//delete vector widgets
	delete strengthEntry;
	delete sharpnessEntry;
	delete iterationsEntry;
	delete xSnapEntry;
	delete ySnapEntry;
	delete zSnapEntry;
	delete xPosEntry;
	delete yPosEntry;
	delete zPosEntry;

	//unref widgets
	gtk_widget_unref( strengthLabel );
	gtk_widget_unref( sharpnessLabel );
	gtk_widget_unref( iterationsLabel );
	gtk_widget_unref( snapLabel );
	gtk_widget_unref( posLabel );
	gtk_widget_unref( doneButton );
}



const ObjectType* ToolXfMagnet::getProcessType() const
{
	return ProcXfMagnet::getTypeStatic();
}



void ToolXfMagnet::initialise()
{
	ToolXformClickAndDrag::initialise();

	if ( hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)journalProcess;

		Point3 position = proc->getPosition();
		xPosEntry->setValue( position.x );
		yPosEntry->setValue( position.y );
		zPosEntry->setValue( position.z );

		double strength = proc->getStrength();
		double sharpness = proc->getSharpness();
		int iterations = proc->getIterations();

		strengthEntry->setValue( strength );
		sharpnessEntry->setValue( sharpness );
		iterationsEntry->setValue( iterations );
	}
}


void ToolXfMagnet::start()
{
	ToolXformClickAndDrag::start();

	if ( hasProcess() )
	{
		//get and store the initial translation
		ProcXfMagnet *proc = (ProcXfMagnet*)journalProcess;
		initialPosition = proc->getPosition();

		Vector3 snapGranularity = Vector3( xSnapEntry->getValue(),
													  ySnapEntry->getValue(),
													  zSnapEntry->getValue() );
					
		snapper = VectorSnap( snapGranularity );
	}
}

void ToolXfMagnet::finish(bool terminated)
{
	ToolXformClickAndDrag::finish(terminated);
	
	if ( hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)journalProcess;
		if ( terminated )
		{
			//set back to the initial translation
			proc->setPosition( initialPosition );
			xPosEntry->setValue( initialPosition.x );
			yPosEntry->setValue( initialPosition.y );
			zPosEntry->setValue( initialPosition.z );
			modified();
		}
	}
}
		


void ToolXfMagnet::function()
{
	if ( hasProcess() )
	{
		ToolXformClickAndDrag::function();
	
		ProcXfMagnet *proc = (ProcXfMagnet*)journalProcess;

/*		Vector3 translation = currentCursorPosition.getWorld() -
									 startCursorPosition.getWorld();


		if (!moveXFlag)
		{
			translation.x = 0.0;
		}
		if (!moveYFlag)
		{
			translation.y = 0.0;
		}
		if (!moveZFlag)
		{
			translation.z = 0.0;
		}

		Point3 newPosition = initialPosition + translation;*/


		Point3 newPosition = currentCursorPosition.getWorld();

		newPosition = snapper.snap(newPosition);

		xPosEntry->setValue( newPosition.x );
		yPosEntry->setValue( newPosition.y );
		zPosEntry->setValue( newPosition.z );

		proc->setPosition(newPosition);

	
		modified();
	}
}



void ToolXfMagnet::positionChanged(NumericEntry *widget, ToolXfMagnet *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)klass->journalProcess;
		Point3 position = Vector3( klass->xPosEntry->getValue(),
											klass->yPosEntry->getValue(),
											klass->zPosEntry->getValue() );
		proc->setPosition( position );
		klass->modified();
	}
}

void ToolXfMagnet::strengthChanged(NumericEntry *widget, ToolXfMagnet *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)klass->journalProcess;
		double strength = widget->getValue();
		proc->setStrength( strength );
		klass->modified();
	}
}

void ToolXfMagnet::sharpnessChanged(NumericEntry *widget, ToolXfMagnet *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)klass->journalProcess;
		double sharpness = widget->getValue();
		proc->setSharpness( sharpness );
		klass->modified();
	}
}

void ToolXfMagnet::iterationsChanged(NumericEntry *widget, ToolXfMagnet *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfMagnet *proc = (ProcXfMagnet*)klass->journalProcess;
		int iter = widget->getValueAsInt();
		proc->setIterations( iter );
		klass->modified();
	}
}

void ToolXfMagnet::doneClicked(GtkWidget *widget, ToolXfMagnet *klass)
{
	klass->done();
}





#endif

