/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFMAGNET_H
#define __TOOLXFMAGNET_H


#include <string>


#include "Vector3.h"
#include "ProcXform.h"
#include "VectorSnap.h"
#include "NumericEntry.h"
#include "ToolXformClickAndDrag.h"



class ProcXfMagnet : public ProcXform
{
private:
	Point3 magnetPosition;
	double strength, sharpness;
	int iterations;
	bool in3dFlag;
	
public:
	ProcXfMagnet();
	ProcXfMagnet(const Point3& p, double str, double sharp, int iter, bool in3d);
	virtual ~ProcXfMagnet();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setPosition(const Point3& p);
	void setStrength(double str);
	void setSharpness(double sharp);
	void setIterations(int iter);
	void setIn3d(bool in3d);
	
	const Point3& getPosition() const;
	double getStrength() const;
	double getSharpness() const;
	int getIterations() const;
	bool getIn3d() const;

protected:
	virtual void transformModel(JournalModel *m) const;
};



class ToolXfMagnet : public ToolXformClickAndDrag
{
private:
	VectorSnap snapper;

	GtkWidget *strengthLabel, *sharpnessLabel, *iterationsLabel;
	NumericEntry *strengthEntry, *sharpnessEntry, *iterationsEntry;

	GtkWidget *snapLabel;
	NumericEntry *xSnapEntry, *ySnapEntry, *zSnapEntry;

	GtkWidget *posLabel;
	NumericEntry *xPosEntry, *yPosEntry, *zPosEntry;
	
	GtkWidget *doneButton;


	Point3 initialPosition;



public:
	ToolXfMagnet(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfMagnet();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();
	
	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();

private:
	static void checkResponse(GtkWidget *widget, bool *flag);
	static void positionChanged(NumericEntry *widget, ToolXfMagnet *klass);
	static void strengthChanged(NumericEntry *widget, ToolXfMagnet *klass);
	static void sharpnessChanged(NumericEntry *widget, ToolXfMagnet *klass);
	static void iterationsChanged(NumericEntry *widget, ToolXfMagnet *klass);
	static void doneClicked(GtkWidget *widget, ToolXfMagnet *klass);
};



#endif
