/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFMOVETO_CPP
#define __TOOLXFMOVETO_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfMoveTo.h"

#include "JournalModel.h"
#include "Matrix4.h"





void* __create_ProcXfMoveTo()
{
	return (void*)( new ProcXfMoveTo() );
}

ObjectType type_ProcXfMoveTo( "ProcXfMoveTo", "ProcXform",
											__create_ProcXfMoveTo );






ProcXfMoveTo::ProcXfMoveTo() : ProcXform()
{
	newPosition = Point3();
}

ProcXfMoveTo::ProcXfMoveTo(const Point3& p) : ProcXform()
{
	newPosition = p;
}

ProcXfMoveTo::~ProcXfMoveTo()
{
}



void ProcXfMoveTo::read(FILE *f)
{
	ProcXform::read(f);

	newPosition.read( f );
}

void ProcXfMoveTo::write(FILE *f)
{
	ProcXform::write(f);

	newPosition.write( f );
}





string ProcXfMoveTo::getDescription() const
{
	return "Move To";
}

const ObjectType* ProcXfMoveTo::getType() const
{
	return &type_ProcXfMoveTo;
}

const ObjectType* ProcXfMoveTo::getTypeStatic()
{
	return &type_ProcXfMoveTo;
}





JournalProcess* ProcXfMoveTo::clone() const
{
	return new ProcXfMoveTo(newPosition);
}


void ProcXfMoveTo::setPosition(const Point3& p)
{
	newPosition = p;
	modified();
}

const Point3& ProcXfMoveTo::getPosition() const
{
	return newPosition;
}



void ProcXfMoveTo::transformModel(JournalModel *model) const
{
	Point3 objectCentre = model->getModel()->getBBox().getCentre();

	Vector3 translation = newPosition - objectCentre;
	
	Matrix4 matrix = Matrix4_translate( translation );
	model->getModel()->transform(matrix);
}






ToolXfMoveTo::ToolXfMoveTo(CadView *v, ObjPainterManager *pman)
									: ToolXformClickAndDrag( v, pman, "Move To",
																	 "Move To", 1 )
{
	snapper = VectorSnap( Vector3() );
	moveXFlag = moveYFlag = moveZFlag = true;


	//build the widgets
	//translation axis check buttons
	moveXCheck = gtk_check_button_new_with_label("X-MoveTo");
	moveYCheck = gtk_check_button_new_with_label("Y-MoveTo");
	moveZCheck = gtk_check_button_new_with_label("Z-MoveTo");
	gtk_widget_ref(moveXCheck);
	gtk_widget_ref(moveYCheck);
	gtk_widget_ref(moveZCheck);
	gtk_widget_show(moveXCheck);
	gtk_widget_show(moveYCheck);
	gtk_widget_show(moveZCheck);

	//connect the check button signals
	gtk_signal_connect(GTK_OBJECT(moveXCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &moveXFlag);
	gtk_signal_connect(GTK_OBJECT(moveYCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &moveYFlag);
	gtk_signal_connect(GTK_OBJECT(moveZCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &moveZFlag);

	//default is all true
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(moveXCheck), moveXFlag );
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(moveYCheck), moveYFlag );
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(moveZCheck), moveZFlag );


	
	//snap
	snapLabel = gtk_label_new("snap to grid:");
	gtk_widget_ref(snapLabel);
	gtk_widget_show(snapLabel);
	xSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	ySnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	zSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	gtk_widget_show( xSnapEntry->getWidget() );
	gtk_widget_show( ySnapEntry->getWidget() );
	gtk_widget_show( zSnapEntry->getWidget() );



	//translation
	xlatLabel = gtk_label_new("position:");
	gtk_widget_ref(xlatLabel);
	gtk_widget_show(xlatLabel);
	xPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	yPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	zPosEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(positionChanged), this );
	gtk_widget_show( xPosEntry->getWidget() );
	gtk_widget_show( yPosEntry->getWidget() );
	gtk_widget_show( zPosEntry->getWidget() );


	//done button
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);



	//connect done button signal
	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
	
		
	
	//add to the layout
	layout.add(moveXCheck);
	layout.newLine();
	layout.add(moveYCheck);
	layout.newLine();
	layout.add(moveZCheck);
	layout.lineSpace(1);
	layout.addAsLine(snapLabel);
	layout.add( xSnapEntry->getWidget() );
	layout.add( ySnapEntry->getWidget() );
	layout.add( zSnapEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(xlatLabel);
	layout.add( xPosEntry->getWidget() );
	layout.add( yPosEntry->getWidget() );
	layout.add( zPosEntry->getWidget() );
	layout.lineSpace(1);
	layout.add(doneButton);
	layout.newLine();
}

ToolXfMoveTo::~ToolXfMoveTo()
{
	//delete vector widgets
	delete xSnapEntry;
	delete ySnapEntry;
	delete zSnapEntry;
	delete xPosEntry;
	delete yPosEntry;
	delete zPosEntry;

	//unref widgets
	gtk_widget_unref(moveXCheck);
	gtk_widget_unref(moveYCheck);
	gtk_widget_unref(moveZCheck);
	gtk_widget_unref(snapLabel);
	gtk_widget_unref(xlatLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfMoveTo::getProcessType() const
{
	return ProcXfMoveTo::getTypeStatic();
}



void ToolXfMoveTo::initialise()
{
	ToolXformClickAndDrag::initialise();

	if ( hasProcess() )
	{
		ProcXfMoveTo *proc = (ProcXfMoveTo*)journalProcess;

		Point3 position = proc->getPosition();
		xPosEntry->setValue( position.x );
		yPosEntry->setValue( position.y );
		zPosEntry->setValue( position.z );
	}
}


void ToolXfMoveTo::start()
{
	ToolXformClickAndDrag::start();

	if ( hasProcess() )
	{
		//get and store the initial translation
		ProcXfMoveTo *proc = (ProcXfMoveTo*)journalProcess;
		initialPosition = proc->getPosition();

		Vector3 snapGranularity = Vector3( xSnapEntry->getValue(),
													  ySnapEntry->getValue(),
													  zSnapEntry->getValue() );
					
		snapper = VectorSnap( snapGranularity );
	}
}

void ToolXfMoveTo::finish(bool terminated)
{
	ToolXformClickAndDrag::finish(terminated);
	
	if ( hasProcess() )
	{
		ProcXfMoveTo *proc = (ProcXfMoveTo*)journalProcess;
		if (terminated)
		{
			//set back to the initial translation
			proc->setPosition( initialPosition );
			xPosEntry->setValue( initialPosition.x );
			yPosEntry->setValue( initialPosition.y );
			zPosEntry->setValue( initialPosition.z );
			modified();
		}
	}
}
		


void ToolXfMoveTo::function()
{
	if ( hasProcess() )
	{
		ToolXformClickAndDrag::function();
	
		ProcXfMoveTo *proc = (ProcXfMoveTo*)journalProcess;

		Vector3 translation = currentCursorPosition.getWorld() -
									 startCursorPosition.getWorld();


		if (!moveXFlag)
		{
			translation.x = 0.0;
		}
		if (!moveYFlag)
		{
			translation.y = 0.0;
		}
		if (!moveZFlag)
		{
			translation.z = 0.0;
		}

		Point3 newPosition = initialPosition + translation;

		newPosition = snapper.snap(newPosition);

		xPosEntry->setValue( newPosition.x );
		yPosEntry->setValue( newPosition.y );
		zPosEntry->setValue( newPosition.z );

		proc->setPosition(newPosition);

	
		modified();
	}
}



void ToolXfMoveTo::checkResponse(GtkWidget *widget, bool *flag)
{
	//set the flag to the value of the button
	if (GTK_TOGGLE_BUTTON(widget)->active)
	{
		*flag = true;
	}
	else
	{
		*flag = false;
	}
}

void ToolXfMoveTo::positionChanged(NumericEntry *widget, ToolXfMoveTo *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfMoveTo *proc = (ProcXfMoveTo*)klass->journalProcess;
		Point3 position = Vector3( klass->xPosEntry->getValue(),
											klass->yPosEntry->getValue(),
											klass->zPosEntry->getValue() );
		proc->setPosition( position );
		klass->modified();
	}
}

void ToolXfMoveTo::doneClicked(GtkWidget *widget, ToolXfMoveTo *klass)
{
	klass->done();
}





#endif

