/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFMOVETO_H
#define __TOOLXFMOVETO_H


#include <string>


#include "Vector3.h"
#include "ProcXform.h"
#include "VectorSnap.h"
#include "NumericEntry.h"
#include "ToolXformClickAndDrag.h"



class ProcXfMoveTo : public ProcXform
{
private:
	Point3 newPosition;
	
public:
	ProcXfMoveTo();
	ProcXfMoveTo(const Point3& p);
	virtual ~ProcXfMoveTo();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setPosition(const Point3& p);
	const Point3& getPosition() const;

protected:
	virtual void transformModel(JournalModel *m) const;
};



class ToolXfMoveTo : public ToolXformClickAndDrag
{
private:
	VectorSnap snapper;
	GtkWidget *moveXCheck, *moveYCheck, *moveZCheck;

	GtkWidget *snapLabel;
	NumericEntry *xSnapEntry, *ySnapEntry, *zSnapEntry;

	GtkWidget *xlatLabel;
	NumericEntry *xPosEntry, *yPosEntry, *zPosEntry;
	
	GtkWidget *doneButton;

	bool moveXFlag, moveYFlag, moveZFlag;

	Point3 initialPosition;


public:
	ToolXfMoveTo(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfMoveTo();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();
	
	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();

private:
	static void checkResponse(GtkWidget *widget, bool *flag);
	static void positionChanged(NumericEntry *widget, ToolXfMoveTo *klass);
	static void doneClicked(GtkWidget *widget, ToolXfMoveTo *klass);
};



#endif
