/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFROTATE_CPP
#define __TOOLXFROTATE_CPP

#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfRotate.h"

#include "JournalModel.h"
#include "Matrix4.h"
#include "radians.h"





void* __create_ProcXfRotate()
{
	return (void*)( new ProcXfRotate() );
}

ObjectType type_ProcXfRotate( "ProcXfRotate", "ProcXformCentreAxis",
										__create_ProcXfRotate );






ProcXfRotate::ProcXfRotate() : ProcXformCentreAxis()
{
	rotation = 0.0;
}

ProcXfRotate::ProcXfRotate(Centre cen, Axis a, double r)
									: ProcXformCentreAxis(cen, a)
{
	rotation = r;
}

ProcXfRotate::~ProcXfRotate()
{
}



void ProcXfRotate::read(FILE *f)
{
	ProcXformCentreAxis::read(f);

	fread(&rotation, sizeof(double), 1, f);
}

void ProcXfRotate::write(FILE *f)
{
	ProcXformCentreAxis::write(f);

	fwrite(&rotation, sizeof(double), 1, f);
}





string ProcXfRotate::getDescription() const
{
	return "Rotate";
}

const ObjectType* ProcXfRotate::getType() const
{
	return &type_ProcXfRotate;
}

const ObjectType* ProcXfRotate::getTypeStatic()
{
	return &type_ProcXfRotate;
}





JournalProcess* ProcXfRotate::clone() const
{
	return new ProcXfRotate(whichCentre, axis, rotation);
}



void ProcXfRotate::setRotation(double r)
{
	rotation = r;
	modified();
}

double ProcXfRotate::getRotation() const
{
	return rotation;
}




void ProcXfRotate::transformModel(JournalModel *model, const Point3& centre,
									const Matrix4& axesMatrix,
									const Matrix4& axesMatrixInverse,
									const BBox& boundingBoxInTransformationSpace) const
{
	Vector3 cenXlat = centre.toVector3();

	Matrix4 m = axesMatrix *
					Matrix4_translate( -cenXlat ) *
					Matrix4_rotateZ( DEGTORAD(-rotation) ) *
					Matrix4_translate( cenXlat ) *
					axesMatrixInverse;

	model->getModel()->transform(m);
}






ToolXfRotate::ToolXfRotate(CadView *v, ObjPainterManager *pman)
									: ToolXformCentreAxis( v, pman, "Rotate",
																  "Rotate", 2 )
{
	snapLabel = gtk_label_new("Snap:");
	gtk_widget_ref(snapLabel);
	gtk_widget_show(snapLabel);
	snapEntry = new NumericEntry( 0.0, 0.0, 360.0, 0.01, 0.5, 0.2, 4,
											NULL, NULL );
	gtk_widget_show( snapEntry->getWidget() );




	rotationLabel = gtk_label_new("Rotate:");
	gtk_widget_ref(rotationLabel);
	gtk_widget_show(rotationLabel);
	rotationEntry = new NumericEntry( 0.0, 0.0, 360.0, 0.1, 1.0, 0.2, 4,
												 GTK_SIGNAL_FUNC(rotationChanged), this );
	rotationEntry->wrap();
	gtk_widget_show( rotationEntry->getWidget() );




	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);


	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
	
		
	gtk_widget_show(doneButton);
	
	layout.lineSpace(1);
	layout.add(snapLabel);
	layout.add( snapEntry->getWidget() );
	layout.add(rotationLabel);
	layout.add( rotationEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolXfRotate::~ToolXfRotate()
{
	delete snapEntry;
	delete rotationEntry;

	gtk_widget_unref(snapLabel);
	gtk_widget_unref(rotationLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfRotate::getProcessType() const
{
	return ProcXfRotate::getTypeStatic();
}




void ToolXfRotate::initialise()
{
	ToolXformCentreAxis::initialise();
	
	if ( hasProcess() )
	{
		ProcXfRotate *proc = (ProcXfRotate*)journalProcess;
		rotationEntry->setValue( proc->getRotation() );
	}
}


void ToolXfRotate::start()
{
	ToolXformCentreAxis::start();


	if ( hasProcess() )
	{
		ProcXfRotate *proc = (ProcXfRotate*)journalProcess;
		initialRotationValue = proc->getRotation();

		snapper = ValueSnap( snapEntry->getValue() );
	}
}

void ToolXfRotate::finish(bool terminated)
{
	ToolXformCentreAxis::finish(terminated);

	if ( hasProcess() )
	{
		ProcXfRotate *proc = (ProcXfRotate*)journalProcess;
		if (terminated)
		{
			proc->setRotation( initialRotationValue );
			rotationEntry->setValue( initialRotationValue );
			modified();
		}
	}
}

void ToolXfRotate::function()
{
	ToolXformCentreAxis::function();

	if ( hasProcess() )
	{
		ProcXfRotate *proc = (ProcXfRotate*)journalProcess;

		double rotation = ( currentCursorPosition.getScreenFloat().x -
								  startCursorPosition.getScreenFloat().x ) * 0.25;
							  
		rotation += initialRotationValue;

		while (rotation < 0.0)
		{
			rotation += 360.0;
		}

		while (rotation >= 360.0)
		{
			rotation -= 360.0;
		}
	
		rotation = snapper.snap(rotation);
		rotationEntry->setValue(rotation);
		proc->setRotation( rotation );

		modified();
	}
}




void ToolXfRotate::rotationChanged(NumericEntry *widget, ToolXfRotate *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfRotate *proc = (ProcXfRotate*)klass->journalProcess;
		proc->setRotation( klass->rotationEntry->getValue() );
		klass->modified();
	}		
}

void ToolXfRotate::doneClicked(GtkWidget *widget, ToolXfRotate *klass)
{
	klass->done();
}


#endif

