/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFROTATE_H
#define __TOOLXFROTATE_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "ToolXformCentreAxis.h"
#include "NumericEntry.h"



class ProcXfRotate : public ProcXformCentreAxis
{
private:
	double rotation;
	
public:
	ProcXfRotate();
	ProcXfRotate(Centre cen, Axis a, double r);
	virtual ~ProcXfRotate();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setRotation(double r);
	double getRotation() const;

protected:
	virtual void transformModel(JournalModel *model, const Point3& centre,
										 const Matrix4& axesMatrix,
										 const Matrix4& axesMatrixInverse,
										 const BBox& boundingBoxInTransformationSpace) const;
};



class ToolXfRotate : public ToolXformCentreAxis
{
private:
	GtkWidget *snapLabel;
	NumericEntry *snapEntry;

	GtkWidget *rotationLabel;
	NumericEntry *rotationEntry;

	GtkWidget *doneButton;


	ValueSnap snapper;


	double initialRotationValue;

public:
	ToolXfRotate(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfRotate();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	
private:
	static void rotationChanged(NumericEntry *widget, ToolXfRotate *klass);
	static void doneClicked(GtkWidget *widget, ToolXfRotate *klass);
};



#endif
