/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFSCALE_CPP
#define __TOOLXFSCALE_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfScale.h"

#include "JournalModel.h"
#include "Matrix4.h"
#include "radians.h"





void* __create_ProcXfScale()
{
	return (void*)( new ProcXfScale() );
}

ObjectType type_ProcXfScale( "ProcXfScale", "ProcXformCentreAxis",
									  __create_ProcXfScale );






ProcXfScale::ProcXfScale() : ProcXformCentreAxis()
{
	value = Vector3(1.0, 1.0, 1.0);
}

ProcXfScale::ProcXfScale(Centre cen, Axis a, const Vector3& val)
								 : ProcXformCentreAxis(cen, a)
{
	value = val;
}

ProcXfScale::~ProcXfScale()
{
}



void ProcXfScale::read(FILE *f)
{
	ProcXformCentreAxis::read(f);

	value.read( f );
}

void ProcXfScale::write(FILE *f)
{
	ProcXformCentreAxis::write(f);

	value.write( f );
}





string ProcXfScale::getDescription() const
{
	return "Scale";
}

const ObjectType* ProcXfScale::getType() const
{
	return &type_ProcXfScale;
}

const ObjectType* ProcXfScale::getTypeStatic()
{
	return &type_ProcXfScale;
}





JournalProcess* ProcXfScale::clone() const
{
	return new ProcXfScale(whichCentre, axis, value);
}



void ProcXfScale::setValue(const Vector3& val)
{
	value = val;
	modified();
}

const Vector3& ProcXfScale::getValue() const
{
	return value;
}


void ProcXfScale::transformModel(JournalModel *model, const Point3& centre,
									const Matrix4& axesMatrix,
									const Matrix4& axesMatrixInverse,
									const BBox& boundingBoxInTransformationSpace) const
{
	Vector3 cenXlat = centre.toVector3();

	Matrix4 m = axesMatrix *
					Matrix4_translate( -cenXlat ) *
					Matrix4_scale( value ) *
					Matrix4_translate( cenXlat ) *
					axesMatrixInverse;

	model->getModel()->transform(m);
}




ToolXfScale::ToolXfScale(CadView *v, ObjPainterManager *pman)
								 : ToolXformCentreAxis( v, pman, "Scale", "Scale", 2 )
{
	snapper = VectorSnap();
	lockValue1Flag = false;
	lockValue2Flag = false;
	uniformFlag = false;
	
	axisChooser->setChoice( AXIS_Z );

	lockVal1Check = gtk_check_button_new_with_label("Lock Horiz");
	lockVal2Check = gtk_check_button_new_with_label("Lock Vert");
	uniformCheck = gtk_check_button_new_with_label("Uniform");
	gtk_widget_ref(lockVal1Check);
	gtk_widget_ref(lockVal2Check);
	gtk_widget_ref(uniformCheck);


	snapLabel = gtk_label_new("Snap (H,V):");
	gtk_widget_ref(snapLabel);


	snapVal1Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );
	snapVal2Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );


	valueLabel = gtk_label_new( "Scale (H,V,D):" );
	gtk_widget_ref(valueLabel);


	val1Entry = new NumericEntry( 1.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );
	val2Entry = new NumericEntry( 1.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );
	val3Entry = new NumericEntry( 1.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );


	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);


	gtk_signal_connect( GTK_OBJECT(lockVal1Check), "clicked",
							  GTK_SIGNAL_FUNC(lockCheckClicked), &lockValue1Flag );
	gtk_signal_connect( GTK_OBJECT(lockVal2Check), "clicked",
							  GTK_SIGNAL_FUNC(lockCheckClicked), &lockValue2Flag );
	gtk_signal_connect( GTK_OBJECT(uniformCheck), "clicked",
							  GTK_SIGNAL_FUNC(uniformClicked), this );

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );
	
		
	gtk_widget_show(lockVal1Check);
	gtk_widget_show(lockVal2Check);
	gtk_widget_show(uniformCheck);
	gtk_widget_show(snapLabel);
	gtk_widget_show( snapVal1Entry->getWidget() );
	gtk_widget_show( snapVal2Entry->getWidget() );
	gtk_widget_show(valueLabel);
	gtk_widget_show( val1Entry->getWidget() );
	gtk_widget_show( val2Entry->getWidget() );
	gtk_widget_show( val3Entry->getWidget() );
	gtk_widget_show(doneButton);



	layout.lineSpace(1);

	layout.add(lockVal1Check);
	layout.add(lockVal2Check);
	layout.add(uniformCheck);
	layout.lineSpace(1);

	layout.addAsLine(snapLabel);
	layout.add( snapVal1Entry->getWidget() );
	layout.add( snapVal2Entry->getWidget() );
	layout.lineSpace(1);

	layout.addAsLine(valueLabel);
	layout.add( val1Entry->getWidget() );
	layout.add( val2Entry->getWidget() );
	layout.add( val3Entry->getWidget() );
	layout.lineSpace(1);
		
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolXfScale::~ToolXfScale()
{
	delete snapVal1Entry;
	delete snapVal2Entry;
	delete val1Entry;
	delete val2Entry;
	delete val3Entry;

	gtk_widget_unref(lockVal1Check);
	gtk_widget_unref(lockVal2Check);
	gtk_widget_unref(uniformCheck);
	gtk_widget_unref(snapLabel);
	gtk_widget_unref(valueLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfScale::getProcessType() const
{
	return ProcXfScale::getTypeStatic();
}




void ToolXfScale::initialise()
{
	ToolXformCentreAxis::initialise();

	if ( hasProcess() )
	{
		ProcXfScale *proc = (ProcXfScale*)journalProcess;
		Vector3 value = proc->getValue();

		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal1Check ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal2Check ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( uniformCheck ), true );

		val1Entry->setValue( value.x );
		val2Entry->setValue( value.y );
		val3Entry->setValue( value.z );
	}
}


void ToolXfScale::start()
{
	ToolXformCentreAxis::start();

	if ( hasProcess() )
	{
		ProcXfScale *proc = (ProcXfScale*)journalProcess;
		initialValue = proc->getValue();

		Vector3 snapValue = Vector3();
		snapValue.x = snapVal1Entry->getValue();
		snapValue.y = snapVal2Entry->getValue();
		snapper = VectorSnap( snapValue );
	}
}

void ToolXfScale::finish(bool terminated)
{
	ToolXformCentreAxis::finish(terminated);

	if ( hasProcess() )
	{
		ProcXfScale *proc = (ProcXfScale*)journalProcess;
		if (terminated)
		{
			proc->setValue( initialValue );
			val1Entry->setValue( initialValue.x );
			val2Entry->setValue( initialValue.y );
			modified();
		}
	}
}

void ToolXfScale::function()
{
	ToolXformCentreAxis::function();

	if ( hasProcess() )
	{
		ProcXfScale *proc = (ProcXfScale*)journalProcess;

		Vector3 newValue;
		newValue.x = ( currentCursorPosition.getScreenFloat().x -
							startCursorPosition.getScreenFloat().x ) * 0.005;
		newValue.y = ( currentCursorPosition.getScreenFloat().y -
							startCursorPosition.getScreenFloat().y ) * 0.005;

		if (lockValue1Flag)
		{
			newValue.x = 0.0;
		}
		if (lockValue2Flag)
		{
			newValue.y = 0.0;
		}
	
		newValue += initialValue;
		newValue = snapper.snap(newValue);

		if (uniformFlag)
		{
			newValue.y = newValue.x;
			newValue.z = newValue.x;
		}

		val1Entry->setValue( newValue.x );
		val2Entry->setValue( newValue.y );
		val3Entry->setValue( newValue.z );

		proc->setValue( newValue );

		modified();
	}
}




void ToolXfScale::lockCheckClicked(GtkWidget *widget, bool *flag)
{
	*flag = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) ) != 0;	
}

void ToolXfScale::uniformClicked(GtkWidget *widget, ToolXfScale *klass)
{
	klass->uniformFlag =
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) ) != 0;	
	if ( klass->uniformFlag )
	{
		if ( klass->hasProcess() )
		{
			ProcXfScale *proc = (ProcXfScale*)klass->journalProcess;
			
			double value1 = klass->val1Entry->getValue();
			klass->val2Entry->setValue(value1);

			proc->setValue( Vector3( value1, value1, 1.0 ) );
			klass->modified();
		}		
	}
}

void ToolXfScale::valueChanged(NumericEntry *widget, ToolXfScale *klass)
{
	if ( klass->hasProcess() )
	{
		Vector3 value;

		if ( klass->uniformFlag )
		{
			value.x = value.y = widget->getValue();
			klass->val1Entry->setValue(value.x);
			klass->val2Entry->setValue(value.x);
			klass->val3Entry->setValue(value.x);
		}
		else
		{	
			value.x = klass->val1Entry->getValue();
			value.y = klass->val2Entry->getValue();
			value.z = klass->val3Entry->getValue();
		}

		ProcXfScale *proc = (ProcXfScale*)klass->journalProcess;

		proc->setValue( value );
		klass->modified();
	}		
}

void ToolXfScale::doneClicked(GtkWidget *widget, ToolXfScale *klass)
{
	klass->done();
}



#endif

