/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFSHEAR_CPP
#define __TOOLXFSHEAR_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfShear.h"

#include "JournalModel.h"
#include "Matrix4.h"
#include "radians.h"





void* __create_ProcXfShear()
{
	return (void*)( new ProcXfShear() );
}

ObjectType type_ProcXfShear( "ProcXfShear", "ProcXformAxis",
									  __create_ProcXfShear );






ProcXfShear::ProcXfShear() : ProcXformAxis()
{
	value = Vector3(0.0, 0.0, 0.0);
}

ProcXfShear::ProcXfShear(Axis a, const Vector3& val)
								 : ProcXformAxis(a)
{
	value = val;
}

ProcXfShear::~ProcXfShear()
{
}



void ProcXfShear::read(FILE *f)
{
	ProcXformAxis::read(f);

	value.read( f );
}

void ProcXfShear::write(FILE *f)
{
	ProcXformAxis::write(f);

	value.write( f );
}





string ProcXfShear::getDescription() const
{
	return "Shear";
}

const ObjectType* ProcXfShear::getType() const
{
	return &type_ProcXfShear;
}

const ObjectType* ProcXfShear::getTypeStatic()
{
	return &type_ProcXfShear;
}





JournalProcess* ProcXfShear::clone() const
{
	return new ProcXfShear(axis, value);
}



void ProcXfShear::setValue(const Vector3& val)
{
	value = val;
	modified();
}

const Vector3& ProcXfShear::getValue() const
{
	return value;
}


void ProcXfShear::transformModel(JournalModel *model, const Point3& centre,
							const Matrix4& axesMatrix,
							const Matrix4& axesMatrixInverse,
							const BBox& boundingBoxInTransformationSpace) const
{
	Vector3 centreXlat =
		boundingBoxInTransformationSpace.getLower().toVector3();

	Matrix4 m = axesMatrix *
					Matrix4_translate( -centreXlat ) *
					Matrix4_shearZ( value.x, value.y ) *
					Matrix4_translate( centreXlat ) *
					axesMatrixInverse;

	model->getModel()->transform(m);
}




ToolXfShear::ToolXfShear(CadView *v, ObjPainterManager *pman)
								 : ToolXformAxis( v, pman, "Shear", "Shear", 2 )
{
	snapper = VectorSnap();
	lockValue1Flag = false;
	lockValue2Flag = false;
	
	axisChooser->setChoice( AXIS_Z );

	lockVal1Check = gtk_check_button_new_with_label("Lock Horiz");
	lockVal2Check = gtk_check_button_new_with_label("Lock Vert");
	gtk_widget_ref(lockVal1Check);
	gtk_widget_ref(lockVal2Check);


	snapLabel = gtk_label_new("Snap (H,V):");
	gtk_widget_ref(snapLabel);


	snapVal1Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );
	snapVal2Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );


	valueLabel = gtk_label_new( "Shear (H,V):" );
	gtk_widget_ref(valueLabel);


	val1Entry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );
	val2Entry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );


	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);


	gtk_signal_connect( GTK_OBJECT(lockVal1Check), "clicked",
							  GTK_SIGNAL_FUNC(checkClicked), this );
	gtk_signal_connect( GTK_OBJECT(lockVal2Check), "clicked",
							  GTK_SIGNAL_FUNC(checkClicked), this );

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );
	
		
	gtk_widget_show(lockVal1Check);
	gtk_widget_show(lockVal2Check);
	gtk_widget_show(snapLabel);
	gtk_widget_show( snapVal1Entry->getWidget() );
	gtk_widget_show( snapVal2Entry->getWidget() );
	gtk_widget_show(valueLabel);
	gtk_widget_show( val1Entry->getWidget() );
	gtk_widget_show( val2Entry->getWidget() );
	gtk_widget_show(doneButton);



	layout.lineSpace(1);

	layout.add(lockVal1Check);
	layout.add(lockVal2Check);
	layout.lineSpace(1);

	layout.addAsLine(snapLabel);
	layout.add( snapVal1Entry->getWidget() );
	layout.add( snapVal2Entry->getWidget() );
	layout.lineSpace(1);

	layout.addAsLine(valueLabel);
	layout.add( val1Entry->getWidget() );
	layout.add( val2Entry->getWidget() );
	layout.lineSpace(1);
		
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolXfShear::~ToolXfShear()
{
	delete snapVal1Entry;
	delete snapVal2Entry;
	delete val1Entry;
	delete val2Entry;

	gtk_widget_unref(lockVal1Check);
	gtk_widget_unref(lockVal2Check);
	gtk_widget_unref(snapLabel);
	gtk_widget_unref(valueLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfShear::getProcessType() const
{
	return ProcXfShear::getTypeStatic();
}




void ToolXfShear::initialise()
{
	ToolXformAxis::initialise();

	if ( hasProcess() )
	{
		ProcXfShear *proc = (ProcXfShear*)journalProcess;
		Vector3 value = proc->getValue();

		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal1Check ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal2Check ), false );
		
		val1Entry->setValue( value.x );
		val2Entry->setValue( value.y );
	}
}


void ToolXfShear::start()
{
	ToolXformAxis::start();

	if ( hasProcess() )
	{
		ProcXfShear *proc = (ProcXfShear*)journalProcess;
		initialValue = proc->getValue();

		Vector3 snapValue;
		snapValue.x = snapVal1Entry->getValue();
		snapValue.y = snapVal2Entry->getValue();
		snapper = VectorSnap( snapValue );
	}
}

void ToolXfShear::finish(bool terminated)
{
	ToolXformAxis::finish(terminated);

	if ( hasProcess() )
	{
		ProcXfShear *proc = (ProcXfShear*)journalProcess;
		if (terminated)
		{
			proc->setValue( initialValue );
			val1Entry->setValue( initialValue.x );
			val2Entry->setValue( initialValue.y );
			modified();
		}
	}
}

void ToolXfShear::function()
{
	ToolXformAxis::function();

	if ( hasProcess() )
	{
		ProcXfShear *proc = (ProcXfShear*)journalProcess;
		
		Vector3 newValue;
		newValue.x = ( currentCursorPosition.getScreenFloat().x -
							startCursorPosition.getScreenFloat().x ) * 0.005;
		newValue.y = ( currentCursorPosition.getScreenFloat().y -
							startCursorPosition.getScreenFloat().y ) * 0.005;

		if (lockValue1Flag)
		{
			newValue.x = 0.0;
		}
		if (lockValue2Flag)
		{
			newValue.y = 0.0;
		}
	
		newValue += initialValue;
		newValue = snapper.snap(newValue);

		val1Entry->setValue( newValue.x );
		val2Entry->setValue( newValue.y );

		proc->setValue( newValue );
		modified();
	}
}




void ToolXfShear::checkClicked(GtkWidget *widget, ToolXfShear *klass)
{
	klass->lockValue1Flag =
		gtk_toggle_button_get_active(
					GTK_TOGGLE_BUTTON(klass->lockVal1Check) ) != 0;
	klass->lockValue2Flag =
		gtk_toggle_button_get_active(
					GTK_TOGGLE_BUTTON(klass->lockVal2Check) ) != 0;
}

void ToolXfShear::valueChanged(NumericEntry *widget, ToolXfShear *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfShear *proc = (ProcXfShear*)klass->journalProcess;

		Vector3 value = Vector3( klass->val1Entry->getValue(),
										 klass->val2Entry->getValue(),
										 proc->getValue().z );
		proc->setValue( value );
		klass->modified();
	}		
}

void ToolXfShear::doneClicked(GtkWidget *widget, ToolXfShear *klass)
{
	klass->done();
}






#endif

