/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFSNAP_CPP
#define __TOOLXFSNAP_CPP

#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtklabel.h>

#include <strstream>

#include "ToolXfSnap.h"

#include "JournalModel.h"
#include "SnapTransformation.h"





void* __create_ProcXfSnap()
{
	return (void*)( new ProcXfSnap() );
}

ObjectType type_ProcXfSnap( "ProcXfSnap", "ProcXform", __create_ProcXfSnap );





ProcXfSnap::ProcXfSnap() : ProcXform()
{
	granularity = Vector3();
}

ProcXfSnap::ProcXfSnap(const Vector3& snapGranularity)
							  : ProcXform()
{
	granularity = snapGranularity;
}

ProcXfSnap::~ProcXfSnap()
{
}



void ProcXfSnap::read(FILE *f)
{
	ProcXform::read(f);

	granularity.read( f );
}

void ProcXfSnap::write(FILE *f)
{
	ProcXform::write(f);

	granularity.write( f );
}





string ProcXfSnap::getDescription() const
{
	return "Snap to grid";
}

const ObjectType* ProcXfSnap::getType() const
{
	return &type_ProcXfSnap;
}

const ObjectType* ProcXfSnap::getTypeStatic()
{
	return &type_ProcXfSnap;
}





JournalProcess* ProcXfSnap::clone() const
{
	return new ProcXfSnap(granularity);
}



void ProcXfSnap::setSnapGranularity(const Vector3& snapGranularity)
{
	granularity = snapGranularity;
	modified();
}

const Vector3& ProcXfSnap::getSnapGranularity() const
{
	return granularity;
}



void ProcXfSnap::transformModel(JournalModel *model) const
{
	SnapTransformation xform(granularity);
	model->getModel()->transform(&xform);
}






ToolXfSnap::ToolXfSnap(CadView *v, ObjPainterManager *pman)
							  : ToolTransformModel( v, pman, "Snap",
															"Snap To Grid", 1 )
{
	granularityLabel = gtk_label_new("Granularity:");
	gtk_widget_ref(granularityLabel);
	gtk_widget_show(granularityLabel);

	xEntry = new NumericEntry( 1.0, 0.0, 1.0e8, 0.0001, 0.01, 0.3, 4,
										GTK_SIGNAL_FUNC(granularityChanged), this );
	yEntry = new NumericEntry( 1.0, 0.0, 1.0e8, 0.0001, 0.01, 0.3, 4,
										GTK_SIGNAL_FUNC(granularityChanged), this );
	zEntry = new NumericEntry( 1.0, 0.0, 1.0e8, 0.0001, 0.01, 0.3, 4,
										GTK_SIGNAL_FUNC(granularityChanged), this );
	gtk_widget_show( xEntry->getWidget() );
	gtk_widget_show( yEntry->getWidget() );
	gtk_widget_show( zEntry->getWidget() );

	
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);


	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
		
	layout.addAsLine(granularityLabel);
	layout.add( xEntry->getWidget() );
	layout.add( yEntry->getWidget() );
	layout.add( zEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolXfSnap::~ToolXfSnap()
{
	delete xEntry;
	delete yEntry;
	delete zEntry;

	gtk_widget_unref(granularityLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfSnap::getProcessType() const
{
	return ProcXfSnap::getTypeStatic();
}




void ToolXfSnap::initialise()
{
	ToolTransformModel::initialise();

	if ( hasProcess() )
	{
		ProcXfSnap *proc = (ProcXfSnap*)journalProcess;
		Vector3 granularity = proc->getSnapGranularity();
		xEntry->setValue( granularity.x );
		yEntry->setValue( granularity.y );
		zEntry->setValue( granularity.z );
	}
}


void ToolXfSnap::granularityChanged(NumericEntry *widget, ToolXfSnap *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfSnap *proc = (ProcXfSnap*)klass->journalProcess;

		Vector3 granularity = Vector3( klass->xEntry->getValue(),
												 klass->yEntry->getValue(),
												 klass->zEntry->getValue() );

		proc->setSnapGranularity( granularity );
		klass->modified();
	}		
}

void ToolXfSnap::doneClicked(GtkWidget *widget, ToolXfSnap *klass)
{
	klass->done();
}













#endif

