/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFSNAP_H
#define __TOOLXFSNAP_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "ToolTransformModel.h"
#include "NumericEntry.h"
#include "CadView.h"



class ProcXfSnap : public ProcXform
{
private:
	Vector3 granularity;
	
public:
	ProcXfSnap();
	ProcXfSnap(const Vector3& snapGranularity);
	virtual ~ProcXfSnap();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setSnapGranularity(const Vector3& snapGranularity);
	const Vector3& getSnapGranularity() const;

protected:
	virtual void transformModel(JournalModel *model) const;
};



class ToolXfSnap : public ToolTransformModel
{
private:
	GtkWidget *granularityLabel;
	NumericEntry *xEntry, *yEntry, *zEntry;

	GtkWidget *doneButton;

public:
	ToolXfSnap(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfSnap();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

private:
	static void granularityChanged(NumericEntry *widget, ToolXfSnap *klass);
	static void doneClicked(GtkWidget *widget, ToolXfSnap *klass);
};



#endif
