/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFTAPER_CPP
#define __TOOLXFTAPER_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfTaper.h"

#include "JournalModel.h"
#include "Matrix4.h"
#include "TaperTransformation.h"





void* __create_ProcXfTaper()
{
	return (void*)( new ProcXfTaper() );
}

ObjectType type_ProcXfTaper( "ProcXfTaper", "ProcXformAxis",
									  __create_ProcXfTaper );






ProcXfTaper::ProcXfTaper() : ProcXformAxis()
{
	value = Vector3(1.0, 1.0, 1.0);
}

ProcXfTaper::ProcXfTaper(Axis a, const Vector3& val)
								 : ProcXformAxis(a)
{
	value = val;
}

ProcXfTaper::~ProcXfTaper()
{
}



void ProcXfTaper::read(FILE *f)
{
	ProcXformAxis::read(f);

	value.read( f );
}

void ProcXfTaper::write(FILE *f)
{
	ProcXformAxis::write(f);

	value.write( f );
}





string ProcXfTaper::getDescription() const
{
	return "Taper";
}

const ObjectType* ProcXfTaper::getType() const
{
	return &type_ProcXfTaper;
}

const ObjectType* ProcXfTaper::getTypeStatic()
{
	return &type_ProcXfTaper;
}





JournalProcess* ProcXfTaper::clone() const
{
	return new ProcXfTaper(axis, value);
}



void ProcXfTaper::setValue(const Vector3& val)
{
	value = val;
	modified();
}

const Vector3& ProcXfTaper::getValue() const
{
	return value;
}


void ProcXfTaper::transformModel(JournalModel *model, const Point3& centre,
									const Matrix4& axesMatrix,
									const Matrix4& axesMatrixInverse,
									const BBox& boundingBoxInTransformationSpace) const
{
	Vector3 cenXlat = centre.toVector3();

	Matrix4 worldToTaperMatrix = axesMatrix *
										  Matrix4_translate( -cenXlat );
	Matrix4 taperToWorldMatrix = Matrix4_translate( cenXlat ) *
										  axesMatrixInverse;

	MatrixTransformation *worldToTaper =
					new MatrixTransformation(worldToTaperMatrix);
	MatrixTransformation *taperToWorld =
					new MatrixTransformation(taperToWorldMatrix);

	TaperTransformation *taper;



	taper = new TaperTransformation(
		boundingBoxInTransformationSpace.getLower().z, 1.0, 1.0,
		boundingBoxInTransformationSpace.getUpper().z, value.x, value.y );


	Transformation *xform = worldToTaper->clone();


	taper->setChild(taperToWorld);
	xform->setChild(taper);


	model->getModel()->transform(xform);		

	delete taper;
	delete xform;
}




ToolXfTaper::ToolXfTaper(CadView *v, ObjPainterManager *pman)
								 : ToolXformAxis( v, pman, "Taper", "Taper", 2 )
{
	snapper = VectorSnap();
	lockValue1Flag = false;
	lockValue2Flag = false;
	uniformFlag = true;
	
	axisChooser->setChoice( AXIS_Z );

	lockVal1Check = gtk_check_button_new_with_label("Lock Horiz");
	lockVal2Check = gtk_check_button_new_with_label("Lock Vert");
	uniformCheck = gtk_check_button_new_with_label("Uniform");
	gtk_widget_ref(lockVal1Check);
	gtk_widget_ref(lockVal2Check);
	gtk_widget_ref(uniformCheck);
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(uniformCheck), true );


	snapLabel = gtk_label_new("Snap (H,V):");
	gtk_widget_ref(snapLabel);


	snapVal1Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );
	snapVal2Entry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.5, 4,
												 NULL, NULL );


	valueLabel = gtk_label_new( "Taper (H,V):" );
	gtk_widget_ref(valueLabel);


	val1Entry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );
	val2Entry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.0005, 0.005, 0.2, 4,
											GTK_SIGNAL_FUNC(valueChanged), this );


	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);


	gtk_signal_connect( GTK_OBJECT(lockVal1Check), "clicked",
							  GTK_SIGNAL_FUNC(lockCheckClicked), &lockValue1Flag );
	gtk_signal_connect( GTK_OBJECT(lockVal2Check), "clicked",
							  GTK_SIGNAL_FUNC(lockCheckClicked), &lockValue2Flag );
	gtk_signal_connect( GTK_OBJECT(uniformCheck), "clicked",
							  GTK_SIGNAL_FUNC(uniformClicked), this );

	gtk_signal_connect( GTK_OBJECT(doneButton), "clicked",
							  GTK_SIGNAL_FUNC(doneClicked), this );
	
		
	gtk_widget_show(lockVal1Check);
	gtk_widget_show(lockVal2Check);
	gtk_widget_show(uniformCheck);
	gtk_widget_show(snapLabel);
	gtk_widget_show( snapVal1Entry->getWidget() );
	gtk_widget_show( snapVal2Entry->getWidget() );
	gtk_widget_show(valueLabel);
	gtk_widget_show( val1Entry->getWidget() );
	gtk_widget_show( val2Entry->getWidget() );
	gtk_widget_show(doneButton);



	layout.lineSpace(1);

	layout.add(lockVal1Check);
	layout.add(lockVal2Check);
	layout.add(uniformCheck);
	layout.lineSpace(1);

	layout.addAsLine(snapLabel);
	layout.add( snapVal1Entry->getWidget() );
	layout.add( snapVal2Entry->getWidget() );
	layout.lineSpace(1);

	layout.addAsLine(valueLabel);
	layout.add( val1Entry->getWidget() );
	layout.add( val2Entry->getWidget() );
	layout.lineSpace(1);
		
	layout.addAsLine(doneButton);
	layout.newLine();
}

ToolXfTaper::~ToolXfTaper()
{
	delete snapVal1Entry;
	delete snapVal2Entry;
	delete val1Entry;
	delete val2Entry;

	gtk_widget_unref(lockVal1Check);
	gtk_widget_unref(lockVal2Check);
	gtk_widget_unref(uniformCheck);
	gtk_widget_unref(snapLabel);
	gtk_widget_unref(valueLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfTaper::getProcessType() const
{
	return ProcXfTaper::getTypeStatic();
}




void ToolXfTaper::initialise()
{
	ToolXformAxis::initialise();

	if ( hasProcess() )
	{
		ProcXfTaper *proc = (ProcXfTaper*)journalProcess;
		Vector3 value = proc->getValue();

		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal1Check ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lockVal2Check ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( uniformCheck ), true );
		
		val1Entry->setValue( value.x );
		val2Entry->setValue( value.y );
	}
}


void ToolXfTaper::start()
{
	ToolXformAxis::start();

	if ( hasProcess() )
	{
		ProcXfTaper *proc = (ProcXfTaper*)journalProcess;
		initialValue = proc->getValue();

		Vector3 snapValue = Vector3();
		snapValue.x = snapVal1Entry->getValue();
		snapValue.y = snapVal2Entry->getValue();
		snapper = VectorSnap( snapValue );
	}
}

void ToolXfTaper::finish(bool terminated)
{
	ToolXformAxis::finish(terminated);

	if ( hasProcess() )
	{
		ProcXfTaper *proc = (ProcXfTaper*)journalProcess;

		if (terminated)
		{
			proc->setValue( initialValue );
			val1Entry->setValue( initialValue.x );
			val2Entry->setValue( initialValue.y );
			modified();
		}
	}
}

void ToolXfTaper::function()
{
	ToolXformAxis::function();

	if ( hasProcess() )
	{
		ProcXfTaper *proc = (ProcXfTaper*)journalProcess;
		
		Vector3 newValue;
		newValue.x = ( currentCursorPosition.getScreenFloat().x -
							startCursorPosition.getScreenFloat().x ) * 0.005;
		newValue.y = ( currentCursorPosition.getScreenFloat().y -
							startCursorPosition.getScreenFloat().y ) * 0.005;

		if (lockValue1Flag)
		{
			newValue.x = 0.0;
		}
		if (lockValue2Flag)
		{
			newValue.y = 0.0;
		}

		newValue += initialValue;
		newValue = snapper.snap(newValue);

		if (uniformFlag)
		{
			newValue.y = newValue.x;
		}

		val1Entry->setValue( newValue.x );
		val2Entry->setValue( newValue.y );

		proc->setValue( newValue );

		modified();
	}
}




void ToolXfTaper::lockCheckClicked(GtkWidget *widget, bool *flag)
{
	*flag = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) ) != 0;	
}

void ToolXfTaper::uniformClicked(GtkWidget *widget, ToolXfTaper *klass)
{
	klass->uniformFlag =
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) ) != 0;	
	if ( klass->uniformFlag )
	{
		if ( klass->hasProcess() )
		{
			ProcXfTaper *proc = (ProcXfTaper*)klass->journalProcess;
			
			double value1 = klass->val1Entry->getValue();
			klass->val2Entry->setValue(value1);

			proc->setValue( Vector3( value1, value1, 1.0 ) );
			klass->modified();
		}		
	}
}

void ToolXfTaper::valueChanged(NumericEntry *widget, ToolXfTaper *klass)
{
	if ( klass->hasProcess() )
	{
		Vector3 value;

		if ( klass->uniformFlag )
		{
			value.x = value.y = widget->getValue();
			klass->val1Entry->setValue(value.x);
			klass->val2Entry->setValue(value.x);
		}
		else
		{	
			value.x = klass->val1Entry->getValue();
			value.y = klass->val2Entry->getValue();
		}

		ProcXfTaper *proc = (ProcXfTaper*)klass->journalProcess;

		proc->setValue( value );
		klass->modified();
	}		
}

void ToolXfTaper::doneClicked(GtkWidget *widget, ToolXfTaper *klass)
{
	klass->done();
}



#endif

