/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFTAPER_H
#define __TOOLXFTAPER_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "VectorSnap.h"
#include "ToolXformAxis.h"
#include "NumericEntry.h"



class ProcXfTaper : public ProcXformAxis
{
protected:
	Vector3 value;
	
public:
	ProcXfTaper();
	ProcXfTaper(Axis a, const Vector3& val);
	virtual ~ProcXfTaper();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setValue(const Vector3& val);
	const Vector3& getValue() const;

protected:
	virtual void transformModel(JournalModel *model, const Point3& centre,
										 const Matrix4& axesMatrix,
										 const Matrix4& axesMatrixInverse,
										 const BBox& boundingBoxInTransformationSpace) const;
};



class ToolXfTaper : public ToolXformAxis
{
private:
	GtkWidget *lockVal1Check, *lockVal2Check, *uniformCheck;

	GtkWidget *snapLabel;
	NumericEntry *snapVal1Entry, *snapVal2Entry;

	GtkWidget *valueLabel;
	NumericEntry *val1Entry, *val2Entry;

	GtkWidget *doneButton;


	VectorSnap snapper;


	Vector3 initialValue;

	bool lockValue1Flag, lockValue2Flag, uniformFlag;

public:
	ToolXfTaper(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfTaper();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	
private:
	static void lockCheckClicked(GtkWidget *widget, bool *flag);
	static void uniformClicked(GtkWidget *widget, ToolXfTaper *klass);
	static void valueChanged(NumericEntry *widget, ToolXfTaper *klass);
	static void doneClicked(GtkWidget *widget, ToolXfTaper *klass);
};



#endif
