/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFTRANSLATE_CPP
#define __TOOLXFTRANSLATE_CPP

#include <gtk/gtkcheckbutton.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXfTranslate.h"

#include "JournalModel.h"
#include "Matrix4.h"





void* __create_ProcXfTranslate()
{
	return (void*)( new ProcXfTranslate() );
}

ObjectType type_ProcXfTranslate( "ProcXfTranslate", "ProcXform",
											__create_ProcXfTranslate );






ProcXfTranslate::ProcXfTranslate() : ProcXform()
{
	xlat = Vector3();
}

ProcXfTranslate::ProcXfTranslate(const Vector3& x) : ProcXform()
{
	xlat = x;
}

ProcXfTranslate::~ProcXfTranslate()
{
}



void ProcXfTranslate::read(FILE *f)
{
	ProcXform::read(f);

	xlat.read( f );
}

void ProcXfTranslate::write(FILE *f)
{
	ProcXform::write(f);

	xlat.write( f );
}





string ProcXfTranslate::getDescription() const
{
	return "Translate";
}

const ObjectType* ProcXfTranslate::getType() const
{
	return &type_ProcXfTranslate;
}

const ObjectType* ProcXfTranslate::getTypeStatic()
{
	return &type_ProcXfTranslate;
}





JournalProcess* ProcXfTranslate::clone() const
{
	return new ProcXfTranslate(xlat);
}


void ProcXfTranslate::setTranslation(const Vector3& x)
{
	xlat = x;
	modified();
}

const Vector3& ProcXfTranslate::getTranslation() const
{
	return xlat;
}



void ProcXfTranslate::transformModel(JournalModel *model) const
{
	Matrix4 m = Matrix4_translate( xlat );
	model->getModel()->transform(m);
}






ToolXfTranslate::ToolXfTranslate(CadView *v, ObjPainterManager *pman)
		: ToolXformClickAndDrag( v, pman, "Xlat", "Translate", 1 )
{
	snapper = VectorSnap( Vector3() );
	translateXFlag = translateYFlag = translateZFlag = true;


	//build the widgets
	//translation axis check buttons
	translateXCheck = gtk_check_button_new_with_label("X-Translate");
	translateYCheck = gtk_check_button_new_with_label("Y-Translate");
	translateZCheck = gtk_check_button_new_with_label("Z-Translate");
	gtk_widget_ref(translateXCheck);
	gtk_widget_ref(translateYCheck);
	gtk_widget_ref(translateZCheck);
	gtk_widget_show(translateXCheck);
	gtk_widget_show(translateYCheck);
	gtk_widget_show(translateZCheck);

	//connect the check button signals
	gtk_signal_connect(GTK_OBJECT(translateXCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &translateXFlag);
	gtk_signal_connect(GTK_OBJECT(translateYCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &translateYFlag);
	gtk_signal_connect(GTK_OBJECT(translateZCheck), "clicked",
							 GTK_SIGNAL_FUNC(checkResponse), &translateZFlag);

	//default is all true
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(translateXCheck), translateXFlag );
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(translateYCheck), translateYFlag );
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(translateZCheck), translateZFlag );


	
	//snap
	snapLabel = gtk_label_new("snap to grid:");
	gtk_widget_ref(snapLabel);
	gtk_widget_show(snapLabel);
	xSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	ySnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	zSnapEntry = new NumericEntry( 0.0, 0.0, 1.0e8, 0.01, 0.5, 0.3, 4,
											 NULL, NULL );
	gtk_widget_show( xSnapEntry->getWidget() );
	gtk_widget_show( ySnapEntry->getWidget() );
	gtk_widget_show( zSnapEntry->getWidget() );



	//translation
	xlatLabel = gtk_label_new("translate:");
	gtk_widget_ref(xlatLabel);
	gtk_widget_show(xlatLabel);
	xXlatEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(translationChanged), this );
	yXlatEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(translationChanged), this );
	zXlatEntry = new NumericEntry( 0.0, -1.0e8, 1.0e8, 0.1, 5.0, 0.3, 4,
											 GTK_SIGNAL_FUNC(translationChanged), this );
	gtk_widget_show( xXlatEntry->getWidget() );
	gtk_widget_show( yXlatEntry->getWidget() );
	gtk_widget_show( zXlatEntry->getWidget() );



	//done button
	doneButton = gtk_button_new_with_label("Done");
	gtk_widget_ref(doneButton);
	gtk_widget_show(doneButton);



	//connect done button signal
	gtk_signal_connect(GTK_OBJECT(doneButton), "clicked",
							 GTK_SIGNAL_FUNC(doneClicked), this);
	
		
	
	//add to the layout
	layout.add(translateXCheck);
	layout.newLine();
	layout.add(translateYCheck);
	layout.newLine();
	layout.add(translateZCheck);
	layout.lineSpace(1);
	layout.addAsLine(snapLabel);
	layout.add( xSnapEntry->getWidget() );
	layout.add( ySnapEntry->getWidget() );
	layout.add( zSnapEntry->getWidget() );
	layout.lineSpace(1);
	layout.addAsLine(xlatLabel);
	layout.add( xXlatEntry->getWidget() );
	layout.add( yXlatEntry->getWidget() );
	layout.add( zXlatEntry->getWidget() );
	layout.lineSpace(1);
	layout.add(doneButton);
	layout.newLine();
}

ToolXfTranslate::~ToolXfTranslate()
{
	//delete vector widgets
	delete xSnapEntry;
	delete ySnapEntry;
	delete zSnapEntry;
	delete xXlatEntry;
	delete yXlatEntry;
	delete zXlatEntry;

	//unref widgets
	gtk_widget_unref(translateXCheck);
	gtk_widget_unref(translateYCheck);
	gtk_widget_unref(translateZCheck);
	gtk_widget_unref(snapLabel);
	gtk_widget_unref(xlatLabel);
	gtk_widget_unref(doneButton);
}



const ObjectType* ToolXfTranslate::getProcessType() const
{
	return ProcXfTranslate::getTypeStatic();
}



void ToolXfTranslate::initialise()
{
	ToolXformClickAndDrag::initialise();

	if ( hasProcess() )
	{
		ProcXfTranslate *proc = (ProcXfTranslate*)journalProcess;
		Vector3 translation = proc->getTranslation();
		xXlatEntry->setValue( translation.x );
		yXlatEntry->setValue( translation.y );
		zXlatEntry->setValue( translation.z );
	}
}



void ToolXfTranslate::start()
{
	ToolXformClickAndDrag::start();

	if ( hasProcess() )
	{
		//get and store the initial translation
		ProcXfTranslate *proc = (ProcXfTranslate*)journalProcess;
		initialTranslationValue = proc->getTranslation();

		Vector3 snapGranularity = Vector3( xSnapEntry->getValue(),
													  ySnapEntry->getValue(),
													  zSnapEntry->getValue() );
					
		snapper = VectorSnap( snapGranularity );
	}
}

void ToolXfTranslate::finish(bool terminated)
{
	ToolXformClickAndDrag::finish(terminated);
	
	if ( hasProcess() )
	{
		ProcXfTranslate *proc = (ProcXfTranslate*)journalProcess;
		if (terminated)
		{
			//set back to the initial translation
			proc->setTranslation( initialTranslationValue );
			xXlatEntry->setValue( initialTranslationValue.x );
			yXlatEntry->setValue( initialTranslationValue.y );
			zXlatEntry->setValue( initialTranslationValue.z );
			modified();
		}
	}
}

void ToolXfTranslate::function()
{
	ToolXformClickAndDrag::function();
	
	if ( hasProcess() )
	{
		ProcXfTranslate *proc = (ProcXfTranslate*)journalProcess;

		Vector3 translation = currentCursorPosition.getWorld() -
									 startCursorPosition.getWorld();


		if (!translateXFlag)
		{
			translation.x = 0.0;
		}
		if (!translateYFlag)
		{
			translation.y = 0.0;
		}
		if (!translateZFlag)
		{
			translation.z = 0.0;
		}

		translation += initialTranslationValue;

		translation = snapper.snap(translation);

		xXlatEntry->setValue( translation.x );
		yXlatEntry->setValue( translation.y );
		zXlatEntry->setValue( translation.z );

		proc->setTranslation(translation);

	
		modified();
	}
}



void ToolXfTranslate::checkResponse(GtkWidget *widget, bool *flag)
{
	//set the flag to the value of the button
	if (GTK_TOGGLE_BUTTON(widget)->active)
	{
		*flag = true;
	}
	else
	{
		*flag = false;
	}
}

void ToolXfTranslate::translationChanged(NumericEntry *widget,
													  ToolXfTranslate *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXfTranslate *proc = (ProcXfTranslate*)klass->journalProcess;
		Vector3 translation = Vector3( klass->xXlatEntry->getValue(),
												 klass->yXlatEntry->getValue(),
												 klass->zXlatEntry->getValue() );
		proc->setTranslation(translation);
		klass->modified();
	}
}

void ToolXfTranslate::doneClicked(GtkWidget *widget, ToolXfTranslate *klass)
{
	klass->done();
}





#endif

