/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFTRANSLATE_H
#define __TOOLXFTRANSLATE_H


#include <string>


#include "Vector3.h"
#include "ProcXform.h"
#include "VectorSnap.h"
#include "NumericEntry.h"
#include "ToolXformClickAndDrag.h"



class ProcXfTranslate : public ProcXform
{
private:
	Vector3 xlat;
	
public:
	ProcXfTranslate();
	ProcXfTranslate(const Vector3& x);
	virtual ~ProcXfTranslate();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setTranslation(const Vector3& x);
	const Vector3& getTranslation() const;

protected:
	virtual void transformModel(JournalModel *m) const;
};



class ToolXfTranslate : public ToolXformClickAndDrag
{
private:
	VectorSnap snapper;
	GtkWidget *translateXCheck, *translateYCheck, *translateZCheck;

	GtkWidget *snapLabel;
	NumericEntry *xSnapEntry, *ySnapEntry, *zSnapEntry;

	GtkWidget *xlatLabel;
	NumericEntry *xXlatEntry, *yXlatEntry, *zXlatEntry;
	
	GtkWidget *doneButton;

	bool translateXFlag, translateYFlag, translateZFlag;

	Vector3 initialTranslationValue;


public:
	ToolXfTranslate(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfTranslate();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();
	
	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();

private:
	static void checkResponse(GtkWidget *widget, bool *flag);
	static void translationChanged(NumericEntry *widget, ToolXfTranslate *klass);
	static void doneClicked(GtkWidget *widget, ToolXfTranslate *klass);
};



#endif
