/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFTWIST_H
#define __TOOLXFTWIST_H


#include <string>


#include "Vector3.h"
#include "Axes3.h"
#include "ProcXform.h"
#include "ValueSnap.h"
#include "ToolXformCentreAxis.h"
#include "NumericEntry.h"
#include "TwistTransformation.h"



class ProcXfTwist : public ProcXformCentreAxis
{
private:
	double twistAngle;
	TwistTransformation::TwistType twistType;
	
public:
	ProcXfTwist();
	ProcXfTwist(Centre cen, Axis a, double t,
					TwistTransformation::TwistType type);
	virtual ~ProcXfTwist();


	virtual void read(FILE *f);
	virtual void write(FILE *f);



	virtual string getDescription() const;
	virtual const ObjectType* getType() const;
	static const ObjectType* getTypeStatic();

	virtual JournalProcess* clone() const;


	void setTwist(double t);
	void setTwistType(TwistTransformation::TwistType type);
	double getTwist() const;
	TwistTransformation::TwistType getTwistType() const;

protected:
	virtual void transformModel(JournalModel *model, const Point3& centre,
										 const Matrix4& axesMatrix,
										 const Matrix4& axesMatrixInverse,
										 const BBox& boundingBoxInTransformationSpace) const;
};



class ToolXfTwist : public ToolXformCentreAxis
{
private:
	GtkWidget *typeLabel, *radioLinear, *radioRadial;
	
	GtkWidget *snapLabel;
	NumericEntry *snapEntry;

	GtkWidget *twistLabel;
	NumericEntry *twistEntry;

	GtkWidget *doneButton;


	ValueSnap snapper;


	double initialTwistValue;

public:
	ToolXfTwist(CadView *v, ObjPainterManager *pman);
	virtual ~ToolXfTwist();


	virtual const ObjectType* getProcessType() const;


protected:
	virtual void initialise();

	virtual void start();
	virtual void finish(bool terminated);
	virtual void function();
	
private:
	static void linearClicked(GtkWidget *widget, ToolXfTwist *klass);
	static void radialClicked(GtkWidget *widget, ToolXfTwist *klass);
	static void twistChanged(NumericEntry *widget, ToolXfTwist *klass);
	static void doneClicked(GtkWidget *widget, ToolXfTwist *klass);
};



#endif
