/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFORMAXIS_CPP
#define __TOOLXFORMAXIS_CPP

#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXformAxis.h"

#include "JournalModel.h"





void* __create_ProcXformAxis()
{
	return (void*)( new ProcXformAxis() );
}

ObjectType type_ProcXformAxis( "ProcXformAxis", "ProcXform",
										 __create_ProcXformAxis );






ProcXformAxis::ProcXformAxis() : ProcXform()
{
	axis = AXIS_Z;
}

ProcXformAxis::ProcXformAxis(Axis a) : ProcXform()
{
	axis = a;
}

ProcXformAxis::~ProcXformAxis()
{
}



void ProcXformAxis::read(FILE *f)
{
	ProcXform::read(f);

	fread(&axis, sizeof(Axis), 1, f);
}

void ProcXformAxis::write(FILE *f)
{
	ProcXform::write(f);

	fwrite(&axis, sizeof(Axis), 1, f);
}





string ProcXformAxis::getDescription() const
{
	return "Transformation Axis";
}

const ObjectType* ProcXformAxis::getType() const
{
	return &type_ProcXformAxis;
}

const ObjectType* ProcXformAxis::getTypeStatic()
{
	return &type_ProcXformAxis;
}



JournalProcess* ProcXformAxis::clone() const
{
	return new ProcXformAxis(axis);
}



void ProcXformAxis::setAxis(Axis a)
{
	axis = a;
	modified();
}

Axis ProcXformAxis::getAxis() const
{
	return axis;
}




void ProcXformAxis::transformModel(JournalModel *model) const
{
	Point3 centre;
	Axes3 axes(axis);
	
	Matrix4 axesMatrixInverse = axes.matrixTransformation();
	Matrix4 axesMatrix = ~axesMatrixInverse;


	BBox boundingBoxInTransformationSpace = model->getModel()->getBBox(axesMatrix);

	centre = boundingBoxInTransformationSpace.getCentre();
	
	transformModel(model, centre, axesMatrix, axesMatrixInverse,
						boundingBoxInTransformationSpace);
}

void ProcXformAxis::transformModel(JournalModel *model, const Point3& centre,
									const Matrix4& axesMatrix,
									const Matrix4& axesMatrixInverse,
									const BBox& boundingBoxInTransformationSpace) const
{
}






ToolXformAxis::ToolXformAxis(CadView *v,
									  ObjPainterManager *pman,
									  const string& funcName,
									  const string& optTitle,
									  int tableWidth)
			: ToolXformClickAndDrag( v, pman, funcName, optTitle, tableWidth )
{
	axisChooser = new CadToolAxisChooser( GTK_SIGNAL_FUNC(axisChanged), this );

	axisChooser->addToLayout(layout);
}

ToolXformAxis::~ToolXformAxis()
{
	delete axisChooser;
}



const ObjectType* ToolXformAxis::getProcessType() const
{
	return ProcXformAxis::getTypeStatic();
}



void ToolXformAxis::initialise()
{
	ToolXformClickAndDrag::initialise();

	ProcXformAxis *proc = (ProcXformAxis*)journalProcess;
	axisChooser->setChoice( proc->getAxis() );
}


void ToolXformAxis::axisChanged(GtkWidget *widget, ToolXformAxis *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXformAxis *proc = (ProcXformAxis*)klass->journalProcess;
		proc->setAxis( klass->axisChooser->getChoice() );
		klass->modified();
	}
}



#endif

