/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFORMCENTRE_CPP
#define __TOOLXFORMCENTRE_CPP

#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXformCentre.h"

#include "JournalModel.h"





void* __create_ProcXformCentre()
{
	return (void*)( new ProcXformCentre() );
}

ObjectType type_ProcXformCentre( "ProcXformCentre", "ProcXform",
											__create_ProcXformCentre );






ProcXformCentre::ProcXformCentre() : ProcXform()
{
	whichCentre = MEDIAN;
}

ProcXformCentre::ProcXformCentre(Centre cen) : ProcXform()
{
	whichCentre = cen;
}

ProcXformCentre::~ProcXformCentre()
{
}



void ProcXformCentre::read(FILE *f)
{
	ProcXform::read(f);

	fread(&whichCentre, sizeof(Centre), 1, f);
}

void ProcXformCentre::write(FILE *f)
{
	ProcXform::write(f);

	fwrite(&whichCentre, sizeof(Centre), 1, f);
}





string ProcXformCentre::getDescription() const
{
	return "Transformation Centre";
}

const ObjectType* ProcXformCentre::getType() const
{
	return &type_ProcXformCentre;
}

const ObjectType* ProcXformCentre::getTypeStatic()
{
	return &type_ProcXformCentre;
}



JournalProcess* ProcXformCentre::clone() const
{
	return new ProcXformCentre(whichCentre);
}



void ProcXformCentre::setCentre(Centre cen)
{
	whichCentre = cen;
	modified();
}

Centre ProcXformCentre::getCentre() const
{
	return whichCentre;
}



void ProcXformCentre::transformModel(JournalModel *model) const
{
	if (whichCentre == MEDIAN)
	{
		transformModel( model, model->getModel()->getBBox().getCentre() );
	}
	else
	{
		transformModel( model, Point3() );
	}
}

void ProcXformCentre::transformModel(JournalModel *model, const Point3& centre) const
{
}






ToolXformCentre::ToolXformCentre(CadView *v,
											ObjPainterManager *pman,
											const string& funcName,
											const string& optTitle,
											int tableWidth)
			: ToolXformClickAndDrag( v, pman, funcName, optTitle, tableWidth )
{
	centreChooser = new CadToolCentreChooser( GTK_SIGNAL_FUNC(centreChanged),
															this );

	centreChooser->addToLayout(layout);
}

ToolXformCentre::~ToolXformCentre()
{
	delete centreChooser;
}



const ObjectType* ToolXformCentre::getProcessType() const
{
	return ProcXformCentre::getTypeStatic();
}



void ToolXformCentre::initialise()
{
	ToolXformClickAndDrag::initialise();

	if ( hasProcess() )
	{
		ProcXformCentre *proc = (ProcXformCentre*)journalProcess;
		centreChooser->setChoice( proc->getCentre() );
	}
}



void ToolXformCentre::centreChanged(GtkWidget *widget, ToolXformCentre *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXformCentre *proc = (ProcXformCentre*)klass->journalProcess;
		proc->setCentre( klass->centreChooser->getChoice() );
		klass->modified();
	}
}



#endif
