/************************
 This code is released under the terms of the GNU GPL, v2 (or any later version).
 The GNU GPL can be found in the file 'COPYING'.  Copyright Geoffrey French 1999-2001.
 ************************/
#ifndef __TOOLXFORMCENTREAXIS_CPP
#define __TOOLXFORMCENTREAXIS_CPP

#include <gtk/gtksignal.h>

#include <strstream>

#include "ToolXformCentreAxis.h"

#include "JournalModel.h"





void* __create_ProcXformCentreAxis()
{
	return (void*)( new ProcXformCentreAxis() );
}

ObjectType type_ProcXformCentreAxis( "ProcXformCentreAxis", "ProcXformCentre",
												 __create_ProcXformCentreAxis );






ProcXformCentreAxis::ProcXformCentreAxis() : ProcXformCentre()
{
	axis = AXIS_Z;
}

ProcXformCentreAxis::ProcXformCentreAxis(Centre cen, Axis a)
													  : ProcXformCentre(cen)
{
	axis = a;
}

ProcXformCentreAxis::~ProcXformCentreAxis()
{
}



void ProcXformCentreAxis::read(FILE *f)
{
	ProcXformCentre::read(f);

	fread(&axis, sizeof(Axis), 1, f);
}

void ProcXformCentreAxis::write(FILE *f)
{
	ProcXformCentre::write(f);

	fwrite(&axis, sizeof(Axis), 1, f);
}





string ProcXformCentreAxis::getDescription() const
{
	return "Transformation Centre and Axes";
}

const ObjectType* ProcXformCentreAxis::getType() const
{
	return &type_ProcXformCentreAxis;
}

const ObjectType* ProcXformCentreAxis::getTypeStatic()
{
	return &type_ProcXformCentreAxis;
}



JournalProcess* ProcXformCentreAxis::clone() const
{
	return new ProcXformCentreAxis(whichCentre, axis);
}



void ProcXformCentreAxis::setAxis(Axis a)
{
	axis = a;
	modified();
}

Axis ProcXformCentreAxis::getAxis() const
{
	return axis;
}




void ProcXformCentreAxis::transformModel(JournalModel *model) const
{
	Point3 centre;
	Axes3 axes(axis);
	
	Matrix4 axesMatrixInverse = axes.matrixTransformation();
	Matrix4 axesMatrix = ~axesMatrixInverse;


	BBox boundingBoxInTransformationSpace = model->getModel()->getBBox(axesMatrix);

	if (whichCentre == MEDIAN)
	{
		centre = boundingBoxInTransformationSpace.getCentre();
	}
	else
	{
		centre = Point3();
	}
	
	transformModel(model, centre, axesMatrix, axesMatrixInverse,
						boundingBoxInTransformationSpace);
}

void ProcXformCentreAxis::transformModel(JournalModel *model, const Point3& centre,
									const Matrix4& axesMatrix,
									const Matrix4& axesMatrixInverse,
									const BBox& boundingBoxInTransformationSpace) const
{
}






ToolXformCentreAxis::ToolXformCentreAxis(CadView *v,
													  ObjPainterManager *pman,
													  const string& funcName,
													  const string& optTitle,
													  int tableWidth)
			: ToolXformCentre(v, pman, funcName, optTitle, tableWidth)
{
	axisChooser = new CadToolAxisChooser( GTK_SIGNAL_FUNC(axisChanged), this );

	axisChooser->addToLayout(layout);
}

ToolXformCentreAxis::~ToolXformCentreAxis()
{
	delete axisChooser;
}



const ObjectType* ToolXformCentreAxis::getProcessType() const
{
	return ProcXformCentreAxis::getTypeStatic();
}



void ToolXformCentreAxis::initialise()
{
	ToolXformCentre::initialise();

	if ( hasProcess() )
	{
		ProcXformCentreAxis *proc = (ProcXformCentreAxis*)journalProcess;
		axisChooser->setChoice( proc->getAxis() );
	}
}


void ToolXformCentreAxis::axisChanged(GtkWidget *widget,
												  ToolXformCentreAxis *klass)
{
	if ( klass->hasProcess() )
	{
		ProcXformCentreAxis *proc = (ProcXformCentreAxis*)klass->journalProcess;
		proc->setAxis( klass->axisChooser->getChoice() );
		klass->modified();
	}
}



#endif

